/********************************************************************************
** Form generated from reading UI file 'soundfonteditortop.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTEDITORTOP_H
#define UI_SOUNDFONTEDITORTOP_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>
#include "styledlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontEditorTop
{
public:
    QHBoxLayout *horizontalLayout;
    StyledLineEdit *lineEdit_name;

    void setupUi(QWidget *SoundfontEditorTop)
    {
        if (SoundfontEditorTop->objectName().isEmpty())
            SoundfontEditorTop->setObjectName("SoundfontEditorTop");
        SoundfontEditorTop->resize(400, 27);
        SoundfontEditorTop->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(SoundfontEditorTop);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(7, 0, -1, 0);
        lineEdit_name = new StyledLineEdit(SoundfontEditorTop);
        lineEdit_name->setObjectName("lineEdit_name");
        QFont font;
        font.setBold(true);
        lineEdit_name->setFont(font);
        lineEdit_name->setMaxLength(255);

        horizontalLayout->addWidget(lineEdit_name);


        retranslateUi(SoundfontEditorTop);

        QMetaObject::connectSlotsByName(SoundfontEditorTop);
    } // setupUi

    void retranslateUi(QWidget *SoundfontEditorTop)
    {
        lineEdit_name->setPlaceholderText(QCoreApplication::translate("SoundfontEditorTop", "Title...", nullptr));
        (void)SoundfontEditorTop;
    } // retranslateUi

};

namespace Ui {
    class SoundfontEditorTop: public Ui_SoundfontEditorTop {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTEDITORTOP_H
