/********************************************************************************
** Form generated from reading UI file 'toolfasteditsmpl_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLFASTEDITSMPL_GUI_H
#define UI_TOOLFASTEDITSMPL_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolFastEditSmpl_gui
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QDoubleSpinBox *doubleSpinMultiply;
    QRadioButton *radioAdd;
    QRadioButton *radioMultiply;
    QComboBox *comboParameter;
    QLabel *label;
    QSpinBox *spinAdd;

    void setupUi(QWidget *ToolFastEditSmpl_gui)
    {
        if (ToolFastEditSmpl_gui->objectName().isEmpty())
            ToolFastEditSmpl_gui->setObjectName("ToolFastEditSmpl_gui");
        ToolFastEditSmpl_gui->resize(207, 136);
        ToolFastEditSmpl_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolFastEditSmpl_gui);
        gridLayout->setObjectName("gridLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolFastEditSmpl_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolFastEditSmpl_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 2);

        doubleSpinMultiply = new QDoubleSpinBox(ToolFastEditSmpl_gui);
        doubleSpinMultiply->setObjectName("doubleSpinMultiply");
        doubleSpinMultiply->setEnabled(false);
        doubleSpinMultiply->setMinimum(-100.000000000000000);
        doubleSpinMultiply->setMaximum(100.000000000000000);
        doubleSpinMultiply->setSingleStep(0.100000000000000);

        gridLayout->addWidget(doubleSpinMultiply, 2, 1, 1, 1);

        radioAdd = new QRadioButton(ToolFastEditSmpl_gui);
        radioAdd->setObjectName("radioAdd");

        gridLayout->addWidget(radioAdd, 1, 0, 1, 1);

        radioMultiply = new QRadioButton(ToolFastEditSmpl_gui);
        radioMultiply->setObjectName("radioMultiply");

        gridLayout->addWidget(radioMultiply, 2, 0, 1, 1);

        comboParameter = new QComboBox(ToolFastEditSmpl_gui);
        comboParameter->setObjectName("comboParameter");

        gridLayout->addWidget(comboParameter, 0, 1, 1, 1);

        label = new QLabel(ToolFastEditSmpl_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        spinAdd = new QSpinBox(ToolFastEditSmpl_gui);
        spinAdd->setObjectName("spinAdd");
        spinAdd->setMinimum(-1000000);
        spinAdd->setMaximum(1000000);

        gridLayout->addWidget(spinAdd, 1, 1, 1, 1);

        QWidget::setTabOrder(comboParameter, radioAdd);
        QWidget::setTabOrder(radioAdd, spinAdd);
        QWidget::setTabOrder(spinAdd, radioMultiply);
        QWidget::setTabOrder(radioMultiply, doubleSpinMultiply);
        QWidget::setTabOrder(doubleSpinMultiply, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolFastEditSmpl_gui);

        QMetaObject::connectSlotsByName(ToolFastEditSmpl_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolFastEditSmpl_gui)
    {
        pushOk->setText(QCoreApplication::translate("ToolFastEditSmpl_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolFastEditSmpl_gui", "&Cancel", nullptr));
        radioAdd->setText(QCoreApplication::translate("ToolFastEditSmpl_gui", "Add", nullptr));
        radioMultiply->setText(QCoreApplication::translate("ToolFastEditSmpl_gui", "Multiply", nullptr));
        label->setText(QCoreApplication::translate("ToolFastEditSmpl_gui", "Parameter", nullptr));
        (void)ToolFastEditSmpl_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolFastEditSmpl_gui: public Ui_ToolFastEditSmpl_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLFASTEDITSMPL_GUI_H
