/********************************************************************************
** Form generated from reading UI file 'dialogchangelog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGCHANGELOG_H
#define UI_DIALOGCHANGELOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_DialogChangeLog
{
public:
    QGridLayout *gridLayout;
    QLabel *labelLogo;
    QLabel *labelTitle;
    QTextBrowser *textChangelog;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushDonate;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushOk;

    void setupUi(QDialog *DialogChangeLog)
    {
        if (DialogChangeLog->objectName().isEmpty())
            DialogChangeLog->setObjectName("DialogChangeLog");
        DialogChangeLog->setWindowModality(Qt::WindowModal);
        DialogChangeLog->resize(414, 423);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogChangeLog->setWindowIcon(icon);
        DialogChangeLog->setModal(true);
        gridLayout = new QGridLayout(DialogChangeLog);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(15);
        gridLayout->setVerticalSpacing(12);
        labelLogo = new QLabel(DialogChangeLog);
        labelLogo->setObjectName("labelLogo");
        labelLogo->setMinimumSize(QSize(100, 100));
        labelLogo->setMaximumSize(QSize(100, 100));
        labelLogo->setScaledContents(true);

        gridLayout->addWidget(labelLogo, 0, 0, 1, 1);

        labelTitle = new QLabel(DialogChangeLog);
        labelTitle->setObjectName("labelTitle");
        QFont font;
        font.setPointSize(15);
        font.setBold(true);
        labelTitle->setFont(font);
#if QT_CONFIG(tooltip)
        labelTitle->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelTitle->setText(QString::fromUtf8("Version ..."));
        labelTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(labelTitle, 0, 1, 1, 1);

        textChangelog = new QTextBrowser(DialogChangeLog);
        textChangelog->setObjectName("textChangelog");
        textChangelog->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);

        gridLayout->addWidget(textChangelog, 1, 0, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushDonate = new QPushButton(DialogChangeLog);
        pushDonate->setObjectName("pushDonate");

        horizontalLayout->addWidget(pushDonate);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushOk = new QPushButton(DialogChangeLog);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 2);

        QWidget::setTabOrder(textChangelog, pushDonate);
        QWidget::setTabOrder(pushDonate, pushOk);

        retranslateUi(DialogChangeLog);

        QMetaObject::connectSlotsByName(DialogChangeLog);
    } // setupUi

    void retranslateUi(QDialog *DialogChangeLog)
    {
        DialogChangeLog->setWindowTitle(QCoreApplication::translate("DialogChangeLog", "Welcome", nullptr));
        labelLogo->setText(QString());
        pushDonate->setText(QCoreApplication::translate("DialogChangeLog", "&Donate", nullptr));
        pushOk->setText(QCoreApplication::translate("DialogChangeLog", "&Ok", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogChangeLog: public Ui_DialogChangeLog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGCHANGELOG_H
