/********************************************************************************
** Form generated from reading UI file 'toolchangeattenuation_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLCHANGEATTENUATION_GUI_H
#define UI_TOOLCHANGEATTENUATION_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolChangeAttenuation_gui
{
public:
    QGridLayout *gridLayout;
    QLabel *labelRange;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QLabel *labelPossibleOffset;
    QLabel *label_2;
    QDoubleSpinBox *doubleSpinValue;
    QLabel *label;
    QLabel *label_4;

    void setupUi(QWidget *ToolChangeAttenuation_gui)
    {
        if (ToolChangeAttenuation_gui->objectName().isEmpty())
            ToolChangeAttenuation_gui->setObjectName("ToolChangeAttenuation_gui");
        ToolChangeAttenuation_gui->resize(219, 108);
        ToolChangeAttenuation_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolChangeAttenuation_gui);
        gridLayout->setObjectName("gridLayout");
        labelRange = new QLabel(ToolChangeAttenuation_gui);
        labelRange->setObjectName("labelRange");
        QFont font;
        font.setBold(false);
        labelRange->setFont(font);
        labelRange->setText(QString::fromUtf8("labelRange"));

        gridLayout->addWidget(labelRange, 0, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolChangeAttenuation_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolChangeAttenuation_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 2);

        labelPossibleOffset = new QLabel(ToolChangeAttenuation_gui);
        labelPossibleOffset->setObjectName("labelPossibleOffset");
        labelPossibleOffset->setFont(font);
        labelPossibleOffset->setText(QString::fromUtf8("labelPossibleOffset"));

        gridLayout->addWidget(labelPossibleOffset, 1, 1, 1, 1);

        label_2 = new QLabel(ToolChangeAttenuation_gui);
        label_2->setObjectName("label_2");
        QFont font1;
        font1.setBold(true);
        label_2->setFont(font1);

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        doubleSpinValue = new QDoubleSpinBox(ToolChangeAttenuation_gui);
        doubleSpinValue->setObjectName("doubleSpinValue");
        doubleSpinValue->setFont(font);
        doubleSpinValue->setDecimals(2);

        gridLayout->addWidget(doubleSpinValue, 2, 1, 1, 1);

        label = new QLabel(ToolChangeAttenuation_gui);
        label->setObjectName("label");
        label->setFont(font1);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_4 = new QLabel(ToolChangeAttenuation_gui);
        label_4->setObjectName("label_4");
        label_4->setFont(font1);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);


        retranslateUi(ToolChangeAttenuation_gui);

        QMetaObject::connectSlotsByName(ToolChangeAttenuation_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolChangeAttenuation_gui)
    {
        pushOk->setText(QCoreApplication::translate("ToolChangeAttenuation_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolChangeAttenuation_gui", "&Cancel", nullptr));
        label_2->setText(QCoreApplication::translate("ToolChangeAttenuation_gui", "Desired value", nullptr));
        doubleSpinValue->setSuffix(QCoreApplication::translate("ToolChangeAttenuation_gui", " dB", nullptr));
        label->setText(QCoreApplication::translate("ToolChangeAttenuation_gui", "Attenuation range", nullptr));
        label_4->setText(QCoreApplication::translate("ToolChangeAttenuation_gui", "Possible offset", nullptr));
        (void)ToolChangeAttenuation_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolChangeAttenuation_gui: public Ui_ToolChangeAttenuation_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLCHANGEATTENUATION_GUI_H
