/********************************************************************************
** Form generated from reading UI file 'toolchords_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLCHORDS_GUI_H
#define UI_TOOLCHORDS_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolChords_gui
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QLineEdit *lineName;
    QLabel *label_4;
    QComboBox *comboFreq;
    QCheckBox *checkLoop;
    QCheckBox *checkStereo;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *labelType9;
    QLabel *label_7;
    QLabel *label_8;
    QLabel *label_2;
    QSlider *sliderType5;
    QLabel *label;
    QLabel *label_6;
    QSlider *sliderType7;
    QComboBox *comboType3;
    QComboBox *comboType1;
    QSlider *sliderType1;
    QComboBox *comboType9;
    QSlider *sliderType3;
    QLabel *label_5;
    QComboBox *comboType7;
    QSlider *sliderType9;
    QComboBox *comboType5;
    QLabel *labelType1;
    QLabel *labelType5;
    QLabel *labelType3;
    QSpinBox *spinInversions;
    QLabel *labelType7;
    QGridLayout *gridLayout_4;
    QSpacerItem *horizontalSpacer_2;
    QSlider *sliderOctave;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout_3;
    QLabel *label_11;
    QLabel *label_10;
    QLabel *label_12;
    QLabel *label_14;
    QLabel *label_13;
    QLabel *label_15;
    QLabel *label_16;
    QLabel *label_9;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolChords_gui)
    {
        if (ToolChords_gui->objectName().isEmpty())
            ToolChords_gui->setObjectName("ToolChords_gui");
        ToolChords_gui->resize(357, 410);
        ToolChords_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolChords_gui);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        label_3 = new QLabel(ToolChords_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        lineName = new QLineEdit(ToolChords_gui);
        lineName->setObjectName("lineName");
        lineName->setMaxLength(20);

        gridLayout->addWidget(lineName, 0, 1, 1, 1);

        label_4 = new QLabel(ToolChords_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        comboFreq = new QComboBox(ToolChords_gui);
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->setObjectName("comboFreq");

        gridLayout->addWidget(comboFreq, 1, 1, 1, 1);

        checkLoop = new QCheckBox(ToolChords_gui);
        checkLoop->setObjectName("checkLoop");
        checkLoop->setChecked(true);

        gridLayout->addWidget(checkLoop, 1, 2, 1, 1);

        checkStereo = new QCheckBox(ToolChords_gui);
        checkStereo->setObjectName("checkStereo");

        gridLayout->addWidget(checkStereo, 0, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        groupBox = new QGroupBox(ToolChords_gui);
        groupBox->setObjectName("groupBox");
        QFont font;
        font.setBold(true);
        groupBox->setFont(font);
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        labelType9 = new QLabel(groupBox);
        labelType9->setObjectName("labelType9");
        QFont font1;
        font1.setBold(false);
        labelType9->setFont(font1);

        gridLayout_2->addWidget(labelType9, 4, 3, 1, 1);

        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");
        label_7->setFont(font1);

        gridLayout_2->addWidget(label_7, 3, 0, 1, 1);

        label_8 = new QLabel(groupBox);
        label_8->setObjectName("label_8");
        label_8->setFont(font1);

        gridLayout_2->addWidget(label_8, 4, 0, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");
        label_2->setFont(font1);

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        sliderType5 = new QSlider(groupBox);
        sliderType5->setObjectName("sliderType5");
        sliderType5->setFont(font1);
        sliderType5->setMinimum(-1440);
        sliderType5->setMaximum(0);
        sliderType5->setSingleStep(10);
        sliderType5->setPageStep(100);
        sliderType5->setOrientation(Qt::Orientation::Horizontal);

        gridLayout_2->addWidget(sliderType5, 2, 2, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");
        label->setFont(font1);

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");
        label_6->setFont(font1);

        gridLayout_2->addWidget(label_6, 2, 0, 1, 1);

        sliderType7 = new QSlider(groupBox);
        sliderType7->setObjectName("sliderType7");
        sliderType7->setFont(font1);
        sliderType7->setMinimum(-1440);
        sliderType7->setMaximum(0);
        sliderType7->setSingleStep(10);
        sliderType7->setPageStep(100);
        sliderType7->setOrientation(Qt::Orientation::Horizontal);

        gridLayout_2->addWidget(sliderType7, 3, 2, 1, 1);

        comboType3 = new QComboBox(groupBox);
        comboType3->addItem(QString());
        comboType3->addItem(QString());
        comboType3->addItem(QString());
        comboType3->setObjectName("comboType3");
        comboType3->setFont(font1);

        gridLayout_2->addWidget(comboType3, 1, 1, 1, 1);

        comboType1 = new QComboBox(groupBox);
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->setObjectName("comboType1");
        comboType1->setFont(font1);

        gridLayout_2->addWidget(comboType1, 0, 1, 1, 1);

        sliderType1 = new QSlider(groupBox);
        sliderType1->setObjectName("sliderType1");
        sliderType1->setFont(font1);
        sliderType1->setMinimum(-1440);
        sliderType1->setMaximum(0);
        sliderType1->setSingleStep(10);
        sliderType1->setPageStep(100);
        sliderType1->setOrientation(Qt::Orientation::Horizontal);

        gridLayout_2->addWidget(sliderType1, 0, 2, 1, 1);

        comboType9 = new QComboBox(groupBox);
        comboType9->addItem(QString());
        comboType9->addItem(QString());
        comboType9->addItem(QString());
        comboType9->setObjectName("comboType9");
        comboType9->setFont(font1);

        gridLayout_2->addWidget(comboType9, 4, 1, 1, 1);

        sliderType3 = new QSlider(groupBox);
        sliderType3->setObjectName("sliderType3");
        sliderType3->setFont(font1);
        sliderType3->setMinimum(-1440);
        sliderType3->setMaximum(0);
        sliderType3->setSingleStep(10);
        sliderType3->setPageStep(100);
        sliderType3->setOrientation(Qt::Orientation::Horizontal);

        gridLayout_2->addWidget(sliderType3, 1, 2, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");
        label_5->setFont(font1);

        gridLayout_2->addWidget(label_5, 5, 0, 1, 1);

        comboType7 = new QComboBox(groupBox);
        comboType7->addItem(QString());
        comboType7->addItem(QString());
        comboType7->addItem(QString());
        comboType7->addItem(QString());
        comboType7->setObjectName("comboType7");
        comboType7->setFont(font1);

        gridLayout_2->addWidget(comboType7, 3, 1, 1, 1);

        sliderType9 = new QSlider(groupBox);
        sliderType9->setObjectName("sliderType9");
        sliderType9->setFont(font1);
        sliderType9->setMinimum(-1440);
        sliderType9->setMaximum(0);
        sliderType9->setSingleStep(10);
        sliderType9->setPageStep(100);
        sliderType9->setOrientation(Qt::Orientation::Horizontal);

        gridLayout_2->addWidget(sliderType9, 4, 2, 1, 1);

        comboType5 = new QComboBox(groupBox);
        comboType5->addItem(QString());
        comboType5->addItem(QString());
        comboType5->addItem(QString());
        comboType5->addItem(QString());
        comboType5->setObjectName("comboType5");
        comboType5->setFont(font1);

        gridLayout_2->addWidget(comboType5, 2, 1, 1, 1);

        labelType1 = new QLabel(groupBox);
        labelType1->setObjectName("labelType1");
        labelType1->setFont(font1);

        gridLayout_2->addWidget(labelType1, 0, 3, 1, 1);

        labelType5 = new QLabel(groupBox);
        labelType5->setObjectName("labelType5");
        labelType5->setFont(font1);

        gridLayout_2->addWidget(labelType5, 2, 3, 1, 1);

        labelType3 = new QLabel(groupBox);
        labelType3->setObjectName("labelType3");
        labelType3->setFont(font1);

        gridLayout_2->addWidget(labelType3, 1, 3, 1, 1);

        spinInversions = new QSpinBox(groupBox);
        spinInversions->setObjectName("spinInversions");
        spinInversions->setFont(font1);
        spinInversions->setMinimum(0);
        spinInversions->setMaximum(4);

        gridLayout_2->addWidget(spinInversions, 5, 1, 1, 3);

        labelType7 = new QLabel(groupBox);
        labelType7->setObjectName("labelType7");
        labelType7->setFont(font1);

        gridLayout_2->addWidget(labelType7, 3, 3, 1, 1);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        horizontalSpacer_2 = new QSpacerItem(6, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_2, 0, 2, 1, 1);

        sliderOctave = new QSlider(groupBox);
        sliderOctave->setObjectName("sliderOctave");
        sliderOctave->setMinimum(-3);
        sliderOctave->setMaximum(3);
        sliderOctave->setOrientation(Qt::Orientation::Horizontal);
        sliderOctave->setTickPosition(QSlider::TickPosition::TicksBelow);

        gridLayout_4->addWidget(sliderOctave, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(6, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 0, 0, 1, 1);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        label_11 = new QLabel(groupBox);
        label_11->setObjectName("label_11");
        label_11->setFont(font1);
        label_11->setText(QString::fromUtf8("-2"));
        label_11->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_11, 1, 2, 1, 1);

        label_10 = new QLabel(groupBox);
        label_10->setObjectName("label_10");
        label_10->setFont(font1);
        label_10->setText(QString::fromUtf8("-3"));
        label_10->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_10, 1, 1, 1, 1);

        label_12 = new QLabel(groupBox);
        label_12->setObjectName("label_12");
        label_12->setFont(font1);
        label_12->setText(QString::fromUtf8("-1"));
        label_12->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_12, 1, 3, 1, 1);

        label_14 = new QLabel(groupBox);
        label_14->setObjectName("label_14");
        label_14->setFont(font1);
        label_14->setText(QString::fromUtf8("+1"));
        label_14->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_14, 1, 5, 1, 1);

        label_13 = new QLabel(groupBox);
        label_13->setObjectName("label_13");
        label_13->setFont(font1);
        label_13->setText(QString::fromUtf8("0"));
        label_13->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_13, 1, 4, 1, 1);

        label_15 = new QLabel(groupBox);
        label_15->setObjectName("label_15");
        label_15->setFont(font1);
        label_15->setText(QString::fromUtf8("+2"));
        label_15->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_15, 1, 6, 1, 1);

        label_16 = new QLabel(groupBox);
        label_16->setObjectName("label_16");
        label_16->setFont(font1);
        label_16->setText(QString::fromUtf8("+3"));
        label_16->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_16, 1, 7, 1, 1);


        gridLayout_4->addLayout(gridLayout_3, 1, 0, 1, 3);


        gridLayout_2->addLayout(gridLayout_4, 6, 1, 1, 3);

        label_9 = new QLabel(groupBox);
        label_9->setObjectName("label_9");
        label_9->setFont(font1);

        gridLayout_2->addWidget(label_9, 6, 0, 1, 1);


        verticalLayout->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolChords_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        pushCancel = new QPushButton(ToolChords_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(lineName, checkStereo);
        QWidget::setTabOrder(checkStereo, comboFreq);
        QWidget::setTabOrder(comboFreq, checkLoop);
        QWidget::setTabOrder(checkLoop, comboType1);
        QWidget::setTabOrder(comboType1, sliderType1);
        QWidget::setTabOrder(sliderType1, comboType3);
        QWidget::setTabOrder(comboType3, sliderType3);
        QWidget::setTabOrder(sliderType3, comboType5);
        QWidget::setTabOrder(comboType5, sliderType5);
        QWidget::setTabOrder(sliderType5, comboType7);
        QWidget::setTabOrder(comboType7, sliderType7);
        QWidget::setTabOrder(sliderType7, comboType9);
        QWidget::setTabOrder(comboType9, sliderType9);
        QWidget::setTabOrder(sliderType9, spinInversions);
        QWidget::setTabOrder(spinInversions, sliderOctave);
        QWidget::setTabOrder(sliderOctave, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolChords_gui);

        QMetaObject::connectSlotsByName(ToolChords_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolChords_gui)
    {
        label_3->setText(QCoreApplication::translate("ToolChords_gui", "Instrument name:", nullptr));
        label_4->setText(QCoreApplication::translate("ToolChords_gui", "Sample creation:", nullptr));
        comboFreq->setItemText(0, QCoreApplication::translate("ToolChords_gui", "for every key", nullptr));
        comboFreq->setItemText(1, QCoreApplication::translate("ToolChords_gui", "every 3 keys", nullptr));
        comboFreq->setItemText(2, QCoreApplication::translate("ToolChords_gui", "every 6 keys", nullptr));

        checkLoop->setText(QCoreApplication::translate("ToolChords_gui", "Auto loop", nullptr));
        checkStereo->setText(QCoreApplication::translate("ToolChords_gui", "Stereo", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ToolChords_gui", "Chord", nullptr));
        labelType9->setText(QString());
        label_7->setText(QCoreApplication::translate("ToolChords_gui", "Seventh", nullptr));
        label_8->setText(QCoreApplication::translate("ToolChords_gui", "Ninth", nullptr));
        label_2->setText(QCoreApplication::translate("ToolChords_gui", "Third", nullptr));
        label->setText(QCoreApplication::translate("ToolChords_gui", "Root key", nullptr));
        label_6->setText(QCoreApplication::translate("ToolChords_gui", "Fifth", nullptr));
        comboType3->setItemText(0, QCoreApplication::translate("ToolChords_gui", "no", nullptr));
        comboType3->setItemText(1, QCoreApplication::translate("ToolChords_gui", "major", nullptr));
        comboType3->setItemText(2, QCoreApplication::translate("ToolChords_gui", "minor", nullptr));

        comboType1->setItemText(0, QCoreApplication::translate("ToolChords_gui", "no", nullptr));
        comboType1->setItemText(1, QCoreApplication::translate("ToolChords_gui", "yes", nullptr));

        comboType9->setItemText(0, QCoreApplication::translate("ToolChords_gui", "no", nullptr));
        comboType9->setItemText(1, QCoreApplication::translate("ToolChords_gui", "major", nullptr));
        comboType9->setItemText(2, QCoreApplication::translate("ToolChords_gui", "minor", nullptr));

        label_5->setText(QCoreApplication::translate("ToolChords_gui", "Inversion number", nullptr));
        comboType7->setItemText(0, QCoreApplication::translate("ToolChords_gui", "no", nullptr));
        comboType7->setItemText(1, QCoreApplication::translate("ToolChords_gui", "minor", nullptr));
        comboType7->setItemText(2, QCoreApplication::translate("ToolChords_gui", "major", nullptr));
        comboType7->setItemText(3, QCoreApplication::translate("ToolChords_gui", "diminished", nullptr));

        comboType5->setItemText(0, QCoreApplication::translate("ToolChords_gui", "no", nullptr));
        comboType5->setItemText(1, QCoreApplication::translate("ToolChords_gui", "perfect", nullptr));
        comboType5->setItemText(2, QCoreApplication::translate("ToolChords_gui", "diminished", nullptr));
        comboType5->setItemText(3, QCoreApplication::translate("ToolChords_gui", "augmented", nullptr));

        labelType1->setText(QString());
        labelType5->setText(QString());
        labelType3->setText(QString());
        labelType7->setText(QString());
        label_9->setText(QCoreApplication::translate("ToolChords_gui", "Octave", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolChords_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolChords_gui", "&Cancel", nullptr));
        (void)ToolChords_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolChords_gui: public Ui_ToolChords_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLCHORDS_GUI_H
