/********************************************************************************
** Form generated from reading UI file 'directorysortmenu.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIRECTORYSORTMENU_H
#define UI_DIRECTORYSORTMENU_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DirectorySortMenu
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *labelSort;
    QPushButton *pushButton;

    void setupUi(QWidget *DirectorySortMenu)
    {
        if (DirectorySortMenu->objectName().isEmpty())
            DirectorySortMenu->setObjectName("DirectorySortMenu");
        DirectorySortMenu->resize(112, 36);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DirectorySortMenu->sizePolicy().hasHeightForWidth());
        DirectorySortMenu->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(DirectorySortMenu);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        labelSort = new QLabel(DirectorySortMenu);
        labelSort->setObjectName("labelSort");
        sizePolicy.setHeightForWidth(labelSort->sizePolicy().hasHeightForWidth());
        labelSort->setSizePolicy(sizePolicy);
        labelSort->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout->addWidget(labelSort);

        pushButton = new QPushButton(DirectorySortMenu);
        pushButton->setObjectName("pushButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushButton->sizePolicy().hasHeightForWidth());
        pushButton->setSizePolicy(sizePolicy1);
        pushButton->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout->addWidget(pushButton);


        retranslateUi(DirectorySortMenu);

        QMetaObject::connectSlotsByName(DirectorySortMenu);
    } // setupUi

    void retranslateUi(QWidget *DirectorySortMenu)
    {
        labelSort->setText(QString());
        (void)DirectorySortMenu;
    } // retranslateUi

};

namespace Ui {
    class DirectorySortMenu: public Ui_DirectorySortMenu {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIRECTORYSORTMENU_H
