/********************************************************************************
** Form generated from reading UI file 'dialog_rename.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOG_RENAME_H
#define UI_DIALOG_RENAME_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>

QT_BEGIN_NAMESPACE

class Ui_DialogRename
{
public:
    QGridLayout *gridLayout;
    QLineEdit *lineText1;
    QComboBox *comboBox;
    QLineEdit *lineText2;
    QLabel *labelString1;
    QLabel *labelString2;
    QSpinBox *spinPos2;
    QLabel *labelPos;
    QSpinBox *spinPos1;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QDialog *DialogRename)
    {
        if (DialogRename->objectName().isEmpty())
            DialogRename->setObjectName("DialogRename");
        DialogRename->setWindowModality(Qt::ApplicationModal);
        DialogRename->resize(451, 169);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DialogRename->sizePolicy().hasHeightForWidth());
        DialogRename->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogRename->setWindowIcon(icon);
        DialogRename->setModal(true);
        gridLayout = new QGridLayout(DialogRename);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        lineText1 = new QLineEdit(DialogRename);
        lineText1->setObjectName("lineText1");
        lineText1->setMaxLength(20);

        gridLayout->addWidget(lineText1, 3, 1, 1, 2);

        comboBox = new QComboBox(DialogRename);
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->setObjectName("comboBox");

        gridLayout->addWidget(comboBox, 0, 0, 1, 3);

        lineText2 = new QLineEdit(DialogRename);
        lineText2->setObjectName("lineText2");
        lineText2->setMaxLength(20);

        gridLayout->addWidget(lineText2, 4, 1, 1, 2);

        labelString1 = new QLabel(DialogRename);
        labelString1->setObjectName("labelString1");
#if QT_CONFIG(tooltip)
        labelString1->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelString1->setText(QString::fromUtf8("Old"));

        gridLayout->addWidget(labelString1, 3, 0, 1, 1);

        labelString2 = new QLabel(DialogRename);
        labelString2->setObjectName("labelString2");
#if QT_CONFIG(tooltip)
        labelString2->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelString2->setText(QString::fromUtf8("New"));

        gridLayout->addWidget(labelString2, 4, 0, 1, 1);

        spinPos2 = new QSpinBox(DialogRename);
        spinPos2->setObjectName("spinPos2");
        spinPos2->setMinimum(1);
        spinPos2->setMaximum(20);

        gridLayout->addWidget(spinPos2, 1, 2, 1, 1);

        labelPos = new QLabel(DialogRename);
        labelPos->setObjectName("labelPos");
#if QT_CONFIG(tooltip)
        labelPos->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)

        gridLayout->addWidget(labelPos, 1, 0, 1, 1);

        spinPos1 = new QSpinBox(DialogRename);
        spinPos1->setObjectName("spinPos1");
        spinPos1->setMinimum(1);
        spinPos1->setMaximum(20);

        gridLayout->addWidget(spinPos1, 1, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(DialogRename);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(DialogRename);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 5, 0, 1, 3);

        QWidget::setTabOrder(comboBox, spinPos1);
        QWidget::setTabOrder(spinPos1, spinPos2);
        QWidget::setTabOrder(spinPos2, lineText1);
        QWidget::setTabOrder(lineText1, lineText2);
        QWidget::setTabOrder(lineText2, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(DialogRename);

        QMetaObject::connectSlotsByName(DialogRename);
    } // setupUi

    void retranslateUi(QDialog *DialogRename)
    {
        DialogRename->setWindowTitle(QCoreApplication::translate("DialogRename", "Bulk rename", nullptr));
        comboBox->setItemText(0, QCoreApplication::translate("DialogRename", "Overwrite existing name with key name as suffix", nullptr));
        comboBox->setItemText(1, QCoreApplication::translate("DialogRename", "Overwrite existing name with key name and velocity as suffix", nullptr));
        comboBox->setItemText(2, QCoreApplication::translate("DialogRename", "Overwrite existing name with numerical ascending suffix", nullptr));
        comboBox->setItemText(3, QCoreApplication::translate("DialogRename", "Replace characters", nullptr));
        comboBox->setItemText(4, QCoreApplication::translate("DialogRename", "Insert at a specific position", nullptr));
        comboBox->setItemText(5, QCoreApplication::translate("DialogRename", "Delete character range", nullptr));

        labelPos->setText(QCoreApplication::translate("DialogRename", "Position", nullptr));
        pushOk->setText(QCoreApplication::translate("DialogRename", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("DialogRename", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogRename: public Ui_DialogRename {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOG_RENAME_H
