/********************************************************************************
** Form generated from reading UI file 'soundfontcell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTCELL_H
#define UI_SOUNDFONTCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "attributeflow.h"
#include "elidedlabel.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontCell
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *line1;
    QHBoxLayout *horizontalLayout2;
    ElidedLabel *labelTitle;
    QFrame *line2;
    QHBoxLayout *horizontalLayout;
    QLabel *iconAuthor;
    QLabel *labelAuthor;
    QSpacerItem *horizontalSpacer;
    AttributeFlow *line3;

    void setupUi(QWidget *SoundfontCell)
    {
        if (SoundfontCell->objectName().isEmpty())
            SoundfontCell->setObjectName("SoundfontCell");
        SoundfontCell->resize(152, 80);
        SoundfontCell->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontCell);
        verticalLayout->setObjectName("verticalLayout");
        line1 = new QFrame(SoundfontCell);
        line1->setObjectName("line1");
        line1->setMinimumSize(QSize(0, 20));
        line1->setMaximumSize(QSize(16777215, 20));
        horizontalLayout2 = new QHBoxLayout(line1);
        horizontalLayout2->setObjectName("horizontalLayout2");
        horizontalLayout2->setContentsMargins(0, 0, 0, 0);
        labelTitle = new ElidedLabel(line1);
        labelTitle->setObjectName("labelTitle");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelTitle->sizePolicy().hasHeightForWidth());
        labelTitle->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        labelTitle->setFont(font);
        labelTitle->setText(QString::fromUtf8("labelTitle"));

        horizontalLayout2->addWidget(labelTitle);


        verticalLayout->addWidget(line1);

        line2 = new QFrame(SoundfontCell);
        line2->setObjectName("line2");
        line2->setMinimumSize(QSize(0, 20));
        line2->setMaximumSize(QSize(16777215, 20));
        horizontalLayout = new QHBoxLayout(line2);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        iconAuthor = new QLabel(line2);
        iconAuthor->setObjectName("iconAuthor");
        iconAuthor->setMinimumSize(QSize(16, 16));
        iconAuthor->setMaximumSize(QSize(16, 16));

        horizontalLayout->addWidget(iconAuthor);

        labelAuthor = new QLabel(line2);
        labelAuthor->setObjectName("labelAuthor");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelAuthor->sizePolicy().hasHeightForWidth());
        labelAuthor->setSizePolicy(sizePolicy1);
        labelAuthor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        labelAuthor->setText(QString::fromUtf8("labelAuthor"));
        labelAuthor->setTextFormat(Qt::RichText);

        horizontalLayout->addWidget(labelAuthor);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(line2);

        line3 = new AttributeFlow(SoundfontCell);
        line3->setObjectName("line3");

        verticalLayout->addWidget(line3);


        retranslateUi(SoundfontCell);

        QMetaObject::connectSlotsByName(SoundfontCell);
    } // setupUi

    void retranslateUi(QWidget *SoundfontCell)
    {
        iconAuthor->setText(QString());
        (void)SoundfontCell;
    } // retranslateUi

};

namespace Ui {
    class SoundfontCell: public Ui_SoundfontCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTCELL_H
