/********************************************************************************
** Form generated from reading UI file 'dialogquestion.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGQUESTION_H
#define UI_DIALOGQUESTION_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_DialogQuestion
{
public:
    QGridLayout *gridLayout;
    QPushButton *pushOk;
    QPushButton *pushCancel;
    QSpacerItem *horizontalSpacer;
    QLineEdit *lineEdit;

    void setupUi(QDialog *DialogQuestion)
    {
        if (DialogQuestion->objectName().isEmpty())
            DialogQuestion->setObjectName("DialogQuestion");
        DialogQuestion->setWindowModality(Qt::WindowModal);
        DialogQuestion->resize(184, 66);
        DialogQuestion->setWindowTitle(QString::fromUtf8("Dialog"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogQuestion->setWindowIcon(icon);
        gridLayout = new QGridLayout(DialogQuestion);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        pushOk = new QPushButton(DialogQuestion);
        pushOk->setObjectName("pushOk");

        gridLayout->addWidget(pushOk, 1, 1, 1, 1);

        pushCancel = new QPushButton(DialogQuestion);
        pushCancel->setObjectName("pushCancel");

        gridLayout->addWidget(pushCancel, 1, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 2, 1, 1);

        lineEdit = new QLineEdit(DialogQuestion);
        lineEdit->setObjectName("lineEdit");

        gridLayout->addWidget(lineEdit, 0, 1, 1, 3);

        QWidget::setTabOrder(lineEdit, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(DialogQuestion);

        QMetaObject::connectSlotsByName(DialogQuestion);
    } // setupUi

    void retranslateUi(QDialog *DialogQuestion)
    {
        pushOk->setText(QCoreApplication::translate("DialogQuestion", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("DialogQuestion", "&Cancel", nullptr));
        (void)DialogQuestion;
    } // retranslateUi

};

namespace Ui {
    class DialogQuestion: public Ui_DialogQuestion {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGQUESTION_H
