/********************************************************************************
** Form generated from reading UI file 'waitingtooldialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WAITINGTOOLDIALOG_H
#define UI_WAITINGTOOLDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>

QT_BEGIN_NAMESPACE

class Ui_WaitingToolDialog
{
public:
    QGridLayout *gridLayout;
    QProgressBar *progressBar;
    QPushButton *pushCancel;
    QLabel *labelProcess;

    void setupUi(QDialog *WaitingToolDialog)
    {
        if (WaitingToolDialog->objectName().isEmpty())
            WaitingToolDialog->setObjectName("WaitingToolDialog");
        WaitingToolDialog->resize(306, 72);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        WaitingToolDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(WaitingToolDialog);
        gridLayout->setObjectName("gridLayout");
        progressBar = new QProgressBar(WaitingToolDialog);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(24);

        gridLayout->addWidget(progressBar, 0, 0, 1, 2);

        pushCancel = new QPushButton(WaitingToolDialog);
        pushCancel->setObjectName("pushCancel");
        pushCancel->setMinimumSize(QSize(28, 28));
        pushCancel->setMaximumSize(QSize(28, 28));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/close.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushCancel->setIcon(icon1);

        gridLayout->addWidget(pushCancel, 0, 2, 1, 1);

        labelProcess = new QLabel(WaitingToolDialog);
        labelProcess->setObjectName("labelProcess");

        gridLayout->addWidget(labelProcess, 1, 0, 1, 3);


        retranslateUi(WaitingToolDialog);

        QMetaObject::connectSlotsByName(WaitingToolDialog);
    } // setupUi

    void retranslateUi(QDialog *WaitingToolDialog)
    {
        pushCancel->setText(QString());
        labelProcess->setText(QCoreApplication::translate("WaitingToolDialog", "Processing...", nullptr));
        (void)WaitingToolDialog;
    } // retranslateUi

};

namespace Ui {
    class WaitingToolDialog: public Ui_WaitingToolDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WAITINGTOOLDIALOG_H
