/********************************************************************************
** Form generated from reading UI file 'toolmergesamples_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLMERGESAMPLES_GUI_H
#define UI_TOOLMERGESAMPLES_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolMergeSamples_gui
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QSpinBox *spinSustainDuration;
    QLabel *label_4;
    QCheckBox *checkStereo;
    QLineEdit *lineName;
    QLabel *label;
    QCheckBox *checkLoop;
    QComboBox *comboFreq;
    QLabel *label_3;
    QSpinBox *spinReleaseDuration;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolMergeSamples_gui)
    {
        if (ToolMergeSamples_gui->objectName().isEmpty())
            ToolMergeSamples_gui->setObjectName("ToolMergeSamples_gui");
        ToolMergeSamples_gui->resize(357, 184);
        ToolMergeSamples_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolMergeSamples_gui);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        spinSustainDuration = new QSpinBox(ToolMergeSamples_gui);
        spinSustainDuration->setObjectName("spinSustainDuration");
        spinSustainDuration->setMinimum(1);
        spinSustainDuration->setMaximum(30);

        gridLayout->addWidget(spinSustainDuration, 2, 2, 1, 1);

        label_4 = new QLabel(ToolMergeSamples_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        checkStereo = new QCheckBox(ToolMergeSamples_gui);
        checkStereo->setObjectName("checkStereo");

        gridLayout->addWidget(checkStereo, 0, 2, 1, 1);

        lineName = new QLineEdit(ToolMergeSamples_gui);
        lineName->setObjectName("lineName");
        lineName->setMaxLength(20);

        gridLayout->addWidget(lineName, 0, 1, 1, 1);

        label = new QLabel(ToolMergeSamples_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 2, 1, 1, 1);

        checkLoop = new QCheckBox(ToolMergeSamples_gui);
        checkLoop->setObjectName("checkLoop");
        checkLoop->setChecked(true);

        gridLayout->addWidget(checkLoop, 1, 2, 1, 1);

        comboFreq = new QComboBox(ToolMergeSamples_gui);
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->setObjectName("comboFreq");

        gridLayout->addWidget(comboFreq, 1, 1, 1, 1);

        label_3 = new QLabel(ToolMergeSamples_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        spinReleaseDuration = new QSpinBox(ToolMergeSamples_gui);
        spinReleaseDuration->setObjectName("spinReleaseDuration");
        spinReleaseDuration->setMaximum(30);

        gridLayout->addWidget(spinReleaseDuration, 3, 2, 1, 1);

        label_2 = new QLabel(ToolMergeSamples_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolMergeSamples_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        pushCancel = new QPushButton(ToolMergeSamples_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(lineName, checkStereo);
        QWidget::setTabOrder(checkStereo, comboFreq);
        QWidget::setTabOrder(comboFreq, checkLoop);
        QWidget::setTabOrder(checkLoop, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolMergeSamples_gui);

        QMetaObject::connectSlotsByName(ToolMergeSamples_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolMergeSamples_gui)
    {
        spinSustainDuration->setSuffix(QCoreApplication::translate("ToolMergeSamples_gui", " s", "unit for second"));
        label_4->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Sample creation:", nullptr));
        checkStereo->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Stereo", nullptr));
        label->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Sustain duration", nullptr));
        checkLoop->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Auto loop", nullptr));
        comboFreq->setItemText(0, QCoreApplication::translate("ToolMergeSamples_gui", "for every key", nullptr));
        comboFreq->setItemText(1, QCoreApplication::translate("ToolMergeSamples_gui", "every 3 keys", nullptr));
        comboFreq->setItemText(2, QCoreApplication::translate("ToolMergeSamples_gui", "every 6 keys", nullptr));

        label_3->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Preset name:", nullptr));
        spinReleaseDuration->setSuffix(QCoreApplication::translate("ToolMergeSamples_gui", " s", "unit for second"));
        label_2->setText(QCoreApplication::translate("ToolMergeSamples_gui", "Release duration", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolMergeSamples_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolMergeSamples_gui", "&Cancel", nullptr));
        (void)ToolMergeSamples_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolMergeSamples_gui: public Ui_ToolMergeSamples_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLMERGESAMPLES_GUI_H
