/********************************************************************************
** Form generated from reading UI file 'dialogcreateelements.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGCREATEELEMENTS_H
#define UI_DIALOGCREATEELEMENTS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DialogCreateElements
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelChoice;
    QRadioButton *radioOnlyOne;
    QRadioButton *radioOnePerElement;
    QWidget *page_2;
    QGridLayout *gridLayout;
    QLabel *labelOneChoice;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QDialog *DialogCreateElements)
    {
        if (DialogCreateElements->objectName().isEmpty())
            DialogCreateElements->setObjectName("DialogCreateElements");
        DialogCreateElements->setWindowModality(Qt::WindowModal);
        DialogCreateElements->resize(192, 122);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogCreateElements->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(DialogCreateElements);
        verticalLayout->setObjectName("verticalLayout");
        stackedWidget = new QStackedWidget(DialogCreateElements);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        labelChoice = new QLabel(page);
        labelChoice->setObjectName("labelChoice");
        QFont font;
        font.setBold(true);
        labelChoice->setFont(font);
        labelChoice->setText(QString::fromUtf8("Create a preset"));

        verticalLayout_2->addWidget(labelChoice);

        radioOnlyOne = new QRadioButton(page);
        radioOnlyOne->setObjectName("radioOnlyOne");
        radioOnlyOne->setText(QString::fromUtf8("comprising X instruments"));

        verticalLayout_2->addWidget(radioOnlyOne);

        radioOnePerElement = new QRadioButton(page);
        radioOnePerElement->setObjectName("radioOnePerElement");
        radioOnePerElement->setText(QString::fromUtf8("for each instrument"));

        verticalLayout_2->addWidget(radioOnePerElement);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        gridLayout = new QGridLayout(page_2);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        labelOneChoice = new QLabel(page_2);
        labelOneChoice->setObjectName("labelOneChoice");
        labelOneChoice->setText(QString::fromUtf8("Create preset \"...\" ?"));

        gridLayout->addWidget(labelOneChoice, 0, 0, 1, 1);

        stackedWidget->addWidget(page_2);

        verticalLayout->addWidget(stackedWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(DialogCreateElements);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(DialogCreateElements);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(DialogCreateElements);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DialogCreateElements);
    } // setupUi

    void retranslateUi(QDialog *DialogCreateElements)
    {
        DialogCreateElements->setWindowTitle(QCoreApplication::translate("DialogCreateElements", "Question", nullptr));
        pushOk->setText(QCoreApplication::translate("DialogCreateElements", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("DialogCreateElements", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogCreateElements: public Ui_DialogCreateElements {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGCREATEELEMENTS_H
