/********************************************************************************
** Form generated from reading UI file 'toolchangevolume_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLCHANGEVOLUME_GUI_H
#define UI_TOOLCHANGEVOLUME_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolChangeVolume_gui
{
public:
    QGridLayout *gridLayout;
    QDoubleSpinBox *doubleSpinMultiply;
    QDoubleSpinBox *doubleSpinAdd;
    QRadioButton *radioNormalize;
    QRadioButton *radioAdd;
    QDoubleSpinBox *doubleSpinNormalize;
    QRadioButton *radioMultiply;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolChangeVolume_gui)
    {
        if (ToolChangeVolume_gui->objectName().isEmpty())
            ToolChangeVolume_gui->setObjectName("ToolChangeVolume_gui");
        ToolChangeVolume_gui->resize(192, 128);
        ToolChangeVolume_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolChangeVolume_gui);
        gridLayout->setObjectName("gridLayout");
        doubleSpinMultiply = new QDoubleSpinBox(ToolChangeVolume_gui);
        doubleSpinMultiply->setObjectName("doubleSpinMultiply");
        doubleSpinMultiply->setEnabled(false);
        doubleSpinMultiply->setMaximum(10.000000000000000);
        doubleSpinMultiply->setSingleStep(0.100000000000000);

        gridLayout->addWidget(doubleSpinMultiply, 1, 1, 1, 1);

        doubleSpinAdd = new QDoubleSpinBox(ToolChangeVolume_gui);
        doubleSpinAdd->setObjectName("doubleSpinAdd");
        doubleSpinAdd->setEnabled(false);
        doubleSpinAdd->setMinimum(-200.000000000000000);
        doubleSpinAdd->setMaximum(200.000000000000000);

        gridLayout->addWidget(doubleSpinAdd, 0, 1, 1, 1);

        radioNormalize = new QRadioButton(ToolChangeVolume_gui);
        radioNormalize->setObjectName("radioNormalize");

        gridLayout->addWidget(radioNormalize, 2, 0, 1, 1);

        radioAdd = new QRadioButton(ToolChangeVolume_gui);
        radioAdd->setObjectName("radioAdd");

        gridLayout->addWidget(radioAdd, 0, 0, 1, 1);

        doubleSpinNormalize = new QDoubleSpinBox(ToolChangeVolume_gui);
        doubleSpinNormalize->setObjectName("doubleSpinNormalize");
        doubleSpinNormalize->setEnabled(false);
        doubleSpinNormalize->setDecimals(2);
        doubleSpinNormalize->setMaximum(100.000000000000000);

        gridLayout->addWidget(doubleSpinNormalize, 2, 1, 1, 1);

        radioMultiply = new QRadioButton(ToolChangeVolume_gui);
        radioMultiply->setObjectName("radioMultiply");

        gridLayout->addWidget(radioMultiply, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolChangeVolume_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolChangeVolume_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 2);

        QWidget::setTabOrder(radioAdd, doubleSpinAdd);
        QWidget::setTabOrder(doubleSpinAdd, radioMultiply);
        QWidget::setTabOrder(radioMultiply, doubleSpinMultiply);
        QWidget::setTabOrder(doubleSpinMultiply, radioNormalize);
        QWidget::setTabOrder(radioNormalize, doubleSpinNormalize);
        QWidget::setTabOrder(doubleSpinNormalize, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolChangeVolume_gui);

        QMetaObject::connectSlotsByName(ToolChangeVolume_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolChangeVolume_gui)
    {
        doubleSpinAdd->setSuffix(QCoreApplication::translate("ToolChangeVolume_gui", " dB", nullptr));
        radioNormalize->setText(QCoreApplication::translate("ToolChangeVolume_gui", "Normalize", nullptr));
        radioAdd->setText(QCoreApplication::translate("ToolChangeVolume_gui", "Add", nullptr));
        doubleSpinNormalize->setSuffix(QCoreApplication::translate("ToolChangeVolume_gui", " %", nullptr));
        radioMultiply->setText(QCoreApplication::translate("ToolChangeVolume_gui", "Multiply", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolChangeVolume_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolChangeVolume_gui", "&Cancel", nullptr));
        (void)ToolChangeVolume_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolChangeVolume_gui: public Ui_ToolChangeVolume_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLCHANGEVOLUME_GUI_H
