/********************************************************************************
** Form generated from reading UI file 'dialogkeyboard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGKEYBOARD_H
#define UI_DIALOGKEYBOARD_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "controllerarea.h"
#include "pianokeybdcustom.h"

QT_BEGIN_NAMESPACE

class Ui_DialogKeyboard
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *topLayout;
    QSpacerItem *verticalSpacer_2;
    PianoKeybdCustom *keyboard;
    QSpacerItem *verticalSpacer;
    ControllerArea *controllerArea;
    QFrame *frameBottom;
    QHBoxLayout *horizontalLayout;
    QComboBox *comboType;
    QSpacerItem *horizontalSpacer;
    QLabel *label;
    QLabel *labelKey;
    QLabel *label_2;
    QLabel *labelVel;
    QLabel *label_3;
    QLabel *labelAftertouch;
    QPushButton *pushExpand;

    void setupUi(QDialog *DialogKeyboard)
    {
        if (DialogKeyboard->objectName().isEmpty())
            DialogKeyboard->setObjectName("DialogKeyboard");
        DialogKeyboard->resize(594, 277);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogKeyboard->setWindowIcon(icon);
        gridLayout = new QGridLayout(DialogKeyboard);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(0);
        gridLayout->setVerticalSpacing(6);
        gridLayout->setContentsMargins(0, 6, 0, 0);
        topLayout = new QVBoxLayout();
        topLayout->setSpacing(0);
        topLayout->setObjectName("topLayout");
        topLayout->setSizeConstraint(QLayout::SizeConstraint::SetNoConstraint);
        topLayout->setContentsMargins(6, 0, 6, -1);
        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        topLayout->addItem(verticalSpacer_2);

        keyboard = new PianoKeybdCustom(DialogKeyboard);
        keyboard->setObjectName("keyboard");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(keyboard->sizePolicy().hasHeightForWidth());
        keyboard->setSizePolicy(sizePolicy);

        topLayout->addWidget(keyboard);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        topLayout->addItem(verticalSpacer);

        controllerArea = new ControllerArea(DialogKeyboard);
        controllerArea->setObjectName("controllerArea");
        controllerArea->setMinimumSize(QSize(0, 130));

        topLayout->addWidget(controllerArea);


        gridLayout->addLayout(topLayout, 0, 0, 1, 1);

        frameBottom = new QFrame(DialogKeyboard);
        frameBottom->setObjectName("frameBottom");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frameBottom->sizePolicy().hasHeightForWidth());
        frameBottom->setSizePolicy(sizePolicy1);
        frameBottom->setMinimumSize(QSize(0, 24));
        frameBottom->setMaximumSize(QSize(16777215, 24));
        frameBottom->setFrameShape(QFrame::Shape::NoFrame);
        frameBottom->setFrameShadow(QFrame::Shadow::Raised);
        horizontalLayout = new QHBoxLayout(frameBottom);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        comboType = new QComboBox(frameBottom);
        comboType->addItem(QString());
        comboType->addItem(QString());
        comboType->addItem(QString());
        comboType->addItem(QString());
        comboType->setObjectName("comboType");

        horizontalLayout->addWidget(comboType);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        label = new QLabel(frameBottom);
        label->setObjectName("label");
        QFont font;
        font.setBold(true);
        label->setFont(font);

        horizontalLayout->addWidget(label);

        labelKey = new QLabel(frameBottom);
        labelKey->setObjectName("labelKey");
        labelKey->setMinimumSize(QSize(50, 0));

        horizontalLayout->addWidget(labelKey);

        label_2 = new QLabel(frameBottom);
        label_2->setObjectName("label_2");
        label_2->setFont(font);

        horizontalLayout->addWidget(label_2);

        labelVel = new QLabel(frameBottom);
        labelVel->setObjectName("labelVel");
        labelVel->setMinimumSize(QSize(50, 0));

        horizontalLayout->addWidget(labelVel);

        label_3 = new QLabel(frameBottom);
        label_3->setObjectName("label_3");
        label_3->setFont(font);

        horizontalLayout->addWidget(label_3);

        labelAftertouch = new QLabel(frameBottom);
        labelAftertouch->setObjectName("labelAftertouch");
        labelAftertouch->setMinimumSize(QSize(50, 0));

        horizontalLayout->addWidget(labelAftertouch);

        pushExpand = new QPushButton(frameBottom);
        pushExpand->setObjectName("pushExpand");
        pushExpand->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushExpand->setFlat(true);

        horizontalLayout->addWidget(pushExpand);


        gridLayout->addWidget(frameBottom, 2, 0, 1, 1);

        QWidget::setTabOrder(comboType, pushExpand);

        retranslateUi(DialogKeyboard);

        QMetaObject::connectSlotsByName(DialogKeyboard);
    } // setupUi

    void retranslateUi(QDialog *DialogKeyboard)
    {
        DialogKeyboard->setWindowTitle(QCoreApplication::translate("DialogKeyboard", "Virtual keyboard", nullptr));
        comboType->setItemText(0, QCoreApplication::translate("DialogKeyboard", "5 octaves", nullptr));
        comboType->setItemText(1, QCoreApplication::translate("DialogKeyboard", "6 octaves", nullptr));
        comboType->setItemText(2, QCoreApplication::translate("DialogKeyboard", "88 keys (piano)", nullptr));
        comboType->setItemText(3, QCoreApplication::translate("DialogKeyboard", "128 keys (full)", nullptr));

        label->setText(QCoreApplication::translate("DialogKeyboard", "Key", nullptr));
        labelKey->setText(QString());
        label_2->setText(QCoreApplication::translate("DialogKeyboard", "Velocity", nullptr));
        labelVel->setText(QString());
        label_3->setText(QCoreApplication::translate("DialogKeyboard", "Aftertouch", nullptr));
        labelAftertouch->setText(QString());
        pushExpand->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DialogKeyboard: public Ui_DialogKeyboard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGKEYBOARD_H
