/****************************************************************************
** Meta object code from reading C++ file 'soundfontbrowser.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "repository/browser/soundfontbrowser.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'soundfontbrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SoundfontBrowserE_t {};
} // unnamed namespace

template <> constexpr inline auto SoundfontBrowser::qt_create_metaobjectdata<qt_meta_tag_ZN16SoundfontBrowserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SoundfontBrowser",
        "applyFilter",
        "",
        "SoundfontFilter*",
        "filter",
        "initialize",
        "soundfontListAvailable",
        "error",
        "on_pushRetry_clicked",
        "on_pushResetFilters_clicked",
        "updateFilter",
        "on_listWidget_itemSelectionChanged",
        "onComboSortCurrentIndexChanged",
        "index",
        "on_pushBecomePremium_clicked",
        "userStatusChanged",
        "UserManager::ConnectionState",
        "state",
        "on_pushGoPrevious_clicked",
        "on_pushGoNext_clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'applyFilter'
        QtMocHelpers::SlotData<void(SoundfontFilter *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'initialize'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'soundfontListAvailable'
        QtMocHelpers::SlotData<void(QString)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'on_pushRetry_clicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushResetFilters_clicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateFilter'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_listWidget_itemSelectionChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onComboSortCurrentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Slot 'on_pushBecomePremium_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'userStatusChanged'
        QtMocHelpers::SlotData<void(UserManager::ConnectionState)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'on_pushGoPrevious_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushGoNext_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SoundfontBrowser, qt_meta_tag_ZN16SoundfontBrowserE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SoundfontBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<Tab::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SoundfontBrowserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SoundfontBrowserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SoundfontBrowserE_t>.metaTypes,
    nullptr
} };

void SoundfontBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SoundfontBrowser *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applyFilter((*reinterpret_cast<std::add_pointer_t<SoundfontFilter*>>(_a[1]))); break;
        case 1: _t->initialize(); break;
        case 2: _t->soundfontListAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->on_pushRetry_clicked(); break;
        case 4: _t->on_pushResetFilters_clicked(); break;
        case 5: _t->updateFilter(); break;
        case 6: _t->on_listWidget_itemSelectionChanged(); break;
        case 7: _t->onComboSortCurrentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->on_pushBecomePremium_clicked(); break;
        case 9: _t->userStatusChanged((*reinterpret_cast<std::add_pointer_t<UserManager::ConnectionState>>(_a[1]))); break;
        case 10: _t->on_pushGoPrevious_clicked(); break;
        case 11: _t->on_pushGoNext_clicked(); break;
        default: ;
        }
    }
}

const QMetaObject *SoundfontBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SoundfontBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SoundfontBrowserE_t>.strings))
        return static_cast<void*>(this);
    return Tab::qt_metacast(_clname);
}

int SoundfontBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Tab::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
