/********************************************************************************
** Form generated from reading UI file 'soundfontcomment.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTCOMMENT_H
#define UI_SOUNDFONTCOMMENT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SoundfontComment
{
public:
    QVBoxLayout *verticalLayout;

    void setupUi(QWidget *SoundfontComment)
    {
        if (SoundfontComment->objectName().isEmpty())
            SoundfontComment->setObjectName("SoundfontComment");
        SoundfontComment->resize(211, 55);
        SoundfontComment->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontComment);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, -1, 0, -1);

        retranslateUi(SoundfontComment);

        QMetaObject::connectSlotsByName(SoundfontComment);
    } // setupUi

    void retranslateUi(QWidget *SoundfontComment)
    {
        (void)SoundfontComment;
    } // retranslateUi

};

namespace Ui {
    class SoundfontComment: public Ui_SoundfontComment {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTCOMMENT_H
