/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2024 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone.io                             **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "abstractcsvfile.h"

#include <QFile>
#include <QTextStream>

AbstractCsvFile::AbstractCsvFile(QString filePath) :
    _file(new QFile(filePath)),
    _stream(nullptr)
{

}

AbstractCsvFile::~AbstractCsvFile()
{
    if (_stream)
    {
        _stream->flush();
        delete _stream;
    }

    if (_file->isOpen())
        _file->close();
    delete _file;
}

bool AbstractCsvFile::writeLine(QStringList elements)
{
    // Possibly start the file with the header
    if (!_file->isOpen())
    {
        if (!_file->open(QIODevice::WriteOnly))
        {
            _error = QObject::tr("cannot create file \"%1\"").arg(_file->fileName());
            return false;
        }
        _stream = new QTextStream(_file);

        (*_stream) << getHeaders().join(',') << Qt::endl;
    }

    (*_stream) << elements.join(',') << Qt::endl;
    return true;
}
