/********************************************************************************
** Form generated from reading UI file 'filtertagcell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERTAGCELL_H
#define UI_FILTERTAGCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>
#include "elidedlabel.h"

QT_BEGIN_NAMESPACE

class Ui_FilterTagCell
{
public:
    QHBoxLayout *horizontalLayout;
    ElidedLabel *labelName;
    QPushButton *pushDelete;

    void setupUi(QWidget *FilterTagCell)
    {
        if (FilterTagCell->objectName().isEmpty())
            FilterTagCell->setObjectName("FilterTagCell");
        FilterTagCell->resize(81, 24);
        FilterTagCell->setMinimumSize(QSize(0, 24));
        FilterTagCell->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(FilterTagCell);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        labelName = new ElidedLabel(FilterTagCell);
        labelName->setObjectName("labelName");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelName->sizePolicy().hasHeightForWidth());
        labelName->setSizePolicy(sizePolicy);
        labelName->setText(QString::fromUtf8("labelName"));

        horizontalLayout->addWidget(labelName);

        pushDelete = new QPushButton(FilterTagCell);
        pushDelete->setObjectName("pushDelete");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushDelete->sizePolicy().hasHeightForWidth());
        pushDelete->setSizePolicy(sizePolicy1);
        pushDelete->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushDelete->setIconSize(QSize(12, 12));
        pushDelete->setFlat(true);

        horizontalLayout->addWidget(pushDelete);


        retranslateUi(FilterTagCell);

        QMetaObject::connectSlotsByName(FilterTagCell);
    } // setupUi

    void retranslateUi(QWidget *FilterTagCell)
    {
        pushDelete->setText(QString());
        (void)FilterTagCell;
    } // retranslateUi

};

namespace Ui {
    class FilterTagCell: public Ui_FilterTagCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERTAGCELL_H
