/********************************************************************************
** Form generated from reading UI file 'toolexternalcommand_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLEXTERNALCOMMAND_GUI_H
#define UI_TOOLEXTERNALCOMMAND_GUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolExternalCommand_gui
{
public:
    QGridLayout *gridLayout;
    QFrame *frame;
    QGridLayout *gridLayout_2;
    QLabel *label_3;
    QLabel *label_4;
    QLabel *label_5;
    QLabel *label;
    QLabel *label_2;
    QCheckBox *checkStereo;
    QLineEdit *lineCommand;
    QPushButton *pushOpen;
    QCheckBox *checkReplaceInfo;
    QComboBox *comboPrevious;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolExternalCommand_gui)
    {
        if (ToolExternalCommand_gui->objectName().isEmpty())
            ToolExternalCommand_gui->setObjectName("ToolExternalCommand_gui");
        ToolExternalCommand_gui->resize(416, 252);
        ToolExternalCommand_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolExternalCommand_gui);
        gridLayout->setObjectName("gridLayout");
        frame = new QFrame(ToolExternalCommand_gui);
        frame->setObjectName("frame");
        frame->setEnabled(true);
        frame->setFrameShape(QFrame::Shape::Box);
        frame->setFrameShadow(QFrame::Shadow::Raised);
        gridLayout_2 = new QGridLayout(frame);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setHorizontalSpacing(3);
        gridLayout_2->setVerticalSpacing(0);
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        label_3 = new QLabel(frame);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 0, 0, 1, 1);

        label_4 = new QLabel(frame);
        label_4->setObjectName("label_4");
        label_4->setText(QString::fromUtf8("\"C:/Program Files (x86)/Audacity/audacity.exe\" {wav}"));

        gridLayout_2->addWidget(label_4, 1, 0, 1, 1);

        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");
        label_5->setText(QString::fromUtf8("sweep {wav}"));

        gridLayout_2->addWidget(label_5, 2, 0, 1, 1);


        gridLayout->addWidget(frame, 4, 0, 1, 3);

        label = new QLabel(ToolExternalCommand_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(ToolExternalCommand_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        checkStereo = new QCheckBox(ToolExternalCommand_gui);
        checkStereo->setObjectName("checkStereo");

        gridLayout->addWidget(checkStereo, 2, 0, 1, 3);

        lineCommand = new QLineEdit(ToolExternalCommand_gui);
        lineCommand->setObjectName("lineCommand");

        gridLayout->addWidget(lineCommand, 0, 1, 1, 1);

        pushOpen = new QPushButton(ToolExternalCommand_gui);
        pushOpen->setObjectName("pushOpen");
        pushOpen->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/search.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushOpen->setIcon(icon);
        pushOpen->setFlat(true);

        gridLayout->addWidget(pushOpen, 0, 2, 1, 1);

        checkReplaceInfo = new QCheckBox(ToolExternalCommand_gui);
        checkReplaceInfo->setObjectName("checkReplaceInfo");

        gridLayout->addWidget(checkReplaceInfo, 3, 0, 1, 3);

        comboPrevious = new QComboBox(ToolExternalCommand_gui);
        comboPrevious->setObjectName("comboPrevious");

        gridLayout->addWidget(comboPrevious, 1, 1, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolExternalCommand_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolExternalCommand_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 5, 0, 1, 3);

        QWidget::setTabOrder(lineCommand, pushOpen);
        QWidget::setTabOrder(pushOpen, comboPrevious);
        QWidget::setTabOrder(comboPrevious, checkStereo);
        QWidget::setTabOrder(checkStereo, checkReplaceInfo);
        QWidget::setTabOrder(checkReplaceInfo, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolExternalCommand_gui);

        QMetaObject::connectSlotsByName(ToolExternalCommand_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolExternalCommand_gui)
    {
        label_3->setText(QCoreApplication::translate("ToolExternalCommand_gui", "Command examples:", nullptr));
#if QT_CONFIG(tooltip)
        label_4->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_5->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("ToolExternalCommand_gui", "Command", nullptr));
        label_2->setText(QCoreApplication::translate("ToolExternalCommand_gui", "History", nullptr));
        checkStereo->setText(QCoreApplication::translate("ToolExternalCommand_gui", "Authorize stereo editing", nullptr));
        checkReplaceInfo->setText(QCoreApplication::translate("ToolExternalCommand_gui", "Replace sample information if possible", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolExternalCommand_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolExternalCommand_gui", "&Cancel", nullptr));
        (void)ToolExternalCommand_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolExternalCommand_gui: public Ui_ToolExternalCommand_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLEXTERNALCOMMAND_GUI_H
