/********************************************************************************
** Form generated from reading UI file 'editordialoginserttable.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITORDIALOGINSERTTABLE_H
#define UI_EDITORDIALOGINSERTTABLE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>

QT_BEGIN_NAMESPACE

class Ui_EditorDialogInsertTable
{
public:
    QGridLayout *gridLayout;
    QDialogButtonBox *buttonBox;
    QSpinBox *spinRows;
    QLabel *label;
    QSpinBox *spinColumns;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *EditorDialogInsertTable)
    {
        if (EditorDialogInsertTable->objectName().isEmpty())
            EditorDialogInsertTable->setObjectName("EditorDialogInsertTable");
        EditorDialogInsertTable->resize(395, 102);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        EditorDialogInsertTable->setWindowIcon(icon);
        gridLayout = new QGridLayout(EditorDialogInsertTable);
        gridLayout->setObjectName("gridLayout");
        buttonBox = new QDialogButtonBox(EditorDialogInsertTable);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 2, 0, 1, 4);

        spinRows = new QSpinBox(EditorDialogInsertTable);
        spinRows->setObjectName("spinRows");
        spinRows->setMinimum(1);
        spinRows->setMaximum(300);
        spinRows->setValue(2);

        gridLayout->addWidget(spinRows, 0, 1, 1, 1);

        label = new QLabel(EditorDialogInsertTable);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 2, 1, 1);

        spinColumns = new QSpinBox(EditorDialogInsertTable);
        spinColumns->setObjectName("spinColumns");
        spinColumns->setMinimum(1);
        spinColumns->setMaximum(100);
        spinColumns->setValue(3);

        gridLayout->addWidget(spinColumns, 0, 3, 1, 1);

        label_2 = new QLabel(EditorDialogInsertTable);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 2, 1, 1);


        retranslateUi(EditorDialogInsertTable);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, EditorDialogInsertTable, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, EditorDialogInsertTable, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(EditorDialogInsertTable);
    } // setupUi

    void retranslateUi(QDialog *EditorDialogInsertTable)
    {
        EditorDialogInsertTable->setWindowTitle(QCoreApplication::translate("EditorDialogInsertTable", "Insert a table", nullptr));
        label->setText(QCoreApplication::translate("EditorDialogInsertTable", "Columns", nullptr));
        label_2->setText(QCoreApplication::translate("EditorDialogInsertTable", "Rows", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditorDialogInsertTable: public Ui_EditorDialogInsertTable {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITORDIALOGINSERTTABLE_H
