/********************************************************************************
** Form generated from reading UI file 'footerprst.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOOTERPRST_H
#define UI_FOOTERPRST_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "footerprst.h"

QT_BEGIN_NAMESPACE

class Ui_FooterPrst
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    SpinBox *spinBank;
    QLabel *label_3;
    SpinBox *spinPreset;
    QSpacerItem *horizontalSpacer;
    QLabel *labelPercussion;

    void setupUi(QWidget *FooterPrst)
    {
        if (FooterPrst->objectName().isEmpty())
            FooterPrst->setObjectName("FooterPrst");
        FooterPrst->resize(381, 35);
        FooterPrst->setWindowTitle(QString::fromUtf8(""));
        horizontalLayout = new QHBoxLayout(FooterPrst);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        label_2 = new QLabel(FooterPrst);
        label_2->setObjectName("label_2");
        label_2->setMinimumSize(QSize(0, 19));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label_2);

        spinBank = new SpinBox(FooterPrst);
        spinBank->setObjectName("spinBank");
        spinBank->setMinimumSize(QSize(70, 23));
        spinBank->setAccelerated(true);
        spinBank->setMaximum(128);

        horizontalLayout->addWidget(spinBank);

        label_3 = new QLabel(FooterPrst);
        label_3->setObjectName("label_3");
        label_3->setMinimumSize(QSize(60, 19));
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(label_3);

        spinPreset = new SpinBox(FooterPrst);
        spinPreset->setObjectName("spinPreset");
        spinPreset->setMinimumSize(QSize(70, 23));
        spinPreset->setAccelerated(true);
        spinPreset->setMaximum(127);

        horizontalLayout->addWidget(spinPreset);

        horizontalSpacer = new QSpacerItem(1, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        labelPercussion = new QLabel(FooterPrst);
        labelPercussion->setObjectName("labelPercussion");

        horizontalLayout->addWidget(labelPercussion);


        retranslateUi(FooterPrst);

        QMetaObject::connectSlotsByName(FooterPrst);
    } // setupUi

    void retranslateUi(QWidget *FooterPrst)
    {
        label_2->setText(QCoreApplication::translate("FooterPrst", "Bank", nullptr));
        label_3->setText(QCoreApplication::translate("FooterPrst", "Preset", nullptr));
        labelPercussion->setText(QCoreApplication::translate("FooterPrst", "(percussion)", nullptr));
        (void)FooterPrst;
    } // retranslateUi

};

namespace Ui {
    class FooterPrst: public Ui_FooterPrst {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FOOTERPRST_H
