/********************************************************************************
** Form generated from reading UI file 'widgetshowhistorycell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WIDGETSHOWHISTORYCELL_H
#define UI_WIDGETSHOWHISTORYCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "elidedlabel.h"

QT_BEGIN_NAMESPACE

class Ui_WidgetShowHistoryCell
{
public:
    QGridLayout *gridLayout;
    QLabel *labelDateTime;
    ElidedLabel *labelLink;
    QLabel *iconFile;

    void setupUi(QWidget *WidgetShowHistoryCell)
    {
        if (WidgetShowHistoryCell->objectName().isEmpty())
            WidgetShowHistoryCell->setObjectName("WidgetShowHistoryCell");
        WidgetShowHistoryCell->resize(301, 82);
        gridLayout = new QGridLayout(WidgetShowHistoryCell);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(12);
        labelDateTime = new QLabel(WidgetShowHistoryCell);
        labelDateTime->setObjectName("labelDateTime");
        labelDateTime->setText(QString::fromUtf8("labelDateTime"));

        gridLayout->addWidget(labelDateTime, 1, 1, 1, 1);

        labelLink = new ElidedLabel(WidgetShowHistoryCell);
        labelLink->setObjectName("labelLink");
        QFont font;
        font.setBold(true);
        labelLink->setFont(font);
        labelLink->setText(QString::fromUtf8("labelLink"));

        gridLayout->addWidget(labelLink, 0, 1, 1, 1);

        iconFile = new QLabel(WidgetShowHistoryCell);
        iconFile->setObjectName("iconFile");
        iconFile->setMinimumSize(QSize(48, 64));
        iconFile->setMaximumSize(QSize(48, 64));
        iconFile->setPixmap(QPixmap(QString::fromUtf8(":/icons/file-audio.svg")));
        iconFile->setScaledContents(true);

        gridLayout->addWidget(iconFile, 0, 0, 2, 1);


        retranslateUi(WidgetShowHistoryCell);

        QMetaObject::connectSlotsByName(WidgetShowHistoryCell);
    } // setupUi

    void retranslateUi(QWidget *WidgetShowHistoryCell)
    {
        iconFile->setText(QString());
        (void)WidgetShowHistoryCell;
    } // retranslateUi

};

namespace Ui {
    class WidgetShowHistoryCell: public Ui_WidgetShowHistoryCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WIDGETSHOWHISTORYCELL_H
