/********************************************************************************
** Form generated from reading UI file 'controllerarea.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONTROLLERAREA_H
#define UI_CONTROLLERAREA_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDial>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "combocc.h"
#include "styledslider.h"

QT_BEGIN_NAMESPACE

class Ui_ControllerArea
{
public:
    QGridLayout *gridLayout;
    QLabel *labelValue3;
    ComboCC *comboControl3;
    QDial *knob3;
    StyledSlider *sliderPressure;
    StyledSlider *sliderSensitivity;
    QPushButton *push4;
    QLabel *labelValue1;
    QLabel *labelValue2;
    QDial *knob2;
    ComboCC *comboControl2;
    ComboCC *comboControl4;
    QLabel *labelValue4;
    StyledSlider *sliderPitchWheel;
    QDial *knob1;
    ComboCC *comboControl1;
    QVBoxLayout *verticalLayout;
    QLabel *label_3;
    QLabel *labelWheelValue;
    QLabel *label_4;
    QLabel *labelSensitivityValue;
    QLabel *label_5;
    QLabel *labelPressureValue;

    void setupUi(QWidget *ControllerArea)
    {
        if (ControllerArea->objectName().isEmpty())
            ControllerArea->setObjectName("ControllerArea");
        ControllerArea->resize(528, 130);
        ControllerArea->setMinimumSize(QSize(0, 130));
        ControllerArea->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ControllerArea);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 6, 0, 0);
        labelValue3 = new QLabel(ControllerArea);
        labelValue3->setObjectName("labelValue3");
        labelValue3->setText(QString::fromUtf8("-"));
        labelValue3->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout->addWidget(labelValue3, 2, 5, 1, 1);

        comboControl3 = new ComboCC(ControllerArea);
        comboControl3->setObjectName("comboControl3");

        gridLayout->addWidget(comboControl3, 0, 5, 1, 1);

        knob3 = new QDial(ControllerArea);
        knob3->setObjectName("knob3");
        knob3->setMaximum(127);

        gridLayout->addWidget(knob3, 1, 5, 1, 1);

        sliderPressure = new StyledSlider(ControllerArea);
        sliderPressure->setObjectName("sliderPressure");
        sliderPressure->setMaximum(127);
        sliderPressure->setOrientation(Qt::Orientation::Horizontal);
        sliderPressure->setTickPosition(QSlider::TickPosition::TicksBelow);
        sliderPressure->setTickInterval(32);

        gridLayout->addWidget(sliderPressure, 2, 2, 1, 1);

        sliderSensitivity = new StyledSlider(ControllerArea);
        sliderSensitivity->setObjectName("sliderSensitivity");
        sliderSensitivity->setMaximum(1500);
        sliderSensitivity->setPageStep(100);
        sliderSensitivity->setValue(200);
        sliderSensitivity->setOrientation(Qt::Orientation::Horizontal);
        sliderSensitivity->setTickPosition(QSlider::TickPosition::TicksBelow);
        sliderSensitivity->setTickInterval(200);

        gridLayout->addWidget(sliderSensitivity, 1, 2, 1, 1);

        push4 = new QPushButton(ControllerArea);
        push4->setObjectName("push4");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(push4->sizePolicy().hasHeightForWidth());
        push4->setSizePolicy(sizePolicy);
        push4->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        push4->setText(QString::fromUtf8(""));
        push4->setIconSize(QSize(36, 36));
        push4->setFlat(true);

        gridLayout->addWidget(push4, 1, 6, 1, 1);

        labelValue1 = new QLabel(ControllerArea);
        labelValue1->setObjectName("labelValue1");
        labelValue1->setText(QString::fromUtf8("-"));
        labelValue1->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout->addWidget(labelValue1, 2, 3, 1, 1);

        labelValue2 = new QLabel(ControllerArea);
        labelValue2->setObjectName("labelValue2");
        labelValue2->setText(QString::fromUtf8("-"));
        labelValue2->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout->addWidget(labelValue2, 2, 4, 1, 1);

        knob2 = new QDial(ControllerArea);
        knob2->setObjectName("knob2");
        knob2->setMaximum(127);

        gridLayout->addWidget(knob2, 1, 4, 1, 1);

        comboControl2 = new ComboCC(ControllerArea);
        comboControl2->setObjectName("comboControl2");

        gridLayout->addWidget(comboControl2, 0, 4, 1, 1);

        comboControl4 = new ComboCC(ControllerArea);
        comboControl4->setObjectName("comboControl4");

        gridLayout->addWidget(comboControl4, 0, 6, 1, 1);

        labelValue4 = new QLabel(ControllerArea);
        labelValue4->setObjectName("labelValue4");
        labelValue4->setText(QString::fromUtf8("off"));
        labelValue4->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout->addWidget(labelValue4, 2, 6, 1, 1);

        sliderPitchWheel = new StyledSlider(ControllerArea);
        sliderPitchWheel->setObjectName("sliderPitchWheel");
        sliderPitchWheel->setMinimumSize(QSize(100, 0));
        sliderPitchWheel->setMaximum(127);
        sliderPitchWheel->setValue(64);
        sliderPitchWheel->setOrientation(Qt::Orientation::Horizontal);
        sliderPitchWheel->setTickPosition(QSlider::TickPosition::TicksBelow);
        sliderPitchWheel->setTickInterval(32);

        gridLayout->addWidget(sliderPitchWheel, 0, 2, 1, 1);

        knob1 = new QDial(ControllerArea);
        knob1->setObjectName("knob1");
        knob1->setMaximum(127);

        gridLayout->addWidget(knob1, 1, 3, 1, 1);

        comboControl1 = new ComboCC(ControllerArea);
        comboControl1->setObjectName("comboControl1");

        gridLayout->addWidget(comboControl1, 0, 3, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        label_3 = new QLabel(ControllerArea);
        label_3->setObjectName("label_3");
        QFont font;
        font.setBold(true);
        label_3->setFont(font);

        verticalLayout->addWidget(label_3);

        labelWheelValue = new QLabel(ControllerArea);
        labelWheelValue->setObjectName("labelWheelValue");
        QFont font1;
        font1.setPointSize(9);
        labelWheelValue->setFont(font1);
        labelWheelValue->setText(QString::fromUtf8(""));

        verticalLayout->addWidget(labelWheelValue);

        label_4 = new QLabel(ControllerArea);
        label_4->setObjectName("label_4");
        label_4->setFont(font);

        verticalLayout->addWidget(label_4);

        labelSensitivityValue = new QLabel(ControllerArea);
        labelSensitivityValue->setObjectName("labelSensitivityValue");
        labelSensitivityValue->setFont(font1);
        labelSensitivityValue->setText(QString::fromUtf8(""));

        verticalLayout->addWidget(labelSensitivityValue);

        label_5 = new QLabel(ControllerArea);
        label_5->setObjectName("label_5");
        label_5->setFont(font);

        verticalLayout->addWidget(label_5);

        labelPressureValue = new QLabel(ControllerArea);
        labelPressureValue->setObjectName("labelPressureValue");
        labelPressureValue->setFont(font1);
        labelPressureValue->setText(QString::fromUtf8(""));

        verticalLayout->addWidget(labelPressureValue);


        gridLayout->addLayout(verticalLayout, 0, 1, 3, 1);

        QWidget::setTabOrder(sliderPitchWheel, sliderSensitivity);
        QWidget::setTabOrder(sliderSensitivity, sliderPressure);
        QWidget::setTabOrder(sliderPressure, comboControl1);
        QWidget::setTabOrder(comboControl1, knob1);
        QWidget::setTabOrder(knob1, comboControl2);
        QWidget::setTabOrder(comboControl2, knob2);
        QWidget::setTabOrder(knob2, comboControl3);
        QWidget::setTabOrder(comboControl3, knob3);
        QWidget::setTabOrder(knob3, comboControl4);
        QWidget::setTabOrder(comboControl4, push4);

        retranslateUi(ControllerArea);

        QMetaObject::connectSlotsByName(ControllerArea);
    } // setupUi

    void retranslateUi(QWidget *ControllerArea)
    {
        label_3->setText(QCoreApplication::translate("ControllerArea", "Pitch wheel", nullptr));
        label_4->setText(QCoreApplication::translate("ControllerArea", "Sensitivity", nullptr));
        label_5->setText(QCoreApplication::translate("ControllerArea", "Pressure", nullptr));
        (void)ControllerArea;
    } // retranslateUi

};

namespace Ui {
    class ControllerArea: public Ui_ControllerArea {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONTROLLERAREA_H
