/********************************************************************************
** Form generated from reading UI file 'mainwindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "mainstackedwidget.h"
#include "maintabbar.h"
#include "showsoundfonts.h"
#include "topbackground.h"
#include "toprightwidget.h"
#include "widgetshowhistory.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QVBoxLayout *verticalLayout;
    TopBackground *topBackground;
    QGridLayout *gridLayoutTop;
    MainTabBar *tabBar;
    TopRightWidget *topRightWidget;
    QPushButton *pushHome;
    MainStackedWidget *stackedWidget;
    QWidget *stackedWidgetPage1;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonNew;
    QPushButton *pushButtonOpen;
    QPushButton *pushButtonOpenDir;
    QWidget *widgetRepo;
    QGridLayout *gridLayout_4;
    QLineEdit *lineSearch;
    QPushButton *pushButtonSearch;
    ShowSoundfonts *widgetShowSoundfonts;
    QFrame *line;
    QLabel *label;
    QPushButton *pushButtonSoundfonts;
    QLabel *label_2;
    WidgetShowHistory *widgetShowHistory;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushButtonDocumentation;
    QPushButton *pushButtonForum;
    QPushButton *pushButtonSettings;

    void setupUi(QWidget *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(642, 367);
        verticalLayout = new QVBoxLayout(MainWindow);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        topBackground = new TopBackground(MainWindow);
        topBackground->setObjectName("topBackground");
        gridLayoutTop = new QGridLayout(topBackground);
        gridLayoutTop->setObjectName("gridLayoutTop");
        gridLayoutTop->setHorizontalSpacing(6);
        gridLayoutTop->setVerticalSpacing(0);
        gridLayoutTop->setContentsMargins(48, 6, 6, 0);
        tabBar = new MainTabBar(topBackground);
        tabBar->setObjectName("tabBar");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabBar->sizePolicy().hasHeightForWidth());
        tabBar->setSizePolicy(sizePolicy);

        gridLayoutTop->addWidget(tabBar, 0, 1, 1, 1);

        topRightWidget = new TopRightWidget(topBackground);
        topRightWidget->setObjectName("topRightWidget");

        gridLayoutTop->addWidget(topRightWidget, 0, 2, 1, 1);

        pushHome = new QPushButton(topBackground);
        pushHome->setObjectName("pushHome");
        pushHome->setMinimumSize(QSize(42, 42));
        pushHome->setMaximumSize(QSize(42, 42));
        pushHome->setText(QString::fromUtf8(""));
        pushHome->setIconSize(QSize(28, 28));
        pushHome->setChecked(false);
        pushHome->setFlat(true);

        gridLayoutTop->addWidget(pushHome, 0, 0, 1, 1);


        verticalLayout->addWidget(topBackground);

        stackedWidget = new MainStackedWidget(MainWindow);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy1);
        stackedWidgetPage1 = new QWidget();
        stackedWidgetPage1->setObjectName("stackedWidgetPage1");
        stackedWidgetPage1->setAutoFillBackground(true);
        gridLayout_2 = new QGridLayout(stackedWidgetPage1);
        gridLayout_2->setObjectName("gridLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushButtonNew = new QPushButton(stackedWidgetPage1);
        pushButtonNew->setObjectName("pushButtonNew");
        pushButtonNew->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonNew->setIconSize(QSize(36, 36));

        horizontalLayout->addWidget(pushButtonNew);

        pushButtonOpen = new QPushButton(stackedWidgetPage1);
        pushButtonOpen->setObjectName("pushButtonOpen");
        pushButtonOpen->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonOpen->setIconSize(QSize(36, 36));

        horizontalLayout->addWidget(pushButtonOpen);

        pushButtonOpenDir = new QPushButton(stackedWidgetPage1);
        pushButtonOpenDir->setObjectName("pushButtonOpenDir");
        pushButtonOpenDir->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonOpenDir->setIconSize(QSize(36, 36));

        horizontalLayout->addWidget(pushButtonOpenDir);


        gridLayout_2->addLayout(horizontalLayout, 0, 0, 1, 1);

        widgetRepo = new QWidget(stackedWidgetPage1);
        widgetRepo->setObjectName("widgetRepo");
        widgetRepo->setMinimumSize(QSize(29, 0));
        gridLayout_4 = new QGridLayout(widgetRepo);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        lineSearch = new QLineEdit(widgetRepo);
        lineSearch->setObjectName("lineSearch");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(lineSearch->sizePolicy().hasHeightForWidth());
        lineSearch->setSizePolicy(sizePolicy2);
        lineSearch->setStyleSheet(QString::fromUtf8("height:52px"));

        gridLayout_4->addWidget(lineSearch, 3, 1, 1, 1);

        pushButtonSearch = new QPushButton(widgetRepo);
        pushButtonSearch->setObjectName("pushButtonSearch");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(pushButtonSearch->sizePolicy().hasHeightForWidth());
        pushButtonSearch->setSizePolicy(sizePolicy3);
        pushButtonSearch->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonSearch->setIconSize(QSize(36, 36));

        gridLayout_4->addWidget(pushButtonSearch, 3, 2, 1, 1);

        widgetShowSoundfonts = new ShowSoundfonts(widgetRepo);
        widgetShowSoundfonts->setObjectName("widgetShowSoundfonts");
        sizePolicy1.setHeightForWidth(widgetShowSoundfonts->sizePolicy().hasHeightForWidth());
        widgetShowSoundfonts->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(widgetShowSoundfonts, 2, 1, 1, 2);

        line = new QFrame(widgetRepo);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_4->addWidget(line, 0, 0, 4, 1);

        label = new QLabel(widgetRepo);
        label->setObjectName("label");
        QFont font;
        font.setBold(true);
        label->setFont(font);

        gridLayout_4->addWidget(label, 1, 1, 1, 2);

        pushButtonSoundfonts = new QPushButton(widgetRepo);
        pushButtonSoundfonts->setObjectName("pushButtonSoundfonts");
        pushButtonSoundfonts->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonSoundfonts->setIconSize(QSize(36, 36));

        gridLayout_4->addWidget(pushButtonSoundfonts, 0, 1, 1, 2);


        gridLayout_2->addWidget(widgetRepo, 0, 1, 4, 1);

        label_2 = new QLabel(stackedWidgetPage1);
        label_2->setObjectName("label_2");
        label_2->setFont(font);
        label_2->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignTop);

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        widgetShowHistory = new WidgetShowHistory(stackedWidgetPage1);
        widgetShowHistory->setObjectName("widgetShowHistory");
        sizePolicy1.setHeightForWidth(widgetShowHistory->sizePolicy().hasHeightForWidth());
        widgetShowHistory->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(widgetShowHistory, 2, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushButtonDocumentation = new QPushButton(stackedWidgetPage1);
        pushButtonDocumentation->setObjectName("pushButtonDocumentation");
        sizePolicy3.setHeightForWidth(pushButtonDocumentation->sizePolicy().hasHeightForWidth());
        pushButtonDocumentation->setSizePolicy(sizePolicy3);
        pushButtonDocumentation->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonDocumentation->setIconSize(QSize(36, 36));

        horizontalLayout_2->addWidget(pushButtonDocumentation);

        pushButtonForum = new QPushButton(stackedWidgetPage1);
        pushButtonForum->setObjectName("pushButtonForum");
        sizePolicy3.setHeightForWidth(pushButtonForum->sizePolicy().hasHeightForWidth());
        pushButtonForum->setSizePolicy(sizePolicy3);
        pushButtonForum->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonForum->setIconSize(QSize(36, 36));

        horizontalLayout_2->addWidget(pushButtonForum);

        pushButtonSettings = new QPushButton(stackedWidgetPage1);
        pushButtonSettings->setObjectName("pushButtonSettings");
        sizePolicy3.setHeightForWidth(pushButtonSettings->sizePolicy().hasHeightForWidth());
        pushButtonSettings->setSizePolicy(sizePolicy3);
        pushButtonSettings->setStyleSheet(QString::fromUtf8("padding: 10px;"));
        pushButtonSettings->setIconSize(QSize(36, 36));

        horizontalLayout_2->addWidget(pushButtonSettings);


        gridLayout_2->addLayout(horizontalLayout_2, 3, 0, 1, 1);

        gridLayout_2->setColumnStretch(0, 6);
        gridLayout_2->setColumnStretch(1, 4);
        gridLayout_2->setColumnMinimumWidth(1, 300);
        stackedWidget->addWidget(stackedWidgetPage1);

        verticalLayout->addWidget(stackedWidget);

        QWidget::setTabOrder(pushHome, pushButtonNew);
        QWidget::setTabOrder(pushButtonNew, pushButtonOpen);
        QWidget::setTabOrder(pushButtonOpen, pushButtonOpenDir);
        QWidget::setTabOrder(pushButtonOpenDir, pushButtonDocumentation);
        QWidget::setTabOrder(pushButtonDocumentation, pushButtonForum);
        QWidget::setTabOrder(pushButtonForum, pushButtonSettings);
        QWidget::setTabOrder(pushButtonSettings, pushButtonSoundfonts);
        QWidget::setTabOrder(pushButtonSoundfonts, lineSearch);
        QWidget::setTabOrder(lineSearch, pushButtonSearch);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QWidget *MainWindow)
    {
        pushButtonNew->setText(QCoreApplication::translate("MainWindow", "  New\n"
"  soundfont", nullptr));
        pushButtonOpen->setText(QCoreApplication::translate("MainWindow", "  Open\n"
"  soundfont", nullptr));
        pushButtonOpenDir->setText(QCoreApplication::translate("MainWindow", "  Open\n"
"  directory", nullptr));
        lineSearch->setPlaceholderText(QCoreApplication::translate("MainWindow", "Search a soundfont...", nullptr));
        pushButtonSearch->setText(QString());
        label->setText(QCoreApplication::translate("MainWindow", "Daily soundfonts", nullptr));
        pushButtonSoundfonts->setText(QCoreApplication::translate("MainWindow", "  Online\n"
"  repository", nullptr));
        label_2->setText(QCoreApplication::translate("MainWindow", "History", nullptr));
        pushButtonDocumentation->setText(QCoreApplication::translate("MainWindow", "  Documentation", nullptr));
        pushButtonForum->setText(QCoreApplication::translate("MainWindow", "  Forum", nullptr));
        pushButtonSettings->setText(QCoreApplication::translate("MainWindow", "  Settings", nullptr));
        (void)MainWindow;
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
