/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2024 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone.io                             **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "playerpresetlist.h"
#include <qmath.h>

PlayerPresetList::PlayerPresetList(QWidget *parent) : QListView(parent)
{
    QFontMetrics fm(this->font());
    _minCellWidth = fm.horizontalAdvance("mmmmmmmmmmmmmmmmmmmm") + 12; // 20 characters + margin
    _cellHeight = fm.height() * 2 + 12; // 2 lines + margin
}

PlayerPresetList::~PlayerPresetList()
{

}

void PlayerPresetList::showEvent(QShowEvent * event)
{
    resizeGrid();
    QListView::showEvent(event);
}

void PlayerPresetList::resizeEvent(QResizeEvent * event)
{
    resizeGrid();
    QListView::resizeEvent(event);
}

void PlayerPresetList::resizeGrid()
{
    this->setGridSize(QSize(this->viewport()->width() / columnCount() - 1, rowHeight()));
}

int PlayerPresetList::columnCount()
{
    return static_cast<int>(qCeil((double)this->viewport()->width() / _minCellWidth) + 0.5);
}

int PlayerPresetList::rowHeight()
{
    return _cellHeight;
}
