/********************************************************************************
** Form generated from reading UI file 'soundfontfilecell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTFILECELL_H
#define UI_SOUNDFONTFILECELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>
#include "styledlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontFileCell
{
public:
    QGridLayout *gridLayout;
    QPushButton *pushDelete;
    QLabel *labelInfo;
    QTextEdit *textDescription;
    StyledLineEdit *lineTitle;
    QPushButton *pushReplace;

    void setupUi(QWidget *SoundfontFileCell)
    {
        if (SoundfontFileCell->objectName().isEmpty())
            SoundfontFileCell->setObjectName("SoundfontFileCell");
        SoundfontFileCell->resize(271, 122);
        SoundfontFileCell->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SoundfontFileCell);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(3);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        pushDelete = new QPushButton(SoundfontFileCell);
        pushDelete->setObjectName("pushDelete");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pushDelete->sizePolicy().hasHeightForWidth());
        pushDelete->setSizePolicy(sizePolicy);
        pushDelete->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushDelete->setIconSize(QSize(12, 12));

        gridLayout->addWidget(pushDelete, 0, 2, 1, 1);

        labelInfo = new QLabel(SoundfontFileCell);
        labelInfo->setObjectName("labelInfo");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelInfo->sizePolicy().hasHeightForWidth());
        labelInfo->setSizePolicy(sizePolicy1);
        QFont font;
        font.setItalic(true);
        labelInfo->setFont(font);
        labelInfo->setText(QString::fromUtf8("size, date OR filepath"));

        gridLayout->addWidget(labelInfo, 3, 0, 1, 3);

        textDescription = new QTextEdit(SoundfontFileCell);
        textDescription->setObjectName("textDescription");

        gridLayout->addWidget(textDescription, 2, 0, 1, 3);

        lineTitle = new StyledLineEdit(SoundfontFileCell);
        lineTitle->setObjectName("lineTitle");
        QFont font1;
        font1.setBold(true);
        lineTitle->setFont(font1);

        gridLayout->addWidget(lineTitle, 0, 0, 1, 1);

        pushReplace = new QPushButton(SoundfontFileCell);
        pushReplace->setObjectName("pushReplace");
        sizePolicy.setHeightForWidth(pushReplace->sizePolicy().hasHeightForWidth());
        pushReplace->setSizePolicy(sizePolicy);
        pushReplace->setIconSize(QSize(12, 12));

        gridLayout->addWidget(pushReplace, 0, 1, 1, 1);


        retranslateUi(SoundfontFileCell);

        QMetaObject::connectSlotsByName(SoundfontFileCell);
    } // setupUi

    void retranslateUi(QWidget *SoundfontFileCell)
    {
#if QT_CONFIG(tooltip)
        pushDelete->setToolTip(QCoreApplication::translate("SoundfontFileCell", "Remove", nullptr));
#endif // QT_CONFIG(tooltip)
        pushDelete->setText(QString());
        textDescription->setPlaceholderText(QCoreApplication::translate("SoundfontFileCell", "Description...", nullptr));
        lineTitle->setPlaceholderText(QCoreApplication::translate("SoundfontFileCell", "Title...", nullptr));
#if QT_CONFIG(tooltip)
        pushReplace->setToolTip(QCoreApplication::translate("SoundfontFileCell", "Replace", nullptr));
#endif // QT_CONFIG(tooltip)
        pushReplace->setText(QString());
        (void)SoundfontFileCell;
    } // retranslateUi

};

namespace Ui {
    class SoundfontFileCell: public Ui_SoundfontFileCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTFILECELL_H
