/********************************************************************************
** Form generated from reading UI file 'dialogcompressquality.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGCOMPRESSQUALITY_H
#define UI_DIALOGCOMPRESSQUALITY_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_DialogCompressQuality
{
public:
    QGridLayout *gridLayout_3;
    QRadioButton *radioButtonMiddle;
    QDialogButtonBox *buttonBox;
    QLabel *label;
    QRadioButton *radioButtonHigh;
    QRadioButton *radioButtonLow;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *DialogCompressQuality)
    {
        if (DialogCompressQuality->objectName().isEmpty())
            DialogCompressQuality->setObjectName("DialogCompressQuality");
        DialogCompressQuality->setWindowModality(Qt::WindowModal);
        DialogCompressQuality->resize(342, 164);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogCompressQuality->setWindowIcon(icon);
        gridLayout_3 = new QGridLayout(DialogCompressQuality);
        gridLayout_3->setObjectName("gridLayout_3");
        radioButtonMiddle = new QRadioButton(DialogCompressQuality);
        radioButtonMiddle->setObjectName("radioButtonMiddle");

        gridLayout_3->addWidget(radioButtonMiddle, 2, 0, 1, 1);

        buttonBox = new QDialogButtonBox(DialogCompressQuality);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout_3->addWidget(buttonBox, 5, 0, 1, 1);

        label = new QLabel(DialogCompressQuality);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 0, 0, 1, 1);

        radioButtonHigh = new QRadioButton(DialogCompressQuality);
        radioButtonHigh->setObjectName("radioButtonHigh");

        gridLayout_3->addWidget(radioButtonHigh, 1, 0, 1, 1);

        radioButtonLow = new QRadioButton(DialogCompressQuality);
        radioButtonLow->setObjectName("radioButtonLow");

        gridLayout_3->addWidget(radioButtonLow, 3, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 4, 0, 1, 1);


        retranslateUi(DialogCompressQuality);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DialogCompressQuality, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DialogCompressQuality, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DialogCompressQuality);
    } // setupUi

    void retranslateUi(QDialog *DialogCompressQuality)
    {
        DialogCompressQuality->setWindowTitle(QCoreApplication::translate("DialogCompressQuality", "Sample compression", nullptr));
        radioButtonMiddle->setText(QCoreApplication::translate("DialogCompressQuality", "Medium", nullptr));
        label->setText(QCoreApplication::translate("DialogCompressQuality", "Quality of the compression:", nullptr));
        radioButtonHigh->setText(QCoreApplication::translate("DialogCompressQuality", "High (good sound quality but larger file size)", nullptr));
        radioButtonLow->setText(QCoreApplication::translate("DialogCompressQuality", "Low (poor sound quality but smaller file size)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogCompressQuality: public Ui_DialogCompressQuality {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGCOMPRESSQUALITY_H
