/********************************************************************************
** Form generated from reading UI file 'toolfrequencyfilter_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLFREQUENCYFILTER_GUI_H
#define UI_TOOLFREQUENCYFILTER_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "frequency_filter/graphfilterfrequencies.h"

QT_BEGIN_NAMESPACE

class Ui_ToolFrequencyFilter_gui
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *frame;
    QGridLayout *gridLayout_2;
    GraphFilterFrequencies *graphFilterFrequencies;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolFrequencyFilter_gui)
    {
        if (ToolFrequencyFilter_gui->objectName().isEmpty())
            ToolFrequencyFilter_gui->setObjectName("ToolFrequencyFilter_gui");
        ToolFrequencyFilter_gui->resize(450, 268);
        ToolFrequencyFilter_gui->setMinimumSize(QSize(450, 0));
        ToolFrequencyFilter_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolFrequencyFilter_gui);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(ToolFrequencyFilter_gui);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(label);

        frame = new QFrame(ToolFrequencyFilter_gui);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Sunken);
        gridLayout_2 = new QGridLayout(frame);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        graphFilterFrequencies = new GraphFilterFrequencies(frame);
        graphFilterFrequencies->setObjectName("graphFilterFrequencies");
        graphFilterFrequencies->setMinimumSize(QSize(300, 200));

        gridLayout_2->addWidget(graphFilterFrequencies, 0, 0, 1, 1);


        verticalLayout->addWidget(frame);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolFrequencyFilter_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolFrequencyFilter_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolFrequencyFilter_gui);

        QMetaObject::connectSlotsByName(ToolFrequencyFilter_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolFrequencyFilter_gui)
    {
        label->setText(QCoreApplication::translate("ToolFrequencyFilter_gui", "Mark with red color where you want to cut the frequencies:", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolFrequencyFilter_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolFrequencyFilter_gui", "&Cancel", nullptr));
        (void)ToolFrequencyFilter_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolFrequencyFilter_gui: public Ui_ToolFrequencyFilter_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLFREQUENCYFILTER_GUI_H
