/********************************************************************************
** Form generated from reading UI file 'widgetshowhistory.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WIDGETSHOWHISTORY_H
#define UI_WIDGETSHOWHISTORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_WidgetShowHistory
{
public:
    QGridLayout *gridLayout;
    QListWidget *listWidget;
    QFrame *frameNoHistory;
    QGridLayout *gridLayout_2;
    QLabel *label;

    void setupUi(QWidget *WidgetShowHistory)
    {
        if (WidgetShowHistory->objectName().isEmpty())
            WidgetShowHistory->setObjectName("WidgetShowHistory");
        WidgetShowHistory->resize(400, 300);
        WidgetShowHistory->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(WidgetShowHistory);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        listWidget = new QListWidget(WidgetShowHistory);
        listWidget->setObjectName("listWidget");
        listWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        listWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);

        gridLayout->addWidget(listWidget, 1, 0, 1, 1);

        frameNoHistory = new QFrame(WidgetShowHistory);
        frameNoHistory->setObjectName("frameNoHistory");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frameNoHistory->sizePolicy().hasHeightForWidth());
        frameNoHistory->setSizePolicy(sizePolicy);
        frameNoHistory->setFrameShape(QFrame::StyledPanel);
        frameNoHistory->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frameNoHistory);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(frameNoHistory);
        label->setObjectName("label");
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(label, 0, 0, 1, 1);


        gridLayout->addWidget(frameNoHistory, 0, 0, 1, 1);


        retranslateUi(WidgetShowHistory);

        QMetaObject::connectSlotsByName(WidgetShowHistory);
    } // setupUi

    void retranslateUi(QWidget *WidgetShowHistory)
    {
        label->setText(QCoreApplication::translate("WidgetShowHistory", "Empty list", nullptr));
        (void)WidgetShowHistory;
    } // retranslateUi

};

namespace Ui {
    class WidgetShowHistory: public Ui_WidgetShowHistory {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WIDGETSHOWHISTORY_H
