/********************************************************************************
** Form generated from reading UI file 'configsectionrepository.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSECTIONREPOSITORY_H
#define UI_CONFIGSECTIONREPOSITORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigSectionRepository
{
public:
    QGridLayout *gridLayout;
    QLabel *label_35;
    QStackedWidget *stackedWidget;
    QWidget *pageCreateAccount;
    QGridLayout *gridLayout_3;
    QToolButton *buttonCreateAccount;
    QWidget *pageBecomePremium;
    QGridLayout *gridLayout_10;
    QToolButton *buttonBecomePremium;
    QWidget *pagePremium;
    QGridLayout *gridLayout_4;
    QLabel *labelPremium;
    QWidget *pageBanned;
    QGridLayout *gridLayout_2;
    QSpacerItem *verticalSpacer;
    QLabel *labelBan2;
    QLabel *labelBan1;
    QSpacerItem *verticalSpacer_2;
    QWidget *pageAdmin;
    QGridLayout *gridLayout_5;
    QLabel *labelAdmin;
    QLineEdit *linePassword;
    QPushButton *pushConnect;
    QLineEdit *lineUser;
    QLabel *label_34;
    QLabel *label_5;
    QLabel *labelMessage;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineDownloadDirectory;
    QPushButton *pushDirectory;
    QSpacerItem *horizontalSpacer;
    QLabel *labelSubTitle1;
    QLabel *labelSubTitle2;

    void setupUi(QWidget *ConfigSectionRepository)
    {
        if (ConfigSectionRepository->objectName().isEmpty())
            ConfigSectionRepository->setObjectName("ConfigSectionRepository");
        ConfigSectionRepository->resize(355, 234);
        ConfigSectionRepository->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ConfigSectionRepository);
        gridLayout->setObjectName("gridLayout");
        label_35 = new QLabel(ConfigSectionRepository);
        label_35->setObjectName("label_35");

        gridLayout->addWidget(label_35, 6, 1, 1, 1);

        stackedWidget = new QStackedWidget(ConfigSectionRepository);
        stackedWidget->setObjectName("stackedWidget");
        stackedWidget->setMaximumSize(QSize(130, 16777215));
        pageCreateAccount = new QWidget();
        pageCreateAccount->setObjectName("pageCreateAccount");
        gridLayout_3 = new QGridLayout(pageCreateAccount);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        buttonCreateAccount = new QToolButton(pageCreateAccount);
        buttonCreateAccount->setObjectName("buttonCreateAccount");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonCreateAccount->sizePolicy().hasHeightForWidth());
        buttonCreateAccount->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        buttonCreateAccount->setFont(font);
        buttonCreateAccount->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonCreateAccount->setIconSize(QSize(48, 48));
        buttonCreateAccount->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

        gridLayout_3->addWidget(buttonCreateAccount, 0, 0, 1, 1);

        stackedWidget->addWidget(pageCreateAccount);
        pageBecomePremium = new QWidget();
        pageBecomePremium->setObjectName("pageBecomePremium");
        gridLayout_10 = new QGridLayout(pageBecomePremium);
        gridLayout_10->setObjectName("gridLayout_10");
        gridLayout_10->setContentsMargins(0, 0, 0, 0);
        buttonBecomePremium = new QToolButton(pageBecomePremium);
        buttonBecomePremium->setObjectName("buttonBecomePremium");
        sizePolicy.setHeightForWidth(buttonBecomePremium->sizePolicy().hasHeightForWidth());
        buttonBecomePremium->setSizePolicy(sizePolicy);
        buttonBecomePremium->setFont(font);
        buttonBecomePremium->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        buttonBecomePremium->setIconSize(QSize(48, 48));
        buttonBecomePremium->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

        gridLayout_10->addWidget(buttonBecomePremium, 0, 0, 1, 1);

        stackedWidget->addWidget(pageBecomePremium);
        pagePremium = new QWidget();
        pagePremium->setObjectName("pagePremium");
        gridLayout_4 = new QGridLayout(pagePremium);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        labelPremium = new QLabel(pagePremium);
        labelPremium->setObjectName("labelPremium");
        labelPremium->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelPremium, 0, 0, 1, 1);

        stackedWidget->addWidget(pagePremium);
        pageBanned = new QWidget();
        pageBanned->setObjectName("pageBanned");
        gridLayout_2 = new QGridLayout(pageBanned);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setVerticalSpacing(0);
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 3, 0, 1, 1);

        labelBan2 = new QLabel(pageBanned);
        labelBan2->setObjectName("labelBan2");
        labelBan2->setFont(font);
        labelBan2->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelBan2, 2, 0, 1, 1);

        labelBan1 = new QLabel(pageBanned);
        labelBan1->setObjectName("labelBan1");
        labelBan1->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelBan1, 1, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 0, 0, 1, 1);

        stackedWidget->addWidget(pageBanned);
        pageAdmin = new QWidget();
        pageAdmin->setObjectName("pageAdmin");
        gridLayout_5 = new QGridLayout(pageAdmin);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        labelAdmin = new QLabel(pageAdmin);
        labelAdmin->setObjectName("labelAdmin");
        labelAdmin->setAlignment(Qt::AlignCenter);

        gridLayout_5->addWidget(labelAdmin, 0, 0, 1, 1);

        stackedWidget->addWidget(pageAdmin);

        gridLayout->addWidget(stackedWidget, 1, 3, 3, 1);

        linePassword = new QLineEdit(ConfigSectionRepository);
        linePassword->setObjectName("linePassword");
        linePassword->setEchoMode(QLineEdit::Password);

        gridLayout->addWidget(linePassword, 2, 2, 1, 1);

        pushConnect = new QPushButton(ConfigSectionRepository);
        pushConnect->setObjectName("pushConnect");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushConnect->sizePolicy().hasHeightForWidth());
        pushConnect->setSizePolicy(sizePolicy1);
        pushConnect->setText(QString::fromUtf8("Connexion"));

        gridLayout->addWidget(pushConnect, 3, 1, 1, 1);

        lineUser = new QLineEdit(ConfigSectionRepository);
        lineUser->setObjectName("lineUser");

        gridLayout->addWidget(lineUser, 1, 2, 1, 1);

        label_34 = new QLabel(ConfigSectionRepository);
        label_34->setObjectName("label_34");

        gridLayout->addWidget(label_34, 2, 1, 1, 1);

        label_5 = new QLabel(ConfigSectionRepository);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 1, 1, 1, 1);

        labelMessage = new QLabel(ConfigSectionRepository);
        labelMessage->setObjectName("labelMessage");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelMessage->sizePolicy().hasHeightForWidth());
        labelMessage->setSizePolicy(sizePolicy2);
        labelMessage->setText(QString::fromUtf8("..."));

        gridLayout->addWidget(labelMessage, 3, 2, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 9, -1, -1);
        lineDownloadDirectory = new QLineEdit(ConfigSectionRepository);
        lineDownloadDirectory->setObjectName("lineDownloadDirectory");

        horizontalLayout->addWidget(lineDownloadDirectory);

        pushDirectory = new QPushButton(ConfigSectionRepository);
        pushDirectory->setObjectName("pushDirectory");
        pushDirectory->setMaximumSize(QSize(26, 26));
        pushDirectory->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushDirectory->setFlat(true);

        horizontalLayout->addWidget(pushDirectory);


        gridLayout->addLayout(horizontalLayout, 6, 2, 1, 2);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 0, 1, 1);

        labelSubTitle1 = new QLabel(ConfigSectionRepository);
        labelSubTitle1->setObjectName("labelSubTitle1");
        labelSubTitle1->setFont(font);

        gridLayout->addWidget(labelSubTitle1, 0, 0, 1, 4);

        labelSubTitle2 = new QLabel(ConfigSectionRepository);
        labelSubTitle2->setObjectName("labelSubTitle2");
        labelSubTitle2->setFont(font);

        gridLayout->addWidget(labelSubTitle2, 5, 0, 1, 4);

        QWidget::setTabOrder(lineUser, linePassword);
        QWidget::setTabOrder(linePassword, pushConnect);
        QWidget::setTabOrder(pushConnect, buttonCreateAccount);
        QWidget::setTabOrder(buttonCreateAccount, lineDownloadDirectory);
        QWidget::setTabOrder(lineDownloadDirectory, pushDirectory);
        QWidget::setTabOrder(pushDirectory, buttonBecomePremium);

        retranslateUi(ConfigSectionRepository);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ConfigSectionRepository);
    } // setupUi

    void retranslateUi(QWidget *ConfigSectionRepository)
    {
        label_35->setText(QCoreApplication::translate("ConfigSectionRepository", "Directory", nullptr));
        buttonCreateAccount->setText(QCoreApplication::translate("ConfigSectionRepository", "Create an account", nullptr));
        buttonBecomePremium->setText(QCoreApplication::translate("ConfigSectionRepository", "Become\n"
"Premium!", nullptr));
        labelPremium->setText(QString());
        labelBan2->setText(QCoreApplication::translate("ConfigSectionRepository", "Banned account", nullptr));
        labelBan1->setText(QString());
        labelAdmin->setText(QString());
        label_34->setText(QCoreApplication::translate("ConfigSectionRepository", "Password", nullptr));
        label_5->setText(QCoreApplication::translate("ConfigSectionRepository", "Email address", nullptr));
        pushDirectory->setText(QString());
        labelSubTitle1->setText(QCoreApplication::translate("ConfigSectionRepository", "Account", nullptr));
        labelSubTitle2->setText(QCoreApplication::translate("ConfigSectionRepository", "Downloads", nullptr));
        (void)ConfigSectionRepository;
    } // retranslateUi

};

namespace Ui {
    class ConfigSectionRepository: public Ui_ConfigSectionRepository {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSECTIONREPOSITORY_H
