/********************************************************************************
** Form generated from reading UI file 'soundfontcommentcell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTCOMMENTCELL_H
#define UI_SOUNDFONTCOMMENTCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SoundfontCommentCell
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *leftBars;
    QGridLayout *gridLayout_2;
    QSpacerItem *horizontalSpacer;
    QLabel *labelUser;
    QLabel *labelDateIcon;
    QLabel *labelUserIcon;
    QLabel *labelDate;
    QTextBrowser *textComment;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *SoundfontCommentCell)
    {
        if (SoundfontCommentCell->objectName().isEmpty())
            SoundfontCommentCell->setObjectName("SoundfontCommentCell");
        SoundfontCommentCell->resize(252, 118);
        SoundfontCommentCell->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SoundfontCommentCell);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(0);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        leftBars = new QHBoxLayout();
        leftBars->setObjectName("leftBars");

        gridLayout->addLayout(leftBars, 0, 0, 2, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setHorizontalSpacing(9);
        gridLayout_2->setVerticalSpacing(12);
        gridLayout_2->setContentsMargins(-1, 9, -1, 9);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 5, 1, 1);

        labelUser = new QLabel(SoundfontCommentCell);
        labelUser->setObjectName("labelUser");
        QFont font;
        font.setBold(true);
        labelUser->setFont(font);
        labelUser->setText(QString::fromUtf8("user"));

        gridLayout_2->addWidget(labelUser, 0, 1, 1, 1);

        labelDateIcon = new QLabel(SoundfontCommentCell);
        labelDateIcon->setObjectName("labelDateIcon");
        labelDateIcon->setMinimumSize(QSize(16, 0));
        labelDateIcon->setMaximumSize(QSize(16, 16777215));
        labelDateIcon->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelDateIcon, 0, 3, 1, 1);

        labelUserIcon = new QLabel(SoundfontCommentCell);
        labelUserIcon->setObjectName("labelUserIcon");
        labelUserIcon->setMinimumSize(QSize(16, 0));
        labelUserIcon->setMaximumSize(QSize(16, 16777215));
        labelUserIcon->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelUserIcon, 0, 0, 1, 1);

        labelDate = new QLabel(SoundfontCommentCell);
        labelDate->setObjectName("labelDate");
        labelDate->setFont(font);
        labelDate->setText(QString::fromUtf8("date"));

        gridLayout_2->addWidget(labelDate, 0, 4, 1, 1);

        textComment = new QTextBrowser(SoundfontCommentCell);
        textComment->setObjectName("textComment");
        textComment->setFrameShape(QFrame::NoFrame);

        gridLayout_2->addWidget(textComment, 1, 0, 1, 6);

        horizontalSpacer_2 = new QSpacerItem(6, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 0, 2, 1, 1);


        gridLayout->addLayout(gridLayout_2, 0, 1, 2, 1);


        retranslateUi(SoundfontCommentCell);

        QMetaObject::connectSlotsByName(SoundfontCommentCell);
    } // setupUi

    void retranslateUi(QWidget *SoundfontCommentCell)
    {
        labelDateIcon->setText(QString());
        labelUserIcon->setText(QString());
        (void)SoundfontCommentCell;
    } // retranslateUi

};

namespace Ui {
    class SoundfontCommentCell: public Ui_SoundfontCommentCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTCOMMENTCELL_H
