/********************************************************************************
** Form generated from reading UI file 'userarea.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_USERAREA_H
#define UI_USERAREA_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_UserArea
{
public:
    QGridLayout *gridLayout;
    QLabel *label;

    void setupUi(QWidget *UserArea)
    {
        if (UserArea->objectName().isEmpty())
            UserArea->setObjectName("UserArea");
        UserArea->resize(462, 328);
        UserArea->setWindowTitle(QString::fromUtf8("Form"));
        UserArea->setAutoFillBackground(true);
        gridLayout = new QGridLayout(UserArea);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(UserArea);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("soon available"));
        label->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(label, 0, 0, 1, 1);


        retranslateUi(UserArea);

        QMetaObject::connectSlotsByName(UserArea);
    } // setupUi

    void retranslateUi(QWidget *UserArea)
    {
        (void)UserArea;
    } // retranslateUi

};

namespace Ui {
    class UserArea: public Ui_UserArea {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_USERAREA_H
