/********************************************************************************
** Form generated from reading UI file 'soundfontdownload.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTDOWNLOAD_H
#define UI_SOUNDFONTDOWNLOAD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SoundfontDownload
{
public:
    QVBoxLayout *verticalLayout;

    void setupUi(QWidget *SoundfontDownload)
    {
        if (SoundfontDownload->objectName().isEmpty())
            SoundfontDownload->setObjectName("SoundfontDownload");
        SoundfontDownload->resize(190, 43);
        SoundfontDownload->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontDownload);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, -1, 0, -1);

        retranslateUi(SoundfontDownload);

        QMetaObject::connectSlotsByName(SoundfontDownload);
    } // setupUi

    void retranslateUi(QWidget *SoundfontDownload)
    {
        (void)SoundfontDownload;
    } // retranslateUi

};

namespace Ui {
    class SoundfontDownload: public Ui_SoundfontDownload {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTDOWNLOAD_H
