/********************************************************************************
** Form generated from reading UI file 'soundfontviewerleft.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTVIEWERLEFT_H
#define UI_SOUNDFONTVIEWERLEFT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "attributeflow.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontViewerLeft
{
public:
    QGridLayout *gridLayout;
    QLabel *iconAuthor;
    QLabel *labelAuthor;
    QLabel *iconDate;
    QLabel *labelDate;
    QLabel *iconLicense;
    QLabel *labelLicense;
    QLabel *iconWebsite;
    QLabel *labelWebsite;
    QSpacerItem *verticalSpacer_6;
    QLabel *labelCategory;
    AttributeFlow *attributeCategory;
    QLabel *labelSampleSource;
    AttributeFlow *attributeSampleSource;
    QLabel *labelTimbre;
    AttributeFlow *attributeTimbre;
    QLabel *labelArticulation;
    AttributeFlow *attributeArticulation;
    QLabel *labelGenre;
    AttributeFlow *attributeGenre;
    QLabel *labelMidiStandard;
    AttributeFlow *attributeMidiStandard;
    QLabel *labelTags;
    AttributeFlow *attributeTag;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *SoundfontViewerLeft)
    {
        if (SoundfontViewerLeft->objectName().isEmpty())
            SoundfontViewerLeft->setObjectName("SoundfontViewerLeft");
        SoundfontViewerLeft->resize(140, 440);
        SoundfontViewerLeft->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SoundfontViewerLeft);
        gridLayout->setObjectName("gridLayout");
        iconAuthor = new QLabel(SoundfontViewerLeft);
        iconAuthor->setObjectName("iconAuthor");
        iconAuthor->setMinimumSize(QSize(16, 16));
        iconAuthor->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconAuthor, 0, 0, 1, 1);

        labelAuthor = new QLabel(SoundfontViewerLeft);
        labelAuthor->setObjectName("labelAuthor");
        labelAuthor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        labelAuthor->setText(QString::fromUtf8("labelAuthor"));
        labelAuthor->setTextFormat(Qt::RichText);
        labelAuthor->setWordWrap(true);

        gridLayout->addWidget(labelAuthor, 0, 1, 1, 1);

        iconDate = new QLabel(SoundfontViewerLeft);
        iconDate->setObjectName("iconDate");
        iconDate->setMinimumSize(QSize(16, 16));
        iconDate->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconDate, 1, 0, 1, 1);

        labelDate = new QLabel(SoundfontViewerLeft);
        labelDate->setObjectName("labelDate");
        labelDate->setText(QString::fromUtf8("labelDate"));

        gridLayout->addWidget(labelDate, 1, 1, 1, 1);

        iconLicense = new QLabel(SoundfontViewerLeft);
        iconLicense->setObjectName("iconLicense");
        iconLicense->setMinimumSize(QSize(16, 16));
        iconLicense->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconLicense, 2, 0, 1, 1);

        labelLicense = new QLabel(SoundfontViewerLeft);
        labelLicense->setObjectName("labelLicense");
        labelLicense->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        labelLicense->setText(QString::fromUtf8("labelLicense"));
        labelLicense->setTextFormat(Qt::RichText);
        labelLicense->setWordWrap(true);
        labelLicense->setOpenExternalLinks(true);

        gridLayout->addWidget(labelLicense, 2, 1, 1, 1);

        iconWebsite = new QLabel(SoundfontViewerLeft);
        iconWebsite->setObjectName("iconWebsite");

        gridLayout->addWidget(iconWebsite, 3, 0, 1, 1);

        labelWebsite = new QLabel(SoundfontViewerLeft);
        labelWebsite->setObjectName("labelWebsite");
        labelWebsite->setText(QString::fromUtf8("labelWebsite"));
        labelWebsite->setWordWrap(true);
        labelWebsite->setOpenExternalLinks(true);

        gridLayout->addWidget(labelWebsite, 3, 1, 1, 1);

        verticalSpacer_6 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(verticalSpacer_6, 4, 0, 1, 1);

        labelCategory = new QLabel(SoundfontViewerLeft);
        labelCategory->setObjectName("labelCategory");
        QFont font;
        font.setBold(true);
        labelCategory->setFont(font);

        gridLayout->addWidget(labelCategory, 5, 0, 1, 2);

        attributeCategory = new AttributeFlow(SoundfontViewerLeft);
        attributeCategory->setObjectName("attributeCategory");

        gridLayout->addWidget(attributeCategory, 6, 0, 1, 2);

        labelSampleSource = new QLabel(SoundfontViewerLeft);
        labelSampleSource->setObjectName("labelSampleSource");
        labelSampleSource->setFont(font);

        gridLayout->addWidget(labelSampleSource, 7, 0, 1, 2);

        attributeSampleSource = new AttributeFlow(SoundfontViewerLeft);
        attributeSampleSource->setObjectName("attributeSampleSource");

        gridLayout->addWidget(attributeSampleSource, 8, 0, 1, 2);

        labelTimbre = new QLabel(SoundfontViewerLeft);
        labelTimbre->setObjectName("labelTimbre");
        labelTimbre->setFont(font);

        gridLayout->addWidget(labelTimbre, 9, 0, 1, 2);

        attributeTimbre = new AttributeFlow(SoundfontViewerLeft);
        attributeTimbre->setObjectName("attributeTimbre");

        gridLayout->addWidget(attributeTimbre, 10, 0, 1, 2);

        labelArticulation = new QLabel(SoundfontViewerLeft);
        labelArticulation->setObjectName("labelArticulation");
        labelArticulation->setFont(font);

        gridLayout->addWidget(labelArticulation, 11, 0, 1, 2);

        attributeArticulation = new AttributeFlow(SoundfontViewerLeft);
        attributeArticulation->setObjectName("attributeArticulation");

        gridLayout->addWidget(attributeArticulation, 12, 0, 1, 2);

        labelGenre = new QLabel(SoundfontViewerLeft);
        labelGenre->setObjectName("labelGenre");
        labelGenre->setFont(font);

        gridLayout->addWidget(labelGenre, 13, 0, 1, 2);

        attributeGenre = new AttributeFlow(SoundfontViewerLeft);
        attributeGenre->setObjectName("attributeGenre");

        gridLayout->addWidget(attributeGenre, 14, 0, 1, 2);

        labelMidiStandard = new QLabel(SoundfontViewerLeft);
        labelMidiStandard->setObjectName("labelMidiStandard");
        labelMidiStandard->setFont(font);

        gridLayout->addWidget(labelMidiStandard, 15, 0, 1, 2);

        attributeMidiStandard = new AttributeFlow(SoundfontViewerLeft);
        attributeMidiStandard->setObjectName("attributeMidiStandard");

        gridLayout->addWidget(attributeMidiStandard, 16, 0, 1, 2);

        labelTags = new QLabel(SoundfontViewerLeft);
        labelTags->setObjectName("labelTags");
        labelTags->setFont(font);

        gridLayout->addWidget(labelTags, 17, 0, 1, 2);

        attributeTag = new AttributeFlow(SoundfontViewerLeft);
        attributeTag->setObjectName("attributeTag");

        gridLayout->addWidget(attributeTag, 18, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 13, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 19, 0, 1, 1);


        retranslateUi(SoundfontViewerLeft);

        QMetaObject::connectSlotsByName(SoundfontViewerLeft);
    } // setupUi

    void retranslateUi(QWidget *SoundfontViewerLeft)
    {
        iconAuthor->setText(QString());
        iconDate->setText(QString());
        iconLicense->setText(QString());
        iconWebsite->setText(QString());
        labelCategory->setText(QCoreApplication::translate("SoundfontViewerLeft", "Category", nullptr));
        labelSampleSource->setText(QCoreApplication::translate("SoundfontViewerLeft", "Sample source", nullptr));
        labelTimbre->setText(QCoreApplication::translate("SoundfontViewerLeft", "Timbre", nullptr));
        labelArticulation->setText(QCoreApplication::translate("SoundfontViewerLeft", "Articulation", nullptr));
        labelGenre->setText(QCoreApplication::translate("SoundfontViewerLeft", "Genre", nullptr));
        labelMidiStandard->setText(QCoreApplication::translate("SoundfontViewerLeft", "MIDI standard", nullptr));
        labelTags->setText(QCoreApplication::translate("SoundfontViewerLeft", "Tags", nullptr));
        (void)SoundfontViewerLeft;
    } // retranslateUi

};

namespace Ui {
    class SoundfontViewerLeft: public Ui_SoundfontViewerLeft {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTVIEWERLEFT_H
