/********************************************************************************
** Form generated from reading UI file 'htmleditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HTMLEDITOR_H
#define UI_HTMLEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QWidget>
#include "customtextedit.h"

QT_BEGIN_NAMESPACE

class Ui_HtmlEditor
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QGridLayout *gridLayout_3;
    CustomTextEdit *textEdit;
    QWidget *page_2;
    QGridLayout *gridLayout_2;
    QPlainTextEdit *plainTextEdit;
    QFrame *frame;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushHtml;
    QFrame *line_3;
    QPushButton *pushStyle;
    QFrame *line;
    QPushButton *pushBold;
    QPushButton *pushItalic;
    QPushButton *pushUnderline;
    QPushButton *pushStrikeThrough;
    QFrame *line_4;
    QPushButton *pushUl;
    QPushButton *pushOl;
    QPushButton *pushAlign;
    QFrame *line_2;
    QPushButton *pushImage;
    QPushButton *pushVideo;
    QPushButton *pushTable;
    QPushButton *pushLink;
    QFrame *line_5;
    QPushButton *pushHr;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *HtmlEditor)
    {
        if (HtmlEditor->objectName().isEmpty())
            HtmlEditor->setObjectName("HtmlEditor");
        HtmlEditor->resize(661, 131);
        HtmlEditor->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(HtmlEditor);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setVerticalSpacing(0);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(HtmlEditor);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        gridLayout_3 = new QGridLayout(page);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        textEdit = new CustomTextEdit(page);
        textEdit->setObjectName("textEdit");

        gridLayout_3->addWidget(textEdit, 0, 0, 1, 1);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        gridLayout_2 = new QGridLayout(page_2);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        plainTextEdit = new QPlainTextEdit(page_2);
        plainTextEdit->setObjectName("plainTextEdit");
        plainTextEdit->setFrameShadow(QFrame::Plain);

        gridLayout_2->addWidget(plainTextEdit, 0, 0, 1, 1);

        stackedWidget->addWidget(page_2);

        gridLayout->addWidget(stackedWidget, 1, 0, 1, 1);

        frame = new QFrame(HtmlEditor);
        frame->setObjectName("frame");
        frame->setStyleSheet(QString::fromUtf8("QFrame{border-bottom:0;}"));
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Plain);
        horizontalLayout = new QHBoxLayout(frame);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        pushHtml = new QPushButton(frame);
        pushHtml->setObjectName("pushHtml");
        pushHtml->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushHtml->setToolTip(QString::fromUtf8("View HTML text"));
#endif // QT_CONFIG(tooltip)
        pushHtml->setCheckable(true);
        pushHtml->setFlat(true);

        horizontalLayout->addWidget(pushHtml);

        line_3 = new QFrame(frame);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::VLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line_3);

        pushStyle = new QPushButton(frame);
        pushStyle->setObjectName("pushStyle");
        pushStyle->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushStyle->setToolTip(QString::fromUtf8("Formatting"));
#endif // QT_CONFIG(tooltip)
        pushStyle->setFlat(true);

        horizontalLayout->addWidget(pushStyle);

        line = new QFrame(frame);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line);

        pushBold = new QPushButton(frame);
        pushBold->setObjectName("pushBold");
        pushBold->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushBold->setToolTip(QString::fromUtf8("Bold"));
#endif // QT_CONFIG(tooltip)
        pushBold->setCheckable(true);
        pushBold->setFlat(true);

        horizontalLayout->addWidget(pushBold);

        pushItalic = new QPushButton(frame);
        pushItalic->setObjectName("pushItalic");
        pushItalic->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushItalic->setToolTip(QString::fromUtf8("Italic"));
#endif // QT_CONFIG(tooltip)
        pushItalic->setCheckable(true);
        pushItalic->setFlat(true);

        horizontalLayout->addWidget(pushItalic);

        pushUnderline = new QPushButton(frame);
        pushUnderline->setObjectName("pushUnderline");
        pushUnderline->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushUnderline->setToolTip(QString::fromUtf8("Underline"));
#endif // QT_CONFIG(tooltip)
        pushUnderline->setCheckable(true);
        pushUnderline->setFlat(true);

        horizontalLayout->addWidget(pushUnderline);

        pushStrikeThrough = new QPushButton(frame);
        pushStrikeThrough->setObjectName("pushStrikeThrough");
        pushStrikeThrough->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushStrikeThrough->setToolTip(QString::fromUtf8("Strikethrough"));
#endif // QT_CONFIG(tooltip)
        pushStrikeThrough->setCheckable(true);
        pushStrikeThrough->setFlat(true);

        horizontalLayout->addWidget(pushStrikeThrough);

        line_4 = new QFrame(frame);
        line_4->setObjectName("line_4");
        line_4->setFrameShape(QFrame::Shape::VLine);
        line_4->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line_4);

        pushUl = new QPushButton(frame);
        pushUl->setObjectName("pushUl");
        pushUl->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushUl->setToolTip(QString::fromUtf8("Unordered list"));
#endif // QT_CONFIG(tooltip)
        pushUl->setCheckable(true);
        pushUl->setFlat(true);

        horizontalLayout->addWidget(pushUl);

        pushOl = new QPushButton(frame);
        pushOl->setObjectName("pushOl");
        pushOl->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushOl->setToolTip(QString::fromUtf8("Ordered list"));
#endif // QT_CONFIG(tooltip)
        pushOl->setCheckable(true);
        pushOl->setFlat(true);

        horizontalLayout->addWidget(pushOl);

        pushAlign = new QPushButton(frame);
        pushAlign->setObjectName("pushAlign");
        pushAlign->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushAlign->setToolTip(QString::fromUtf8("Alignment"));
#endif // QT_CONFIG(tooltip)
        pushAlign->setFlat(true);

        horizontalLayout->addWidget(pushAlign);

        line_2 = new QFrame(frame);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line_2);

        pushImage = new QPushButton(frame);
        pushImage->setObjectName("pushImage");
        pushImage->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushImage->setToolTip(QString::fromUtf8("Insert an image"));
#endif // QT_CONFIG(tooltip)
        pushImage->setFlat(true);

        horizontalLayout->addWidget(pushImage);

        pushVideo = new QPushButton(frame);
        pushVideo->setObjectName("pushVideo");
        pushVideo->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushVideo->setToolTip(QString::fromUtf8("Insert a YouTube video"));
#endif // QT_CONFIG(tooltip)
        pushVideo->setFlat(true);

        horizontalLayout->addWidget(pushVideo);

        pushTable = new QPushButton(frame);
        pushTable->setObjectName("pushTable");
        pushTable->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushTable->setToolTip(QString::fromUtf8("Table"));
#endif // QT_CONFIG(tooltip)
        pushTable->setFlat(true);

        horizontalLayout->addWidget(pushTable);

        pushLink = new QPushButton(frame);
        pushLink->setObjectName("pushLink");
        pushLink->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushLink->setToolTip(QString::fromUtf8("Link"));
#endif // QT_CONFIG(tooltip)
        pushLink->setFlat(true);

        horizontalLayout->addWidget(pushLink);

        line_5 = new QFrame(frame);
        line_5->setObjectName("line_5");
        line_5->setFrameShape(QFrame::Shape::VLine);
        line_5->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line_5);

        pushHr = new QPushButton(frame);
        pushHr->setObjectName("pushHr");
        pushHr->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
#if QT_CONFIG(tooltip)
        pushHr->setToolTip(QString::fromUtf8("Insert a horizontal line"));
#endif // QT_CONFIG(tooltip)
        pushHr->setFlat(true);

        horizontalLayout->addWidget(pushHr);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addWidget(frame, 0, 0, 1, 1);


        retranslateUi(HtmlEditor);

        QMetaObject::connectSlotsByName(HtmlEditor);
    } // setupUi

    void retranslateUi(QWidget *HtmlEditor)
    {
        pushHtml->setText(QString());
        pushStyle->setText(QString());
        pushBold->setText(QString());
        pushItalic->setText(QString());
        pushUnderline->setText(QString());
        pushStrikeThrough->setText(QString());
        pushUl->setText(QString());
        pushOl->setText(QString());
        pushAlign->setText(QString());
        pushImage->setText(QString());
        pushVideo->setText(QString());
        pushTable->setText(QString());
        pushLink->setText(QString());
        pushHr->setText(QString());
        (void)HtmlEditor;
    } // retranslateUi

};

namespace Ui {
    class HtmlEditor: public Ui_HtmlEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HTMLEDITOR_H
