/********************************************************************************
** Form generated from reading UI file 'editor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITOR_H
#define UI_EDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <editortoolbar.h>
#include "backgroundwidget.h"
#include "footerlinkedto.h"
#include "footeroverview.h"
#include "footerprst.h"
#include "footersf2.h"
#include "rotatingspinner.h"
#include "treeview.h"

QT_BEGIN_NAMESPACE

class Ui_Editor
{
public:
    QVBoxLayout *verticalLayout;
    EditorToolBar *toolBar;
    QHBoxLayout *horizontalLayout;
    BackgroundWidget *leftPart;
    QVBoxLayout *verticalLayout_2;
    TreeView *treeView;
    QFrame *frameSearch;
    QGridLayout *gridLayout;
    QLineEdit *editFilter;
    QFrame *rightPart;
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *stackedMain;
    QWidget *pageWait;
    QGridLayout *gridLayout_2;
    RotatingSpinner *rotatingSpinner;
    QSpacerItem *horizontalSpacer;
    QLabel *labelWait;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QWidget *pageError;
    QGridLayout *gridLayout_3;
    QLabel *iconWarning;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *verticalSpacer_4;
    QLabel *labelError;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *horizontalSpacer_3;
    QLabel *labelReason;
    QWidget *pageLogo;
    QGridLayout *gridLayout_5;
    QSpacerItem *horizontalSpacer_5;
    QLabel *iconLogo;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *verticalSpacer_6;
    QStackedWidget *stackedFooter;
    QWidget *footerDefault;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelFileName;
    FooterSf2 *footerSf2;
    FooterOverview *footerOverview;
    FooterLinkedTo *footerLinkedTo;
    FooterPrst *footerPrst;

    void setupUi(QWidget *Editor)
    {
        if (Editor->objectName().isEmpty())
            Editor->setObjectName("Editor");
        Editor->resize(569, 372);
        verticalLayout = new QVBoxLayout(Editor);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        toolBar = new EditorToolBar(Editor);
        toolBar->setObjectName("toolBar");
        toolBar->setMinimumSize(QSize(0, 42));
        toolBar->setMaximumSize(QSize(16777215, 42));

        verticalLayout->addWidget(toolBar);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        leftPart = new BackgroundWidget(Editor);
        leftPart->setObjectName("leftPart");
        leftPart->setMaximumSize(QSize(285, 16777215));
        verticalLayout_2 = new QVBoxLayout(leftPart);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        treeView = new TreeView(leftPart);
        treeView->setObjectName("treeView");
        treeView->setMinimumSize(QSize(160, 0));
        treeView->setContextMenuPolicy(Qt::CustomContextMenu);
        treeView->setAcceptDrops(true);
        treeView->setFrameShape(QFrame::NoFrame);
        treeView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        treeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        treeView->setDragEnabled(true);
        treeView->setDragDropMode(QAbstractItemView::DragDrop);
        treeView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        treeView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        treeView->setIndentation(0);
        treeView->setSortingEnabled(true);
        treeView->setHeaderHidden(true);

        verticalLayout_2->addWidget(treeView);

        frameSearch = new QFrame(leftPart);
        frameSearch->setObjectName("frameSearch");
        frameSearch->setMinimumSize(QSize(0, 24));
        frameSearch->setMaximumSize(QSize(16777215, 24));
        frameSearch->setFrameShape(QFrame::NoFrame);
        gridLayout = new QGridLayout(frameSearch);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        editFilter = new QLineEdit(frameSearch);
        editFilter->setObjectName("editFilter");
        editFilter->setMinimumSize(QSize(0, 24));
        editFilter->setMaximumSize(QSize(16777215, 23));
        editFilter->setFrame(false);
        editFilter->setClearButtonEnabled(true);

        gridLayout->addWidget(editFilter, 0, 0, 1, 1);


        verticalLayout_2->addWidget(frameSearch);


        horizontalLayout->addWidget(leftPart);

        rightPart = new QFrame(Editor);
        rightPart->setObjectName("rightPart");
        verticalLayout_3 = new QVBoxLayout(rightPart);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        stackedMain = new QStackedWidget(rightPart);
        stackedMain->setObjectName("stackedMain");
        stackedMain->setAutoFillBackground(true);
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_2 = new QGridLayout(pageWait);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        rotatingSpinner = new RotatingSpinner(pageWait);
        rotatingSpinner->setObjectName("rotatingSpinner");
        rotatingSpinner->setMinimumSize(QSize(32, 32));
        rotatingSpinner->setMaximumSize(QSize(32, 32));

        gridLayout_2->addWidget(rotatingSpinner, 3, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 3, 0, 1, 1);

        labelWait = new QLabel(pageWait);
        labelWait->setObjectName("labelWait");
        labelWait->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelWait, 1, 0, 1, 3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 3, 2, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 4, 1, 1, 1);

        stackedMain->addWidget(pageWait);
        pageError = new QWidget();
        pageError->setObjectName("pageError");
        gridLayout_3 = new QGridLayout(pageError);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        iconWarning = new QLabel(pageError);
        iconWarning->setObjectName("iconWarning");
        iconWarning->setMinimumSize(QSize(64, 64));
        iconWarning->setMaximumSize(QSize(64, 64));
        iconWarning->setText(QString::fromUtf8(""));

        gridLayout_3->addWidget(iconWarning, 1, 1, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_4, 1, 2, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_4, 4, 1, 1, 1);

        labelError = new QLabel(pageError);
        labelError->setObjectName("labelError");
        QFont font;
        font.setBold(true);
        labelError->setFont(font);
        labelError->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(labelError, 2, 0, 1, 3);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 0, 1, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_3, 1, 0, 1, 1);

        labelReason = new QLabel(pageError);
        labelReason->setObjectName("labelReason");
        labelReason->setText(QString::fromUtf8("reason"));
        labelReason->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(labelReason, 3, 0, 1, 3);

        stackedMain->addWidget(pageError);
        pageLogo = new QWidget();
        pageLogo->setObjectName("pageLogo");
        gridLayout_5 = new QGridLayout(pageLogo);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_5, 1, 2, 1, 1);

        iconLogo = new QLabel(pageLogo);
        iconLogo->setObjectName("iconLogo");
        iconLogo->setMinimumSize(QSize(128, 128));
        iconLogo->setMaximumSize(QSize(128, 128));
        iconLogo->setText(QString::fromUtf8(""));
        iconLogo->setAlignment(Qt::AlignCenter);

        gridLayout_5->addWidget(iconLogo, 1, 1, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_6, 1, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 79, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_5->addItem(verticalSpacer_5, 0, 1, 1, 1);

        verticalSpacer_6 = new QSpacerItem(20, 79, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_5->addItem(verticalSpacer_6, 2, 1, 1, 1);

        stackedMain->addWidget(pageLogo);

        verticalLayout_3->addWidget(stackedMain);

        stackedFooter = new QStackedWidget(rightPart);
        stackedFooter->setObjectName("stackedFooter");
        stackedFooter->setMinimumSize(QSize(0, 24));
        stackedFooter->setMaximumSize(QSize(16777215, 24));
        footerDefault = new QWidget();
        footerDefault->setObjectName("footerDefault");
        horizontalLayout_2 = new QHBoxLayout(footerDefault);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(6, 0, 6, 0);
        labelFileName = new QLabel(footerDefault);
        labelFileName->setObjectName("labelFileName");
        labelFileName->setText(QString::fromUtf8("File..."));
        labelFileName->setWordWrap(true);

        horizontalLayout_2->addWidget(labelFileName);

        stackedFooter->addWidget(footerDefault);
        footerSf2 = new FooterSf2();
        footerSf2->setObjectName("footerSf2");
        stackedFooter->addWidget(footerSf2);
        footerOverview = new FooterOverview();
        footerOverview->setObjectName("footerOverview");
        stackedFooter->addWidget(footerOverview);
        footerLinkedTo = new FooterLinkedTo();
        footerLinkedTo->setObjectName("footerLinkedTo");
        stackedFooter->addWidget(footerLinkedTo);
        footerPrst = new FooterPrst();
        footerPrst->setObjectName("footerPrst");
        stackedFooter->addWidget(footerPrst);

        verticalLayout_3->addWidget(stackedFooter);


        horizontalLayout->addWidget(rightPart);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(Editor);

        stackedMain->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Editor);
    } // setupUi

    void retranslateUi(QWidget *Editor)
    {
        editFilter->setText(QString());
        editFilter->setPlaceholderText(QCoreApplication::translate("Editor", "Search...", nullptr));
        labelWait->setText(QCoreApplication::translate("Editor", "Opening file...", nullptr));
        labelError->setText(QCoreApplication::translate("Editor", "Cannot open the file", nullptr));
        (void)Editor;
    } // retranslateUi

};

namespace Ui {
    class Editor: public Ui_Editor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITOR_H
