/********************************************************************************
** Form generated from reading UI file 'pagesmpl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGESMPL_H
#define UI_PAGESMPL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <transparentframe.h>
#include "equalizer.h"
#include "graphicsfourier.h"
#include "graphicswave.h"
#include "spinboxcents.h"
#include "spinboxkey.h"

QT_BEGIN_NAMESPACE

class Ui_PageSmpl
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *frameGraph;
    QVBoxLayout *verticalLayout_5;
    GraphicsWave *waveDisplay;
    QScrollBar *sliderGraphe;
    QFrame *frame_5;
    QGridLayout *gridLayout;
    QFrame *frameRight;
    QGridLayout *gridLayout_3;
    Equalizer *widgetEqualizer;
    QFrame *frame;
    QGridLayout *frameMiddle;
    SpinBoxKey *spinRootKey;
    SpinBoxCents *spinTune;
    QPushButton *pushAutoTune;
    TransparentFrame *framePlayArea;
    QGridLayout *gridLayout_6;
    QCheckBox *checkLectureBoucle;
    QCheckBox *checkLectureSinus;
    QCheckBox *checkLectureLien;
    QSlider *sliderLectureVolume;
    QPushButton *pushLecture;
    QFrame *frame_6;
    QHBoxLayout *horizontalLayout_4;
    GraphicsFourier *grapheFourier;
    QLabel *label_22;
    QFrame *frameLeft;
    QHBoxLayout *horizontalLayout_2;
    QGridLayout *gridLayout_2;
    QLabel *label_2;
    QSpinBox *spinEndLoop;
    QSpinBox *spinStartLoop;
    QComboBox *comboLink;
    QComboBox *comboSampleRate;
    QPushButton *pushFullLength;
    QLabel *labelTaille;
    QLabel *iconLoopWarning;
    QLabel *label;
    QLabel *label_6;
    QLabel *label_7;
    QLabel *label_5;
    QComboBox *comboType;
    QPushButton *pushCompress;
    QLabel *label_20;
    QFrame *frame_2;
    QLabel *label_21;

    void setupUi(QWidget *PageSmpl)
    {
        if (PageSmpl->objectName().isEmpty())
            PageSmpl->setObjectName("PageSmpl");
        PageSmpl->resize(815, 251);
        PageSmpl->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(PageSmpl);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        frameGraph = new QFrame(PageSmpl);
        frameGraph->setObjectName("frameGraph");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frameGraph->sizePolicy().hasHeightForWidth());
        frameGraph->setSizePolicy(sizePolicy);
        frameGraph->setFrameShape(QFrame::Shape::NoFrame);
        verticalLayout_5 = new QVBoxLayout(frameGraph);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        waveDisplay = new GraphicsWave(frameGraph);
        waveDisplay->setObjectName("waveDisplay");
        sizePolicy.setHeightForWidth(waveDisplay->sizePolicy().hasHeightForWidth());
        waveDisplay->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(waveDisplay);

        sliderGraphe = new QScrollBar(frameGraph);
        sliderGraphe->setObjectName("sliderGraphe");
        sliderGraphe->setMaximum(0);
        sliderGraphe->setSingleStep(200);
        sliderGraphe->setPageStep(10000);
        sliderGraphe->setOrientation(Qt::Orientation::Horizontal);

        verticalLayout_5->addWidget(sliderGraphe);


        verticalLayout->addWidget(frameGraph);

        frame_5 = new QFrame(PageSmpl);
        frame_5->setObjectName("frame_5");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame_5->sizePolicy().hasHeightForWidth());
        frame_5->setSizePolicy(sizePolicy1);
        frame_5->setFrameShape(QFrame::Shape::NoFrame);
        frame_5->setFrameShadow(QFrame::Shadow::Raised);
        gridLayout = new QGridLayout(frame_5);
        gridLayout->setSpacing(10);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        frameRight = new QFrame(frame_5);
        frameRight->setObjectName("frameRight");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(frameRight->sizePolicy().hasHeightForWidth());
        frameRight->setSizePolicy(sizePolicy2);
        frameRight->setFrameShape(QFrame::Shape::NoFrame);
        gridLayout_3 = new QGridLayout(frameRight);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        widgetEqualizer = new Equalizer(frameRight);
        widgetEqualizer->setObjectName("widgetEqualizer");

        gridLayout_3->addWidget(widgetEqualizer, 0, 0, 1, 1);


        gridLayout->addWidget(frameRight, 1, 4, 1, 1);

        frame = new QFrame(frame_5);
        frame->setObjectName("frame");
        frame->setMinimumSize(QSize(2, 0));
        frame->setMaximumSize(QSize(2, 16777215));
        frame->setFrameShape(QFrame::Shape::Panel);
        frame->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(frame, 1, 1, 1, 1);

        frameMiddle = new QGridLayout();
        frameMiddle->setObjectName("frameMiddle");
        frameMiddle->setHorizontalSpacing(3);
        frameMiddle->setVerticalSpacing(0);
        spinRootKey = new SpinBoxKey(frame_5);
        spinRootKey->setObjectName("spinRootKey");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(spinRootKey->sizePolicy().hasHeightForWidth());
        spinRootKey->setSizePolicy(sizePolicy3);
        spinRootKey->setCorrectionMode(QAbstractSpinBox::CorrectionMode::CorrectToNearestValue);
        spinRootKey->setMaximum(127);

        frameMiddle->addWidget(spinRootKey, 2, 0, 1, 1);

        spinTune = new SpinBoxCents(frame_5);
        spinTune->setObjectName("spinTune");
        sizePolicy3.setHeightForWidth(spinTune->sizePolicy().hasHeightForWidth());
        spinTune->setSizePolicy(sizePolicy3);
        spinTune->setCorrectionMode(QAbstractSpinBox::CorrectionMode::CorrectToNearestValue);
        spinTune->setMinimum(-100);
        spinTune->setMaximum(100);

        frameMiddle->addWidget(spinTune, 2, 1, 1, 1);

        pushAutoTune = new QPushButton(frame_5);
        pushAutoTune->setObjectName("pushAutoTune");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(pushAutoTune->sizePolicy().hasHeightForWidth());
        pushAutoTune->setSizePolicy(sizePolicy4);
        pushAutoTune->setIconSize(QSize(14, 14));

        frameMiddle->addWidget(pushAutoTune, 2, 2, 1, 1);

        framePlayArea = new TransparentFrame(frame_5);
        framePlayArea->setObjectName("framePlayArea");
        framePlayArea->setMinimumSize(QSize(0, 30));
        framePlayArea->setFrameShape(QFrame::Shape::StyledPanel);
        framePlayArea->setFrameShadow(QFrame::Shadow::Raised);
        gridLayout_6 = new QGridLayout(framePlayArea);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setHorizontalSpacing(6);
        gridLayout_6->setVerticalSpacing(0);
        gridLayout_6->setContentsMargins(3, 3, 6, 3);
        checkLectureBoucle = new QCheckBox(framePlayArea);
        checkLectureBoucle->setObjectName("checkLectureBoucle");

        gridLayout_6->addWidget(checkLectureBoucle, 0, 1, 1, 1);

        checkLectureSinus = new QCheckBox(framePlayArea);
        checkLectureSinus->setObjectName("checkLectureSinus");

        gridLayout_6->addWidget(checkLectureSinus, 0, 3, 1, 1);

        checkLectureLien = new QCheckBox(framePlayArea);
        checkLectureLien->setObjectName("checkLectureLien");

        gridLayout_6->addWidget(checkLectureLien, 0, 2, 1, 1);

        sliderLectureVolume = new QSlider(framePlayArea);
        sliderLectureVolume->setObjectName("sliderLectureVolume");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(sliderLectureVolume->sizePolicy().hasHeightForWidth());
        sliderLectureVolume->setSizePolicy(sizePolicy5);
        sliderLectureVolume->setMinimum(-50);
        sliderLectureVolume->setMaximum(50);
        sliderLectureVolume->setOrientation(Qt::Orientation::Horizontal);
        sliderLectureVolume->setTickPosition(QSlider::TickPosition::TicksBelow);
        sliderLectureVolume->setTickInterval(10);

        gridLayout_6->addWidget(sliderLectureVolume, 1, 1, 1, 3);

        pushLecture = new QPushButton(framePlayArea);
        pushLecture->setObjectName("pushLecture");
        sizePolicy2.setHeightForWidth(pushLecture->sizePolicy().hasHeightForWidth());
        pushLecture->setSizePolicy(sizePolicy2);
        pushLecture->setMinimumSize(QSize(42, 0));
        pushLecture->setMaximumSize(QSize(42, 16777215));
        pushLecture->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushLecture->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/play.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushLecture->setIcon(icon);
        pushLecture->setIconSize(QSize(36, 36));
        pushLecture->setCheckable(true);
        pushLecture->setFlat(true);

        gridLayout_6->addWidget(pushLecture, 0, 0, 2, 1);


        frameMiddle->addWidget(framePlayArea, 1, 0, 1, 3);

        frame_6 = new QFrame(frame_5);
        frame_6->setObjectName("frame_6");
        sizePolicy.setHeightForWidth(frame_6->sizePolicy().hasHeightForWidth());
        frame_6->setSizePolicy(sizePolicy);
        frame_6->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout_4 = new QHBoxLayout(frame_6);
        horizontalLayout_4->setSpacing(0);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        grapheFourier = new GraphicsFourier(frame_6);
        grapheFourier->setObjectName("grapheFourier");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(grapheFourier->sizePolicy().hasHeightForWidth());
        grapheFourier->setSizePolicy(sizePolicy6);

        horizontalLayout_4->addWidget(grapheFourier);


        frameMiddle->addWidget(frame_6, 0, 0, 1, 3);

        frameMiddle->setColumnStretch(0, 5);
        frameMiddle->setColumnStretch(1, 5);

        gridLayout->addLayout(frameMiddle, 1, 2, 1, 1);

        label_22 = new QLabel(frame_5);
        label_22->setObjectName("label_22");
        label_22->setMaximumSize(QSize(16777215, 20));
        QFont font;
        font.setBold(true);
        label_22->setFont(font);

        gridLayout->addWidget(label_22, 0, 2, 1, 1);

        frameLeft = new QFrame(frame_5);
        frameLeft->setObjectName("frameLeft");
        frameLeft->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout_2 = new QHBoxLayout(frameLeft);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(3);
        gridLayout_2->setObjectName("gridLayout_2");
        label_2 = new QLabel(frameLeft);
        label_2->setObjectName("label_2");

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        spinEndLoop = new QSpinBox(frameLeft);
        spinEndLoop->setObjectName("spinEndLoop");
        sizePolicy3.setHeightForWidth(spinEndLoop->sizePolicy().hasHeightForWidth());
        spinEndLoop->setSizePolicy(sizePolicy3);
        spinEndLoop->setMinimumSize(QSize(65, 0));
        spinEndLoop->setCorrectionMode(QAbstractSpinBox::CorrectionMode::CorrectToNearestValue);

        gridLayout_2->addWidget(spinEndLoop, 1, 3, 1, 1);

        spinStartLoop = new QSpinBox(frameLeft);
        spinStartLoop->setObjectName("spinStartLoop");
        sizePolicy3.setHeightForWidth(spinStartLoop->sizePolicy().hasHeightForWidth());
        spinStartLoop->setSizePolicy(sizePolicy3);
        spinStartLoop->setMinimumSize(QSize(65, 0));
        spinStartLoop->setCorrectionMode(QAbstractSpinBox::CorrectionMode::CorrectToNearestValue);

        gridLayout_2->addWidget(spinStartLoop, 1, 2, 1, 1);

        comboLink = new QComboBox(frameLeft);
        comboLink->setObjectName("comboLink");

        gridLayout_2->addWidget(comboLink, 4, 2, 1, 3);

        comboSampleRate = new QComboBox(frameLeft);
        comboSampleRate->addItem(QString::fromUtf8("11025"));
        comboSampleRate->addItem(QString::fromUtf8("22050"));
        comboSampleRate->addItem(QString::fromUtf8("32000"));
        comboSampleRate->addItem(QString::fromUtf8("44100"));
        comboSampleRate->addItem(QString::fromUtf8("48000"));
        comboSampleRate->addItem(QString::fromUtf8("96000"));
        comboSampleRate->addItem(QString::fromUtf8("192000"));
        comboSampleRate->setObjectName("comboSampleRate");

        gridLayout_2->addWidget(comboSampleRate, 2, 2, 1, 3);

        pushFullLength = new QPushButton(frameLeft);
        pushFullLength->setObjectName("pushFullLength");
        sizePolicy4.setHeightForWidth(pushFullLength->sizePolicy().hasHeightForWidth());
        pushFullLength->setSizePolicy(sizePolicy4);
        pushFullLength->setStyleSheet(QString::fromUtf8("QPushButton{padding: 0 0 0 0;}"));
        pushFullLength->setIconSize(QSize(20, 18));

        gridLayout_2->addWidget(pushFullLength, 1, 4, 1, 1);

        labelTaille = new QLabel(frameLeft);
        labelTaille->setObjectName("labelTaille");
        labelTaille->setText(QString::fromUtf8("TextLabel"));

        gridLayout_2->addWidget(labelTaille, 0, 2, 1, 3);

        iconLoopWarning = new QLabel(frameLeft);
        iconLoopWarning->setObjectName("iconLoopWarning");
        iconLoopWarning->setMinimumSize(QSize(16, 16));
        iconLoopWarning->setMaximumSize(QSize(16, 16));

        gridLayout_2->addWidget(iconLoopWarning, 1, 1, 1, 1);

        label = new QLabel(frameLeft);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 0, 0, 1, 2);

        label_6 = new QLabel(frameLeft);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 2, 0, 1, 2);

        label_7 = new QLabel(frameLeft);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 3, 0, 1, 2);

        label_5 = new QLabel(frameLeft);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 4, 0, 1, 2);

        comboType = new QComboBox(frameLeft);
        comboType->setObjectName("comboType");

        gridLayout_2->addWidget(comboType, 3, 2, 1, 2);

        pushCompress = new QPushButton(frameLeft);
        pushCompress->setObjectName("pushCompress");
        sizePolicy4.setHeightForWidth(pushCompress->sizePolicy().hasHeightForWidth());
        pushCompress->setSizePolicy(sizePolicy4);
        pushCompress->setStyleSheet(QString::fromUtf8("QPushButton{padding: 0 0 0 0;}"));
        pushCompress->setIconSize(QSize(20, 18));

        gridLayout_2->addWidget(pushCompress, 3, 4, 1, 1);


        horizontalLayout_2->addLayout(gridLayout_2);


        gridLayout->addWidget(frameLeft, 1, 0, 1, 1);

        label_20 = new QLabel(frame_5);
        label_20->setObjectName("label_20");
        label_20->setMinimumSize(QSize(0, 0));
        label_20->setMaximumSize(QSize(16777215, 16));
        label_20->setFont(font);

        gridLayout->addWidget(label_20, 0, 0, 1, 1);

        frame_2 = new QFrame(frame_5);
        frame_2->setObjectName("frame_2");
        frame_2->setMinimumSize(QSize(2, 0));
        frame_2->setMaximumSize(QSize(2, 16777215));
        frame_2->setFrameShape(QFrame::Shape::Panel);
        frame_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(frame_2, 1, 3, 1, 1);

        label_21 = new QLabel(frame_5);
        label_21->setObjectName("label_21");
        label_21->setMaximumSize(QSize(16777215, 20));
        label_21->setFont(font);

        gridLayout->addWidget(label_21, 0, 4, 1, 1);


        verticalLayout->addWidget(frame_5);

        QWidget::setTabOrder(pushLecture, checkLectureBoucle);
        QWidget::setTabOrder(checkLectureBoucle, checkLectureLien);
        QWidget::setTabOrder(checkLectureLien, checkLectureSinus);
        QWidget::setTabOrder(checkLectureSinus, sliderLectureVolume);
        QWidget::setTabOrder(sliderLectureVolume, spinStartLoop);
        QWidget::setTabOrder(spinStartLoop, spinEndLoop);
        QWidget::setTabOrder(spinEndLoop, pushFullLength);
        QWidget::setTabOrder(pushFullLength, comboSampleRate);
        QWidget::setTabOrder(comboSampleRate, comboType);
        QWidget::setTabOrder(comboType, comboLink);
        QWidget::setTabOrder(comboLink, spinRootKey);
        QWidget::setTabOrder(spinRootKey, spinTune);
        QWidget::setTabOrder(spinTune, pushAutoTune);

        retranslateUi(PageSmpl);
        QObject::connect(spinTune, SIGNAL(editingFinished()), PageSmpl, SLOT(setTune()));
        QObject::connect(spinTune, SIGNAL(valueChanged(int)), PageSmpl, SLOT(setTune(int)));
        QObject::connect(spinStartLoop, SIGNAL(editingFinished()), PageSmpl, SLOT(setStartLoop()));
        QObject::connect(spinStartLoop, SIGNAL(valueChanged(int)), PageSmpl, SLOT(setStartLoop(int)));
        QObject::connect(spinStartLoop, SIGNAL(valueChanged(int)), waveDisplay, SLOT(setStartLoop(int)));
        QObject::connect(spinRootKey, SIGNAL(editingFinished()), PageSmpl, SLOT(setRootKey()));
        QObject::connect(spinRootKey, SIGNAL(valueChanged(int)), PageSmpl, SLOT(setRootKey(int)));
        QObject::connect(spinEndLoop, SIGNAL(editingFinished()), PageSmpl, SLOT(setEndLoop()));
        QObject::connect(spinEndLoop, SIGNAL(valueChanged(int)), PageSmpl, SLOT(setEndLoop(int)));
        QObject::connect(spinEndLoop, SIGNAL(valueChanged(int)), waveDisplay, SLOT(setEndLoop(int)));
        QObject::connect(sliderLectureVolume, SIGNAL(valueChanged(int)), PageSmpl, SLOT(setGainSample(int)));
        QObject::connect(sliderGraphe, SIGNAL(valueChanged(int)), waveDisplay, SLOT(setPosX(int)));
        QObject::connect(pushLecture, SIGNAL(clicked()), PageSmpl, SLOT(lecture()));
        QObject::connect(waveDisplay, SIGNAL(endLoopChanged()), PageSmpl, SLOT(setEndLoop()));
        QObject::connect(waveDisplay, SIGNAL(startLoopChanged()), PageSmpl, SLOT(setStartLoop()));
        QObject::connect(comboType, SIGNAL(currentIndexChanged(int)), PageSmpl, SLOT(setType(int)));
        QObject::connect(comboSampleRate, SIGNAL(currentIndexChanged(int)), PageSmpl, SLOT(setRate(int)));
        QObject::connect(comboLink, SIGNAL(currentIndexChanged(int)), PageSmpl, SLOT(setLinkedSmpl(int)));
        QObject::connect(checkLectureLien, SIGNAL(clicked(bool)), PageSmpl, SLOT(setStereo(bool)));
        QObject::connect(checkLectureSinus, SIGNAL(clicked(bool)), PageSmpl, SLOT(setSinusEnabled(bool)));

        QMetaObject::connectSlotsByName(PageSmpl);
    } // setupUi

    void retranslateUi(QWidget *PageSmpl)
    {
#if QT_CONFIG(tooltip)
        spinRootKey->setToolTip(QCoreApplication::translate("PageSmpl", "Root key", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinTune->setToolTip(QCoreApplication::translate("PageSmpl", "Correction", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushAutoTune->setToolTip(QCoreApplication::translate("PageSmpl", "Use the estimated pitch and correction as values", nullptr));
#endif // QT_CONFIG(tooltip)
        pushAutoTune->setText(QString());
        checkLectureBoucle->setText(QCoreApplication::translate("PageSmpl", "loop", nullptr));
        checkLectureSinus->setText(QCoreApplication::translate("PageSmpl", "sine", nullptr));
        checkLectureLien->setText(QCoreApplication::translate("PageSmpl", "stereo", nullptr));
#if QT_CONFIG(tooltip)
        pushLecture->setToolTip(QCoreApplication::translate("PageSmpl", "Play", nullptr));
#endif // QT_CONFIG(tooltip)
        label_22->setText(QCoreApplication::translate("PageSmpl", "Tuning", nullptr));
        label_2->setText(QCoreApplication::translate("PageSmpl", "Loop", nullptr));

#if QT_CONFIG(tooltip)
        pushFullLength->setToolTip(QCoreApplication::translate("PageSmpl", "Use the full length of the sample as loop", nullptr));
#endif // QT_CONFIG(tooltip)
        pushFullLength->setText(QString());
        iconLoopWarning->setText(QString());
        label->setText(QCoreApplication::translate("PageSmpl", "Size", nullptr));
        label_6->setText(QCoreApplication::translate("PageSmpl", "Sample rate", nullptr));
        label_7->setText(QCoreApplication::translate("PageSmpl", "Type", nullptr));
        label_5->setText(QCoreApplication::translate("PageSmpl", "Link", nullptr));
        pushCompress->setText(QString());
        label_20->setText(QCoreApplication::translate("PageSmpl", "Information", nullptr));
        label_21->setText(QCoreApplication::translate("PageSmpl", "Equalizer (\302\26115 dB)", nullptr));
        (void)PageSmpl;
    } // retranslateUi

};

namespace Ui {
    class PageSmpl: public Ui_PageSmpl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGESMPL_H
