/********************************************************************************
** Form generated from reading UI file 'soundfontviewer.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTVIEWER_H
#define UI_SOUNDFONTVIEWER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "rotatingspinner.h"
#include "soundfonteditorcenter.h"
#include "soundfonteditorleft.h"
#include "soundfonteditortop.h"
#include "soundfontviewercenter.h"
#include "soundfontviewerleft.h"
#include "soundfontviewertop.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontViewer
{
public:
    QGridLayout *gridLayout_6;
    QStackedWidget *stackWidgetMain;
    QWidget *pageViewer;
    QGridLayout *gridLayout_5;
    QFrame *frameTitle;
    QHBoxLayout *horizontalLayout;
    SoundfontViewerTop *viewerTop;
    QPushButton *pushEdit;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout;
    SoundfontViewerLeft *viewerLeft;
    QStackedWidget *stackedWidget;
    QWidget *pageWait;
    QGridLayout *gridLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer_3;
    RotatingSpinner *spinner;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *verticalSpacer_3;
    QWidget *pageSoundfont;
    QGridLayout *gridLayout_2;
    QScrollArea *scrollArea_2;
    QWidget *scrollMainArea;
    QVBoxLayout *verticalLayout;
    SoundfontViewerCenter *viewerCenter;
    QWidget *pageFailed;
    QGridLayout *gridLayout_4;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *verticalSpacer_4;
    QLabel *label_4;
    QPushButton *pushRetry;
    QSpacerItem *horizontalSpacer_5;
    QWidget *pageEditor;
    QGridLayout *gridLayout_8;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_2;
    QGridLayout *gridLayout_7;
    SoundfontEditorLeft *editorLeft;
    QFrame *frameTitle_2;
    QHBoxLayout *horizontalLayout_2;
    SoundfontEditorTop *editorTop;
    QPushButton *pushCancel;
    QPushButton *pushSave;
    QStackedWidget *stackedWidget_2;
    QWidget *page_7;
    QGridLayout *gridLayout_9;
    QScrollArea *scrollArea_4;
    QWidget *scrollMainArea_2;
    QVBoxLayout *verticalLayout_2;
    SoundfontEditorCenter *editorCenter;
    QWidget *page_8;

    void setupUi(QWidget *SoundfontViewer)
    {
        if (SoundfontViewer->objectName().isEmpty())
            SoundfontViewer->setObjectName("SoundfontViewer");
        SoundfontViewer->resize(567, 385);
        SoundfontViewer->setWindowTitle(QString::fromUtf8("Form"));
        SoundfontViewer->setAutoFillBackground(true);
        gridLayout_6 = new QGridLayout(SoundfontViewer);
        gridLayout_6->setSpacing(0);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        stackWidgetMain = new QStackedWidget(SoundfontViewer);
        stackWidgetMain->setObjectName("stackWidgetMain");
        pageViewer = new QWidget();
        pageViewer->setObjectName("pageViewer");
        gridLayout_5 = new QGridLayout(pageViewer);
        gridLayout_5->setSpacing(0);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        frameTitle = new QFrame(pageViewer);
        frameTitle->setObjectName("frameTitle");
        frameTitle->setMinimumSize(QSize(0, 42));
        frameTitle->setMaximumSize(QSize(16777215, 42));
        frameTitle->setFrameShape(QFrame::NoFrame);
        horizontalLayout = new QHBoxLayout(frameTitle);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        viewerTop = new SoundfontViewerTop(frameTitle);
        viewerTop->setObjectName("viewerTop");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(viewerTop->sizePolicy().hasHeightForWidth());
        viewerTop->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(viewerTop);

        pushEdit = new QPushButton(frameTitle);
        pushEdit->setObjectName("pushEdit");
        pushEdit->setMinimumSize(QSize(39, 39));
        pushEdit->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushEdit->setIconSize(QSize(22, 22));
        pushEdit->setFlat(true);

        horizontalLayout->addWidget(pushEdit);


        gridLayout_5->addWidget(frameTitle, 0, 0, 1, 2);

        scrollArea = new QScrollArea(pageViewer);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 250, 343));
        gridLayout = new QGridLayout(scrollAreaWidgetContents);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        viewerLeft = new SoundfontViewerLeft(scrollAreaWidgetContents);
        viewerLeft->setObjectName("viewerLeft");

        gridLayout->addWidget(viewerLeft, 0, 0, 1, 1);

        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout_5->addWidget(scrollArea, 1, 0, 1, 1);

        stackedWidget = new QStackedWidget(pageViewer);
        stackedWidget->setObjectName("stackedWidget");
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_3 = new QGridLayout(pageWait);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 1, 0, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_3, 1, 2, 1, 1);

        spinner = new RotatingSpinner(pageWait);
        spinner->setObjectName("spinner");
        spinner->setMinimumSize(QSize(32, 32));

        gridLayout_3->addWidget(spinner, 1, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 0, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 2, 1, 1, 1);

        stackedWidget->addWidget(pageWait);
        pageSoundfont = new QWidget();
        pageSoundfont->setObjectName("pageSoundfont");
        gridLayout_2 = new QGridLayout(pageSoundfont);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        scrollArea_2 = new QScrollArea(pageSoundfont);
        scrollArea_2->setObjectName("scrollArea_2");
        scrollArea_2->setFrameShape(QFrame::NoFrame);
        scrollArea_2->setWidgetResizable(true);
        scrollMainArea = new QWidget();
        scrollMainArea->setObjectName("scrollMainArea");
        scrollMainArea->setGeometry(QRect(0, 0, 317, 343));
        verticalLayout = new QVBoxLayout(scrollMainArea);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(-1, 9, 9, 9);
        viewerCenter = new SoundfontViewerCenter(scrollMainArea);
        viewerCenter->setObjectName("viewerCenter");

        verticalLayout->addWidget(viewerCenter);

        scrollArea_2->setWidget(scrollMainArea);

        gridLayout_2->addWidget(scrollArea_2, 0, 0, 1, 1);

        stackedWidget->addWidget(pageSoundfont);
        pageFailed = new QWidget();
        pageFailed->setObjectName("pageFailed");
        gridLayout_4 = new QGridLayout(pageFailed);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_5, 3, 1, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_4, 2, 0, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_4, 0, 1, 1, 1);

        label_4 = new QLabel(pageFailed);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(label_4, 1, 0, 1, 3);

        pushRetry = new QPushButton(pageFailed);
        pushRetry->setObjectName("pushRetry");
        pushRetry->setIconSize(QSize(16, 16));
        pushRetry->setFlat(true);

        gridLayout_4->addWidget(pushRetry, 2, 1, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_5, 2, 2, 1, 1);

        stackedWidget->addWidget(pageFailed);

        gridLayout_5->addWidget(stackedWidget, 1, 1, 2, 1);

        gridLayout_5->setColumnStretch(0, 3);
        gridLayout_5->setColumnStretch(1, 10);
        gridLayout_5->setColumnMinimumWidth(0, 250);
        stackWidgetMain->addWidget(pageViewer);
        pageEditor = new QWidget();
        pageEditor->setObjectName("pageEditor");
        gridLayout_8 = new QGridLayout(pageEditor);
        gridLayout_8->setSpacing(0);
        gridLayout_8->setObjectName("gridLayout_8");
        gridLayout_8->setContentsMargins(0, 0, 0, 0);
        scrollArea_3 = new QScrollArea(pageEditor);
        scrollArea_3->setObjectName("scrollArea_3");
        scrollArea_3->setFrameShape(QFrame::NoFrame);
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 250, 343));
        gridLayout_7 = new QGridLayout(scrollAreaWidgetContents_2);
        gridLayout_7->setObjectName("gridLayout_7");
        gridLayout_7->setContentsMargins(0, 0, 0, 0);
        editorLeft = new SoundfontEditorLeft(scrollAreaWidgetContents_2);
        editorLeft->setObjectName("editorLeft");

        gridLayout_7->addWidget(editorLeft, 0, 0, 1, 1);

        scrollArea_3->setWidget(scrollAreaWidgetContents_2);

        gridLayout_8->addWidget(scrollArea_3, 1, 0, 1, 1);

        frameTitle_2 = new QFrame(pageEditor);
        frameTitle_2->setObjectName("frameTitle_2");
        frameTitle_2->setMinimumSize(QSize(0, 42));
        frameTitle_2->setMaximumSize(QSize(16777215, 42));
        frameTitle_2->setFrameShape(QFrame::NoFrame);
        horizontalLayout_2 = new QHBoxLayout(frameTitle_2);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        editorTop = new SoundfontEditorTop(frameTitle_2);
        editorTop->setObjectName("editorTop");
        sizePolicy.setHeightForWidth(editorTop->sizePolicy().hasHeightForWidth());
        editorTop->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(editorTop);

        pushCancel = new QPushButton(frameTitle_2);
        pushCancel->setObjectName("pushCancel");
        pushCancel->setMinimumSize(QSize(39, 39));
        pushCancel->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushCancel->setIconSize(QSize(22, 22));
        pushCancel->setFlat(true);

        horizontalLayout_2->addWidget(pushCancel);

        pushSave = new QPushButton(frameTitle_2);
        pushSave->setObjectName("pushSave");
        pushSave->setMinimumSize(QSize(39, 39));
        pushSave->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushSave->setFlat(true);

        horizontalLayout_2->addWidget(pushSave);


        gridLayout_8->addWidget(frameTitle_2, 0, 0, 1, 2);

        stackedWidget_2 = new QStackedWidget(pageEditor);
        stackedWidget_2->setObjectName("stackedWidget_2");
        page_7 = new QWidget();
        page_7->setObjectName("page_7");
        gridLayout_9 = new QGridLayout(page_7);
        gridLayout_9->setSpacing(0);
        gridLayout_9->setObjectName("gridLayout_9");
        gridLayout_9->setContentsMargins(0, 0, 0, 0);
        scrollArea_4 = new QScrollArea(page_7);
        scrollArea_4->setObjectName("scrollArea_4");
        scrollArea_4->setFrameShape(QFrame::NoFrame);
        scrollArea_4->setWidgetResizable(true);
        scrollMainArea_2 = new QWidget();
        scrollMainArea_2->setObjectName("scrollMainArea_2");
        scrollMainArea_2->setGeometry(QRect(0, 0, 317, 343));
        verticalLayout_2 = new QVBoxLayout(scrollMainArea_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        editorCenter = new SoundfontEditorCenter(scrollMainArea_2);
        editorCenter->setObjectName("editorCenter");

        verticalLayout_2->addWidget(editorCenter);

        scrollArea_4->setWidget(scrollMainArea_2);

        gridLayout_9->addWidget(scrollArea_4, 0, 0, 1, 1);

        stackedWidget_2->addWidget(page_7);
        page_8 = new QWidget();
        page_8->setObjectName("page_8");
        stackedWidget_2->addWidget(page_8);

        gridLayout_8->addWidget(stackedWidget_2, 1, 1, 1, 1);

        gridLayout_8->setColumnStretch(0, 3);
        gridLayout_8->setColumnStretch(1, 10);
        gridLayout_8->setColumnMinimumWidth(0, 250);
        stackWidgetMain->addWidget(pageEditor);

        gridLayout_6->addWidget(stackWidgetMain, 0, 0, 1, 1);


        retranslateUi(SoundfontViewer);

        stackWidgetMain->setCurrentIndex(0);
        stackedWidget->setCurrentIndex(1);
        stackedWidget_2->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(SoundfontViewer);
    } // setupUi

    void retranslateUi(QWidget *SoundfontViewer)
    {
#if QT_CONFIG(tooltip)
        pushEdit->setToolTip(QCoreApplication::translate("SoundfontViewer", "Edit the soundfont description", nullptr));
#endif // QT_CONFIG(tooltip)
        pushEdit->setText(QString());
        label_4->setText(QCoreApplication::translate("SoundfontViewer", "Oops! Cannot\n"
"download content", nullptr));
        pushRetry->setText(QString());
#if QT_CONFIG(tooltip)
        pushCancel->setToolTip(QCoreApplication::translate("SoundfontViewer", "Discard changes", nullptr));
#endif // QT_CONFIG(tooltip)
        pushCancel->setText(QString());
#if QT_CONFIG(tooltip)
        pushSave->setToolTip(QCoreApplication::translate("SoundfontViewer", "Submit the changes", nullptr));
#endif // QT_CONFIG(tooltip)
        pushSave->setText(QString());
        (void)SoundfontViewer;
    } // retranslateUi

};

namespace Ui {
    class SoundfontViewer: public Ui_SoundfontViewer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTVIEWER_H
