/********************************************************************************
** Form generated from reading UI file 'pagerange.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGERANGE_H
#define UI_PAGERANGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QWidget>
#include "graphicsviewrange.h"

QT_BEGIN_NAMESPACE

class Ui_PageRange
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QGridLayout *gridLayout_2;
    GraphicsViewRange *rangeEditor;
    QWidget *page_2;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *PageRange)
    {
        if (PageRange->objectName().isEmpty())
            PageRange->setObjectName("PageRange");
        PageRange->resize(400, 300);
        PageRange->setWindowTitle(QString::fromUtf8(""));
        gridLayout = new QGridLayout(PageRange);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(PageRange);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        gridLayout_2 = new QGridLayout(page);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        rangeEditor = new GraphicsViewRange(page);
        rangeEditor->setObjectName("rangeEditor");
        rangeEditor->setFrameShape(QFrame::NoFrame);

        gridLayout_2->addWidget(rangeEditor, 0, 0, 1, 1);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        gridLayout_3 = new QGridLayout(page_2);
        gridLayout_3->setObjectName("gridLayout_3");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 0, 1, 1, 1);

        label = new QLabel(page_2);
        label->setObjectName("label");
        QFont font;
        font.setBold(true);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label, 1, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer, 1, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 2, 1, 1, 1);

        stackedWidget->addWidget(page_2);

        gridLayout->addWidget(stackedWidget, 0, 0, 1, 1);


        retranslateUi(PageRange);

        QMetaObject::connectSlotsByName(PageRange);
    } // setupUi

    void retranslateUi(QWidget *PageRange)
    {
        label->setText(QCoreApplication::translate("PageRange", "Select a single instrument or preset\n"
"to display its division ranges", nullptr));
        (void)PageRange;
    } // retranslateUi

};

namespace Ui {
    class PageRange: public Ui_PageRange {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGERANGE_H
