/****************************************************************************
** Meta object code from reading C++ file 'editortoolbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "editor/widgets/editortoolbar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editortoolbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13EditorToolBarE_t {};
} // unnamed namespace

template <> constexpr inline auto EditorToolBar::qt_create_metaobjectdata<qt_meta_tag_ZN13EditorToolBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EditorToolBar",
        "displayOptionChanged",
        "",
        "displayOption",
        "recorderDisplayChanged",
        "isDisplayed",
        "keyboardDisplayChanged",
        "selectionChanged",
        "IdList",
        "id",
        "onSelectionChanged",
        "ids",
        "disableDisplayOption",
        "QList<int>",
        "disabledOptions",
        "selectDisplayOption",
        "onDisplayActionClicked",
        "on_pushAddSample_clicked",
        "on_pushAddInstrument_clicked",
        "onNewInstClicked",
        "name",
        "linkElements",
        "on_pushAddPreset_clicked",
        "onNewPrstClicked",
        "on_pushUndo_clicked",
        "on_pushRedo_clicked",
        "on_pushSave_clicked",
        "on_pushShowRecorder_clicked",
        "on_pushShowKeyboard_clicked",
        "onMidiExtensionActionClicked",
        "onExtensionDialogClosed",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'displayOptionChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'recorderDisplayChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'keyboardDisplayChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void(IdList)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'onSelectionChanged'
        QtMocHelpers::SlotData<void(IdList)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 11 },
        }}),
        // Slot 'disableDisplayOption'
        QtMocHelpers::SlotData<void(QList<int>)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'selectDisplayOption'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'onDisplayActionClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushAddSample_clicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushAddInstrument_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onNewInstClicked'
        QtMocHelpers::SlotData<void(QString, bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'on_pushAddPreset_clicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onNewPrstClicked'
        QtMocHelpers::SlotData<void(QString, bool)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'on_pushUndo_clicked'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushRedo_clicked'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushSave_clicked'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushShowRecorder_clicked'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushShowKeyboard_clicked'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMidiExtensionActionClicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onExtensionDialogClosed'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EditorToolBar, qt_meta_tag_ZN13EditorToolBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EditorToolBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditorToolBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditorToolBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13EditorToolBarE_t>.metaTypes,
    nullptr
} };

void EditorToolBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EditorToolBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->displayOptionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->recorderDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->keyboardDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<IdList>>(_a[1]))); break;
        case 4: _t->onSelectionChanged((*reinterpret_cast<std::add_pointer_t<IdList>>(_a[1]))); break;
        case 5: _t->disableDisplayOption((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 6: _t->selectDisplayOption((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->onDisplayActionClicked(); break;
        case 8: _t->on_pushAddSample_clicked(); break;
        case 9: _t->on_pushAddInstrument_clicked(); break;
        case 10: _t->onNewInstClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->on_pushAddPreset_clicked(); break;
        case 12: _t->onNewPrstClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->on_pushUndo_clicked(); break;
        case 14: _t->on_pushRedo_clicked(); break;
        case 15: _t->on_pushSave_clicked(); break;
        case 16: _t->on_pushShowRecorder_clicked(); break;
        case 17: _t->on_pushShowKeyboard_clicked(); break;
        case 18: _t->onMidiExtensionActionClicked(); break;
        case 19: _t->onExtensionDialogClosed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EditorToolBar::*)(int )>(_a, &EditorToolBar::displayOptionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorToolBar::*)(bool )>(_a, &EditorToolBar::recorderDisplayChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorToolBar::*)(bool )>(_a, &EditorToolBar::keyboardDisplayChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorToolBar::*)(IdList )>(_a, &EditorToolBar::selectionChanged, 3))
            return;
    }
}

const QMetaObject *EditorToolBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditorToolBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13EditorToolBarE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int EditorToolBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void EditorToolBar::displayOptionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void EditorToolBar::recorderDisplayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void EditorToolBar::keyboardDisplayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void EditorToolBar::selectionChanged(IdList _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
