/********************************************************************************
** Form generated from reading UI file 'fivestarswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FIVESTARSWIDGET_H
#define UI_FIVESTARSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FiveStarsWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *star1;
    QLabel *star2;
    QLabel *star3;
    QLabel *star4;
    QLabel *star5;

    void setupUi(QWidget *FiveStarsWidget)
    {
        if (FiveStarsWidget->objectName().isEmpty())
            FiveStarsWidget->setObjectName("FiveStarsWidget");
        FiveStarsWidget->resize(132, 24);
        FiveStarsWidget->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(FiveStarsWidget);
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        star1 = new QLabel(FiveStarsWidget);
        star1->setObjectName("star1");
        star1->setMinimumSize(QSize(24, 24));
        star1->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(star1);

        star2 = new QLabel(FiveStarsWidget);
        star2->setObjectName("star2");
        star2->setMinimumSize(QSize(24, 24));
        star2->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(star2);

        star3 = new QLabel(FiveStarsWidget);
        star3->setObjectName("star3");
        star3->setMinimumSize(QSize(24, 24));
        star3->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(star3);

        star4 = new QLabel(FiveStarsWidget);
        star4->setObjectName("star4");
        star4->setMinimumSize(QSize(24, 24));
        star4->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(star4);

        star5 = new QLabel(FiveStarsWidget);
        star5->setObjectName("star5");
        star5->setMinimumSize(QSize(24, 24));
        star5->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(star5);


        retranslateUi(FiveStarsWidget);

        QMetaObject::connectSlotsByName(FiveStarsWidget);
    } // setupUi

    void retranslateUi(QWidget *FiveStarsWidget)
    {
        star1->setText(QString());
        star2->setText(QString());
        star3->setText(QString());
        star4->setText(QString());
        star5->setText(QString());
        (void)FiveStarsWidget;
    } // retranslateUi

};

namespace Ui {
    class FiveStarsWidget: public Ui_FiveStarsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FIVESTARSWIDGET_H
