/********************************************************************************
** Form generated from reading UI file 'tooldivisionduplication_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLDIVISIONDUPLICATION_GUI_H
#define UI_TOOLDIVISIONDUPLICATION_GUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolDivisionDuplication_gui
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *checkForEachKey;
    QCheckBox *checkForEachVelocityRange;
    QGridLayout *gridLayout_2;
    QSpinBox *spinMinVel;
    QListWidget *listVelocites;
    QSpinBox *spinMaxVel;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushAdd;
    QPushButton *pushRemove;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolDivisionDuplication_gui)
    {
        if (ToolDivisionDuplication_gui->objectName().isEmpty())
            ToolDivisionDuplication_gui->setObjectName("ToolDivisionDuplication_gui");
        ToolDivisionDuplication_gui->resize(202, 183);
        ToolDivisionDuplication_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolDivisionDuplication_gui);
        verticalLayout->setObjectName("verticalLayout");
        checkForEachKey = new QCheckBox(ToolDivisionDuplication_gui);
        checkForEachKey->setObjectName("checkForEachKey");
        checkForEachKey->setChecked(true);

        verticalLayout->addWidget(checkForEachKey);

        checkForEachVelocityRange = new QCheckBox(ToolDivisionDuplication_gui);
        checkForEachVelocityRange->setObjectName("checkForEachVelocityRange");

        verticalLayout->addWidget(checkForEachVelocityRange);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        spinMinVel = new QSpinBox(ToolDivisionDuplication_gui);
        spinMinVel->setObjectName("spinMinVel");
        spinMinVel->setMaximum(127);

        gridLayout_2->addWidget(spinMinVel, 1, 1, 1, 1);

        listVelocites = new QListWidget(ToolDivisionDuplication_gui);
        listVelocites->setObjectName("listVelocites");

        gridLayout_2->addWidget(listVelocites, 0, 0, 3, 1);

        spinMaxVel = new QSpinBox(ToolDivisionDuplication_gui);
        spinMaxVel->setObjectName("spinMaxVel");
        spinMaxVel->setMaximum(127);

        gridLayout_2->addWidget(spinMaxVel, 2, 1, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushAdd = new QPushButton(ToolDivisionDuplication_gui);
        pushAdd->setObjectName("pushAdd");
        pushAdd->setMinimumSize(QSize(24, 24));
        pushAdd->setMaximumSize(QSize(24, 24));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/document-new.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushAdd->setIcon(icon);

        horizontalLayout_2->addWidget(pushAdd);

        pushRemove = new QPushButton(ToolDivisionDuplication_gui);
        pushRemove->setObjectName("pushRemove");
        pushRemove->setMinimumSize(QSize(24, 24));
        pushRemove->setMaximumSize(QSize(24, 24));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/minus.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushRemove->setIcon(icon1);

        horizontalLayout_2->addWidget(pushRemove);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        gridLayout_2->addLayout(horizontalLayout_2, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolDivisionDuplication_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        pushCancel = new QPushButton(ToolDivisionDuplication_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(checkForEachKey, checkForEachVelocityRange);
        QWidget::setTabOrder(checkForEachVelocityRange, listVelocites);
        QWidget::setTabOrder(listVelocites, pushAdd);
        QWidget::setTabOrder(pushAdd, pushRemove);
        QWidget::setTabOrder(pushRemove, spinMinVel);
        QWidget::setTabOrder(spinMinVel, spinMaxVel);
        QWidget::setTabOrder(spinMaxVel, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolDivisionDuplication_gui);

        QMetaObject::connectSlotsByName(ToolDivisionDuplication_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolDivisionDuplication_gui)
    {
        checkForEachKey->setText(QCoreApplication::translate("ToolDivisionDuplication_gui", "A division for each key", nullptr));
        checkForEachVelocityRange->setText(QCoreApplication::translate("ToolDivisionDuplication_gui", "A division for each velocity range", nullptr));
        pushAdd->setText(QString());
        pushRemove->setText(QString());
        pushOk->setText(QCoreApplication::translate("ToolDivisionDuplication_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolDivisionDuplication_gui", "&Cancel", nullptr));
        (void)ToolDivisionDuplication_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolDivisionDuplication_gui: public Ui_ToolDivisionDuplication_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLDIVISIONDUPLICATION_GUI_H
