/********************************************************************************
** Form generated from reading UI file 'modulatoreditor.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MODULATOREDITOR_H
#define UI_MODULATOREDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "modulatorlistwidget.h"

QT_BEGIN_NAMESPACE

class Ui_ModulatorEditor
{
public:
    QGridLayout *gridLayout;
    QFrame *frameExpanded;
    QHBoxLayout *frameExpanded_layout;
    QFrame *frameButtons;
    QVBoxLayout *verticalLayout;
    QPushButton *pushAdd;
    QPushButton *pushCopy;
    QPushButton *pushPaste;
    QPushButton *pushClone;
    QPushButton *pushDelete;
    QSpacerItem *verticalSpacer;
    QPushButton *pushCollapse;
    QStackedWidget *stackedWidget;
    QWidget *pageNoModulators;
    QGridLayout *gridLayout_2;
    QLabel *labelNoModulators;
    QWidget *pageSelectDivision;
    QGridLayout *gridLayout_4;
    QLabel *labelSelectDivision;
    QWidget *pageModulatorList;
    QGridLayout *gridLayout_3;
    ModulatorListWidget *listWidget;
    QFrame *frameCollapsed;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushExpand;
    QLabel *labelModSummary;

    void setupUi(QWidget *ModulatorEditor)
    {
        if (ModulatorEditor->objectName().isEmpty())
            ModulatorEditor->setObjectName("ModulatorEditor");
        ModulatorEditor->resize(437, 208);
        ModulatorEditor->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ModulatorEditor);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        frameExpanded = new QFrame(ModulatorEditor);
        frameExpanded->setObjectName("frameExpanded");
        frameExpanded_layout = new QHBoxLayout(frameExpanded);
        frameExpanded_layout->setSpacing(0);
        frameExpanded_layout->setObjectName("frameExpanded_layout");
        frameExpanded_layout->setContentsMargins(0, 0, 0, 0);
        frameButtons = new QFrame(frameExpanded);
        frameButtons->setObjectName("frameButtons");
        verticalLayout = new QVBoxLayout(frameButtons);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        pushAdd = new QPushButton(frameButtons);
        pushAdd->setObjectName("pushAdd");
        pushAdd->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushAdd->setFlat(true);

        verticalLayout->addWidget(pushAdd);

        pushCopy = new QPushButton(frameButtons);
        pushCopy->setObjectName("pushCopy");
        pushCopy->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushCopy->setFlat(true);

        verticalLayout->addWidget(pushCopy);

        pushPaste = new QPushButton(frameButtons);
        pushPaste->setObjectName("pushPaste");
        pushPaste->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushPaste->setFlat(true);

        verticalLayout->addWidget(pushPaste);

        pushClone = new QPushButton(frameButtons);
        pushClone->setObjectName("pushClone");
        pushClone->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushClone->setFlat(true);

        verticalLayout->addWidget(pushClone);

        pushDelete = new QPushButton(frameButtons);
        pushDelete->setObjectName("pushDelete");
        pushDelete->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushDelete->setFlat(true);

        verticalLayout->addWidget(pushDelete);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        pushCollapse = new QPushButton(frameButtons);
        pushCollapse->setObjectName("pushCollapse");
        pushCollapse->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushCollapse->setFlat(true);

        verticalLayout->addWidget(pushCollapse);


        frameExpanded_layout->addWidget(frameButtons);

        stackedWidget = new QStackedWidget(frameExpanded);
        stackedWidget->setObjectName("stackedWidget");
        pageNoModulators = new QWidget();
        pageNoModulators->setObjectName("pageNoModulators");
        gridLayout_2 = new QGridLayout(pageNoModulators);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        labelNoModulators = new QLabel(pageNoModulators);
        labelNoModulators->setObjectName("labelNoModulators");
        QFont font;
        font.setBold(true);
        labelNoModulators->setFont(font);
        labelNoModulators->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelNoModulators, 0, 0, 1, 1);

        stackedWidget->addWidget(pageNoModulators);
        pageSelectDivision = new QWidget();
        pageSelectDivision->setObjectName("pageSelectDivision");
        gridLayout_4 = new QGridLayout(pageSelectDivision);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        labelSelectDivision = new QLabel(pageSelectDivision);
        labelSelectDivision->setObjectName("labelSelectDivision");
        labelSelectDivision->setFont(font);
        labelSelectDivision->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelSelectDivision, 0, 0, 1, 1);

        stackedWidget->addWidget(pageSelectDivision);
        pageModulatorList = new QWidget();
        pageModulatorList->setObjectName("pageModulatorList");
        gridLayout_3 = new QGridLayout(pageModulatorList);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setHorizontalSpacing(0);
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        listWidget = new ModulatorListWidget(pageModulatorList);
        listWidget->setObjectName("listWidget");
        listWidget->setStyleSheet(QString::fromUtf8("QListWidget {\n"
"border-top: 0;\n"
"border-right: 0;\n"
"border-bottom: 0;\n"
"}"));
        listWidget->setFrameShape(QFrame::NoFrame);
        listWidget->setAlternatingRowColors(true);
        listWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
        listWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setResizeMode(QListView::Adjust);

        gridLayout_3->addWidget(listWidget, 0, 1, 1, 1);

        stackedWidget->addWidget(pageModulatorList);

        frameExpanded_layout->addWidget(stackedWidget);


        gridLayout->addWidget(frameExpanded, 1, 0, 1, 1);

        frameCollapsed = new QFrame(ModulatorEditor);
        frameCollapsed->setObjectName("frameCollapsed");
        horizontalLayout = new QHBoxLayout(frameCollapsed);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(1, 6, 6, 6);
        pushExpand = new QPushButton(frameCollapsed);
        pushExpand->setObjectName("pushExpand");
        pushExpand->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushExpand->setFlat(true);

        horizontalLayout->addWidget(pushExpand);

        labelModSummary = new QLabel(frameCollapsed);
        labelModSummary->setObjectName("labelModSummary");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelModSummary->sizePolicy().hasHeightForWidth());
        labelModSummary->setSizePolicy(sizePolicy);
        labelModSummary->setText(QString::fromUtf8("Modulator summary"));

        horizontalLayout->addWidget(labelModSummary);


        gridLayout->addWidget(frameCollapsed, 0, 0, 1, 1);


        retranslateUi(ModulatorEditor);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(ModulatorEditor);
    } // setupUi

    void retranslateUi(QWidget *ModulatorEditor)
    {
#if QT_CONFIG(tooltip)
        pushAdd->setToolTip(QCoreApplication::translate("ModulatorEditor", "Add a modulator", nullptr));
#endif // QT_CONFIG(tooltip)
        pushAdd->setText(QString());
        pushCopy->setText(QString());
#if QT_CONFIG(tooltip)
        pushPaste->setToolTip(QCoreApplication::translate("ModulatorEditor", "Paste the copied modulators", nullptr));
#endif // QT_CONFIG(tooltip)
        pushPaste->setText(QString());
        pushClone->setText(QString());
#if QT_CONFIG(tooltip)
        pushDelete->setToolTip(QCoreApplication::translate("ModulatorEditor", "Delete a modulator", nullptr));
#endif // QT_CONFIG(tooltip)
        pushDelete->setText(QString());
#if QT_CONFIG(tooltip)
        pushCollapse->setToolTip(QCoreApplication::translate("ModulatorEditor", "Collapse the modulator section", nullptr));
#endif // QT_CONFIG(tooltip)
        pushCollapse->setText(QString());
        labelNoModulators->setText(QCoreApplication::translate("ModulatorEditor", "No modulators", nullptr));
        labelSelectDivision->setText(QCoreApplication::translate("ModulatorEditor", "Select a single division to display the modulator list", nullptr));
#if QT_CONFIG(tooltip)
        pushExpand->setToolTip(QCoreApplication::translate("ModulatorEditor", "Expand the modulator section", nullptr));
#endif // QT_CONFIG(tooltip)
        pushExpand->setText(QString());
        (void)ModulatorEditor;
    } // retranslateUi

};

namespace Ui {
    class ModulatorEditor: public Ui_ModulatorEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MODULATOREDITOR_H
