/********************************************************************************
** Form generated from reading UI file 'soundfontdownloadcell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTDOWNLOADCELL_H
#define UI_SOUNDFONTDOWNLOADCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SoundfontDownloadCell
{
public:
    QGridLayout *gridLayout;
    QLabel *labelTitle;
    QLabel *labelInfo;
    QPushButton *pushDownload;
    QLabel *labelDescription;

    void setupUi(QWidget *SoundfontDownloadCell)
    {
        if (SoundfontDownloadCell->objectName().isEmpty())
            SoundfontDownloadCell->setObjectName("SoundfontDownloadCell");
        SoundfontDownloadCell->resize(272, 72);
        SoundfontDownloadCell->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SoundfontDownloadCell);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        labelTitle = new QLabel(SoundfontDownloadCell);
        labelTitle->setObjectName("labelTitle");
        QFont font;
        font.setBold(true);
        labelTitle->setFont(font);
        labelTitle->setText(QString::fromUtf8("Title"));

        gridLayout->addWidget(labelTitle, 0, 0, 1, 1);

        labelInfo = new QLabel(SoundfontDownloadCell);
        labelInfo->setObjectName("labelInfo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelInfo->sizePolicy().hasHeightForWidth());
        labelInfo->setSizePolicy(sizePolicy);
        labelInfo->setText(QString::fromUtf8("(10Mo, 15 August, 2018)"));

        gridLayout->addWidget(labelInfo, 0, 1, 1, 1);

        pushDownload = new QPushButton(SoundfontDownloadCell);
        pushDownload->setObjectName("pushDownload");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushDownload->sizePolicy().hasHeightForWidth());
        pushDownload->setSizePolicy(sizePolicy1);
        pushDownload->setMinimumSize(QSize(48, 48));
        pushDownload->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushDownload->setIconSize(QSize(24, 24));

        gridLayout->addWidget(pushDownload, 0, 2, 1, 1);

        labelDescription = new QLabel(SoundfontDownloadCell);
        labelDescription->setObjectName("labelDescription");
        labelDescription->setText(QString::fromUtf8("Description"));
        labelDescription->setWordWrap(true);

        gridLayout->addWidget(labelDescription, 1, 0, 1, 3);


        retranslateUi(SoundfontDownloadCell);

        QMetaObject::connectSlotsByName(SoundfontDownloadCell);
    } // setupUi

    void retranslateUi(QWidget *SoundfontDownloadCell)
    {
#if QT_CONFIG(tooltip)
        pushDownload->setToolTip(QCoreApplication::translate("SoundfontDownloadCell", "Download", nullptr));
#endif // QT_CONFIG(tooltip)
        pushDownload->setText(QString());
        (void)SoundfontDownloadCell;
    } // retranslateUi

};

namespace Ui {
    class SoundfontDownloadCell: public Ui_SoundfontDownloadCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTDOWNLOADCELL_H
