/********************************************************************************
** Form generated from reading UI file 'soundfonteditorfiles.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTEDITORFILES_H
#define UI_SOUNDFONTEDITORFILES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SoundfontEditorFiles
{
public:
    QVBoxLayout *verticalLayout;

    void setupUi(QWidget *SoundfontEditorFiles)
    {
        if (SoundfontEditorFiles->objectName().isEmpty())
            SoundfontEditorFiles->setObjectName("SoundfontEditorFiles");
        SoundfontEditorFiles->resize(210, 18);
        SoundfontEditorFiles->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontEditorFiles);
        verticalLayout->setSpacing(9);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);

        retranslateUi(SoundfontEditorFiles);

        QMetaObject::connectSlotsByName(SoundfontEditorFiles);
    } // setupUi

    void retranslateUi(QWidget *SoundfontEditorFiles)
    {
        (void)SoundfontEditorFiles;
    } // retranslateUi

};

namespace Ui {
    class SoundfontEditorFiles: public Ui_SoundfontEditorFiles {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTEDITORFILES_H
