/********************************************************************************
** Form generated from reading UI file 'soundfonteditorcenter.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTEDITORCENTER_H
#define UI_SOUNDFONTEDITORCENTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "htmleditor.h"
#include "soundfonteditorfiles.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontEditorCenter
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *labelDescription;
    HtmlEditor *editorDescription;
    QSpacerItem *verticalSpacer;
    QFrame *frameEditDownload;
    QHBoxLayout *horizontalLayout_5;
    QLabel *labelDownload;
    QPushButton *pushAddFile;
    SoundfontEditorFiles *editorFiles;

    void setupUi(QWidget *SoundfontEditorCenter)
    {
        if (SoundfontEditorCenter->objectName().isEmpty())
            SoundfontEditorCenter->setObjectName("SoundfontEditorCenter");
        SoundfontEditorCenter->resize(400, 300);
        SoundfontEditorCenter->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontEditorCenter);
        verticalLayout->setSpacing(9);
        verticalLayout->setObjectName("verticalLayout");
        labelDescription = new QLabel(SoundfontEditorCenter);
        labelDescription->setObjectName("labelDescription");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelDescription->sizePolicy().hasHeightForWidth());
        labelDescription->setSizePolicy(sizePolicy);
        labelDescription->setMinimumSize(QSize(0, 28));
        QFont font;
        font.setPointSize(13);
        font.setBold(true);
        labelDescription->setFont(font);

        verticalLayout->addWidget(labelDescription);

        editorDescription = new HtmlEditor(SoundfontEditorCenter);
        editorDescription->setObjectName("editorDescription");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(editorDescription->sizePolicy().hasHeightForWidth());
        editorDescription->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(editorDescription);

        verticalSpacer = new QSpacerItem(20, 2, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer);

        frameEditDownload = new QFrame(SoundfontEditorCenter);
        frameEditDownload->setObjectName("frameEditDownload");
        frameEditDownload->setMaximumSize(QSize(16777215, 28));
        frameEditDownload->setFrameShape(QFrame::StyledPanel);
        frameEditDownload->setFrameShadow(QFrame::Raised);
        horizontalLayout_5 = new QHBoxLayout(frameEditDownload);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(-1, 0, 0, 0);
        labelDownload = new QLabel(frameEditDownload);
        labelDownload->setObjectName("labelDownload");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Maximum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelDownload->sizePolicy().hasHeightForWidth());
        labelDownload->setSizePolicy(sizePolicy2);
        labelDownload->setFont(font);

        horizontalLayout_5->addWidget(labelDownload);

        pushAddFile = new QPushButton(frameEditDownload);
        pushAddFile->setObjectName("pushAddFile");
        pushAddFile->setMinimumSize(QSize(0, 28));
        pushAddFile->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout_5->addWidget(pushAddFile);


        verticalLayout->addWidget(frameEditDownload);

        editorFiles = new SoundfontEditorFiles(SoundfontEditorCenter);
        editorFiles->setObjectName("editorFiles");
        sizePolicy.setHeightForWidth(editorFiles->sizePolicy().hasHeightForWidth());
        editorFiles->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(editorFiles);


        retranslateUi(SoundfontEditorCenter);

        QMetaObject::connectSlotsByName(SoundfontEditorCenter);
    } // setupUi

    void retranslateUi(QWidget *SoundfontEditorCenter)
    {
        labelDescription->setText(QCoreApplication::translate("SoundfontEditorCenter", "Description", nullptr));
        labelDownload->setText(QCoreApplication::translate("SoundfontEditorCenter", "Files", nullptr));
#if QT_CONFIG(tooltip)
        pushAddFile->setToolTip(QCoreApplication::translate("SoundfontEditorCenter", "Add a file", nullptr));
#endif // QT_CONFIG(tooltip)
        pushAddFile->setText(QString());
        (void)SoundfontEditorCenter;
    } // retranslateUi

};

namespace Ui {
    class SoundfontEditorCenter: public Ui_SoundfontEditorCenter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTEDITORCENTER_H
