/********************************************************************************
** Form generated from reading UI file 'pageenvelope.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEENVELOPE_H
#define UI_PAGEENVELOPE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "graphicsviewenvelop.h"

QT_BEGIN_NAMESPACE

class Ui_PageEnvelope
{
public:
    QVBoxLayout *verticalLayout_2;
    GraphicsViewEnvelop *graphicsView;
    QScrollBar *sliderGraph;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QPushButton *pushKeyDecay;
    QLabel *labelDecay;
    QLabel *labelRelease;
    QPushButton *pushHold;
    QDoubleSpinBox *doubleSpinDelay;
    QLabel *labelKeyDecay;
    QPushButton *pushKeyHold;
    QDoubleSpinBox *doubleSpinRelease;
    QDoubleSpinBox *doubleSpinAttack;
    QPushButton *pushRelease;
    QLabel *labelHold;
    QLabel *labelDelay;
    QPushButton *pushDecay;
    QSpinBox *spinKeyHold;
    QSpinBox *spinKeyDecay;
    QLabel *labelAttack;
    QDoubleSpinBox *doubleSpinDecay;
    QPushButton *pushDelay;
    QDoubleSpinBox *doubleSpinSustain;
    QLabel *labelKeyHold;
    QDoubleSpinBox *doubleSpinHold;
    QLabel *labelSustain;
    QPushButton *pushAttack;
    QPushButton *pushSustain;
    QPushButton *pushVolume;
    QPushButton *pushModulation;

    void setupUi(QWidget *PageEnvelope)
    {
        if (PageEnvelope->objectName().isEmpty())
            PageEnvelope->setObjectName("PageEnvelope");
        PageEnvelope->resize(1120, 265);
        PageEnvelope->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout_2 = new QVBoxLayout(PageEnvelope);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        graphicsView = new GraphicsViewEnvelop(PageEnvelope);
        graphicsView->setObjectName("graphicsView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(graphicsView->sizePolicy().hasHeightForWidth());
        graphicsView->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(graphicsView);

        sliderGraph = new QScrollBar(PageEnvelope);
        sliderGraph->setObjectName("sliderGraph");
        sliderGraph->setMaximum(0);
        sliderGraph->setSingleStep(200);
        sliderGraph->setPageStep(10000);
        sliderGraph->setOrientation(Qt::Horizontal);

        verticalLayout_2->addWidget(sliderGraph);

        frame = new QFrame(PageEnvelope);
        frame->setObjectName("frame");
        frame->setMaximumSize(QSize(16777215, 1));
        frame->setFrameShape(QFrame::Box);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);

        verticalLayout_2->addWidget(frame);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setHorizontalSpacing(6);
        gridLayout_2->setVerticalSpacing(3);
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        pushKeyDecay = new QPushButton(PageEnvelope);
        pushKeyDecay->setObjectName("pushKeyDecay");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/edit-undo.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushKeyDecay->setIcon(icon);
        pushKeyDecay->setFlat(true);

        gridLayout_2->addWidget(pushKeyDecay, 2, 11, 1, 1);

        labelDecay = new QLabel(PageEnvelope);
        labelDecay->setObjectName("labelDecay");
        labelDecay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelDecay, 2, 6, 1, 1);

        labelRelease = new QLabel(PageEnvelope);
        labelRelease->setObjectName("labelRelease");
        labelRelease->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelRelease, 2, 3, 1, 1);

        pushHold = new QPushButton(PageEnvelope);
        pushHold->setObjectName("pushHold");
        pushHold->setIcon(icon);
        pushHold->setFlat(true);

        gridLayout_2->addWidget(pushHold, 1, 8, 1, 1);

        doubleSpinDelay = new QDoubleSpinBox(PageEnvelope);
        doubleSpinDelay->setObjectName("doubleSpinDelay");
        doubleSpinDelay->setDecimals(3);
        doubleSpinDelay->setMinimum(0.001000000000000);
        doubleSpinDelay->setMaximum(18.000000000000000);
        doubleSpinDelay->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinDelay, 1, 1, 1, 1);

        labelKeyDecay = new QLabel(PageEnvelope);
        labelKeyDecay->setObjectName("labelKeyDecay");
        labelKeyDecay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelKeyDecay, 2, 9, 1, 1);

        pushKeyHold = new QPushButton(PageEnvelope);
        pushKeyHold->setObjectName("pushKeyHold");
        pushKeyHold->setIcon(icon);
        pushKeyHold->setFlat(true);

        gridLayout_2->addWidget(pushKeyHold, 1, 11, 1, 1);

        doubleSpinRelease = new QDoubleSpinBox(PageEnvelope);
        doubleSpinRelease->setObjectName("doubleSpinRelease");
        doubleSpinRelease->setDecimals(3);
        doubleSpinRelease->setMinimum(0.001000000000000);
        doubleSpinRelease->setMaximum(102.000000000000000);
        doubleSpinRelease->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinRelease, 2, 4, 1, 1);

        doubleSpinAttack = new QDoubleSpinBox(PageEnvelope);
        doubleSpinAttack->setObjectName("doubleSpinAttack");
        doubleSpinAttack->setDecimals(3);
        doubleSpinAttack->setMinimum(0.001000000000000);
        doubleSpinAttack->setMaximum(102.000000000000000);
        doubleSpinAttack->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinAttack, 1, 4, 1, 1);

        pushRelease = new QPushButton(PageEnvelope);
        pushRelease->setObjectName("pushRelease");
        pushRelease->setIcon(icon);
        pushRelease->setFlat(true);

        gridLayout_2->addWidget(pushRelease, 2, 5, 1, 1);

        labelHold = new QLabel(PageEnvelope);
        labelHold->setObjectName("labelHold");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelHold->sizePolicy().hasHeightForWidth());
        labelHold->setSizePolicy(sizePolicy1);
        labelHold->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelHold, 1, 6, 1, 1);

        labelDelay = new QLabel(PageEnvelope);
        labelDelay->setObjectName("labelDelay");
        sizePolicy1.setHeightForWidth(labelDelay->sizePolicy().hasHeightForWidth());
        labelDelay->setSizePolicy(sizePolicy1);
        labelDelay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelDelay, 1, 0, 1, 1);

        pushDecay = new QPushButton(PageEnvelope);
        pushDecay->setObjectName("pushDecay");
        pushDecay->setIcon(icon);
        pushDecay->setFlat(true);

        gridLayout_2->addWidget(pushDecay, 2, 8, 1, 1);

        spinKeyHold = new QSpinBox(PageEnvelope);
        spinKeyHold->setObjectName("spinKeyHold");
        spinKeyHold->setMinimum(-1200);
        spinKeyHold->setMaximum(1200);

        gridLayout_2->addWidget(spinKeyHold, 1, 10, 1, 1);

        spinKeyDecay = new QSpinBox(PageEnvelope);
        spinKeyDecay->setObjectName("spinKeyDecay");
        spinKeyDecay->setMinimum(-1200);
        spinKeyDecay->setMaximum(1200);

        gridLayout_2->addWidget(spinKeyDecay, 2, 10, 1, 1);

        labelAttack = new QLabel(PageEnvelope);
        labelAttack->setObjectName("labelAttack");
        sizePolicy1.setHeightForWidth(labelAttack->sizePolicy().hasHeightForWidth());
        labelAttack->setSizePolicy(sizePolicy1);
        labelAttack->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelAttack, 1, 3, 1, 1);

        doubleSpinDecay = new QDoubleSpinBox(PageEnvelope);
        doubleSpinDecay->setObjectName("doubleSpinDecay");
        doubleSpinDecay->setDecimals(3);
        doubleSpinDecay->setMinimum(0.001000000000000);
        doubleSpinDecay->setMaximum(102.000000000000000);
        doubleSpinDecay->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinDecay, 2, 7, 1, 1);

        pushDelay = new QPushButton(PageEnvelope);
        pushDelay->setObjectName("pushDelay");
        pushDelay->setIcon(icon);
        pushDelay->setFlat(true);

        gridLayout_2->addWidget(pushDelay, 1, 2, 1, 1);

        doubleSpinSustain = new QDoubleSpinBox(PageEnvelope);
        doubleSpinSustain->setObjectName("doubleSpinSustain");
        doubleSpinSustain->setDecimals(1);
        doubleSpinSustain->setMaximum(144.000000000000000);

        gridLayout_2->addWidget(doubleSpinSustain, 2, 1, 1, 1);

        labelKeyHold = new QLabel(PageEnvelope);
        labelKeyHold->setObjectName("labelKeyHold");
        sizePolicy1.setHeightForWidth(labelKeyHold->sizePolicy().hasHeightForWidth());
        labelKeyHold->setSizePolicy(sizePolicy1);
        labelKeyHold->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelKeyHold, 1, 9, 1, 1);

        doubleSpinHold = new QDoubleSpinBox(PageEnvelope);
        doubleSpinHold->setObjectName("doubleSpinHold");
        doubleSpinHold->setDecimals(3);
        doubleSpinHold->setMinimum(0.001000000000000);
        doubleSpinHold->setMaximum(18.000000000000000);
        doubleSpinHold->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinHold, 1, 7, 1, 1);

        labelSustain = new QLabel(PageEnvelope);
        labelSustain->setObjectName("labelSustain");
        labelSustain->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelSustain, 2, 0, 1, 1);

        pushAttack = new QPushButton(PageEnvelope);
        pushAttack->setObjectName("pushAttack");
        pushAttack->setIcon(icon);
        pushAttack->setFlat(true);

        gridLayout_2->addWidget(pushAttack, 1, 5, 1, 1);

        pushSustain = new QPushButton(PageEnvelope);
        pushSustain->setObjectName("pushSustain");
        pushSustain->setIcon(icon);
        pushSustain->setFlat(true);

        gridLayout_2->addWidget(pushSustain, 2, 2, 1, 1);

        pushVolume = new QPushButton(PageEnvelope);
        pushVolume->setObjectName("pushVolume");
        pushVolume->setCheckable(true);
        pushVolume->setChecked(true);
        pushVolume->setAutoExclusive(true);
        pushVolume->setFlat(true);

        gridLayout_2->addWidget(pushVolume, 0, 0, 1, 6);

        pushModulation = new QPushButton(PageEnvelope);
        pushModulation->setObjectName("pushModulation");
        pushModulation->setCheckable(true);
        pushModulation->setAutoExclusive(true);
        pushModulation->setFlat(true);

        gridLayout_2->addWidget(pushModulation, 0, 6, 1, 6);


        verticalLayout_2->addLayout(gridLayout_2);

        QWidget::setTabOrder(pushVolume, pushModulation);
        QWidget::setTabOrder(pushModulation, doubleSpinDelay);
        QWidget::setTabOrder(doubleSpinDelay, pushDelay);
        QWidget::setTabOrder(pushDelay, doubleSpinAttack);
        QWidget::setTabOrder(doubleSpinAttack, pushAttack);
        QWidget::setTabOrder(pushAttack, doubleSpinHold);
        QWidget::setTabOrder(doubleSpinHold, pushHold);
        QWidget::setTabOrder(pushHold, spinKeyHold);
        QWidget::setTabOrder(spinKeyHold, pushKeyHold);
        QWidget::setTabOrder(pushKeyHold, doubleSpinSustain);
        QWidget::setTabOrder(doubleSpinSustain, pushSustain);
        QWidget::setTabOrder(pushSustain, doubleSpinRelease);
        QWidget::setTabOrder(doubleSpinRelease, pushRelease);
        QWidget::setTabOrder(pushRelease, doubleSpinDecay);
        QWidget::setTabOrder(doubleSpinDecay, pushDecay);
        QWidget::setTabOrder(pushDecay, spinKeyDecay);
        QWidget::setTabOrder(spinKeyDecay, pushKeyDecay);

        retranslateUi(PageEnvelope);

        QMetaObject::connectSlotsByName(PageEnvelope);
    } // setupUi

    void retranslateUi(QWidget *PageEnvelope)
    {
        pushKeyDecay->setText(QString());
        labelDecay->setText(QCoreApplication::translate("PageEnvelope", "Decay", nullptr));
        labelRelease->setText(QCoreApplication::translate("PageEnvelope", "Release", nullptr));
        pushHold->setText(QString());
        doubleSpinDelay->setSuffix(QCoreApplication::translate("PageEnvelope", " s", "unit for second"));
        labelKeyDecay->setText(QCoreApplication::translate("PageEnvelope", "Key \342\206\222 Decay", nullptr));
        pushKeyHold->setText(QString());
        doubleSpinRelease->setSuffix(QCoreApplication::translate("PageEnvelope", " s", "unit for second"));
        doubleSpinAttack->setSuffix(QCoreApplication::translate("PageEnvelope", " s", "unit for second"));
        pushRelease->setText(QString());
        labelHold->setText(QCoreApplication::translate("PageEnvelope", "Hold", nullptr));
        labelDelay->setText(QCoreApplication::translate("PageEnvelope", "Delay", nullptr));
        pushDecay->setText(QString());
        labelAttack->setText(QCoreApplication::translate("PageEnvelope", "Attack", nullptr));
        doubleSpinDecay->setSuffix(QCoreApplication::translate("PageEnvelope", " s", "unit for second"));
        pushDelay->setText(QString());
        doubleSpinSustain->setSuffix(QCoreApplication::translate("PageEnvelope", " dB", "unit for decibels"));
        labelKeyHold->setText(QCoreApplication::translate("PageEnvelope", "Key \342\206\222 Hold", nullptr));
        doubleSpinHold->setSuffix(QCoreApplication::translate("PageEnvelope", " s", "unit for second"));
        labelSustain->setText(QCoreApplication::translate("PageEnvelope", "Sustain", nullptr));
        pushAttack->setText(QString());
        pushSustain->setText(QString());
        pushVolume->setText(QCoreApplication::translate("PageEnvelope", "Volume envelope", nullptr));
        pushModulation->setText(QCoreApplication::translate("PageEnvelope", "Modulation envelope", nullptr));
        (void)PageEnvelope;
    } // retranslateUi

};

namespace Ui {
    class PageEnvelope: public Ui_PageEnvelope {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEENVELOPE_H
