/****************************************************************************
** Meta object code from reading C++ file 'pagesmpl.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "editor/pagesmpl.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagesmpl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8PageSmplE_t {};
} // unnamed namespace

template <> constexpr inline auto PageSmpl::qt_create_metaobjectdata<qt_meta_tag_ZN8PageSmplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageSmpl",
        "lecture",
        "",
        "lecteurFinished",
        "token",
        "setStartLoop",
        "val",
        "setEndLoop",
        "on_pushFullLength_clicked",
        "setRootKey",
        "setTune",
        "setType",
        "index",
        "setLinkedSmpl",
        "setRate",
        "setSinusEnabled",
        "setGainSample",
        "setStereo",
        "on_pushAutoTune_clicked",
        "onCutOrdered",
        "start",
        "end",
        "cutSample",
        "EltID",
        "id",
        "EltID&",
        "createdSmplId",
        "findDuplicateName",
        "smplId",
        "getName",
        "name",
        "maxCharacters",
        "suffixNumber",
        "on_checkLectureBoucle_clicked",
        "checked",
        "on_pushCompress_clicked",
        "onCompress",
        "oggQuality"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'lecture'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'lecteurFinished'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'setStartLoop'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setStartLoop'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setEndLoop'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setEndLoop'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'on_pushFullLength_clicked'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setRootKey'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setRootKey'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setTune'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setTune'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setType'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setLinkedSmpl'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setRate'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'setSinusEnabled'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'setGainSample'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setStereo'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'on_pushAutoTune_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCutOrdered'
        QtMocHelpers::SlotData<void(int, int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'cutSample'
        QtMocHelpers::SlotData<bool(EltID, quint32, quint32, EltID &)>(22, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { 0x80000000 | 23, 24 }, { QMetaType::UInt, 20 }, { QMetaType::UInt, 21 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'findDuplicateName'
        QtMocHelpers::SlotData<QString(EltID)>(27, 2, QMC::AccessPrivate, QMetaType::QString, {{
            { 0x80000000 | 23, 28 },
        }}),
        // Slot 'getName'
        QtMocHelpers::SlotData<QString(QString, int, int)>(29, 2, QMC::AccessPrivate, QMetaType::QString, {{
            { QMetaType::QString, 30 }, { QMetaType::Int, 31 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'on_checkLectureBoucle_clicked'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 34 },
        }}),
        // Slot 'on_pushCompress_clicked'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCompress'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageSmpl, qt_meta_tag_ZN8PageSmplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageSmpl::staticMetaObject = { {
    QMetaObject::SuperData::link<Page::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageSmplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageSmplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8PageSmplE_t>.metaTypes,
    nullptr
} };

void PageSmpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageSmpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->lecture(); break;
        case 1: _t->lecteurFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->setStartLoop(); break;
        case 3: _t->setStartLoop((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->setEndLoop(); break;
        case 5: _t->setEndLoop((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->on_pushFullLength_clicked(); break;
        case 7: _t->setRootKey(); break;
        case 8: _t->setRootKey((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->setTune(); break;
        case 10: _t->setTune((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setLinkedSmpl((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->setRate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->setSinusEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->setGainSample((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->setStereo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->on_pushAutoTune_clicked(); break;
        case 18: _t->onCutOrdered((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 19: { bool _r = _t->cutSample((*reinterpret_cast<std::add_pointer_t<EltID>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EltID&>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->findDuplicateName((*reinterpret_cast<std::add_pointer_t<EltID>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: { QString _r = _t->getName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->on_checkLectureBoucle_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->on_pushCompress_clicked(); break;
        case 24: _t->onCompress((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EltID >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EltID >(); break;
            }
            break;
        }
    }
}

const QMetaObject *PageSmpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageSmpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageSmplE_t>.strings))
        return static_cast<void*>(this);
    return Page::qt_metacast(_clname);
}

int PageSmpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Page::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
