/********************************************************************************
** Form generated from reading UI file 'pageprst.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEPRST_H
#define UI_PAGEPRST_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "modulatoreditor.h"
#include "pageprst.h"

QT_BEGIN_NAMESPACE

class Ui_PagePrst
{
public:
    QVBoxLayout *verticalLayout;
    TableWidgetPrst *tablePrst;
    ModulatorEditor *modulatorEditor;

    void setupUi(QWidget *PagePrst)
    {
        if (PagePrst->objectName().isEmpty())
            PagePrst->setObjectName("PagePrst");
        PagePrst->resize(622, 428);
        PagePrst->setWindowTitle(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(PagePrst);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tablePrst = new TableWidgetPrst(PagePrst);
        tablePrst->setObjectName("tablePrst");
        tablePrst->setFrameShape(QFrame::NoFrame);
        tablePrst->setAutoScrollMargin(16);
        tablePrst->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::SelectedClicked);
        tablePrst->setSelectionMode(QAbstractItemView::ExtendedSelection);
        tablePrst->setTextElideMode(Qt::ElideMiddle);
        tablePrst->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        tablePrst->setCornerButtonEnabled(false);
        tablePrst->verticalHeader()->setMinimumSectionSize(10);

        verticalLayout->addWidget(tablePrst);

        modulatorEditor = new ModulatorEditor(PagePrst);
        modulatorEditor->setObjectName("modulatorEditor");

        verticalLayout->addWidget(modulatorEditor);


        retranslateUi(PagePrst);
        QObject::connect(tablePrst, SIGNAL(cellChanged(int,int)), PagePrst, SLOT(set(int,int)));
        QObject::connect(tablePrst, SIGNAL(itemSelectionChanged()), PagePrst, SLOT(selected()));

        QMetaObject::connectSlotsByName(PagePrst);
    } // setupUi

    void retranslateUi(QWidget *PagePrst)
    {
        (void)PagePrst;
    } // retranslateUi

};

namespace Ui {
    class PagePrst: public Ui_PagePrst {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEPRST_H
