/********************************************************************************
** Form generated from reading UI file 'toolrelease_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLRELEASE_GUI_H
#define UI_TOOLRELEASE_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolRelease_gui
{
public:
    QGridLayout *gridLayout;
    QDoubleSpinBox *doubleDuree36;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QLabel *label_2;
    QDoubleSpinBox *doubleDivision;
    QDoubleSpinBox *doubleDeTune;
    QLabel *label;
    QLabel *label_3;

    void setupUi(QWidget *ToolRelease_gui)
    {
        if (ToolRelease_gui->objectName().isEmpty())
            ToolRelease_gui->setObjectName("ToolRelease_gui");
        ToolRelease_gui->resize(324, 145);
        ToolRelease_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolRelease_gui);
        gridLayout->setObjectName("gridLayout");
        doubleDuree36 = new QDoubleSpinBox(ToolRelease_gui);
        doubleDuree36->setObjectName("doubleDuree36");
        doubleDuree36->setDecimals(3);
        doubleDuree36->setMinimum(0.001000000000000);
        doubleDuree36->setMaximum(10.000000000000000);
        doubleDuree36->setSingleStep(0.100000000000000);
        doubleDuree36->setValue(0.300000000000000);

        gridLayout->addWidget(doubleDuree36, 0, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolRelease_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolRelease_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 2);

        label_2 = new QLabel(ToolRelease_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        doubleDivision = new QDoubleSpinBox(ToolRelease_gui);
        doubleDivision->setObjectName("doubleDivision");
        doubleDivision->setDecimals(2);
        doubleDivision->setMinimum(1.000000000000000);
        doubleDivision->setMaximum(10.000000000000000);
        doubleDivision->setSingleStep(0.100000000000000);
        doubleDivision->setValue(2.000000000000000);

        gridLayout->addWidget(doubleDivision, 1, 1, 1, 1);

        doubleDeTune = new QDoubleSpinBox(ToolRelease_gui);
        doubleDeTune->setObjectName("doubleDeTune");
        doubleDeTune->setMinimum(-12.000000000000000);
        doubleDeTune->setMaximum(12.000000000000000);
        doubleDeTune->setSingleStep(0.500000000000000);

        gridLayout->addWidget(doubleDeTune, 2, 1, 1, 1);

        label = new QLabel(ToolRelease_gui);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("Duration of release for C2 (key 36)"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_3 = new QLabel(ToolRelease_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);


        retranslateUi(ToolRelease_gui);

        QMetaObject::connectSlotsByName(ToolRelease_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolRelease_gui)
    {
        pushOk->setText(QCoreApplication::translate("ToolRelease_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolRelease_gui", "&Cancel", nullptr));
        label_2->setText(QCoreApplication::translate("ToolRelease_gui", "Division to the next octave", nullptr));
        label_3->setText(QCoreApplication::translate("ToolRelease_gui", "Detuning induced (semi-tones)", nullptr));
        (void)ToolRelease_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolRelease_gui: public Ui_ToolRelease_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLRELEASE_GUI_H
