/********************************************************************************
** Form generated from reading UI file 'configpanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGPANEL_H
#define UI_CONFIGPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "configsectiongeneral.h"
#include "configsectioninterface.h"
#include "configsectionkeyboard.h"
#include "configsectionrepository.h"
#include "configsectionsound.h"
#include "configtoc.h"

QT_BEGIN_NAMESPACE

class Ui_ConfigPanel
{
public:
    QGridLayout *gridLayout_2;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *generalLabel;
    ConfigSectionGeneral *generalWidget;
    QSpacerItem *verticalSpacer_3;
    QLabel *interfaceLabel;
    ConfigSectionInterface *interfaceWidget;
    QSpacerItem *verticalSpacer_2;
    QLabel *soundLabel;
    ConfigSectionSound *soundWidget;
    QSpacerItem *verticalSpacer;
    QLabel *keyboardLabel;
    ConfigSectionKeyboard *keyboardWidget;
    QSpacerItem *verticalSpacer_6;
    QLabel *repositoryLabel;
    ConfigSectionRepository *repositoryWidget;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_4;
    ConfigToc *widgetToc;

    void setupUi(QWidget *ConfigPanel)
    {
        if (ConfigPanel->objectName().isEmpty())
            ConfigPanel->setObjectName("ConfigPanel");
        ConfigPanel->resize(445, 265);
        ConfigPanel->setWindowTitle(QString::fromUtf8(""));
        ConfigPanel->setAutoFillBackground(true);
        gridLayout_2 = new QGridLayout(ConfigPanel);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(ConfigPanel);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setLineWidth(0);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 411, 283));
        horizontalLayout = new QHBoxLayout(scrollAreaWidgetContents);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 9, -1, 9);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        generalLabel = new QLabel(scrollAreaWidgetContents);
        generalLabel->setObjectName("generalLabel");
        QFont font;
        font.setPointSize(13);
        font.setBold(true);
        generalLabel->setFont(font);

        verticalLayout->addWidget(generalLabel);

        generalWidget = new ConfigSectionGeneral(scrollAreaWidgetContents);
        generalWidget->setObjectName("generalWidget");

        verticalLayout->addWidget(generalWidget);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_3);

        interfaceLabel = new QLabel(scrollAreaWidgetContents);
        interfaceLabel->setObjectName("interfaceLabel");
        interfaceLabel->setFont(font);

        verticalLayout->addWidget(interfaceLabel);

        interfaceWidget = new ConfigSectionInterface(scrollAreaWidgetContents);
        interfaceWidget->setObjectName("interfaceWidget");

        verticalLayout->addWidget(interfaceWidget);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_2);

        soundLabel = new QLabel(scrollAreaWidgetContents);
        soundLabel->setObjectName("soundLabel");
        soundLabel->setFont(font);

        verticalLayout->addWidget(soundLabel);

        soundWidget = new ConfigSectionSound(scrollAreaWidgetContents);
        soundWidget->setObjectName("soundWidget");

        verticalLayout->addWidget(soundWidget);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer);

        keyboardLabel = new QLabel(scrollAreaWidgetContents);
        keyboardLabel->setObjectName("keyboardLabel");
        keyboardLabel->setFont(font);

        verticalLayout->addWidget(keyboardLabel);

        keyboardWidget = new ConfigSectionKeyboard(scrollAreaWidgetContents);
        keyboardWidget->setObjectName("keyboardWidget");

        verticalLayout->addWidget(keyboardWidget);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_6);

        repositoryLabel = new QLabel(scrollAreaWidgetContents);
        repositoryLabel->setObjectName("repositoryLabel");
        repositoryLabel->setFont(font);

        verticalLayout->addWidget(repositoryLabel);

        repositoryWidget = new ConfigSectionRepository(scrollAreaWidgetContents);
        repositoryWidget->setObjectName("repositoryWidget");

        verticalLayout->addWidget(repositoryWidget);


        horizontalLayout->addLayout(verticalLayout);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout_2->addWidget(scrollArea, 0, 1, 3, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_4, 2, 0, 1, 1);

        widgetToc = new ConfigToc(ConfigPanel);
        widgetToc->setObjectName("widgetToc");

        gridLayout_2->addWidget(widgetToc, 1, 0, 1, 1);


        retranslateUi(ConfigPanel);

        QMetaObject::connectSlotsByName(ConfigPanel);
    } // setupUi

    void retranslateUi(QWidget *ConfigPanel)
    {
        generalLabel->setText(QCoreApplication::translate("ConfigPanel", "General", nullptr));
        interfaceLabel->setText(QCoreApplication::translate("ConfigPanel", "Interface", nullptr));
        soundLabel->setText(QCoreApplication::translate("ConfigPanel", "Sound", nullptr));
        keyboardLabel->setText(QCoreApplication::translate("ConfigPanel", "Keyboard", nullptr));
        repositoryLabel->setText(QCoreApplication::translate("ConfigPanel", "Online repository", nullptr));
        (void)ConfigPanel;
    } // retranslateUi

};

namespace Ui {
    class ConfigPanel: public Ui_ConfigPanel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGPANEL_H
