/****************************************************************************
** Meta object code from reading C++ file 'pageenvelope.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "editor/pageenvelope.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageenvelope.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12PageEnvelopeE_t {};
} // unnamed namespace

template <> constexpr inline auto PageEnvelope::qt_create_metaobjectdata<qt_meta_tag_ZN12PageEnvelopeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageEnvelope",
        "on_pushVolume_clicked",
        "",
        "on_pushModulation_clicked",
        "populate",
        "on_doubleSpinDelay_editingFinished",
        "on_doubleSpinAttack_editingFinished",
        "on_doubleSpinHold_editingFinished",
        "on_doubleSpinDecay_editingFinished",
        "on_doubleSpinSustain_editingFinished",
        "on_doubleSpinRelease_editingFinished",
        "on_spinKeyHold_editingFinished",
        "on_spinKeyDecay_editingFinished",
        "on_pushAttack_clicked",
        "on_pushDelay_clicked",
        "on_pushHold_clicked",
        "on_pushKeyHold_clicked",
        "on_pushSustain_clicked",
        "on_pushRelease_clicked",
        "on_pushDecay_clicked",
        "on_pushKeyDecay_clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_pushVolume_clicked'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushModulation_clicked'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populate'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinDelay_editingFinished'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinAttack_editingFinished'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinHold_editingFinished'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinDecay_editingFinished'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinSustain_editingFinished'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_doubleSpinRelease_editingFinished'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_spinKeyHold_editingFinished'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_spinKeyDecay_editingFinished'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushAttack_clicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushDelay_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushHold_clicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushKeyHold_clicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushSustain_clicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushRelease_clicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushDecay_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushKeyDecay_clicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageEnvelope, qt_meta_tag_ZN12PageEnvelopeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageEnvelope::staticMetaObject = { {
    QMetaObject::SuperData::link<Page::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PageEnvelopeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PageEnvelopeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12PageEnvelopeE_t>.metaTypes,
    nullptr
} };

void PageEnvelope::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageEnvelope *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_pushVolume_clicked(); break;
        case 1: _t->on_pushModulation_clicked(); break;
        case 2: _t->populate(); break;
        case 3: _t->on_doubleSpinDelay_editingFinished(); break;
        case 4: _t->on_doubleSpinAttack_editingFinished(); break;
        case 5: _t->on_doubleSpinHold_editingFinished(); break;
        case 6: _t->on_doubleSpinDecay_editingFinished(); break;
        case 7: _t->on_doubleSpinSustain_editingFinished(); break;
        case 8: _t->on_doubleSpinRelease_editingFinished(); break;
        case 9: _t->on_spinKeyHold_editingFinished(); break;
        case 10: _t->on_spinKeyDecay_editingFinished(); break;
        case 11: _t->on_pushAttack_clicked(); break;
        case 12: _t->on_pushDelay_clicked(); break;
        case 13: _t->on_pushHold_clicked(); break;
        case 14: _t->on_pushKeyHold_clicked(); break;
        case 15: _t->on_pushSustain_clicked(); break;
        case 16: _t->on_pushRelease_clicked(); break;
        case 17: _t->on_pushDecay_clicked(); break;
        case 18: _t->on_pushKeyDecay_clicked(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *PageEnvelope::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageEnvelope::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PageEnvelopeE_t>.strings))
        return static_cast<void*>(this);
    return Page::qt_metacast(_clname);
}

int PageEnvelope::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Page::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
