/********************************************************************************
** Form generated from reading UI file 'dialognewelement.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGNEWELEMENT_H
#define UI_DIALOGNEWELEMENT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_DialogNewElement
{
public:
    QGridLayout *gridLayout;
    QLineEdit *lineEdit;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QPushButton *pushOk;
    QCheckBox *checkLink;

    void setupUi(QDialog *DialogNewElement)
    {
        if (DialogNewElement->objectName().isEmpty())
            DialogNewElement->setObjectName("DialogNewElement");
        DialogNewElement->setWindowModality(Qt::ApplicationModal);
        DialogNewElement->resize(184, 94);
        DialogNewElement->setWindowTitle(QString::fromUtf8("Dialog"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogNewElement->setWindowIcon(icon);
        gridLayout = new QGridLayout(DialogNewElement);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        lineEdit = new QLineEdit(DialogNewElement);
        lineEdit->setObjectName("lineEdit");

        gridLayout->addWidget(lineEdit, 0, 0, 1, 3);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 1, 1, 1);

        pushCancel = new QPushButton(DialogNewElement);
        pushCancel->setObjectName("pushCancel");

        gridLayout->addWidget(pushCancel, 2, 2, 1, 1);

        pushOk = new QPushButton(DialogNewElement);
        pushOk->setObjectName("pushOk");

        gridLayout->addWidget(pushOk, 2, 0, 1, 1);

        checkLink = new QCheckBox(DialogNewElement);
        checkLink->setObjectName("checkLink");
        checkLink->setText(QString::fromUtf8("Link selected elements"));

        gridLayout->addWidget(checkLink, 1, 0, 1, 3);

        QWidget::setTabOrder(lineEdit, checkLink);
        QWidget::setTabOrder(checkLink, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(DialogNewElement);

        QMetaObject::connectSlotsByName(DialogNewElement);
    } // setupUi

    void retranslateUi(QDialog *DialogNewElement)
    {
        pushCancel->setText(QCoreApplication::translate("DialogNewElement", "&Cancel", nullptr));
        pushOk->setText(QCoreApplication::translate("DialogNewElement", "&Ok", nullptr));
        (void)DialogNewElement;
    } // retranslateUi

};

namespace Ui {
    class DialogNewElement: public Ui_DialogNewElement {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGNEWELEMENT_H
