/********************************************************************************
** Form generated from reading UI file 'toolcelestetuning_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLCELESTETUNING_GUI_H
#define UI_TOOLCELESTETUNING_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolCelesteTuning_gui
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QLabel *label_2;
    QDoubleSpinBox *doubleSpinDiv;
    QDoubleSpinBox *doubleSpinHerz;
    QLabel *label_4;

    void setupUi(QWidget *ToolCelesteTuning_gui)
    {
        if (ToolCelesteTuning_gui->objectName().isEmpty())
            ToolCelesteTuning_gui->setObjectName("ToolCelesteTuning_gui");
        ToolCelesteTuning_gui->resize(345, 118);
        ToolCelesteTuning_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolCelesteTuning_gui);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ToolCelesteTuning_gui);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("Nombre de battements par secondes au do 4 (note 60)"));

        gridLayout->addWidget(label, 0, 0, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolCelesteTuning_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolCelesteTuning_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 3);

        label_2 = new QLabel(ToolCelesteTuning_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 2);

        doubleSpinDiv = new QDoubleSpinBox(ToolCelesteTuning_gui);
        doubleSpinDiv->setObjectName("doubleSpinDiv");
        doubleSpinDiv->setMinimum(0.500000000000000);
        doubleSpinDiv->setMaximum(2.000000000000000);
        doubleSpinDiv->setSingleStep(0.100000000000000);
        doubleSpinDiv->setValue(1.000000000000000);

        gridLayout->addWidget(doubleSpinDiv, 2, 2, 1, 1);

        doubleSpinHerz = new QDoubleSpinBox(ToolCelesteTuning_gui);
        doubleSpinHerz->setObjectName("doubleSpinHerz");
        doubleSpinHerz->setDecimals(1);
        doubleSpinHerz->setMinimum(-10.000000000000000);
        doubleSpinHerz->setMaximum(10.000000000000000);
        doubleSpinHerz->setValue(4.000000000000000);

        gridLayout->addWidget(doubleSpinHerz, 0, 2, 1, 1);

        label_4 = new QLabel(ToolCelesteTuning_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 2, 0, 1, 2);

        QWidget::setTabOrder(doubleSpinHerz, doubleSpinDiv);
        QWidget::setTabOrder(doubleSpinDiv, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolCelesteTuning_gui);

        QMetaObject::connectSlotsByName(ToolCelesteTuning_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolCelesteTuning_gui)
    {
        pushOk->setText(QCoreApplication::translate("ToolCelesteTuning_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolCelesteTuning_gui", "&Cancel", nullptr));
        label_2->setText(QCoreApplication::translate("ToolCelesteTuning_gui", "The algebraic sign determines the tuning direction.", nullptr));
        label_4->setText(QCoreApplication::translate("ToolCelesteTuning_gui", "Division to the next octave", nullptr));
        (void)ToolCelesteTuning_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolCelesteTuning_gui: public Ui_ToolCelesteTuning_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLCELESTETUNING_GUI_H
