/********************************************************************************
** Form generated from reading UI file 'toolsoundspatialization_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLSOUNDSPATIALIZATION_GUI_H
#define UI_TOOLSOUNDSPATIALIZATION_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "sound_spatialization/graphspace.h"

QT_BEGIN_NAMESPACE

class Ui_ToolSoundSpatialization_gui
{
public:
    QGridLayout *gridLayout;
    QComboBox *comboPattern;
    QSpinBox *spinSpreading;
    QLabel *label_3;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushOk;
    QPushButton *pushCancel;
    QLabel *label;
    QSpinBox *spinNbDivision;
    QLabel *label_5;
    QSpinBox *spinFilling;
    QLabel *label_6;
    QCheckBox *checkFlop;
    QSpinBox *spinOffset;
    QCheckBox *checkFlip;
    QLabel *label_4;
    QFrame *frame_3;
    QVBoxLayout *verticalLayout_4;
    GraphSpace *grapheSpace;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ToolSoundSpatialization_gui)
    {
        if (ToolSoundSpatialization_gui->objectName().isEmpty())
            ToolSoundSpatialization_gui->setObjectName("ToolSoundSpatialization_gui");
        ToolSoundSpatialization_gui->resize(500, 220);
        ToolSoundSpatialization_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolSoundSpatialization_gui);
        gridLayout->setObjectName("gridLayout");
        comboPattern = new QComboBox(ToolSoundSpatialization_gui);
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->setObjectName("comboPattern");

        gridLayout->addWidget(comboPattern, 0, 2, 1, 2);

        spinSpreading = new QSpinBox(ToolSoundSpatialization_gui);
        spinSpreading->setObjectName("spinSpreading");
        spinSpreading->setMaximum(100);
        spinSpreading->setValue(100);

        gridLayout->addWidget(spinSpreading, 3, 2, 1, 2);

        label_3 = new QLabel(ToolSoundSpatialization_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 3, 1, 1, 1);

        label_2 = new QLabel(ToolSoundSpatialization_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushOk = new QPushButton(ToolSoundSpatialization_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        pushCancel = new QPushButton(ToolSoundSpatialization_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 9, 1, 1, 3);

        label = new QLabel(ToolSoundSpatialization_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 1, 1, 1);

        spinNbDivision = new QSpinBox(ToolSoundSpatialization_gui);
        spinNbDivision->setObjectName("spinNbDivision");
        spinNbDivision->setMinimum(1);
        spinNbDivision->setMaximum(10);

        gridLayout->addWidget(spinNbDivision, 1, 2, 1, 2);

        label_5 = new QLabel(ToolSoundSpatialization_gui);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 6, 1, 1, 1);

        spinFilling = new QSpinBox(ToolSoundSpatialization_gui);
        spinFilling->setObjectName("spinFilling");
        spinFilling->setMaximum(100);
        spinFilling->setValue(100);

        gridLayout->addWidget(spinFilling, 2, 2, 1, 2);

        label_6 = new QLabel(ToolSoundSpatialization_gui);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 7, 1, 1, 1);

        checkFlop = new QCheckBox(ToolSoundSpatialization_gui);
        checkFlop->setObjectName("checkFlop");
        checkFlop->setMaximumSize(QSize(40, 16777215));

        gridLayout->addWidget(checkFlop, 7, 3, 1, 1);

        spinOffset = new QSpinBox(ToolSoundSpatialization_gui);
        spinOffset->setObjectName("spinOffset");
        spinOffset->setMaximum(100);
        spinOffset->setValue(50);

        gridLayout->addWidget(spinOffset, 6, 2, 1, 2);

        checkFlip = new QCheckBox(ToolSoundSpatialization_gui);
        checkFlip->setObjectName("checkFlip");
        checkFlip->setMaximumSize(QSize(40, 16777215));

        gridLayout->addWidget(checkFlip, 7, 2, 1, 1);

        label_4 = new QLabel(ToolSoundSpatialization_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 2, 1, 1, 1);

        frame_3 = new QFrame(ToolSoundSpatialization_gui);
        frame_3->setObjectName("frame_3");
        frame_3->setFrameShape(QFrame::Panel);
        frame_3->setFrameShadow(QFrame::Sunken);
        verticalLayout_4 = new QVBoxLayout(frame_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        grapheSpace = new GraphSpace(frame_3);
        grapheSpace->setObjectName("grapheSpace");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(grapheSpace->sizePolicy().hasHeightForWidth());
        grapheSpace->setSizePolicy(sizePolicy);
        grapheSpace->setMinimumSize(QSize(300, 200));

        verticalLayout_4->addWidget(grapheSpace);


        gridLayout->addWidget(frame_3, 0, 0, 10, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 8, 1, 1, 1);

        QWidget::setTabOrder(comboPattern, spinNbDivision);
        QWidget::setTabOrder(spinNbDivision, spinFilling);
        QWidget::setTabOrder(spinFilling, spinSpreading);
        QWidget::setTabOrder(spinSpreading, spinOffset);
        QWidget::setTabOrder(spinOffset, checkFlip);
        QWidget::setTabOrder(checkFlip, checkFlop);
        QWidget::setTabOrder(checkFlop, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolSoundSpatialization_gui);

        QMetaObject::connectSlotsByName(ToolSoundSpatialization_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolSoundSpatialization_gui)
    {
        comboPattern->setItemText(0, QCoreApplication::translate("ToolSoundSpatialization_gui", "Ascending", nullptr));
        comboPattern->setItemText(1, QCoreApplication::translate("ToolSoundSpatialization_gui", "Descending", nullptr));
        comboPattern->setItemText(2, QCoreApplication::translate("ToolSoundSpatialization_gui", "Hollow", nullptr));
        comboPattern->setItemText(3, QCoreApplication::translate("ToolSoundSpatialization_gui", "Spike", nullptr));
        comboPattern->setItemText(4, QCoreApplication::translate("ToolSoundSpatialization_gui", "Random", nullptr));

        label_3->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Spreading (%)", nullptr));
        label_2->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Division number", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "&Cancel", nullptr));
        label->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Pattern", nullptr));
        label_5->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Offset (0-100)", nullptr));
        label_6->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Inversions", nullptr));
        checkFlop->setText(QString());
        checkFlip->setText(QString());
        label_4->setText(QCoreApplication::translate("ToolSoundSpatialization_gui", "Filling (%)", nullptr));
        (void)ToolSoundSpatialization_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolSoundSpatialization_gui: public Ui_ToolSoundSpatialization_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLSOUNDSPATIALIZATION_GUI_H
