/********************************************************************************
** Form generated from reading UI file 'soundfontbrowser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTBROWSER_H
#define UI_SOUNDFONTBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "browsersortmenu.h"
#include "filterflow.h"
#include "filtertag.h"
#include "rotatingspinner.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontBrowser
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidget;
    QWidget *pageWait;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *horizontalSpacer_2;
    RotatingSpinner *spinner;
    QSpacerItem *verticalSpacer_3;
    QWidget *widgetColored2;
    QWidget *pageSoundfonts;
    QGridLayout *gridLayout_2;
    QLabel *labelNoResult;
    QFrame *frameSearch;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *lineSearch;
    BrowserSortMenu *widgetSortMenu;
    QFrame *frameTitle;
    QHBoxLayout *horizontalLayout;
    QLabel *labelFilters;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushResetFilters;
    QPushButton *pushBecomePremium;
    QListWidget *listWidget;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    FilterFlow *filterCategory;
    QLabel *label_5;
    FilterFlow *filterLicense;
    QLabel *label_6;
    FilterFlow *filterSampleSource;
    QLabel *label_7;
    FilterFlow *filterTimbre;
    QLabel *label_8;
    FilterFlow *filterArticulation;
    QLabel *label_9;
    FilterFlow *filterGenre;
    QLabel *label_10;
    FilterFlow *filterMidiStandard;
    QLabel *label_11;
    FilterTag *filterTag;
    QSpacerItem *verticalSpacer;
    QFrame *framePagination;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *pushGoPrevious;
    QLabel *labelPagination;
    QPushButton *pushGoNext;
    QWidget *pageFailed;
    QGridLayout *gridLayout_4;
    QPushButton *pushRetry;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *horizontalSpacer_5;
    QSpacerItem *horizontalSpacer_4;
    QLabel *label_4;
    QSpacerItem *verticalSpacer_4;
    QWidget *widgetColored;

    void setupUi(QWidget *SoundfontBrowser)
    {
        if (SoundfontBrowser->objectName().isEmpty())
            SoundfontBrowser->setObjectName("SoundfontBrowser");
        SoundfontBrowser->resize(811, 389);
        SoundfontBrowser->setWindowTitle(QString::fromUtf8("Form"));
        SoundfontBrowser->setAutoFillBackground(true);
        gridLayout = new QGridLayout(SoundfontBrowser);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(SoundfontBrowser);
        stackedWidget->setObjectName("stackedWidget");
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_3 = new QGridLayout(pageWait);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 1, 1, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_3, 2, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 2, 0, 1, 1);

        spinner = new RotatingSpinner(pageWait);
        spinner->setObjectName("spinner");
        spinner->setMinimumSize(QSize(32, 32));
        spinner->setMaximumSize(QSize(32, 32));

        gridLayout_3->addWidget(spinner, 2, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 3, 1, 1, 1);

        widgetColored2 = new QWidget(pageWait);
        widgetColored2->setObjectName("widgetColored2");
        widgetColored2->setMinimumSize(QSize(0, 6));

        gridLayout_3->addWidget(widgetColored2, 0, 0, 1, 3);

        stackedWidget->addWidget(pageWait);
        pageSoundfonts = new QWidget();
        pageSoundfonts->setObjectName("pageSoundfonts");
        gridLayout_2 = new QGridLayout(pageSoundfonts);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        labelNoResult = new QLabel(pageSoundfonts);
        labelNoResult->setObjectName("labelNoResult");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelNoResult->sizePolicy().hasHeightForWidth());
        labelNoResult->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        labelNoResult->setFont(font);
        labelNoResult->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_2->addWidget(labelNoResult, 2, 1, 1, 1);

        frameSearch = new QFrame(pageSoundfonts);
        frameSearch->setObjectName("frameSearch");
        frameSearch->setMinimumSize(QSize(0, 42));
        frameSearch->setMaximumSize(QSize(16777215, 42));
        frameSearch->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout_2 = new QHBoxLayout(frameSearch);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 5, 6, 6);
        lineSearch = new QLineEdit(frameSearch);
        lineSearch->setObjectName("lineSearch");
        sizePolicy.setHeightForWidth(lineSearch->sizePolicy().hasHeightForWidth());
        lineSearch->setSizePolicy(sizePolicy);
        lineSearch->setClearButtonEnabled(true);

        horizontalLayout_2->addWidget(lineSearch);

        widgetSortMenu = new BrowserSortMenu(frameSearch);
        widgetSortMenu->setObjectName("widgetSortMenu");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(widgetSortMenu->sizePolicy().hasHeightForWidth());
        widgetSortMenu->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(widgetSortMenu);


        gridLayout_2->addWidget(frameSearch, 0, 1, 1, 1);

        frameTitle = new QFrame(pageSoundfonts);
        frameTitle->setObjectName("frameTitle");
        frameTitle->setMinimumSize(QSize(0, 42));
        frameTitle->setMaximumSize(QSize(16777215, 42));
        frameTitle->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout = new QHBoxLayout(frameTitle);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(6, 5, -1, 6);
        labelFilters = new QLabel(frameTitle);
        labelFilters->setObjectName("labelFilters");
        labelFilters->setFont(font);

        horizontalLayout->addWidget(labelFilters);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushResetFilters = new QPushButton(frameTitle);
        pushResetFilters->setObjectName("pushResetFilters");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(pushResetFilters->sizePolicy().hasHeightForWidth());
        pushResetFilters->setSizePolicy(sizePolicy2);
        pushResetFilters->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout->addWidget(pushResetFilters);


        gridLayout_2->addWidget(frameTitle, 0, 0, 1, 1);

        pushBecomePremium = new QPushButton(pageSoundfonts);
        pushBecomePremium->setObjectName("pushBecomePremium");
        pushBecomePremium->setFont(font);
        pushBecomePremium->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushBecomePremium->setFlat(true);

        gridLayout_2->addWidget(pushBecomePremium, 1, 1, 1, 1);

        listWidget = new QListWidget(pageSoundfonts);
        listWidget->setObjectName("listWidget");
        listWidget->setFrameShape(QFrame::Shape::NoFrame);
        listWidget->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOn);
        listWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        listWidget->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        listWidget->setProperty("showDropIndicator", QVariant(false));
        listWidget->setAlternatingRowColors(true);
        listWidget->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        listWidget->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);

        gridLayout_2->addWidget(listWidget, 4, 1, 1, 1);

        scrollArea = new QScrollArea(pageSoundfonts);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::Shape::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 250, 347));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(scrollAreaWidgetContents);
        label->setObjectName("label");
        label->setFont(font);

        verticalLayout->addWidget(label);

        filterCategory = new FilterFlow(scrollAreaWidgetContents);
        filterCategory->setObjectName("filterCategory");

        verticalLayout->addWidget(filterCategory);

        label_5 = new QLabel(scrollAreaWidgetContents);
        label_5->setObjectName("label_5");
        label_5->setFont(font);

        verticalLayout->addWidget(label_5);

        filterLicense = new FilterFlow(scrollAreaWidgetContents);
        filterLicense->setObjectName("filterLicense");

        verticalLayout->addWidget(filterLicense);

        label_6 = new QLabel(scrollAreaWidgetContents);
        label_6->setObjectName("label_6");
        label_6->setFont(font);

        verticalLayout->addWidget(label_6);

        filterSampleSource = new FilterFlow(scrollAreaWidgetContents);
        filterSampleSource->setObjectName("filterSampleSource");

        verticalLayout->addWidget(filterSampleSource);

        label_7 = new QLabel(scrollAreaWidgetContents);
        label_7->setObjectName("label_7");
        label_7->setFont(font);

        verticalLayout->addWidget(label_7);

        filterTimbre = new FilterFlow(scrollAreaWidgetContents);
        filterTimbre->setObjectName("filterTimbre");

        verticalLayout->addWidget(filterTimbre);

        label_8 = new QLabel(scrollAreaWidgetContents);
        label_8->setObjectName("label_8");
        label_8->setFont(font);

        verticalLayout->addWidget(label_8);

        filterArticulation = new FilterFlow(scrollAreaWidgetContents);
        filterArticulation->setObjectName("filterArticulation");

        verticalLayout->addWidget(filterArticulation);

        label_9 = new QLabel(scrollAreaWidgetContents);
        label_9->setObjectName("label_9");
        label_9->setFont(font);

        verticalLayout->addWidget(label_9);

        filterGenre = new FilterFlow(scrollAreaWidgetContents);
        filterGenre->setObjectName("filterGenre");

        verticalLayout->addWidget(filterGenre);

        label_10 = new QLabel(scrollAreaWidgetContents);
        label_10->setObjectName("label_10");
        label_10->setFont(font);

        verticalLayout->addWidget(label_10);

        filterMidiStandard = new FilterFlow(scrollAreaWidgetContents);
        filterMidiStandard->setObjectName("filterMidiStandard");

        verticalLayout->addWidget(filterMidiStandard);

        label_11 = new QLabel(scrollAreaWidgetContents);
        label_11->setObjectName("label_11");
        label_11->setFont(font);

        verticalLayout->addWidget(label_11);

        filterTag = new FilterTag(scrollAreaWidgetContents);
        filterTag->setObjectName("filterTag");

        verticalLayout->addWidget(filterTag);

        verticalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout_2->addWidget(scrollArea, 1, 0, 4, 1);

        framePagination = new QFrame(pageSoundfonts);
        framePagination->setObjectName("framePagination");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(framePagination->sizePolicy().hasHeightForWidth());
        framePagination->setSizePolicy(sizePolicy3);
        framePagination->setMinimumSize(QSize(150, 50));
        framePagination->setMaximumSize(QSize(150, 50));
        framePagination->setFrameShape(QFrame::Shape::StyledPanel);
        framePagination->setFrameShadow(QFrame::Shadow::Raised);
        horizontalLayout_3 = new QHBoxLayout(framePagination);
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(3, 3, 3, 3);
        pushGoPrevious = new QPushButton(framePagination);
        pushGoPrevious->setObjectName("pushGoPrevious");
        pushGoPrevious->setMaximumSize(QSize(40, 16777215));
        pushGoPrevious->setIconSize(QSize(30, 30));
        pushGoPrevious->setFlat(true);

        horizontalLayout_3->addWidget(pushGoPrevious);

        labelPagination = new QLabel(framePagination);
        labelPagination->setObjectName("labelPagination");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(labelPagination->sizePolicy().hasHeightForWidth());
        labelPagination->setSizePolicy(sizePolicy4);
        labelPagination->setAlignment(Qt::AlignmentFlag::AlignCenter);

        horizontalLayout_3->addWidget(labelPagination);

        pushGoNext = new QPushButton(framePagination);
        pushGoNext->setObjectName("pushGoNext");
        pushGoNext->setMaximumSize(QSize(40, 16777215));
        pushGoNext->setIconSize(QSize(30, 30));
        pushGoNext->setFlat(true);

        horizontalLayout_3->addWidget(pushGoNext);


        gridLayout_2->addWidget(framePagination, 3, 1, 1, 1);

        gridLayout_2->setColumnStretch(0, 3);
        gridLayout_2->setColumnStretch(1, 10);
        gridLayout_2->setColumnMinimumWidth(0, 250);
        stackedWidget->addWidget(pageSoundfonts);
        pageFailed = new QWidget();
        pageFailed->setObjectName("pageFailed");
        gridLayout_4 = new QGridLayout(pageFailed);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        pushRetry = new QPushButton(pageFailed);
        pushRetry->setObjectName("pushRetry");
        pushRetry->setIconSize(QSize(16, 16));
        pushRetry->setFlat(true);

        gridLayout_4->addWidget(pushRetry, 3, 1, 1, 1);

        verticalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_5, 4, 1, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_5, 3, 2, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_4, 3, 0, 1, 1);

        label_4 = new QLabel(pageFailed);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_4->addWidget(label_4, 2, 0, 1, 3);

        verticalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_4, 1, 1, 1, 1);

        widgetColored = new QWidget(pageFailed);
        widgetColored->setObjectName("widgetColored");
        widgetColored->setMinimumSize(QSize(0, 6));

        gridLayout_4->addWidget(widgetColored, 0, 0, 1, 3);

        stackedWidget->addWidget(pageFailed);

        gridLayout->addWidget(stackedWidget, 0, 0, 1, 1);

        gridLayout->setColumnStretch(0, 2);

        retranslateUi(SoundfontBrowser);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(SoundfontBrowser);
    } // setupUi

    void retranslateUi(QWidget *SoundfontBrowser)
    {
        labelNoResult->setText(QCoreApplication::translate("SoundfontBrowser", "No results", nullptr));
        lineSearch->setText(QString());
        lineSearch->setPlaceholderText(QCoreApplication::translate("SoundfontBrowser", "Search a soundfont...", nullptr));
        labelFilters->setText(QCoreApplication::translate("SoundfontBrowser", "Filters", nullptr));
        pushResetFilters->setText(QCoreApplication::translate("SoundfontBrowser", "Reset", nullptr));
        pushBecomePremium->setText(QCoreApplication::translate("SoundfontBrowser", "Subscribe to a Premium account to get all the features!", nullptr));
        label->setText(QCoreApplication::translate("SoundfontBrowser", "Category", nullptr));
        label_5->setText(QCoreApplication::translate("SoundfontBrowser", "License", nullptr));
        label_6->setText(QCoreApplication::translate("SoundfontBrowser", "Sample source", nullptr));
        label_7->setText(QCoreApplication::translate("SoundfontBrowser", "Timbre", nullptr));
        label_8->setText(QCoreApplication::translate("SoundfontBrowser", "Articulation", nullptr));
        label_9->setText(QCoreApplication::translate("SoundfontBrowser", "Genre", nullptr));
        label_10->setText(QCoreApplication::translate("SoundfontBrowser", "MIDI standard", nullptr));
        label_11->setText(QCoreApplication::translate("SoundfontBrowser", "Tags", nullptr));
#if QT_CONFIG(tooltip)
        pushGoPrevious->setToolTip(QCoreApplication::translate("SoundfontBrowser", "Go to the previous page", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushGoNext->setToolTip(QCoreApplication::translate("SoundfontBrowser", "Go to the next page", nullptr));
#endif // QT_CONFIG(tooltip)
        pushRetry->setText(QString());
        label_4->setText(QCoreApplication::translate("SoundfontBrowser", "Oops! Cannot\n"
"download the list", nullptr));
        (void)SoundfontBrowser;
    } // retranslateUi

};

namespace Ui {
    class SoundfontBrowser: public Ui_SoundfontBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTBROWSER_H
