/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2024 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone.io                             **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "toolsoundfontexport_parameters.h"
#include "contextmanager.h"

void ToolSoundfontExport_parameters::loadConfiguration()
{
    _directory = ContextManager::recentFile()->getLastDirectory(RecentFileManager::FILE_TYPE_EXPORT);
    _format = getToolValue("type", 0).toInt();
    _quality = getToolValue("quality", 1).toInt();
    _presetPrefix = getToolValue("preset_prefix", true).toBool();
    _bankDirectory = getToolValue("bank_directory", false).toBool();
    _gmSort = getToolValue("gm_sort", false).toBool();
    _filePreset = getToolValue("file_preset", false).toBool();
    _csvRaw = getToolValue("csv_raw", false).toBool();
}

void ToolSoundfontExport_parameters::saveConfiguration()
{
    ContextManager::recentFile()->addRecentFile(RecentFileManager::FILE_TYPE_EXPORT, _directory + "/soundfont.sfz");
    setToolValue("type", _format);
    setToolValue("quality", _quality);
    setToolValue("preset_prefix", _presetPrefix);
    setToolValue("bank_directory", _bankDirectory);
    setToolValue("gm_sort", _gmSort);
    setToolValue("file_preset", _filePreset);
    setToolValue("csv_raw", _csvRaw);
}
