/********************************************************************************
** Form generated from reading UI file 'showsoundfonts.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHOWSOUNDFONTS_H
#define UI_SHOWSOUNDFONTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QWidget>
#include "rotatingspinner.h"

QT_BEGIN_NAMESPACE

class Ui_ShowSoundfonts
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidget;
    QWidget *pageWait;
    QGridLayout *gridLayout_3;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer;
    RotatingSpinner *spinner;
    QSpacerItem *verticalSpacer_2;
    QWidget *pageSoundfonts;
    QGridLayout *gridLayout_2;
    QListWidget *listWidget;
    QWidget *pageFailed;
    QGridLayout *gridLayout_4;
    QSpacerItem *verticalSpacer_3;
    QPushButton *pushRetry;
    QSpacerItem *verticalSpacer_4;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label;

    void setupUi(QWidget *ShowSoundfonts)
    {
        if (ShowSoundfonts->objectName().isEmpty())
            ShowSoundfonts->setObjectName("ShowSoundfonts");
        ShowSoundfonts->resize(400, 300);
        ShowSoundfonts->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ShowSoundfonts);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(ShowSoundfonts);
        stackedWidget->setObjectName("stackedWidget");
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_3 = new QGridLayout(pageWait);
        gridLayout_3->setObjectName("gridLayout_3");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer, 1, 0, 1, 1);

        spinner = new RotatingSpinner(pageWait);
        spinner->setObjectName("spinner");
        spinner->setMinimumSize(QSize(32, 32));

        gridLayout_3->addWidget(spinner, 1, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 2, 1, 1, 1);

        stackedWidget->addWidget(pageWait);
        pageSoundfonts = new QWidget();
        pageSoundfonts->setObjectName("pageSoundfonts");
        gridLayout_2 = new QGridLayout(pageSoundfonts);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        listWidget = new QListWidget(pageSoundfonts);
        listWidget->setObjectName("listWidget");
        listWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        listWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listWidget->setAlternatingRowColors(true);
        listWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
        listWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);

        gridLayout_2->addWidget(listWidget, 0, 0, 1, 1);

        stackedWidget->addWidget(pageSoundfonts);
        pageFailed = new QWidget();
        pageFailed->setObjectName("pageFailed");
        gridLayout_4 = new QGridLayout(pageFailed);
        gridLayout_4->setObjectName("gridLayout_4");
        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_3, 3, 1, 1, 1);

        pushRetry = new QPushButton(pageFailed);
        pushRetry->setObjectName("pushRetry");
        pushRetry->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushRetry->setIconSize(QSize(16, 16));
        pushRetry->setFlat(true);

        gridLayout_4->addWidget(pushRetry, 2, 1, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_4->addItem(verticalSpacer_4, 0, 1, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_4, 2, 2, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_3, 2, 0, 1, 1);

        label = new QLabel(pageFailed);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(label, 1, 0, 1, 3);

        stackedWidget->addWidget(pageFailed);

        gridLayout->addWidget(stackedWidget, 0, 0, 1, 1);


        retranslateUi(ShowSoundfonts);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(ShowSoundfonts);
    } // setupUi

    void retranslateUi(QWidget *ShowSoundfonts)
    {
        pushRetry->setText(QString());
        label->setText(QCoreApplication::translate("ShowSoundfonts", "Oops! Cannot\n"
"download the list", nullptr));
        (void)ShowSoundfonts;
    } // retranslateUi

};

namespace Ui {
    class ShowSoundfonts: public Ui_ShowSoundfonts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHOWSOUNDFONTS_H
