/********************************************************************************
** Form generated from reading UI file 'toolloadfrominst_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLLOADFROMINST_GUI_H
#define UI_TOOLLOADFROMINST_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolLoadFromInst_gui
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *radioRootKey;
    QRadioButton *radioTuningSemiTones;
    QRadioButton *radioTuningCents;
    QRadioButton *radioLoop;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolLoadFromInst_gui)
    {
        if (ToolLoadFromInst_gui->objectName().isEmpty())
            ToolLoadFromInst_gui->setObjectName("ToolLoadFromInst_gui");
        ToolLoadFromInst_gui->resize(186, 149);
        ToolLoadFromInst_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolLoadFromInst_gui);
        verticalLayout->setObjectName("verticalLayout");
        radioRootKey = new QRadioButton(ToolLoadFromInst_gui);
        radioRootKey->setObjectName("radioRootKey");

        verticalLayout->addWidget(radioRootKey);

        radioTuningSemiTones = new QRadioButton(ToolLoadFromInst_gui);
        radioTuningSemiTones->setObjectName("radioTuningSemiTones");

        verticalLayout->addWidget(radioTuningSemiTones);

        radioTuningCents = new QRadioButton(ToolLoadFromInst_gui);
        radioTuningCents->setObjectName("radioTuningCents");

        verticalLayout->addWidget(radioTuningCents);

        radioLoop = new QRadioButton(ToolLoadFromInst_gui);
        radioLoop->setObjectName("radioLoop");

        verticalLayout->addWidget(radioLoop);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolLoadFromInst_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolLoadFromInst_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(radioRootKey, radioTuningSemiTones);
        QWidget::setTabOrder(radioTuningSemiTones, radioTuningCents);
        QWidget::setTabOrder(radioTuningCents, radioLoop);
        QWidget::setTabOrder(radioLoop, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolLoadFromInst_gui);

        QMetaObject::connectSlotsByName(ToolLoadFromInst_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolLoadFromInst_gui)
    {
        radioRootKey->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "Root key", nullptr));
        radioTuningSemiTones->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "Tuning (semi-tones)", nullptr));
        radioTuningCents->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "Tuning (cents)", nullptr));
        radioLoop->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "Loop start / loop end", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolLoadFromInst_gui", "&Cancel", nullptr));
        (void)ToolLoadFromInst_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolLoadFromInst_gui: public Ui_ToolLoadFromInst_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLLOADFROMINST_GUI_H
