/********************************************************************************
** Form generated from reading UI file 'footerlinkedto.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOOTERLINKEDTO_H
#define UI_FOOTERLINKEDTO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "linkedtowidget.h"
#include "pushstereoediting.h"

QT_BEGIN_NAMESPACE

class Ui_FooterLinkedTo
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *labelLinkedTo;
    LinkedToWidget *widgetLinkedTo;
    PushStereoEditing *pushStereoEditing;

    void setupUi(QWidget *FooterLinkedTo)
    {
        if (FooterLinkedTo->objectName().isEmpty())
            FooterLinkedTo->setObjectName("FooterLinkedTo");
        FooterLinkedTo->resize(347, 36);
        FooterLinkedTo->setWindowTitle(QString::fromUtf8(""));
        horizontalLayout = new QHBoxLayout(FooterLinkedTo);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, 0, 0);
        labelLinkedTo = new QLabel(FooterLinkedTo);
        labelLinkedTo->setObjectName("labelLinkedTo");
        QFont font;
        font.setBold(true);
        labelLinkedTo->setFont(font);
        labelLinkedTo->setText(QString::fromUtf8("Element linked to:"));
        labelLinkedTo->setTextInteractionFlags(Qt::NoTextInteraction);

        horizontalLayout->addWidget(labelLinkedTo);

        widgetLinkedTo = new LinkedToWidget(FooterLinkedTo);
        widgetLinkedTo->setObjectName("widgetLinkedTo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(widgetLinkedTo->sizePolicy().hasHeightForWidth());
        widgetLinkedTo->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(widgetLinkedTo);

        pushStereoEditing = new PushStereoEditing(FooterLinkedTo);
        pushStereoEditing->setObjectName("pushStereoEditing");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushStereoEditing->sizePolicy().hasHeightForWidth());
        pushStereoEditing->setSizePolicy(sizePolicy1);
        pushStereoEditing->setMinimumSize(QSize(30, 0));
        pushStereoEditing->setMaximumSize(QSize(30, 16777215));
        pushStereoEditing->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushStereoEditing->setIconSize(QSize(18, 18));
        pushStereoEditing->setFlat(true);

        horizontalLayout->addWidget(pushStereoEditing);


        retranslateUi(FooterLinkedTo);

        QMetaObject::connectSlotsByName(FooterLinkedTo);
    } // setupUi

    void retranslateUi(QWidget *FooterLinkedTo)
    {
        (void)FooterLinkedTo;
    } // retranslateUi

};

namespace Ui {
    class FooterLinkedTo: public Ui_FooterLinkedTo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FOOTERLINKEDTO_H
