/********************************************************************************
** Form generated from reading UI file 'editordialoginsertvideo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITORDIALOGINSERTVIDEO_H
#define UI_EDITORDIALOGINSERTVIDEO_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_EditorDialogInsertVideo
{
public:
    QGridLayout *gridLayout;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer;
    QLineEdit *lineUrl;
    QLabel *labelWarning;

    void setupUi(QDialog *EditorDialogInsertVideo)
    {
        if (EditorDialogInsertVideo->objectName().isEmpty())
            EditorDialogInsertVideo->setObjectName("EditorDialogInsertVideo");
        EditorDialogInsertVideo->resize(513, 121);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        EditorDialogInsertVideo->setWindowIcon(icon);
        gridLayout = new QGridLayout(EditorDialogInsertVideo);
        gridLayout->setObjectName("gridLayout");
        buttonBox = new QDialogButtonBox(EditorDialogInsertVideo);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 3, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        lineUrl = new QLineEdit(EditorDialogInsertVideo);
        lineUrl->setObjectName("lineUrl");
        lineUrl->setPlaceholderText(QString::fromUtf8("YouTube URL..."));

        gridLayout->addWidget(lineUrl, 0, 0, 1, 1);

        labelWarning = new QLabel(EditorDialogInsertVideo);
        labelWarning->setObjectName("labelWarning");
        QFont font;
        font.setBold(true);
        labelWarning->setFont(font);
        labelWarning->setText(QString::fromUtf8("Invalid URL (example: https://www.youtube.com/watch?v=YOpa5Ec3i4s)"));

        gridLayout->addWidget(labelWarning, 1, 0, 1, 1);


        retranslateUi(EditorDialogInsertVideo);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, EditorDialogInsertVideo, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, EditorDialogInsertVideo, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(EditorDialogInsertVideo);
    } // setupUi

    void retranslateUi(QDialog *EditorDialogInsertVideo)
    {
        EditorDialogInsertVideo->setWindowTitle(QCoreApplication::translate("EditorDialogInsertVideo", "Insert a YouTube video", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditorDialogInsertVideo: public Ui_EditorDialogInsertVideo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITORDIALOGINSERTVIDEO_H
