/********************************************************************************
** Form generated from reading UI file 'filtertag.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERTAG_H
#define UI_FILTERTAG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FilterTag
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *lineEdit;
    QWidget *tagArea;

    void setupUi(QWidget *FilterTag)
    {
        if (FilterTag->objectName().isEmpty())
            FilterTag->setObjectName("FilterTag");
        FilterTag->resize(364, 44);
        FilterTag->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(FilterTag);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        lineEdit = new QLineEdit(FilterTag);
        lineEdit->setObjectName("lineEdit");

        verticalLayout->addWidget(lineEdit);

        tagArea = new QWidget(FilterTag);
        tagArea->setObjectName("tagArea");

        verticalLayout->addWidget(tagArea);


        retranslateUi(FilterTag);

        QMetaObject::connectSlotsByName(FilterTag);
    } // setupUi

    void retranslateUi(QWidget *FilterTag)
    {
        lineEdit->setPlaceholderText(QCoreApplication::translate("FilterTag", "Tag name...", nullptr));
        (void)FilterTag;
    } // retranslateUi

};

namespace Ui {
    class FilterTag: public Ui_FilterTag {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERTAG_H
