/********************************************************************************
** Form generated from reading UI file 'configsectionsound.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSECTIONSOUND_H
#define UI_CONFIGSECTIONSOUND_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDial>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigSectionSound
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout_5;
    QLabel *label_10;
    QDial *dialChoFrequence;
    QLabel *label_12;
    QDial *dialChoAmplitude;
    QDial *dialChoNiveau;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_11;
    QLabel *label_33;
    QComboBox *comboVelToFilter;
    QGridLayout *gridLayout_3;
    QDial *dialRevProfondeur;
    QDial *dialRevAttenuation;
    QLabel *label_7;
    QLabel *label_6;
    QDial *dialRevDensite;
    QLabel *label_8;
    QLabel *label_9;
    QDial *dialRevNiveau;
    QSpacerItem *horizontalSpacer_4;
    QSlider *sliderGain;
    QLabel *labelGain;
    QLabel *label;
    QSpacerItem *horizontalSpacer_5;
    QLabel *labelSubTitle1;
    QLabel *labelSubTitle2;
    QLabel *labelSubTitle3;

    void setupUi(QWidget *ConfigSectionSound)
    {
        if (ConfigSectionSound->objectName().isEmpty())
            ConfigSectionSound->setObjectName("ConfigSectionSound");
        ConfigSectionSound->resize(525, 424);
        ConfigSectionSound->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout_2 = new QGridLayout(ConfigSectionSound);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");
        label_10 = new QLabel(ConfigSectionSound);
        label_10->setObjectName("label_10");
        label_10->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_5->addWidget(label_10, 1, 0, 1, 1);

        dialChoFrequence = new QDial(ConfigSectionSound);
        dialChoFrequence->setObjectName("dialChoFrequence");
        dialChoFrequence->setMaximum(100);
        dialChoFrequence->setNotchTarget(10.000000000000000);
        dialChoFrequence->setNotchesVisible(true);

        gridLayout_5->addWidget(dialChoFrequence, 0, 2, 1, 1);

        label_12 = new QLabel(ConfigSectionSound);
        label_12->setObjectName("label_12");
        label_12->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_5->addWidget(label_12, 1, 2, 1, 1);

        dialChoAmplitude = new QDial(ConfigSectionSound);
        dialChoAmplitude->setObjectName("dialChoAmplitude");
        dialChoAmplitude->setMaximum(100);
        dialChoAmplitude->setNotchTarget(10.000000000000000);
        dialChoAmplitude->setNotchesVisible(true);

        gridLayout_5->addWidget(dialChoAmplitude, 0, 1, 1, 1);

        dialChoNiveau = new QDial(ConfigSectionSound);
        dialChoNiveau->setObjectName("dialChoNiveau");
        dialChoNiveau->setMinimumSize(QSize(100, 100));
        dialChoNiveau->setMaximum(100);
        dialChoNiveau->setNotchTarget(10.000000000000000);
        dialChoNiveau->setNotchesVisible(true);

        gridLayout_5->addWidget(dialChoNiveau, 0, 0, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_2, 0, 3, 1, 1);

        label_11 = new QLabel(ConfigSectionSound);
        label_11->setObjectName("label_11");
        label_11->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_5->addWidget(label_11, 1, 1, 1, 1);


        gridLayout_2->addLayout(gridLayout_5, 6, 1, 1, 3);

        label_33 = new QLabel(ConfigSectionSound);
        label_33->setObjectName("label_33");

        gridLayout_2->addWidget(label_33, 1, 1, 1, 1);

        comboVelToFilter = new QComboBox(ConfigSectionSound);
        comboVelToFilter->addItem(QString::fromUtf8("SoundFont 2.01"));
        comboVelToFilter->addItem(QString::fromUtf8("SoundFont 2.04"));
        comboVelToFilter->addItem(QString());
        comboVelToFilter->setObjectName("comboVelToFilter");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comboVelToFilter->sizePolicy().hasHeightForWidth());
        comboVelToFilter->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(comboVelToFilter, 2, 2, 1, 2);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(-1, 5, -1, -1);
        dialRevProfondeur = new QDial(ConfigSectionSound);
        dialRevProfondeur->setObjectName("dialRevProfondeur");
        dialRevProfondeur->setMaximum(100);
        dialRevProfondeur->setNotchTarget(10.000000000000000);
        dialRevProfondeur->setNotchesVisible(true);

        gridLayout_3->addWidget(dialRevProfondeur, 0, 1, 1, 1);

        dialRevAttenuation = new QDial(ConfigSectionSound);
        dialRevAttenuation->setObjectName("dialRevAttenuation");
        dialRevAttenuation->setMaximum(100);
        dialRevAttenuation->setNotchTarget(10.000000000000000);
        dialRevAttenuation->setNotchesVisible(true);

        gridLayout_3->addWidget(dialRevAttenuation, 0, 3, 1, 1);

        label_7 = new QLabel(ConfigSectionSound);
        label_7->setObjectName("label_7");
        QFont font;
        font.setBold(false);
        label_7->setFont(font);
        label_7->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_7, 1, 1, 1, 1);

        label_6 = new QLabel(ConfigSectionSound);
        label_6->setObjectName("label_6");
        label_6->setFont(font);
        label_6->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_6, 1, 0, 1, 1);

        dialRevDensite = new QDial(ConfigSectionSound);
        dialRevDensite->setObjectName("dialRevDensite");
        dialRevDensite->setMaximum(100);
        dialRevDensite->setNotchTarget(10.000000000000000);
        dialRevDensite->setNotchesVisible(true);

        gridLayout_3->addWidget(dialRevDensite, 0, 2, 1, 1);

        label_8 = new QLabel(ConfigSectionSound);
        label_8->setObjectName("label_8");
        label_8->setFont(font);
        label_8->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_8, 1, 2, 1, 1);

        label_9 = new QLabel(ConfigSectionSound);
        label_9->setObjectName("label_9");
        label_9->setFont(font);
        label_9->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(label_9, 1, 3, 1, 1);

        dialRevNiveau = new QDial(ConfigSectionSound);
        dialRevNiveau->setObjectName("dialRevNiveau");
        dialRevNiveau->setMinimumSize(QSize(100, 100));
        dialRevNiveau->setMaximum(100);
        dialRevNiveau->setNotchTarget(10.000000000000000);
        dialRevNiveau->setNotchesVisible(true);

        gridLayout_3->addWidget(dialRevNiveau, 0, 0, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_4, 0, 4, 1, 1);


        gridLayout_2->addLayout(gridLayout_3, 4, 1, 1, 3);

        sliderGain = new QSlider(ConfigSectionSound);
        sliderGain->setObjectName("sliderGain");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(sliderGain->sizePolicy().hasHeightForWidth());
        sliderGain->setSizePolicy(sizePolicy1);
        sliderGain->setMinimum(-50);
        sliderGain->setMaximum(50);
        sliderGain->setOrientation(Qt::Orientation::Horizontal);
        sliderGain->setTickPosition(QSlider::TickPosition::NoTicks);
        sliderGain->setTickInterval(25);

        gridLayout_2->addWidget(sliderGain, 1, 2, 1, 1);

        labelGain = new QLabel(ConfigSectionSound);
        labelGain->setObjectName("labelGain");
        labelGain->setMinimumSize(QSize(50, 0));
        labelGain->setText(QString::fromUtf8("+0 dB"));
        labelGain->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_2->addWidget(labelGain, 1, 3, 1, 1);

        label = new QLabel(ConfigSectionSound);
        label->setObjectName("label");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(label, 2, 1, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_5, 1, 0, 1, 1);

        labelSubTitle1 = new QLabel(ConfigSectionSound);
        labelSubTitle1->setObjectName("labelSubTitle1");
        QFont font1;
        font1.setBold(true);
        labelSubTitle1->setFont(font1);

        gridLayout_2->addWidget(labelSubTitle1, 0, 0, 1, 4);

        labelSubTitle2 = new QLabel(ConfigSectionSound);
        labelSubTitle2->setObjectName("labelSubTitle2");
        labelSubTitle2->setFont(font1);

        gridLayout_2->addWidget(labelSubTitle2, 3, 0, 1, 2);

        labelSubTitle3 = new QLabel(ConfigSectionSound);
        labelSubTitle3->setObjectName("labelSubTitle3");
        labelSubTitle3->setFont(font1);

        gridLayout_2->addWidget(labelSubTitle3, 5, 0, 1, 2);

        QWidget::setTabOrder(sliderGain, comboVelToFilter);
        QWidget::setTabOrder(comboVelToFilter, dialRevNiveau);
        QWidget::setTabOrder(dialRevNiveau, dialRevProfondeur);
        QWidget::setTabOrder(dialRevProfondeur, dialRevDensite);
        QWidget::setTabOrder(dialRevDensite, dialRevAttenuation);
        QWidget::setTabOrder(dialRevAttenuation, dialChoNiveau);
        QWidget::setTabOrder(dialChoNiveau, dialChoAmplitude);
        QWidget::setTabOrder(dialChoAmplitude, dialChoFrequence);

        retranslateUi(ConfigSectionSound);

        QMetaObject::connectSlotsByName(ConfigSectionSound);
    } // setupUi

    void retranslateUi(QWidget *ConfigSectionSound)
    {
        label_10->setText(QCoreApplication::translate("ConfigSectionSound", "Level", nullptr));
        label_12->setText(QCoreApplication::translate("ConfigSectionSound", "Frequency", nullptr));
        label_11->setText(QCoreApplication::translate("ConfigSectionSound", "Depth", nullptr));
        label_33->setText(QCoreApplication::translate("ConfigSectionSound", "General volume", nullptr));
        comboVelToFilter->setItemText(2, QCoreApplication::translate("ConfigSectionSound", "disabled", nullptr));

        label_7->setText(QCoreApplication::translate("ConfigSectionSound", "Room", nullptr));
        label_6->setText(QCoreApplication::translate("ConfigSectionSound", "Level", nullptr));
        label_8->setText(QCoreApplication::translate("ConfigSectionSound", "Width", nullptr));
        label_9->setText(QCoreApplication::translate("ConfigSectionSound", "Damp", nullptr));
        label->setText(QCoreApplication::translate("ConfigSectionSound", "Default modulator \"velocity \342\206\222 filter cutoff\"", nullptr));
        labelSubTitle1->setText(QCoreApplication::translate("ConfigSectionSound", "Options", nullptr));
        labelSubTitle2->setText(QCoreApplication::translate("ConfigSectionSound", "Reverb", nullptr));
        labelSubTitle3->setText(QCoreApplication::translate("ConfigSectionSound", "Chorus", nullptr));
        (void)ConfigSectionSound;
    } // retranslateUi

};

namespace Ui {
    class ConfigSectionSound: public Ui_ConfigSectionSound {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSECTIONSOUND_H
