/********************************************************************************
** Form generated from reading UI file 'dialog_list.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOG_LIST_H
#define UI_DIALOG_LIST_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_DialogList
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QListWidget *listWidget;
    QLineEdit *lineSearch;

    void setupUi(QDialog *DialogList)
    {
        if (DialogList->objectName().isEmpty())
            DialogList->setObjectName("DialogList");
        DialogList->setWindowModality(Qt::WindowModal);
        DialogList->resize(207, 320);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogList->setWindowIcon(icon);
        gridLayout = new QGridLayout(DialogList);
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(DialogList);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(DialogList);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 3, 0, 1, 1);

        listWidget = new QListWidget(DialogList);
        listWidget->setObjectName("listWidget");

        gridLayout->addWidget(listWidget, 1, 0, 1, 1);

        lineSearch = new QLineEdit(DialogList);
        lineSearch->setObjectName("lineSearch");
        lineSearch->setClearButtonEnabled(true);

        gridLayout->addWidget(lineSearch, 0, 0, 1, 1);

        QWidget::setTabOrder(lineSearch, listWidget);
        QWidget::setTabOrder(listWidget, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(DialogList);

        QMetaObject::connectSlotsByName(DialogList);
    } // setupUi

    void retranslateUi(QDialog *DialogList)
    {
        pushOk->setText(QCoreApplication::translate("DialogList", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("DialogList", "&Cancel", nullptr));
        lineSearch->setPlaceholderText(QCoreApplication::translate("DialogList", "Search...", nullptr));
        (void)DialogList;
    } // retranslateUi

};

namespace Ui {
    class DialogList: public Ui_DialogList {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOG_LIST_H
