/********************************************************************************
** Form generated from reading UI file 'pageinst.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEINST_H
#define UI_PAGEINST_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "modulatoreditor.h"
#include "pageinst.h"

QT_BEGIN_NAMESPACE

class Ui_PageInst
{
public:
    QVBoxLayout *verticalLayout;
    TableWidgetInst *tableInst;
    ModulatorEditor *modulatorEditor;

    void setupUi(QWidget *PageInst)
    {
        if (PageInst->objectName().isEmpty())
            PageInst->setObjectName("PageInst");
        PageInst->resize(607, 385);
        PageInst->setWindowTitle(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(PageInst);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tableInst = new TableWidgetInst(PageInst);
        tableInst->setObjectName("tableInst");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableInst->sizePolicy().hasHeightForWidth());
        tableInst->setSizePolicy(sizePolicy);
        tableInst->setFrameShape(QFrame::NoFrame);
        tableInst->setAutoScroll(true);
        tableInst->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::SelectedClicked);
        tableInst->setProperty("showDropIndicator", QVariant(false));
        tableInst->setDragDropOverwriteMode(false);
        tableInst->setSelectionMode(QAbstractItemView::ExtendedSelection);
        tableInst->setTextElideMode(Qt::ElideMiddle);
        tableInst->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        tableInst->setCornerButtonEnabled(false);
        tableInst->verticalHeader()->setMinimumSectionSize(10);

        verticalLayout->addWidget(tableInst);

        modulatorEditor = new ModulatorEditor(PageInst);
        modulatorEditor->setObjectName("modulatorEditor");

        verticalLayout->addWidget(modulatorEditor);


        retranslateUi(PageInst);
        QObject::connect(tableInst, SIGNAL(cellChanged(int,int)), PageInst, SLOT(set(int,int)));
        QObject::connect(tableInst, SIGNAL(itemSelectionChanged()), PageInst, SLOT(selected()));

        QMetaObject::connectSlotsByName(PageInst);
    } // setupUi

    void retranslateUi(QWidget *PageInst)
    {
        (void)PageInst;
    } // retranslateUi

};

namespace Ui {
    class PageInst: public Ui_PageInst {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEINST_H
