/********************************************************************************
** Form generated from reading UI file 'toolsoundfontexport_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLSOUNDFONTEXPORT_GUI_H
#define UI_TOOLSOUNDFONTEXPORT_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolSoundfontExport_gui
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushTick;
    QPushButton *pushUntick;
    QTreeWidget *listPresets;
    QGridLayout *gridLayout;
    QLabel *labelQuality;
    QLineEdit *lineFolder;
    QCheckBox *checkGM;
    QPushButton *pushFolder;
    QLabel *label_2;
    QCheckBox *checkPreset;
    QComboBox *comboFormat;
    QLabel *label;
    QCheckBox *checkBank;
    QComboBox *comboQuality;
    QCheckBox *checkFilePreset;
    QCheckBox *checkRawValues;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushExport;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushAnnuler;

    void setupUi(QWidget *ToolSoundfontExport_gui)
    {
        if (ToolSoundfontExport_gui->objectName().isEmpty())
            ToolSoundfontExport_gui->setObjectName("ToolSoundfontExport_gui");
        ToolSoundfontExport_gui->resize(320, 402);
        ToolSoundfontExport_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolSoundfontExport_gui);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushTick = new QPushButton(ToolSoundfontExport_gui);
        pushTick->setObjectName("pushTick");

        horizontalLayout_2->addWidget(pushTick);

        pushUntick = new QPushButton(ToolSoundfontExport_gui);
        pushUntick->setObjectName("pushUntick");

        horizontalLayout_2->addWidget(pushUntick);


        verticalLayout->addLayout(horizontalLayout_2);

        listPresets = new QTreeWidget(ToolSoundfontExport_gui);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        listPresets->setHeaderItem(__qtreewidgetitem);
        listPresets->setObjectName("listPresets");
        listPresets->header()->setVisible(false);
        listPresets->header()->setDefaultSectionSize(120);

        verticalLayout->addWidget(listPresets);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(6);
        gridLayout->setVerticalSpacing(5);
        labelQuality = new QLabel(ToolSoundfontExport_gui);
        labelQuality->setObjectName("labelQuality");

        gridLayout->addWidget(labelQuality, 2, 0, 1, 1);

        lineFolder = new QLineEdit(ToolSoundfontExport_gui);
        lineFolder->setObjectName("lineFolder");

        gridLayout->addWidget(lineFolder, 0, 1, 1, 1);

        checkGM = new QCheckBox(ToolSoundfontExport_gui);
        checkGM->setObjectName("checkGM");

        gridLayout->addWidget(checkGM, 5, 1, 1, 2);

        pushFolder = new QPushButton(ToolSoundfontExport_gui);
        pushFolder->setObjectName("pushFolder");
        pushFolder->setMaximumSize(QSize(26, 26));
        pushFolder->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushFolder->setFlat(true);

        gridLayout->addWidget(pushFolder, 0, 2, 1, 1);

        label_2 = new QLabel(ToolSoundfontExport_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 0, 1, 1);

        checkPreset = new QCheckBox(ToolSoundfontExport_gui);
        checkPreset->setObjectName("checkPreset");

        gridLayout->addWidget(checkPreset, 3, 1, 1, 2);

        comboFormat = new QComboBox(ToolSoundfontExport_gui);
        comboFormat->addItem(QString::fromUtf8(".sf2"));
        comboFormat->addItem(QString::fromUtf8(".sf3"));
        comboFormat->addItem(QString::fromUtf8(".sfz"));
        comboFormat->addItem(QString::fromUtf8(".csv"));
        comboFormat->setObjectName("comboFormat");

        gridLayout->addWidget(comboFormat, 1, 1, 1, 2);

        label = new QLabel(ToolSoundfontExport_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        checkBank = new QCheckBox(ToolSoundfontExport_gui);
        checkBank->setObjectName("checkBank");

        gridLayout->addWidget(checkBank, 4, 1, 1, 2);

        comboQuality = new QComboBox(ToolSoundfontExport_gui);
        comboQuality->addItem(QString());
        comboQuality->addItem(QString());
        comboQuality->addItem(QString());
        comboQuality->setObjectName("comboQuality");

        gridLayout->addWidget(comboQuality, 2, 1, 1, 2);

        checkFilePreset = new QCheckBox(ToolSoundfontExport_gui);
        checkFilePreset->setObjectName("checkFilePreset");

        gridLayout->addWidget(checkFilePreset, 6, 1, 1, 2);

        checkRawValues = new QCheckBox(ToolSoundfontExport_gui);
        checkRawValues->setObjectName("checkRawValues");

        gridLayout->addWidget(checkRawValues, 7, 1, 1, 2);


        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushExport = new QPushButton(ToolSoundfontExport_gui);
        pushExport->setObjectName("pushExport");

        horizontalLayout->addWidget(pushExport);

        horizontalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushAnnuler = new QPushButton(ToolSoundfontExport_gui);
        pushAnnuler->setObjectName("pushAnnuler");

        horizontalLayout->addWidget(pushAnnuler);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(pushTick, pushUntick);
        QWidget::setTabOrder(pushUntick, listPresets);
        QWidget::setTabOrder(listPresets, lineFolder);
        QWidget::setTabOrder(lineFolder, comboFormat);
        QWidget::setTabOrder(comboFormat, checkPreset);
        QWidget::setTabOrder(checkPreset, checkBank);
        QWidget::setTabOrder(checkBank, checkGM);
        QWidget::setTabOrder(checkGM, pushExport);
        QWidget::setTabOrder(pushExport, pushAnnuler);

        retranslateUi(ToolSoundfontExport_gui);

        comboQuality->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(ToolSoundfontExport_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolSoundfontExport_gui)
    {
        pushTick->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "Select all", nullptr));
        pushUntick->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "Unselect all", nullptr));
        labelQuality->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "Quality", nullptr));
        checkGM->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "General MIDI classification", nullptr));
        pushFolder->setText(QString());
        label_2->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "Destination", nullptr));
        checkPreset->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "preset number as prefix", nullptr));

        label->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "Format", nullptr));
        checkBank->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "one directory per bank", nullptr));
        comboQuality->setItemText(0, QCoreApplication::translate("ToolSoundfontExport_gui", "High", nullptr));
        comboQuality->setItemText(1, QCoreApplication::translate("ToolSoundfontExport_gui", "Medium", nullptr));
        comboQuality->setItemText(2, QCoreApplication::translate("ToolSoundfontExport_gui", "Low", nullptr));

        checkFilePreset->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "one file per preset", nullptr));
        checkRawValues->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "raw values", nullptr));
        pushExport->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "&Export", nullptr));
        pushAnnuler->setText(QCoreApplication::translate("ToolSoundfontExport_gui", "&Cancel", nullptr));
        (void)ToolSoundfontExport_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolSoundfontExport_gui: public Ui_ToolSoundfontExport_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLSOUNDFONTEXPORT_GUI_H
