/********************************************************************************
** Form generated from reading UI file 'toolfrequencypeaks_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLFREQUENCYPEAKS_GUI_H
#define UI_TOOLFREQUENCYPEAKS_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolFrequencyPeaks_gui
{
public:
    QGridLayout *gridLayout;
    QPushButton *pushExport;
    QPushButton *pushClose;
    QTableWidget *table;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *ToolFrequencyPeaks_gui)
    {
        if (ToolFrequencyPeaks_gui->objectName().isEmpty())
            ToolFrequencyPeaks_gui->setObjectName("ToolFrequencyPeaks_gui");
        ToolFrequencyPeaks_gui->resize(580, 240);
        ToolFrequencyPeaks_gui->setMinimumSize(QSize(580, 240));
        ToolFrequencyPeaks_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolFrequencyPeaks_gui);
        gridLayout->setObjectName("gridLayout");
        pushExport = new QPushButton(ToolFrequencyPeaks_gui);
        pushExport->setObjectName("pushExport");

        gridLayout->addWidget(pushExport, 1, 0, 1, 1);

        pushClose = new QPushButton(ToolFrequencyPeaks_gui);
        pushClose->setObjectName("pushClose");

        gridLayout->addWidget(pushClose, 1, 2, 1, 1);

        table = new QTableWidget(ToolFrequencyPeaks_gui);
        table->setObjectName("table");
        table->setEditTriggers(QAbstractItemView::NoEditTriggers);
        table->setAlternatingRowColors(true);
        table->setSelectionMode(QAbstractItemView::SingleSelection);
        table->setSelectionBehavior(QAbstractItemView::SelectRows);
        table->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        table->horizontalHeader()->setStretchLastSection(true);
        table->verticalHeader()->setVisible(false);
        table->verticalHeader()->setMinimumSectionSize(20);
        table->verticalHeader()->setDefaultSectionSize(20);

        gridLayout->addWidget(table, 0, 0, 1, 3);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 1, 1, 1);

        QWidget::setTabOrder(table, pushExport);
        QWidget::setTabOrder(pushExport, pushClose);

        retranslateUi(ToolFrequencyPeaks_gui);

        QMetaObject::connectSlotsByName(ToolFrequencyPeaks_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolFrequencyPeaks_gui)
    {
        pushExport->setText(QCoreApplication::translate("ToolFrequencyPeaks_gui", "&Export", nullptr));
        pushClose->setText(QCoreApplication::translate("ToolFrequencyPeaks_gui", "&Close", nullptr));
        (void)ToolFrequencyPeaks_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolFrequencyPeaks_gui: public Ui_ToolFrequencyPeaks_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLFREQUENCYPEAKS_GUI_H
