/********************************************************************************
** Form generated from reading UI file 'toolmonitor_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLMONITOR_GUI_H
#define UI_TOOLMONITOR_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "monitor/graphlegend.h"
#include "monitor/graphvisualizer.h"

QT_BEGIN_NAMESPACE

class Ui_ToolMonitor_gui
{
public:
    QVBoxLayout *verticalLayout;
    GraphVisualizer *graphVisualizer;
    QFrame *frame;
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_5;
    QComboBox *comboParameter;
    QCheckBox *checkLog;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_2;
    GraphLegend *widgetLegendMoyenne;
    GraphLegend *widgetLegendDefined;
    QLabel *label_5;
    GraphLegend *widgetLegendDefault;
    QLabel *label_3;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer_3;

    void setupUi(QWidget *ToolMonitor_gui)
    {
        if (ToolMonitor_gui->objectName().isEmpty())
            ToolMonitor_gui->setObjectName("ToolMonitor_gui");
        ToolMonitor_gui->resize(466, 277);
        ToolMonitor_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolMonitor_gui);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        graphVisualizer = new GraphVisualizer(ToolMonitor_gui);
        graphVisualizer->setObjectName("graphVisualizer");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(graphVisualizer->sizePolicy().hasHeightForWidth());
        graphVisualizer->setSizePolicy(sizePolicy);
        graphVisualizer->setMinimumSize(QSize(0, 160));

        verticalLayout->addWidget(graphVisualizer);

        frame = new QFrame(ToolMonitor_gui);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(frame);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(6, 6, 6, 6);
        groupBox = new QGroupBox(frame);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        QFont font;
        font.setBold(true);
        groupBox->setFont(font);
        verticalLayout_5 = new QVBoxLayout(groupBox);
        verticalLayout_5->setSpacing(3);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(6, 6, 6, 6);
        comboParameter = new QComboBox(groupBox);
        comboParameter->setObjectName("comboParameter");
        QFont font1;
        font1.setBold(false);
        comboParameter->setFont(font1);

        verticalLayout_5->addWidget(comboParameter);

        checkLog = new QCheckBox(groupBox);
        checkLog->setObjectName("checkLog");
        checkLog->setMaximumSize(QSize(200, 16777215));
        checkLog->setFont(font1);

        verticalLayout_5->addWidget(checkLog);


        gridLayout->addWidget(groupBox, 0, 0, 3, 1);

        groupBox_2 = new QGroupBox(frame);
        groupBox_2->setObjectName("groupBox_2");
        sizePolicy1.setHeightForWidth(groupBox_2->sizePolicy().hasHeightForWidth());
        groupBox_2->setSizePolicy(sizePolicy1);
        groupBox_2->setFont(font);
        gridLayout_2 = new QGridLayout(groupBox_2);
        gridLayout_2->setSpacing(3);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        widgetLegendMoyenne = new GraphLegend(groupBox_2);
        widgetLegendMoyenne->setObjectName("widgetLegendMoyenne");

        gridLayout_2->addWidget(widgetLegendMoyenne, 3, 0, 1, 1);

        widgetLegendDefined = new GraphLegend(groupBox_2);
        widgetLegendDefined->setObjectName("widgetLegendDefined");

        gridLayout_2->addWidget(widgetLegendDefined, 0, 0, 1, 1);

        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");
        label_5->setFont(font1);

        gridLayout_2->addWidget(label_5, 2, 1, 1, 2);

        widgetLegendDefault = new GraphLegend(groupBox_2);
        widgetLegendDefault->setObjectName("widgetLegendDefault");

        gridLayout_2->addWidget(widgetLegendDefault, 2, 0, 1, 1);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");
        label_3->setFont(font1);

        gridLayout_2->addWidget(label_3, 3, 1, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName("label_4");
        label_4->setFont(font1);

        gridLayout_2->addWidget(label_4, 0, 1, 1, 1);


        gridLayout->addWidget(groupBox_2, 0, 1, 3, 1);

        horizontalSpacer_3 = new QSpacerItem(3, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 1, 2, 1, 1);


        verticalLayout->addWidget(frame);


        retranslateUi(ToolMonitor_gui);

        QMetaObject::connectSlotsByName(ToolMonitor_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolMonitor_gui)
    {
        groupBox->setTitle(QCoreApplication::translate("ToolMonitor_gui", "Display options", nullptr));
        checkLog->setText(QCoreApplication::translate("ToolMonitor_gui", "Log scale", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ToolMonitor_gui", "Legend", nullptr));
        label_5->setText(QCoreApplication::translate("ToolMonitor_gui", "Defined value in the global division", nullptr));
        label_3->setText(QCoreApplication::translate("ToolMonitor_gui", "Default value for the selected parameter", nullptr));
        label_4->setText(QCoreApplication::translate("ToolMonitor_gui", "Defined value in a division", nullptr));
        (void)ToolMonitor_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolMonitor_gui: public Ui_ToolMonitor_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLMONITOR_GUI_H
