/********************************************************************************
** Form generated from reading UI file 'tooltransposesmpl_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLTRANSPOSESMPL_GUI_H
#define UI_TOOLTRANSPOSESMPL_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolTransposeSmpl_gui
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QDoubleSpinBox *spinTon;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolTransposeSmpl_gui)
    {
        if (ToolTransposeSmpl_gui->objectName().isEmpty())
            ToolTransposeSmpl_gui->setObjectName("ToolTransposeSmpl_gui");
        ToolTransposeSmpl_gui->resize(192, 70);
        ToolTransposeSmpl_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolTransposeSmpl_gui);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ToolTransposeSmpl_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        spinTon = new QDoubleSpinBox(ToolTransposeSmpl_gui);
        spinTon->setObjectName("spinTon");
        spinTon->setMinimum(-120.000000000000000);
        spinTon->setMaximum(120.000000000000000);

        gridLayout->addWidget(spinTon, 0, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolTransposeSmpl_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolTransposeSmpl_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 1, 0, 1, 2);


        retranslateUi(ToolTransposeSmpl_gui);

        QMetaObject::connectSlotsByName(ToolTransposeSmpl_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolTransposeSmpl_gui)
    {
        label->setText(QCoreApplication::translate("ToolTransposeSmpl_gui", "Shift in semitones", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolTransposeSmpl_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolTransposeSmpl_gui", "&Cancel", nullptr));
        (void)ToolTransposeSmpl_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolTransposeSmpl_gui: public Ui_ToolTransposeSmpl_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLTRANSPOSESMPL_GUI_H
