/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2024 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone.io                             **
**             Date: 01.01.2013                                           **
***************************************************************************/

#ifndef ACTION_H
#define ACTION_H

#include "basetypes.h"

class Action
{
public:
    Action();

    // Type of action
    typedef enum
    {
        TypeRemoval = 0,
        TypeCreation = 1,
        TypeUpdate = 2,
        TypeChangeFromDefault = 3,
        TypeChangeToDefault = 4,
        TypeNull = 5
    } ActionType;

    // Data
    ActionType typeAction;
    EltID id;
    AttributeType champ;
    QString qNewValue;
    QString qOldValue;
    AttributeValue vNewValue;
    AttributeValue vOldValue;
    QByteArray bNewValue;
    QByteArray bOldValue;
};

#endif // ACTION_H
