/********************************************************************************
** Form generated from reading UI file 'soundfontviewertop.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTVIEWERTOP_H
#define UI_SOUNDFONTVIEWERTOP_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "elidedlabel.h"
#include "fivestarswidget.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontViewerTop
{
public:
    QHBoxLayout *horizontalLayout;
    ElidedLabel *labelTitle;
    QLabel *iconComment;
    QLabel *labelCommentNumber;
    QLabel *iconDownload;
    QLabel *labelDownloadNumber;
    FiveStarsWidget *widgetStars;

    void setupUi(QWidget *SoundfontViewerTop)
    {
        if (SoundfontViewerTop->objectName().isEmpty())
            SoundfontViewerTop->setObjectName("SoundfontViewerTop");
        SoundfontViewerTop->resize(280, 24);
        SoundfontViewerTop->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(SoundfontViewerTop);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        labelTitle = new ElidedLabel(SoundfontViewerTop);
        labelTitle->setObjectName("labelTitle");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelTitle->sizePolicy().hasHeightForWidth());
        labelTitle->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        labelTitle->setFont(font);
        labelTitle->setText(QString::fromUtf8("labelTitle"));

        horizontalLayout->addWidget(labelTitle);

        iconComment = new QLabel(SoundfontViewerTop);
        iconComment->setObjectName("iconComment");
        iconComment->setMinimumSize(QSize(24, 24));
        iconComment->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(iconComment);

        labelCommentNumber = new QLabel(SoundfontViewerTop);
        labelCommentNumber->setObjectName("labelCommentNumber");
        labelCommentNumber->setMinimumSize(QSize(30, 0));
        labelCommentNumber->setText(QString::fromUtf8("0"));
        labelCommentNumber->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(labelCommentNumber);

        iconDownload = new QLabel(SoundfontViewerTop);
        iconDownload->setObjectName("iconDownload");
        iconDownload->setMinimumSize(QSize(24, 24));
        iconDownload->setMaximumSize(QSize(24, 24));

        horizontalLayout->addWidget(iconDownload);

        labelDownloadNumber = new QLabel(SoundfontViewerTop);
        labelDownloadNumber->setObjectName("labelDownloadNumber");
        labelDownloadNumber->setMinimumSize(QSize(30, 0));
        labelDownloadNumber->setText(QString::fromUtf8("0"));
        labelDownloadNumber->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(labelDownloadNumber);

        widgetStars = new FiveStarsWidget(SoundfontViewerTop);
        widgetStars->setObjectName("widgetStars");

        horizontalLayout->addWidget(widgetStars);


        retranslateUi(SoundfontViewerTop);

        QMetaObject::connectSlotsByName(SoundfontViewerTop);
    } // setupUi

    void retranslateUi(QWidget *SoundfontViewerTop)
    {
        iconComment->setText(QString());
        iconDownload->setText(QString());
        (void)SoundfontViewerTop;
    } // retranslateUi

};

namespace Ui {
    class SoundfontViewerTop: public Ui_SoundfontViewerTop {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTVIEWERTOP_H
