/********************************************************************************
** Form generated from reading UI file 'pageoverview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEOVERVIEW_H
#define UI_PAGEOVERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QWidget>
#include "tablepageoverview.h"

QT_BEGIN_NAMESPACE

class Ui_PageOverview
{
public:
    QGridLayout *gridLayout;
    TablePageOverview *table;

    void setupUi(QWidget *PageOverview)
    {
        if (PageOverview->objectName().isEmpty())
            PageOverview->setObjectName("PageOverview");
        PageOverview->resize(400, 300);
        PageOverview->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(PageOverview);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        table = new TablePageOverview(PageOverview);
        table->setObjectName("table");
        table->setFrameShape(QFrame::NoFrame);
        table->setEditTriggers(QAbstractItemView::NoEditTriggers);
        table->setProperty("showDropIndicator", QVariant(false));
        table->setDragDropOverwriteMode(false);
        table->setSelectionMode(QAbstractItemView::SingleSelection);
        table->setSelectionBehavior(QAbstractItemView::SelectRows);
        table->setTextElideMode(Qt::ElideMiddle);
        table->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        table->setSortingEnabled(true);
        table->setCornerButtonEnabled(false);
        table->horizontalHeader()->setDefaultSectionSize(110);
        table->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        table->verticalHeader()->setVisible(false);
        table->verticalHeader()->setMinimumSectionSize(19);
        table->verticalHeader()->setDefaultSectionSize(20);

        gridLayout->addWidget(table, 0, 0, 1, 1);


        retranslateUi(PageOverview);

        QMetaObject::connectSlotsByName(PageOverview);
    } // setupUi

    void retranslateUi(QWidget *PageOverview)
    {
        (void)PageOverview;
    } // retranslateUi

};

namespace Ui {
    class PageOverview: public Ui_PageOverview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEOVERVIEW_H
