/********************************************************************************
** Form generated from reading UI file 'tooldefaultmod_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLDEFAULTMOD_GUI_H
#define UI_TOOLDEFAULTMOD_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "modulatorlistwidget.h"

QT_BEGIN_NAMESPACE

class Ui_ToolDefaultMod_gui
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    ModulatorListWidget *listWidget;
    QPushButton *pushOverride;
    QPushButton *pushDisable;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolDefaultMod_gui)
    {
        if (ToolDefaultMod_gui->objectName().isEmpty())
            ToolDefaultMod_gui->setObjectName("ToolDefaultMod_gui");
        ToolDefaultMod_gui->resize(718, 276);
        ToolDefaultMod_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolDefaultMod_gui);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ToolDefaultMod_gui);
        label->setObjectName("label");
        label->setWordWrap(true);

        gridLayout->addWidget(label, 0, 0, 1, 4);

        listWidget = new ModulatorListWidget(ToolDefaultMod_gui);
        listWidget->setObjectName("listWidget");
        listWidget->setMinimumSize(QSize(700, 0));
        listWidget->setAlternatingRowColors(true);
        listWidget->setSelectionMode(QAbstractItemView::ExtendedSelection);
        listWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
        listWidget->setResizeMode(QListView::Adjust);

        gridLayout->addWidget(listWidget, 1, 0, 1, 4);

        pushOverride = new QPushButton(ToolDefaultMod_gui);
        pushOverride->setObjectName("pushOverride");
        pushOverride->setEnabled(false);

        gridLayout->addWidget(pushOverride, 2, 0, 1, 1);

        pushDisable = new QPushButton(ToolDefaultMod_gui);
        pushDisable->setObjectName("pushDisable");
        pushDisable->setEnabled(false);

        gridLayout->addWidget(pushDisable, 2, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 2, 1, 1);

        pushCancel = new QPushButton(ToolDefaultMod_gui);
        pushCancel->setObjectName("pushCancel");

        gridLayout->addWidget(pushCancel, 2, 3, 1, 1);

        QWidget::setTabOrder(listWidget, pushOverride);
        QWidget::setTabOrder(pushOverride, pushDisable);
        QWidget::setTabOrder(pushDisable, pushCancel);

        retranslateUi(ToolDefaultMod_gui);

        QMetaObject::connectSlotsByName(ToolDefaultMod_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolDefaultMod_gui)
    {
        label->setText(QCoreApplication::translate("ToolDefaultMod_gui", "Here is the list of modulators automatically set for every instrument. You can override or disable them by selecting the corresponding modulators and action below.", nullptr));
        pushOverride->setText(QCoreApplication::translate("ToolDefaultMod_gui", "&Override", nullptr));
        pushDisable->setText(QCoreApplication::translate("ToolDefaultMod_gui", "&Disable", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolDefaultMod_gui", "&Cancel", nullptr));
        (void)ToolDefaultMod_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolDefaultMod_gui: public Ui_ToolDefaultMod_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLDEFAULTMOD_GUI_H
