/********************************************************************************
** Form generated from reading UI file 'pagesf2.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGESF2_H
#define UI_PAGESF2_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "pagesf2.h"
#include "styledlineedit.h"
#include "styledlineeditwithcalendar.h"

QT_BEGIN_NAMESPACE

class Ui_PageSf2
{
public:
    QVBoxLayout *verticalLayout_4;
    StyledLineEdit *lineEdit_name;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelUser;
    StyledLineEdit *lineEdit_author;
    QLabel *labelDate;
    StyledLineEditWithCalendar *lineEdit_date;
    QSpacerItem *horizontalSpacer;
    QLabel *labelInfo;
    QGridLayout *gridLayout_2;
    QLabel *label_25;
    QLabel *label_26;
    QLineEdit *lineEdit_copyright;
    QLabel *label_23;
    QLineEdit *lineEdit_product;
    TextEdit *textEdit_Com;
    QFrame *elementFrame;
    QGridLayout *gridLayout;
    QFrame *framePresetTitle;
    QHBoxLayout *horizontalLayout_5;
    QLabel *labelPreset;
    QLabel *label_11;
    QPushButton *pushViewPresets;
    QFrame *frame_7;
    QFrame *frameSampleTitle;
    QHBoxLayout *horizontalLayout;
    QLabel *labelSample;
    QLabel *label;
    QPushButton *pushViewSamples;
    QFrame *frame_5;
    QFrame *frameSample;
    QGridLayout *gridLayout_4;
    QLabel *labelCompression;
    QLabel *label_7;
    QLabel *label_nbSmpl;
    QLabel *labelBitDepth;
    QComboBox *comboBitDepth;
    QComboBox *comboCompression;
    QFrame *frameInstrument;
    QGridLayout *gridLayout_5;
    QLabel *label_14;
    QLabel *label_10;
    QLabel *label_nbInst;
    QLabel *label_nbInstMod;
    QLabel *label_12;
    QLabel *label_nbInstGen;
    QFrame *framePreset;
    QGridLayout *gridLayout_6;
    QLabel *label_8;
    QLabel *label_13;
    QLabel *label_nbPrstMod;
    QLabel *label_nbPrstGen;
    QLabel *label_nbPrst;
    QLabel *label_15;
    QFrame *frameInstrumentTitle;
    QHBoxLayout *horizontalLayout_4;
    QLabel *labelInstrument;
    QLabel *label_9;
    QPushButton *pushViewInstruments;

    void setupUi(QWidget *PageSf2)
    {
        if (PageSf2->objectName().isEmpty())
            PageSf2->setObjectName("PageSf2");
        PageSf2->resize(717, 488);
        PageSf2->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout_4 = new QVBoxLayout(PageSf2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        lineEdit_name = new StyledLineEdit(PageSf2);
        lineEdit_name->setObjectName("lineEdit_name");
        lineEdit_name->setMinimumSize(QSize(0, 52));
        lineEdit_name->setMaximumSize(QSize(16777215, 52));
        QFont font;
        font.setPointSize(30);
        font.setBold(true);
        lineEdit_name->setFont(font);
        lineEdit_name->setMaxLength(255);

        verticalLayout_4->addWidget(lineEdit_name);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelUser = new QLabel(PageSf2);
        labelUser->setObjectName("labelUser");
        labelUser->setMinimumSize(QSize(16, 18));
        labelUser->setMaximumSize(QSize(16, 18));
        labelUser->setPixmap(QPixmap(QString::fromUtf8(":/icons/user.svg")));
        labelUser->setScaledContents(true);

        horizontalLayout_2->addWidget(labelUser);

        lineEdit_author = new StyledLineEdit(PageSf2);
        lineEdit_author->setObjectName("lineEdit_author");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lineEdit_author->sizePolicy().hasHeightForWidth());
        lineEdit_author->setSizePolicy(sizePolicy);
        lineEdit_author->setMinimumSize(QSize(0, 28));
        lineEdit_author->setMaximumSize(QSize(16777215, 28));
        lineEdit_author->setMaxLength(255);

        horizontalLayout_2->addWidget(lineEdit_author);

        labelDate = new QLabel(PageSf2);
        labelDate->setObjectName("labelDate");
        labelDate->setMinimumSize(QSize(16, 18));
        labelDate->setMaximumSize(QSize(16, 18));
        labelDate->setPixmap(QPixmap(QString::fromUtf8(":/icons/calendar.svg")));
        labelDate->setScaledContents(true);

        horizontalLayout_2->addWidget(labelDate);

        lineEdit_date = new StyledLineEditWithCalendar(PageSf2);
        lineEdit_date->setObjectName("lineEdit_date");
        sizePolicy.setHeightForWidth(lineEdit_date->sizePolicy().hasHeightForWidth());
        lineEdit_date->setSizePolicy(sizePolicy);
        lineEdit_date->setMinimumSize(QSize(0, 28));
        lineEdit_date->setMaximumSize(QSize(16777215, 28));
        lineEdit_date->setMaxLength(255);

        horizontalLayout_2->addWidget(lineEdit_date);

        horizontalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        labelInfo = new QLabel(PageSf2);
        labelInfo->setObjectName("labelInfo");
        labelInfo->setMinimumSize(QSize(18, 18));
        labelInfo->setMaximumSize(QSize(18, 18));
        labelInfo->setTextFormat(Qt::TextFormat::PlainText);
        labelInfo->setPixmap(QPixmap(QString::fromUtf8(":/icons/info.svg")));
        labelInfo->setScaledContents(true);

        horizontalLayout_2->addWidget(labelInfo);


        verticalLayout_4->addLayout(horizontalLayout_2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(-1, -1, -1, 2);
        label_25 = new QLabel(PageSf2);
        label_25->setObjectName("label_25");

        gridLayout_2->addWidget(label_25, 0, 0, 1, 1);

        label_26 = new QLabel(PageSf2);
        label_26->setObjectName("label_26");
        label_26->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignTop);

        gridLayout_2->addWidget(label_26, 2, 0, 1, 1);

        lineEdit_copyright = new QLineEdit(PageSf2);
        lineEdit_copyright->setObjectName("lineEdit_copyright");
        lineEdit_copyright->setMaxLength(255);

        gridLayout_2->addWidget(lineEdit_copyright, 1, 1, 1, 1);

        label_23 = new QLabel(PageSf2);
        label_23->setObjectName("label_23");

        gridLayout_2->addWidget(label_23, 1, 0, 1, 1);

        lineEdit_product = new QLineEdit(PageSf2);
        lineEdit_product->setObjectName("lineEdit_product");
        lineEdit_product->setMaxLength(255);

        gridLayout_2->addWidget(lineEdit_product, 0, 1, 1, 1);

        textEdit_Com = new TextEdit(PageSf2);
        textEdit_Com->setObjectName("textEdit_Com");
        textEdit_Com->setAcceptRichText(false);

        gridLayout_2->addWidget(textEdit_Com, 2, 1, 1, 1);


        verticalLayout_4->addLayout(gridLayout_2);

        elementFrame = new QFrame(PageSf2);
        elementFrame->setObjectName("elementFrame");
        gridLayout = new QGridLayout(elementFrame);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(10);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        framePresetTitle = new QFrame(elementFrame);
        framePresetTitle->setObjectName("framePresetTitle");
        framePresetTitle->setMaximumSize(QSize(16777215, 28));
        framePresetTitle->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout_5 = new QHBoxLayout(framePresetTitle);
        horizontalLayout_5->setSpacing(8);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(8, 0, 0, 0);
        labelPreset = new QLabel(framePresetTitle);
        labelPreset->setObjectName("labelPreset");
        labelPreset->setMinimumSize(QSize(13, 16));
        labelPreset->setMaximumSize(QSize(13, 16));
        labelPreset->setPixmap(QPixmap(QString::fromUtf8(":/icons/preset.svg")));
        labelPreset->setScaledContents(true);

        horizontalLayout_5->addWidget(labelPreset);

        label_11 = new QLabel(framePresetTitle);
        label_11->setObjectName("label_11");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy1);
        QFont font1;
        font1.setBold(true);
        label_11->setFont(font1);

        horizontalLayout_5->addWidget(label_11);

        pushViewPresets = new QPushButton(framePresetTitle);
        pushViewPresets->setObjectName("pushViewPresets");
        pushViewPresets->setMinimumSize(QSize(0, 28));
        pushViewPresets->setMaximumSize(QSize(16777215, 28));
        pushViewPresets->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout_5->addWidget(pushViewPresets);


        gridLayout->addWidget(framePresetTitle, 0, 4, 1, 1);

        frame_7 = new QFrame(elementFrame);
        frame_7->setObjectName("frame_7");
        frame_7->setMinimumSize(QSize(2, 0));
        frame_7->setMaximumSize(QSize(2, 16777215));
        frame_7->setFrameShape(QFrame::Shape::NoFrame);

        gridLayout->addWidget(frame_7, 1, 1, 1, 1);

        frameSampleTitle = new QFrame(elementFrame);
        frameSampleTitle->setObjectName("frameSampleTitle");
        frameSampleTitle->setMinimumSize(QSize(0, 28));
        frameSampleTitle->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout = new QHBoxLayout(frameSampleTitle);
        horizontalLayout->setSpacing(8);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(8, 0, 0, 0);
        labelSample = new QLabel(frameSampleTitle);
        labelSample->setObjectName("labelSample");
        labelSample->setMinimumSize(QSize(13, 16));
        labelSample->setMaximumSize(QSize(13, 16));
        labelSample->setPixmap(QPixmap(QString::fromUtf8(":/icons/sample.svg")));
        labelSample->setScaledContents(true);

        horizontalLayout->addWidget(labelSample);

        label = new QLabel(frameSampleTitle);
        label->setObjectName("label");
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setFont(font1);

        horizontalLayout->addWidget(label);

        pushViewSamples = new QPushButton(frameSampleTitle);
        pushViewSamples->setObjectName("pushViewSamples");
        pushViewSamples->setMinimumSize(QSize(0, 28));
        pushViewSamples->setMaximumSize(QSize(16777215, 28));
        pushViewSamples->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout->addWidget(pushViewSamples);


        gridLayout->addWidget(frameSampleTitle, 0, 0, 1, 1);

        frame_5 = new QFrame(elementFrame);
        frame_5->setObjectName("frame_5");
        frame_5->setMinimumSize(QSize(2, 0));
        frame_5->setMaximumSize(QSize(2, 16777215));
        frame_5->setFrameShape(QFrame::Shape::NoFrame);

        gridLayout->addWidget(frame_5, 1, 3, 1, 1);

        frameSample = new QFrame(elementFrame);
        frameSample->setObjectName("frameSample");
        frameSample->setMinimumSize(QSize(0, 30));
        frameSample->setFrameShape(QFrame::Shape::NoFrame);
        gridLayout_4 = new QGridLayout(frameSample);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        labelCompression = new QLabel(frameSample);
        labelCompression->setObjectName("labelCompression");

        gridLayout_4->addWidget(labelCompression, 2, 0, 1, 1);

        label_7 = new QLabel(frameSample);
        label_7->setObjectName("label_7");

        gridLayout_4->addWidget(label_7, 0, 0, 1, 1);

        label_nbSmpl = new QLabel(frameSample);
        label_nbSmpl->setObjectName("label_nbSmpl");
        label_nbSmpl->setText(QString::fromUtf8("TextLabel"));

        gridLayout_4->addWidget(label_nbSmpl, 0, 1, 1, 1);

        labelBitDepth = new QLabel(frameSample);
        labelBitDepth->setObjectName("labelBitDepth");

        gridLayout_4->addWidget(labelBitDepth, 1, 0, 1, 1);

        comboBitDepth = new QComboBox(frameSample);
        comboBitDepth->addItem(QString());
        comboBitDepth->addItem(QString());
        comboBitDepth->setObjectName("comboBitDepth");
        comboBitDepth->setMaximumSize(QSize(16777215, 20));

        gridLayout_4->addWidget(comboBitDepth, 1, 1, 1, 1);

        comboCompression = new QComboBox(frameSample);
        comboCompression->addItem(QString());
        comboCompression->addItem(QString());
        comboCompression->setObjectName("comboCompression");
        comboCompression->setMaximumSize(QSize(16777215, 20));

        gridLayout_4->addWidget(comboCompression, 2, 1, 1, 1);


        gridLayout->addWidget(frameSample, 1, 0, 1, 1);

        frameInstrument = new QFrame(elementFrame);
        frameInstrument->setObjectName("frameInstrument");
        frameInstrument->setFrameShape(QFrame::Shape::NoFrame);
        gridLayout_5 = new QGridLayout(frameInstrument);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        label_14 = new QLabel(frameInstrument);
        label_14->setObjectName("label_14");

        gridLayout_5->addWidget(label_14, 2, 0, 1, 1);

        label_10 = new QLabel(frameInstrument);
        label_10->setObjectName("label_10");

        gridLayout_5->addWidget(label_10, 0, 0, 1, 1);

        label_nbInst = new QLabel(frameInstrument);
        label_nbInst->setObjectName("label_nbInst");
        label_nbInst->setText(QString::fromUtf8("TextLabel"));

        gridLayout_5->addWidget(label_nbInst, 0, 1, 1, 1);

        label_nbInstMod = new QLabel(frameInstrument);
        label_nbInstMod->setObjectName("label_nbInstMod");
        label_nbInstMod->setText(QString::fromUtf8("TextLabel"));

        gridLayout_5->addWidget(label_nbInstMod, 2, 1, 1, 1);

        label_12 = new QLabel(frameInstrument);
        label_12->setObjectName("label_12");

        gridLayout_5->addWidget(label_12, 1, 0, 1, 1);

        label_nbInstGen = new QLabel(frameInstrument);
        label_nbInstGen->setObjectName("label_nbInstGen");
        label_nbInstGen->setText(QString::fromUtf8("TextLabel"));

        gridLayout_5->addWidget(label_nbInstGen, 1, 1, 1, 1);


        gridLayout->addWidget(frameInstrument, 1, 2, 1, 1);

        framePreset = new QFrame(elementFrame);
        framePreset->setObjectName("framePreset");
        framePreset->setFrameShape(QFrame::Shape::NoFrame);
        gridLayout_6 = new QGridLayout(framePreset);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        label_8 = new QLabel(framePreset);
        label_8->setObjectName("label_8");

        gridLayout_6->addWidget(label_8, 1, 0, 1, 1);

        label_13 = new QLabel(framePreset);
        label_13->setObjectName("label_13");

        gridLayout_6->addWidget(label_13, 0, 0, 1, 1);

        label_nbPrstMod = new QLabel(framePreset);
        label_nbPrstMod->setObjectName("label_nbPrstMod");
        label_nbPrstMod->setText(QString::fromUtf8("TextLabel"));

        gridLayout_6->addWidget(label_nbPrstMod, 2, 1, 1, 1);

        label_nbPrstGen = new QLabel(framePreset);
        label_nbPrstGen->setObjectName("label_nbPrstGen");
        label_nbPrstGen->setText(QString::fromUtf8("TextLabel"));

        gridLayout_6->addWidget(label_nbPrstGen, 1, 1, 1, 1);

        label_nbPrst = new QLabel(framePreset);
        label_nbPrst->setObjectName("label_nbPrst");
        label_nbPrst->setText(QString::fromUtf8("TextLabel"));

        gridLayout_6->addWidget(label_nbPrst, 0, 1, 1, 1);

        label_15 = new QLabel(framePreset);
        label_15->setObjectName("label_15");

        gridLayout_6->addWidget(label_15, 2, 0, 1, 1);


        gridLayout->addWidget(framePreset, 1, 4, 1, 1);

        frameInstrumentTitle = new QFrame(elementFrame);
        frameInstrumentTitle->setObjectName("frameInstrumentTitle");
        frameInstrumentTitle->setMaximumSize(QSize(16777215, 28));
        frameInstrumentTitle->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout_4 = new QHBoxLayout(frameInstrumentTitle);
        horizontalLayout_4->setSpacing(8);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(8, 0, 0, 0);
        labelInstrument = new QLabel(frameInstrumentTitle);
        labelInstrument->setObjectName("labelInstrument");
        labelInstrument->setMinimumSize(QSize(13, 16));
        labelInstrument->setMaximumSize(QSize(13, 16));
        labelInstrument->setPixmap(QPixmap(QString::fromUtf8(":/icons/instrument.svg")));
        labelInstrument->setScaledContents(true);

        horizontalLayout_4->addWidget(labelInstrument);

        label_9 = new QLabel(frameInstrumentTitle);
        label_9->setObjectName("label_9");
        sizePolicy1.setHeightForWidth(label_9->sizePolicy().hasHeightForWidth());
        label_9->setSizePolicy(sizePolicy1);
        label_9->setMinimumSize(QSize(13, 0));
        label_9->setFont(font1);

        horizontalLayout_4->addWidget(label_9);

        pushViewInstruments = new QPushButton(frameInstrumentTitle);
        pushViewInstruments->setObjectName("pushViewInstruments");
        pushViewInstruments->setMinimumSize(QSize(0, 28));
        pushViewInstruments->setMaximumSize(QSize(16777215, 28));
        pushViewInstruments->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));

        horizontalLayout_4->addWidget(pushViewInstruments);


        gridLayout->addWidget(frameInstrumentTitle, 0, 2, 1, 1);

        gridLayout->setColumnStretch(0, 1);
        gridLayout->setColumnStretch(2, 1);
        gridLayout->setColumnStretch(4, 1);

        verticalLayout_4->addWidget(elementFrame);

        QWidget::setTabOrder(lineEdit_name, lineEdit_author);
        QWidget::setTabOrder(lineEdit_author, lineEdit_date);
        QWidget::setTabOrder(lineEdit_date, lineEdit_product);
        QWidget::setTabOrder(lineEdit_product, lineEdit_copyright);
        QWidget::setTabOrder(lineEdit_copyright, textEdit_Com);
        QWidget::setTabOrder(textEdit_Com, pushViewSamples);
        QWidget::setTabOrder(pushViewSamples, comboBitDepth);
        QWidget::setTabOrder(comboBitDepth, comboCompression);
        QWidget::setTabOrder(comboCompression, pushViewInstruments);
        QWidget::setTabOrder(pushViewInstruments, pushViewPresets);

        retranslateUi(PageSf2);
        QObject::connect(lineEdit_name, SIGNAL(editingFinished()), PageSf2, SLOT(setName()));
        QObject::connect(textEdit_Com, SIGNAL(editingFinished()), PageSf2, SLOT(setCommentaire()));
        QObject::connect(lineEdit_product, SIGNAL(editingFinished()), PageSf2, SLOT(setProduct()));
        QObject::connect(lineEdit_author, SIGNAL(editingFinished()), PageSf2, SLOT(setAuthor()));
        QObject::connect(lineEdit_copyright, SIGNAL(editingFinished()), PageSf2, SLOT(setCopyright()));
        QObject::connect(lineEdit_date, SIGNAL(editingFinished()), PageSf2, SLOT(setDate()));

        QMetaObject::connectSlotsByName(PageSf2);
    } // setupUi

    void retranslateUi(QWidget *PageSf2)
    {
        lineEdit_name->setPlaceholderText(QCoreApplication::translate("PageSf2", "Title...", nullptr));
#if QT_CONFIG(tooltip)
        labelUser->setToolTip(QCoreApplication::translate("PageSf2", "Author", nullptr));
#endif // QT_CONFIG(tooltip)
        labelUser->setText(QString());
        lineEdit_author->setPlaceholderText(QCoreApplication::translate("PageSf2", "Author...", nullptr));
#if QT_CONFIG(tooltip)
        labelDate->setToolTip(QCoreApplication::translate("PageSf2", "Date", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDate->setText(QString());
        lineEdit_date->setPlaceholderText(QCoreApplication::translate("PageSf2", "Date...", nullptr));
        label_25->setText(QCoreApplication::translate("PageSf2", "Product", nullptr));
        label_26->setText(QCoreApplication::translate("PageSf2", "Comments", nullptr));
        label_23->setText(QCoreApplication::translate("PageSf2", "Copyright", nullptr));
        labelPreset->setText(QString());
        label_11->setText(QCoreApplication::translate("PageSf2", "Presets", nullptr));
        pushViewPresets->setText(QCoreApplication::translate("PageSf2", "Details", nullptr));
        labelSample->setText(QString());
        label->setText(QCoreApplication::translate("PageSf2", "Samples", nullptr));
        pushViewSamples->setText(QCoreApplication::translate("PageSf2", "Details", nullptr));
        labelCompression->setText(QCoreApplication::translate("PageSf2", "Compression", nullptr));
        label_7->setText(QCoreApplication::translate("PageSf2", "Count", nullptr));
        labelBitDepth->setText(QCoreApplication::translate("PageSf2", "Bit depth", nullptr));
        comboBitDepth->setItemText(0, QCoreApplication::translate("PageSf2", "16 bits", nullptr));
        comboBitDepth->setItemText(1, QCoreApplication::translate("PageSf2", "24 bits", nullptr));

        comboCompression->setItemText(0, QCoreApplication::translate("PageSf2", "no", nullptr));
        comboCompression->setItemText(1, QCoreApplication::translate("PageSf2", "possible", nullptr));

        label_14->setText(QCoreApplication::translate("PageSf2", "Modulators", nullptr));
        label_10->setText(QCoreApplication::translate("PageSf2", "Count", nullptr));
        label_12->setText(QCoreApplication::translate("PageSf2", "Parameters", nullptr));
        label_8->setText(QCoreApplication::translate("PageSf2", "Parameters", nullptr));
        label_13->setText(QCoreApplication::translate("PageSf2", "Count", nullptr));
        label_15->setText(QCoreApplication::translate("PageSf2", "Modulators", nullptr));
        labelInstrument->setText(QString());
        label_9->setText(QCoreApplication::translate("PageSf2", "Instruments", nullptr));
        pushViewInstruments->setText(QCoreApplication::translate("PageSf2", "Details", nullptr));
        (void)PageSf2;
    } // retranslateUi

};

namespace Ui {
    class PageSf2: public Ui_PageSf2 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGESF2_H
