/********************************************************************************
** Form generated from reading UI file 'editortoolbar.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITORTOOLBAR_H
#define UI_EDITORTOOLBAR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "styledaction.h"

QT_BEGIN_NAMESPACE

class Ui_EditorToolBar
{
public:
    QHBoxLayout *horizontalLayout;
    StyledAction *pushAddSample;
    StyledAction *pushAddInstrument;
    StyledAction *pushAddPreset;
    StyledAction *pushToolBox;
    QFrame *separator_1;
    StyledAction *pushUndo;
    StyledAction *pushRedo;
    QFrame *separator_2;
    StyledAction *pushSave;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *layoutDisplay;
    QFrame *separator_3;
    StyledAction *pushShowRecorder;
    StyledAction *pushShowKeyboard;

    void setupUi(QWidget *EditorToolBar)
    {
        if (EditorToolBar->objectName().isEmpty())
            EditorToolBar->setObjectName("EditorToolBar");
        EditorToolBar->resize(535, 30);
        horizontalLayout = new QHBoxLayout(EditorToolBar);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 2, 2, 2);
        pushAddSample = new StyledAction(EditorToolBar);
        pushAddSample->setObjectName("pushAddSample");

        horizontalLayout->addWidget(pushAddSample);

        pushAddInstrument = new StyledAction(EditorToolBar);
        pushAddInstrument->setObjectName("pushAddInstrument");

        horizontalLayout->addWidget(pushAddInstrument);

        pushAddPreset = new StyledAction(EditorToolBar);
        pushAddPreset->setObjectName("pushAddPreset");

        horizontalLayout->addWidget(pushAddPreset);

        pushToolBox = new StyledAction(EditorToolBar);
        pushToolBox->setObjectName("pushToolBox");

        horizontalLayout->addWidget(pushToolBox);

        separator_1 = new QFrame(EditorToolBar);
        separator_1->setObjectName("separator_1");
        separator_1->setMinimumSize(QSize(8, 0));
        separator_1->setMaximumSize(QSize(8, 16777215));
        separator_1->setFrameShape(QFrame::StyledPanel);
        separator_1->setFrameShadow(QFrame::Plain);

        horizontalLayout->addWidget(separator_1);

        pushUndo = new StyledAction(EditorToolBar);
        pushUndo->setObjectName("pushUndo");

        horizontalLayout->addWidget(pushUndo);

        pushRedo = new StyledAction(EditorToolBar);
        pushRedo->setObjectName("pushRedo");

        horizontalLayout->addWidget(pushRedo);

        separator_2 = new QFrame(EditorToolBar);
        separator_2->setObjectName("separator_2");
        separator_2->setMinimumSize(QSize(8, 0));
        separator_2->setMaximumSize(QSize(8, 16777215));
        separator_2->setFrameShape(QFrame::StyledPanel);
        separator_2->setFrameShadow(QFrame::Plain);

        horizontalLayout->addWidget(separator_2);

        pushSave = new StyledAction(EditorToolBar);
        pushSave->setObjectName("pushSave");

        horizontalLayout->addWidget(pushSave);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        layoutDisplay = new QHBoxLayout();
        layoutDisplay->setObjectName("layoutDisplay");

        horizontalLayout->addLayout(layoutDisplay);

        separator_3 = new QFrame(EditorToolBar);
        separator_3->setObjectName("separator_3");
        separator_3->setMinimumSize(QSize(8, 0));
        separator_3->setMaximumSize(QSize(8, 16777215));
        separator_3->setFrameShape(QFrame::StyledPanel);
        separator_3->setFrameShadow(QFrame::Plain);

        horizontalLayout->addWidget(separator_3);

        pushShowRecorder = new StyledAction(EditorToolBar);
        pushShowRecorder->setObjectName("pushShowRecorder");
        pushShowRecorder->setCheckable(true);

        horizontalLayout->addWidget(pushShowRecorder);

        pushShowKeyboard = new StyledAction(EditorToolBar);
        pushShowKeyboard->setObjectName("pushShowKeyboard");
        pushShowKeyboard->setCheckable(true);

        horizontalLayout->addWidget(pushShowKeyboard);


        retranslateUi(EditorToolBar);

        QMetaObject::connectSlotsByName(EditorToolBar);
    } // setupUi

    void retranslateUi(QWidget *EditorToolBar)
    {
        (void)EditorToolBar;
    } // retranslateUi

};

namespace Ui {
    class EditorToolBar: public Ui_EditorToolBar {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITORTOOLBAR_H
