/********************************************************************************
** Form generated from reading UI file 'dialogselection.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGSELECTION_H
#define UI_DIALOGSELECTION_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_DialogSelection
{
public:
    QGridLayout *gridLayout;
    QListWidget *listWidget;
    QPushButton *pushSelectAll;
    QPushButton *pushSelectNone;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;

    void setupUi(QDialog *DialogSelection)
    {
        if (DialogSelection->objectName().isEmpty())
            DialogSelection->setObjectName("DialogSelection");
        DialogSelection->setWindowModality(Qt::ApplicationModal);
        DialogSelection->resize(206, 339);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogSelection->setWindowIcon(icon);
        gridLayout = new QGridLayout(DialogSelection);
        gridLayout->setObjectName("gridLayout");
        listWidget = new QListWidget(DialogSelection);
        listWidget->setObjectName("listWidget");

        gridLayout->addWidget(listWidget, 2, 0, 1, 2);

        pushSelectAll = new QPushButton(DialogSelection);
        pushSelectAll->setObjectName("pushSelectAll");

        gridLayout->addWidget(pushSelectAll, 0, 0, 1, 1);

        pushSelectNone = new QPushButton(DialogSelection);
        pushSelectNone->setObjectName("pushSelectNone");

        gridLayout->addWidget(pushSelectNone, 0, 1, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushOk = new QPushButton(DialogSelection);
        pushOk->setObjectName("pushOk");

        horizontalLayout_2->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        pushCancel = new QPushButton(DialogSelection);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout_2->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout_2, 4, 0, 1, 2);

        QWidget::setTabOrder(pushSelectAll, pushSelectNone);
        QWidget::setTabOrder(pushSelectNone, listWidget);
        QWidget::setTabOrder(listWidget, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(DialogSelection);

        QMetaObject::connectSlotsByName(DialogSelection);
    } // setupUi

    void retranslateUi(QDialog *DialogSelection)
    {
        DialogSelection->setWindowTitle(QCoreApplication::translate("DialogSelection", "Duplicate", nullptr));
        pushSelectAll->setText(QCoreApplication::translate("DialogSelection", "Select all", nullptr));
        pushSelectNone->setText(QCoreApplication::translate("DialogSelection", "Unselect all", nullptr));
        pushOk->setText(QCoreApplication::translate("DialogSelection", "&Duplicate", nullptr));
        pushCancel->setText(QCoreApplication::translate("DialogSelection", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogSelection: public Ui_DialogSelection {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGSELECTION_H
