/********************************************************************************
** Form generated from reading UI file 'soundfonteditorleft.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTEDITORLEFT_H
#define UI_SOUNDFONTEDITORLEFT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "filterflow.h"
#include "filtertag.h"
#include "smallcombobox.h"
#include "styledlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontEditorLeft
{
public:
    QGridLayout *gridLayout;
    FilterFlow *filterArticulation;
    QLabel *labelDate;
    QLabel *labelTimbre;
    QLabel *labelAuthor;
    FilterFlow *filterSampleSource;
    QLabel *labelGenre;
    QLabel *iconLicense;
    QLabel *labelArticulation;
    FilterFlow *filterGenre;
    FilterTag *filterTag;
    QLabel *labelTags;
    FilterFlow *filterMidiStandard;
    SmallComboBox *comboCategory;
    QLabel *iconDate;
    QLabel *iconWebsite;
    SmallComboBox *comboLicense;
    QSpacerItem *verticalSpacer_2;
    QLabel *labelCategory;
    StyledLineEdit *lineWebsite;
    QLabel *labelSampleSource;
    FilterFlow *filterTimbre;
    QSpacerItem *verticalSpacer;
    QLabel *iconAuthor;
    QLabel *labelMidiStandard;

    void setupUi(QWidget *SoundfontEditorLeft)
    {
        if (SoundfontEditorLeft->objectName().isEmpty())
            SoundfontEditorLeft->setObjectName("SoundfontEditorLeft");
        SoundfontEditorLeft->resize(203, 479);
        SoundfontEditorLeft->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SoundfontEditorLeft);
        gridLayout->setObjectName("gridLayout");
        filterArticulation = new FilterFlow(SoundfontEditorLeft);
        filterArticulation->setObjectName("filterArticulation");

        gridLayout->addWidget(filterArticulation, 12, 0, 1, 2);

        labelDate = new QLabel(SoundfontEditorLeft);
        labelDate->setObjectName("labelDate");
        labelDate->setText(QString::fromUtf8("labelDate"));

        gridLayout->addWidget(labelDate, 1, 1, 1, 1);

        labelTimbre = new QLabel(SoundfontEditorLeft);
        labelTimbre->setObjectName("labelTimbre");
        QFont font;
        font.setBold(true);
        labelTimbre->setFont(font);

        gridLayout->addWidget(labelTimbre, 9, 0, 1, 2);

        labelAuthor = new QLabel(SoundfontEditorLeft);
        labelAuthor->setObjectName("labelAuthor");
        labelAuthor->setText(QString::fromUtf8("labelAuthor"));
        labelAuthor->setWordWrap(true);

        gridLayout->addWidget(labelAuthor, 0, 1, 1, 1);

        filterSampleSource = new FilterFlow(SoundfontEditorLeft);
        filterSampleSource->setObjectName("filterSampleSource");

        gridLayout->addWidget(filterSampleSource, 8, 0, 1, 2);

        labelGenre = new QLabel(SoundfontEditorLeft);
        labelGenre->setObjectName("labelGenre");
        labelGenre->setFont(font);

        gridLayout->addWidget(labelGenre, 13, 0, 1, 2);

        iconLicense = new QLabel(SoundfontEditorLeft);
        iconLicense->setObjectName("iconLicense");
        iconLicense->setMinimumSize(QSize(16, 16));
        iconLicense->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconLicense, 2, 0, 1, 1);

        labelArticulation = new QLabel(SoundfontEditorLeft);
        labelArticulation->setObjectName("labelArticulation");
        labelArticulation->setFont(font);

        gridLayout->addWidget(labelArticulation, 11, 0, 1, 2);

        filterGenre = new FilterFlow(SoundfontEditorLeft);
        filterGenre->setObjectName("filterGenre");

        gridLayout->addWidget(filterGenre, 14, 0, 1, 2);

        filterTag = new FilterTag(SoundfontEditorLeft);
        filterTag->setObjectName("filterTag");

        gridLayout->addWidget(filterTag, 18, 0, 1, 2);

        labelTags = new QLabel(SoundfontEditorLeft);
        labelTags->setObjectName("labelTags");
        labelTags->setFont(font);

        gridLayout->addWidget(labelTags, 17, 0, 1, 2);

        filterMidiStandard = new FilterFlow(SoundfontEditorLeft);
        filterMidiStandard->setObjectName("filterMidiStandard");

        gridLayout->addWidget(filterMidiStandard, 16, 0, 1, 2);

        comboCategory = new SmallComboBox(SoundfontEditorLeft);
        comboCategory->setObjectName("comboCategory");

        gridLayout->addWidget(comboCategory, 6, 0, 1, 2);

        iconDate = new QLabel(SoundfontEditorLeft);
        iconDate->setObjectName("iconDate");
        iconDate->setMinimumSize(QSize(16, 16));
        iconDate->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconDate, 1, 0, 1, 1);

        iconWebsite = new QLabel(SoundfontEditorLeft);
        iconWebsite->setObjectName("iconWebsite");

        gridLayout->addWidget(iconWebsite, 3, 0, 1, 1);

        comboLicense = new SmallComboBox(SoundfontEditorLeft);
        comboLicense->setObjectName("comboLicense");

        gridLayout->addWidget(comboLicense, 2, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(verticalSpacer_2, 4, 0, 1, 1);

        labelCategory = new QLabel(SoundfontEditorLeft);
        labelCategory->setObjectName("labelCategory");
        labelCategory->setFont(font);

        gridLayout->addWidget(labelCategory, 5, 0, 1, 2);

        lineWebsite = new StyledLineEdit(SoundfontEditorLeft);
        lineWebsite->setObjectName("lineWebsite");

        gridLayout->addWidget(lineWebsite, 3, 1, 1, 1);

        labelSampleSource = new QLabel(SoundfontEditorLeft);
        labelSampleSource->setObjectName("labelSampleSource");
        labelSampleSource->setFont(font);

        gridLayout->addWidget(labelSampleSource, 7, 0, 1, 2);

        filterTimbre = new FilterFlow(SoundfontEditorLeft);
        filterTimbre->setObjectName("filterTimbre");

        gridLayout->addWidget(filterTimbre, 10, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 19, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 19, 0, 1, 1);

        iconAuthor = new QLabel(SoundfontEditorLeft);
        iconAuthor->setObjectName("iconAuthor");
        iconAuthor->setMinimumSize(QSize(16, 16));
        iconAuthor->setMaximumSize(QSize(16, 16));

        gridLayout->addWidget(iconAuthor, 0, 0, 1, 1);

        labelMidiStandard = new QLabel(SoundfontEditorLeft);
        labelMidiStandard->setObjectName("labelMidiStandard");
        labelMidiStandard->setFont(font);

        gridLayout->addWidget(labelMidiStandard, 15, 0, 1, 2);


        retranslateUi(SoundfontEditorLeft);

        QMetaObject::connectSlotsByName(SoundfontEditorLeft);
    } // setupUi

    void retranslateUi(QWidget *SoundfontEditorLeft)
    {
        labelTimbre->setText(QCoreApplication::translate("SoundfontEditorLeft", "Timbre", nullptr));
        labelGenre->setText(QCoreApplication::translate("SoundfontEditorLeft", "Genre", nullptr));
        iconLicense->setText(QString());
        labelArticulation->setText(QCoreApplication::translate("SoundfontEditorLeft", "Articulation", nullptr));
        labelTags->setText(QCoreApplication::translate("SoundfontEditorLeft", "Tags", nullptr));
        iconDate->setText(QString());
        iconWebsite->setText(QString());
        labelCategory->setText(QCoreApplication::translate("SoundfontEditorLeft", "Category", nullptr));
        lineWebsite->setPlaceholderText(QCoreApplication::translate("SoundfontEditorLeft", "Website...", nullptr));
        labelSampleSource->setText(QCoreApplication::translate("SoundfontEditorLeft", "Sample source", nullptr));
        iconAuthor->setText(QString());
        labelMidiStandard->setText(QCoreApplication::translate("SoundfontEditorLeft", "MIDI standard", nullptr));
        (void)SoundfontEditorLeft;
    } // retranslateUi

};

namespace Ui {
    class SoundfontEditorLeft: public Ui_SoundfontEditorLeft {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTEDITORLEFT_H
