/********************************************************************************
** Form generated from reading UI file 'equalizer.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EQUALIZER_H
#define UI_EQUALIZER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Equalizer
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_3;
    QSlider *verticalSlider_9;
    QLabel *label_12;
    QLabel *label_15;
    QLabel *label_13;
    QSlider *verticalSlider_2;
    QSlider *verticalSlider_3;
    QSlider *verticalSlider_4;
    QLabel *label_8;
    QLabel *label_9;
    QSlider *verticalSlider_1;
    QLabel *label_16;
    QSlider *verticalSlider_7;
    QSlider *verticalSlider_8;
    QLabel *label_10;
    QSlider *verticalSlider_5;
    QSlider *verticalSlider_6;
    QLabel *label_11;
    QLabel *label_14;
    QSlider *verticalSlider_10;
    QLabel *label_17;
    QFrame *frame_9;
    QHBoxLayout *horizontalLayout;
    QCheckBox *checkEqualizerPreview;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *pushEgalRestore;
    QPushButton *pushEgaliser;

    void setupUi(QWidget *Equalizer)
    {
        if (Equalizer->objectName().isEmpty())
            Equalizer->setObjectName("Equalizer");
        Equalizer->resize(260, 120);
        Equalizer->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(Equalizer);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setHorizontalSpacing(2);
        verticalSlider_9 = new QSlider(Equalizer);
        verticalSlider_9->setObjectName("verticalSlider_9");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(verticalSlider_9->sizePolicy().hasHeightForWidth());
        verticalSlider_9->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_9->setToolTip(QString::fromUtf8("8000 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_9->setMinimum(-15);
        verticalSlider_9->setMaximum(15);
        verticalSlider_9->setPageStep(1);
        verticalSlider_9->setOrientation(Qt::Vertical);
        verticalSlider_9->setTickPosition(QSlider::TicksBelow);
        verticalSlider_9->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_9, 0, 8, 1, 1);

        label_12 = new QLabel(Equalizer);
        label_12->setObjectName("label_12");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_12->sizePolicy().hasHeightForWidth());
        label_12->setSizePolicy(sizePolicy1);
        QFont font;
        font.setPointSize(9);
        label_12->setFont(font);
        label_12->setText(QString::fromUtf8("500"));
        label_12->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_12, 1, 4, 1, 1);

        label_15 = new QLabel(Equalizer);
        label_15->setObjectName("label_15");
        sizePolicy1.setHeightForWidth(label_15->sizePolicy().hasHeightForWidth());
        label_15->setSizePolicy(sizePolicy1);
        label_15->setFont(font);
        label_15->setText(QString::fromUtf8("4k"));
        label_15->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_15, 1, 7, 1, 1);

        label_13 = new QLabel(Equalizer);
        label_13->setObjectName("label_13");
        sizePolicy1.setHeightForWidth(label_13->sizePolicy().hasHeightForWidth());
        label_13->setSizePolicy(sizePolicy1);
        label_13->setFont(font);
        label_13->setText(QString::fromUtf8("1k"));
        label_13->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_13, 1, 5, 1, 1);

        verticalSlider_2 = new QSlider(Equalizer);
        verticalSlider_2->setObjectName("verticalSlider_2");
        sizePolicy.setHeightForWidth(verticalSlider_2->sizePolicy().hasHeightForWidth());
        verticalSlider_2->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_2->setToolTip(QString::fromUtf8("64 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_2->setMinimum(-15);
        verticalSlider_2->setMaximum(15);
        verticalSlider_2->setPageStep(1);
        verticalSlider_2->setOrientation(Qt::Vertical);
        verticalSlider_2->setTickPosition(QSlider::TicksBelow);
        verticalSlider_2->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_2, 0, 1, 1, 1);

        verticalSlider_3 = new QSlider(Equalizer);
        verticalSlider_3->setObjectName("verticalSlider_3");
        sizePolicy.setHeightForWidth(verticalSlider_3->sizePolicy().hasHeightForWidth());
        verticalSlider_3->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_3->setToolTip(QString::fromUtf8("125 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_3->setMinimum(-15);
        verticalSlider_3->setMaximum(15);
        verticalSlider_3->setPageStep(1);
        verticalSlider_3->setValue(0);
        verticalSlider_3->setOrientation(Qt::Vertical);
        verticalSlider_3->setTickPosition(QSlider::TicksBelow);
        verticalSlider_3->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_3, 0, 2, 1, 1);

        verticalSlider_4 = new QSlider(Equalizer);
        verticalSlider_4->setObjectName("verticalSlider_4");
        sizePolicy.setHeightForWidth(verticalSlider_4->sizePolicy().hasHeightForWidth());
        verticalSlider_4->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_4->setToolTip(QString::fromUtf8("250 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_4->setMinimum(-15);
        verticalSlider_4->setMaximum(15);
        verticalSlider_4->setPageStep(1);
        verticalSlider_4->setOrientation(Qt::Vertical);
        verticalSlider_4->setTickPosition(QSlider::TicksBelow);
        verticalSlider_4->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_4, 0, 3, 1, 1);

        label_8 = new QLabel(Equalizer);
        label_8->setObjectName("label_8");
        sizePolicy1.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy1);
        label_8->setFont(font);
        label_8->setText(QString::fromUtf8("32"));
        label_8->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_8, 1, 0, 1, 1);

        label_9 = new QLabel(Equalizer);
        label_9->setObjectName("label_9");
        sizePolicy1.setHeightForWidth(label_9->sizePolicy().hasHeightForWidth());
        label_9->setSizePolicy(sizePolicy1);
        label_9->setFont(font);
        label_9->setText(QString::fromUtf8("64"));
        label_9->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_9, 1, 1, 1, 1);

        verticalSlider_1 = new QSlider(Equalizer);
        verticalSlider_1->setObjectName("verticalSlider_1");
        sizePolicy.setHeightForWidth(verticalSlider_1->sizePolicy().hasHeightForWidth());
        verticalSlider_1->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_1->setToolTip(QString::fromUtf8("32 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_1->setMinimum(-15);
        verticalSlider_1->setMaximum(15);
        verticalSlider_1->setPageStep(1);
        verticalSlider_1->setOrientation(Qt::Vertical);
        verticalSlider_1->setTickPosition(QSlider::TicksBelow);
        verticalSlider_1->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_1, 0, 0, 1, 1);

        label_16 = new QLabel(Equalizer);
        label_16->setObjectName("label_16");
        sizePolicy1.setHeightForWidth(label_16->sizePolicy().hasHeightForWidth());
        label_16->setSizePolicy(sizePolicy1);
        label_16->setFont(font);
        label_16->setText(QString::fromUtf8("8k"));
        label_16->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_16, 1, 8, 1, 1);

        verticalSlider_7 = new QSlider(Equalizer);
        verticalSlider_7->setObjectName("verticalSlider_7");
        sizePolicy.setHeightForWidth(verticalSlider_7->sizePolicy().hasHeightForWidth());
        verticalSlider_7->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_7->setToolTip(QString::fromUtf8("2000 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_7->setMinimum(-15);
        verticalSlider_7->setMaximum(15);
        verticalSlider_7->setPageStep(1);
        verticalSlider_7->setOrientation(Qt::Vertical);
        verticalSlider_7->setTickPosition(QSlider::TicksBelow);
        verticalSlider_7->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_7, 0, 6, 1, 1);

        verticalSlider_8 = new QSlider(Equalizer);
        verticalSlider_8->setObjectName("verticalSlider_8");
        sizePolicy.setHeightForWidth(verticalSlider_8->sizePolicy().hasHeightForWidth());
        verticalSlider_8->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_8->setToolTip(QString::fromUtf8("4000 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_8->setMinimum(-15);
        verticalSlider_8->setMaximum(15);
        verticalSlider_8->setPageStep(1);
        verticalSlider_8->setOrientation(Qt::Vertical);
        verticalSlider_8->setTickPosition(QSlider::TicksBelow);
        verticalSlider_8->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_8, 0, 7, 1, 1);

        label_10 = new QLabel(Equalizer);
        label_10->setObjectName("label_10");
        sizePolicy1.setHeightForWidth(label_10->sizePolicy().hasHeightForWidth());
        label_10->setSizePolicy(sizePolicy1);
        label_10->setFont(font);
        label_10->setText(QString::fromUtf8("125"));
        label_10->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_10, 1, 2, 1, 1);

        verticalSlider_5 = new QSlider(Equalizer);
        verticalSlider_5->setObjectName("verticalSlider_5");
        sizePolicy.setHeightForWidth(verticalSlider_5->sizePolicy().hasHeightForWidth());
        verticalSlider_5->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_5->setToolTip(QString::fromUtf8("500 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_5->setMinimum(-15);
        verticalSlider_5->setMaximum(15);
        verticalSlider_5->setPageStep(1);
        verticalSlider_5->setOrientation(Qt::Vertical);
        verticalSlider_5->setTickPosition(QSlider::TicksBelow);
        verticalSlider_5->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_5, 0, 4, 1, 1);

        verticalSlider_6 = new QSlider(Equalizer);
        verticalSlider_6->setObjectName("verticalSlider_6");
        sizePolicy.setHeightForWidth(verticalSlider_6->sizePolicy().hasHeightForWidth());
        verticalSlider_6->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_6->setToolTip(QString::fromUtf8("1000 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_6->setMinimum(-15);
        verticalSlider_6->setMaximum(15);
        verticalSlider_6->setPageStep(1);
        verticalSlider_6->setOrientation(Qt::Vertical);
        verticalSlider_6->setTickPosition(QSlider::TicksBelow);
        verticalSlider_6->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_6, 0, 5, 1, 1);

        label_11 = new QLabel(Equalizer);
        label_11->setObjectName("label_11");
        sizePolicy1.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy1);
        label_11->setFont(font);
        label_11->setText(QString::fromUtf8("250"));
        label_11->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_11, 1, 3, 1, 1);

        label_14 = new QLabel(Equalizer);
        label_14->setObjectName("label_14");
        sizePolicy1.setHeightForWidth(label_14->sizePolicy().hasHeightForWidth());
        label_14->setSizePolicy(sizePolicy1);
        label_14->setFont(font);
        label_14->setText(QString::fromUtf8("2k"));
        label_14->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_14, 1, 6, 1, 1);

        verticalSlider_10 = new QSlider(Equalizer);
        verticalSlider_10->setObjectName("verticalSlider_10");
        sizePolicy.setHeightForWidth(verticalSlider_10->sizePolicy().hasHeightForWidth());
        verticalSlider_10->setSizePolicy(sizePolicy);
#if QT_CONFIG(tooltip)
        verticalSlider_10->setToolTip(QString::fromUtf8("16000 Hz"));
#endif // QT_CONFIG(tooltip)
        verticalSlider_10->setMinimum(-15);
        verticalSlider_10->setMaximum(15);
        verticalSlider_10->setPageStep(1);
        verticalSlider_10->setOrientation(Qt::Vertical);
        verticalSlider_10->setTickPosition(QSlider::TicksBelow);
        verticalSlider_10->setTickInterval(5);

        gridLayout_3->addWidget(verticalSlider_10, 0, 9, 1, 1);

        label_17 = new QLabel(Equalizer);
        label_17->setObjectName("label_17");
        sizePolicy1.setHeightForWidth(label_17->sizePolicy().hasHeightForWidth());
        label_17->setSizePolicy(sizePolicy1);
        label_17->setFont(font);
        label_17->setText(QString::fromUtf8("16k"));
        label_17->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(label_17, 1, 9, 1, 1);

        gridLayout_3->setColumnStretch(0, 1);
        gridLayout_3->setColumnStretch(1, 1);
        gridLayout_3->setColumnStretch(2, 1);
        gridLayout_3->setColumnStretch(3, 1);
        gridLayout_3->setColumnStretch(4, 1);
        gridLayout_3->setColumnStretch(5, 1);
        gridLayout_3->setColumnStretch(6, 1);
        gridLayout_3->setColumnStretch(7, 1);
        gridLayout_3->setColumnStretch(8, 1);
        gridLayout_3->setColumnStretch(9, 1);

        verticalLayout->addLayout(gridLayout_3);

        frame_9 = new QFrame(Equalizer);
        frame_9->setObjectName("frame_9");
        frame_9->setFrameShape(QFrame::NoFrame);
        frame_9->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(frame_9);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(3, 6, 0, 0);
        checkEqualizerPreview = new QCheckBox(frame_9);
        checkEqualizerPreview->setObjectName("checkEqualizerPreview");

        horizontalLayout->addWidget(checkEqualizerPreview);

        horizontalSpacer_2 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        pushEgalRestore = new QPushButton(frame_9);
        pushEgalRestore->setObjectName("pushEgalRestore");
        sizePolicy1.setHeightForWidth(pushEgalRestore->sizePolicy().hasHeightForWidth());
        pushEgalRestore->setSizePolicy(sizePolicy1);
        pushEgalRestore->setMaximumSize(QSize(16777215, 27));
        pushEgalRestore->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/reset_eq.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushEgalRestore->setIcon(icon);

        horizontalLayout->addWidget(pushEgalRestore);

        pushEgaliser = new QPushButton(frame_9);
        pushEgaliser->setObjectName("pushEgaliser");
        sizePolicy1.setHeightForWidth(pushEgaliser->sizePolicy().hasHeightForWidth());
        pushEgaliser->setSizePolicy(sizePolicy1);
        pushEgaliser->setMaximumSize(QSize(16777215, 27));
        pushEgaliser->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/check.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushEgaliser->setIcon(icon1);

        horizontalLayout->addWidget(pushEgaliser);


        verticalLayout->addWidget(frame_9);

        QWidget::setTabOrder(verticalSlider_1, verticalSlider_2);
        QWidget::setTabOrder(verticalSlider_2, verticalSlider_3);
        QWidget::setTabOrder(verticalSlider_3, verticalSlider_4);
        QWidget::setTabOrder(verticalSlider_4, verticalSlider_5);
        QWidget::setTabOrder(verticalSlider_5, verticalSlider_6);
        QWidget::setTabOrder(verticalSlider_6, verticalSlider_7);
        QWidget::setTabOrder(verticalSlider_7, verticalSlider_8);
        QWidget::setTabOrder(verticalSlider_8, verticalSlider_9);
        QWidget::setTabOrder(verticalSlider_9, verticalSlider_10);
        QWidget::setTabOrder(verticalSlider_10, checkEqualizerPreview);
        QWidget::setTabOrder(checkEqualizerPreview, pushEgalRestore);
        QWidget::setTabOrder(pushEgalRestore, pushEgaliser);

        retranslateUi(Equalizer);
        QObject::connect(verticalSlider_1, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_2, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_3, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_5, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_7, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_4, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_6, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_8, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_9, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));
        QObject::connect(verticalSlider_10, SIGNAL(valueChanged(int)), Equalizer, SLOT(on_verticalSliderMoved(int)));

        QMetaObject::connectSlotsByName(Equalizer);
    } // setupUi

    void retranslateUi(QWidget *Equalizer)
    {
        checkEqualizerPreview->setText(QCoreApplication::translate("Equalizer", "preview", nullptr));
#if QT_CONFIG(tooltip)
        pushEgalRestore->setToolTip(QCoreApplication::translate("Equalizer", "Reset", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushEgaliser->setToolTip(QCoreApplication::translate("Equalizer", "Apply", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)Equalizer;
    } // retranslateUi

};

namespace Ui {
    class Equalizer: public Ui_Equalizer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EQUALIZER_H
