/********************************************************************************
** Form generated from reading UI file 'downloadprogresscell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOWNLOADPROGRESSCELL_H
#define UI_DOWNLOADPROGRESSCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DownloadProgressCell
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *iconFile;
    QLabel *labelTitle;
    QSpacerItem *horizontalSpacer;
    QLabel *labelPercent;
    QPushButton *pushCancel;

    void setupUi(QWidget *DownloadProgressCell)
    {
        if (DownloadProgressCell->objectName().isEmpty())
            DownloadProgressCell->setObjectName("DownloadProgressCell");
        DownloadProgressCell->resize(327, 30);
        DownloadProgressCell->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(DownloadProgressCell);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(10, 5, 15, 5);
        iconFile = new QLabel(DownloadProgressCell);
        iconFile->setObjectName("iconFile");
        iconFile->setMinimumSize(QSize(26, 0));
        iconFile->setMaximumSize(QSize(16777215, 18));
        iconFile->setText(QString::fromUtf8(""));
        iconFile->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(iconFile);

        labelTitle = new QLabel(DownloadProgressCell);
        labelTitle->setObjectName("labelTitle");
        labelTitle->setText(QString::fromUtf8("Soundfont title"));

        horizontalLayout->addWidget(labelTitle);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        labelPercent = new QLabel(DownloadProgressCell);
        labelPercent->setObjectName("labelPercent");
        QFont font;
        font.setBold(true);
        labelPercent->setFont(font);
        labelPercent->setText(QString::fromUtf8("Percent"));
        labelPercent->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(labelPercent);

        pushCancel = new QPushButton(DownloadProgressCell);
        pushCancel->setObjectName("pushCancel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pushCancel->sizePolicy().hasHeightForWidth());
        pushCancel->setSizePolicy(sizePolicy);
        pushCancel->setMinimumSize(QSize(26, 0));
        pushCancel->setMaximumSize(QSize(26, 20));
        pushCancel->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushCancel->setFlat(true);

        horizontalLayout->addWidget(pushCancel);


        retranslateUi(DownloadProgressCell);

        QMetaObject::connectSlotsByName(DownloadProgressCell);
    } // setupUi

    void retranslateUi(QWidget *DownloadProgressCell)
    {
#if QT_CONFIG(tooltip)
        pushCancel->setToolTip(QCoreApplication::translate("DownloadProgressCell", "Cancel download", nullptr));
#endif // QT_CONFIG(tooltip)
        pushCancel->setText(QString());
        (void)DownloadProgressCell;
    } // retranslateUi

};

namespace Ui {
    class DownloadProgressCell: public Ui_DownloadProgressCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DOWNLOADPROGRESSCELL_H
