/********************************************************************************
** Form generated from reading UI file 'toolmixturecreation_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLMIXTURECREATION_GUI_H
#define UI_TOOLMIXTURECREATION_GUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "spinboxkey.h"

QT_BEGIN_NAMESPACE

class Ui_ToolMixtureCreation_gui
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QLineEdit *lineNom;
    QLabel *label_4;
    QComboBox *comboFreq;
    QCheckBox *checkBouclage;
    QCheckBox *checkStereo;
    QHBoxLayout *horizontalLayout_5;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    SpinBoxKey *spinDivStart;
    QPushButton *pushAddDiv;
    QListWidget *listDivisions;
    QPushButton *pushRemoveDiv;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *horizontalSpacer;
    SpinBoxKey *spinDivEnd;
    QLabel *label;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer_4;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *pushRemoveRank;
    QLabel *label_2;
    QComboBox *comboType1;
    QComboBox *comboType2;
    QPushButton *pushAddRank;
    QSpacerItem *verticalSpacer_2;
    QListWidget *listRangs;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolMixtureCreation_gui)
    {
        if (ToolMixtureCreation_gui->objectName().isEmpty())
            ToolMixtureCreation_gui->setObjectName("ToolMixtureCreation_gui");
        ToolMixtureCreation_gui->resize(540, 261);
        ToolMixtureCreation_gui->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ToolMixtureCreation_gui);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        label_3 = new QLabel(ToolMixtureCreation_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        lineNom = new QLineEdit(ToolMixtureCreation_gui);
        lineNom->setObjectName("lineNom");
        lineNom->setMaxLength(20);

        gridLayout->addWidget(lineNom, 0, 1, 1, 1);

        label_4 = new QLabel(ToolMixtureCreation_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        comboFreq = new QComboBox(ToolMixtureCreation_gui);
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->addItem(QString());
        comboFreq->setObjectName("comboFreq");

        gridLayout->addWidget(comboFreq, 1, 1, 1, 1);

        checkBouclage = new QCheckBox(ToolMixtureCreation_gui);
        checkBouclage->setObjectName("checkBouclage");
        checkBouclage->setChecked(true);

        gridLayout->addWidget(checkBouclage, 1, 2, 1, 1);

        checkStereo = new QCheckBox(ToolMixtureCreation_gui);
        checkStereo->setObjectName("checkStereo");

        gridLayout->addWidget(checkStereo, 0, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(-1, -1, -1, 0);
        groupBox = new QGroupBox(ToolMixtureCreation_gui);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setVerticalSpacing(6);
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        spinDivStart = new SpinBoxKey(groupBox);
        spinDivStart->setObjectName("spinDivStart");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spinDivStart->sizePolicy().hasHeightForWidth());
        spinDivStart->setSizePolicy(sizePolicy);
        spinDivStart->setMaximum(127);
        spinDivStart->setValue(36);

        gridLayout_2->addWidget(spinDivStart, 3, 1, 1, 3);

        pushAddDiv = new QPushButton(groupBox);
        pushAddDiv->setObjectName("pushAddDiv");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushAddDiv->sizePolicy().hasHeightForWidth());
        pushAddDiv->setSizePolicy(sizePolicy1);
        pushAddDiv->setMinimumSize(QSize(24, 0));
        pushAddDiv->setMaximumSize(QSize(24, 24));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/add"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushAddDiv->setIcon(icon);

        gridLayout_2->addWidget(pushAddDiv, 0, 1, 1, 1);

        listDivisions = new QListWidget(groupBox);
        listDivisions->setObjectName("listDivisions");
        listDivisions->setMinimumSize(QSize(150, 0));
        listDivisions->setEditTriggers(QAbstractItemView::NoEditTriggers);

        gridLayout_2->addWidget(listDivisions, 0, 0, 6, 1);

        pushRemoveDiv = new QPushButton(groupBox);
        pushRemoveDiv->setObjectName("pushRemoveDiv");
        sizePolicy1.setHeightForWidth(pushRemoveDiv->sizePolicy().hasHeightForWidth());
        pushRemoveDiv->setSizePolicy(sizePolicy1);
        pushRemoveDiv->setMinimumSize(QSize(24, 0));
        pushRemoveDiv->setMaximumSize(QSize(24, 24));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/remove"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushRemoveDiv->setIcon(icon1);

        gridLayout_2->addWidget(pushRemoveDiv, 0, 2, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 6, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout_2->addItem(verticalSpacer_3, 1, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 3, 1, 1);

        spinDivEnd = new SpinBoxKey(groupBox);
        spinDivEnd->setObjectName("spinDivEnd");
        sizePolicy.setHeightForWidth(spinDivEnd->sizePolicy().hasHeightForWidth());
        spinDivEnd->setSizePolicy(sizePolicy);
        spinDivEnd->setMaximum(127);
        spinDivEnd->setValue(96);

        gridLayout_2->addWidget(spinDivEnd, 4, 1, 1, 3);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 2, 1, 1, 3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 5, 1, 1, 1);

        gridLayout_2->setColumnStretch(0, 100);

        horizontalLayout_5->addWidget(groupBox);

        groupBox_2 = new QGroupBox(ToolMixtureCreation_gui);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_3 = new QGridLayout(groupBox_2);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setVerticalSpacing(6);
        gridLayout_3->setContentsMargins(6, 6, 6, 6);
        verticalSpacer_4 = new QSpacerItem(20, 6, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout_3->addItem(verticalSpacer_4, 1, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 0, 3, 1, 1);

        pushRemoveRank = new QPushButton(groupBox_2);
        pushRemoveRank->setObjectName("pushRemoveRank");
        sizePolicy1.setHeightForWidth(pushRemoveRank->sizePolicy().hasHeightForWidth());
        pushRemoveRank->setSizePolicy(sizePolicy1);
        pushRemoveRank->setMaximumSize(QSize(24, 24));
        pushRemoveRank->setIcon(icon1);

        gridLayout_3->addWidget(pushRemoveRank, 0, 2, 1, 1);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName("label_2");

        gridLayout_3->addWidget(label_2, 2, 1, 1, 3);

        comboType1 = new QComboBox(groupBox_2);
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->addItem(QString());
        comboType1->setObjectName("comboType1");

        gridLayout_3->addWidget(comboType1, 3, 1, 1, 3);

        comboType2 = new QComboBox(groupBox_2);
        comboType2->setObjectName("comboType2");

        gridLayout_3->addWidget(comboType2, 4, 1, 1, 3);

        pushAddRank = new QPushButton(groupBox_2);
        pushAddRank->setObjectName("pushAddRank");
        sizePolicy1.setHeightForWidth(pushAddRank->sizePolicy().hasHeightForWidth());
        pushAddRank->setSizePolicy(sizePolicy1);
        pushAddRank->setMaximumSize(QSize(24, 24));
        pushAddRank->setIcon(icon);

        gridLayout_3->addWidget(pushAddRank, 0, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 5, 1, 1, 1);

        listRangs = new QListWidget(groupBox_2);
        listRangs->setObjectName("listRangs");
        listRangs->setMinimumSize(QSize(150, 0));
        listRangs->setEditTriggers(QAbstractItemView::NoEditTriggers);

        gridLayout_3->addWidget(listRangs, 0, 0, 6, 1);

        gridLayout_3->setColumnStretch(0, 100);

        horizontalLayout_5->addWidget(groupBox_2);


        verticalLayout->addLayout(horizontalLayout_5);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolMixtureCreation_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        pushCancel = new QPushButton(ToolMixtureCreation_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(lineNom, checkStereo);
        QWidget::setTabOrder(checkStereo, comboFreq);
        QWidget::setTabOrder(comboFreq, checkBouclage);
        QWidget::setTabOrder(checkBouclage, listDivisions);
        QWidget::setTabOrder(listDivisions, pushAddDiv);
        QWidget::setTabOrder(pushAddDiv, pushRemoveDiv);
        QWidget::setTabOrder(pushRemoveDiv, spinDivStart);
        QWidget::setTabOrder(spinDivStart, spinDivEnd);
        QWidget::setTabOrder(spinDivEnd, listRangs);
        QWidget::setTabOrder(listRangs, pushAddRank);
        QWidget::setTabOrder(pushAddRank, pushRemoveRank);
        QWidget::setTabOrder(pushRemoveRank, comboType1);
        QWidget::setTabOrder(comboType1, comboType2);
        QWidget::setTabOrder(comboType2, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolMixtureCreation_gui);

        QMetaObject::connectSlotsByName(ToolMixtureCreation_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolMixtureCreation_gui)
    {
        label_3->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Name of the mixture:", nullptr));
        label_4->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Sample creation:", nullptr));
        comboFreq->setItemText(0, QCoreApplication::translate("ToolMixtureCreation_gui", "for every key", nullptr));
        comboFreq->setItemText(1, QCoreApplication::translate("ToolMixtureCreation_gui", "every 3 keys", nullptr));
        comboFreq->setItemText(2, QCoreApplication::translate("ToolMixtureCreation_gui", "every 6 keys", nullptr));

        checkBouclage->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Auto loop", nullptr));
        checkStereo->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Stereo", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ToolMixtureCreation_gui", "Divisions", nullptr));
        pushAddDiv->setText(QString());
        pushRemoveDiv->setText(QString());
        label->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Key range", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ToolMixtureCreation_gui", "Ranks", nullptr));
        pushRemoveRank->setText(QString());
        label_2->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "Type", nullptr));
        comboType1->setItemText(0, QCoreApplication::translate("ToolMixtureCreation_gui", "octave", nullptr));
        comboType1->setItemText(1, QCoreApplication::translate("ToolMixtureCreation_gui", "fifth", nullptr));
        comboType1->setItemText(2, QCoreApplication::translate("ToolMixtureCreation_gui", "third", nullptr));
        comboType1->setItemText(3, QCoreApplication::translate("ToolMixtureCreation_gui", "seventh", nullptr));
        comboType1->setItemText(4, QCoreApplication::translate("ToolMixtureCreation_gui", "ninth", nullptr));
        comboType1->setItemText(5, QCoreApplication::translate("ToolMixtureCreation_gui", "eleventh", nullptr));
        comboType1->setItemText(6, QCoreApplication::translate("ToolMixtureCreation_gui", "thirteenth", nullptr));
        comboType1->setItemText(7, QCoreApplication::translate("ToolMixtureCreation_gui", "fifteenth", nullptr));
        comboType1->setItemText(8, QCoreApplication::translate("ToolMixtureCreation_gui", "seventeenth", nullptr));
        comboType1->setItemText(9, QCoreApplication::translate("ToolMixtureCreation_gui", "nineteenth", nullptr));
        comboType1->setItemText(10, QCoreApplication::translate("ToolMixtureCreation_gui", "twenty-first", nullptr));
        comboType1->setItemText(11, QCoreApplication::translate("ToolMixtureCreation_gui", "twenty-third", nullptr));
        comboType1->setItemText(12, QCoreApplication::translate("ToolMixtureCreation_gui", "twenty-fifth", nullptr));
        comboType1->setItemText(13, QCoreApplication::translate("ToolMixtureCreation_gui", "twenty-seventh", nullptr));

        pushAddRank->setText(QString());
        pushOk->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolMixtureCreation_gui", "&Cancel", nullptr));
        (void)ToolMixtureCreation_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolMixtureCreation_gui: public Ui_ToolMixtureCreation_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLMIXTURECREATION_GUI_H
