/********************************************************************************
** Form generated from reading UI file 'footeroverview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOOTEROVERVIEW_H
#define UI_FOOTEROVERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FooterOverview
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *labelInformation;

    void setupUi(QWidget *FooterOverview)
    {
        if (FooterOverview->objectName().isEmpty())
            FooterOverview->setObjectName("FooterOverview");
        FooterOverview->resize(166, 19);
        FooterOverview->setWindowTitle(QString::fromUtf8(""));
        horizontalLayout = new QHBoxLayout(FooterOverview);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        labelInformation = new QLabel(FooterOverview);
        labelInformation->setObjectName("labelInformation");
        labelInformation->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout->addWidget(labelInformation);


        retranslateUi(FooterOverview);

        QMetaObject::connectSlotsByName(FooterOverview);
    } // setupUi

    void retranslateUi(QWidget *FooterOverview)
    {
        (void)FooterOverview;
    } // retranslateUi

};

namespace Ui {
    class FooterOverview: public Ui_FooterOverview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FOOTEROVERVIEW_H
