/********************************************************************************
** Form generated from reading UI file 'soundfontviewercenter.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTVIEWERCENTER_H
#define UI_SOUNDFONTVIEWERCENTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "customtextedit.h"
#include "soundfontcomment.h"
#include "soundfontdownload.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontViewerCenter
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *labelDescription;
    CustomTextEdit *widgetDescription;
    QLabel *labelDownload;
    SoundfontDownload *widgetDownload;
    QLabel *labelComment;
    SoundfontComment *widgetComment;

    void setupUi(QWidget *SoundfontViewerCenter)
    {
        if (SoundfontViewerCenter->objectName().isEmpty())
            SoundfontViewerCenter->setObjectName("SoundfontViewerCenter");
        SoundfontViewerCenter->resize(304, 204);
        SoundfontViewerCenter->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontViewerCenter);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        labelDescription = new QLabel(SoundfontViewerCenter);
        labelDescription->setObjectName("labelDescription");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelDescription->sizePolicy().hasHeightForWidth());
        labelDescription->setSizePolicy(sizePolicy);
        labelDescription->setMinimumSize(QSize(0, 28));
        QFont font;
        font.setPointSize(13);
        font.setBold(true);
        labelDescription->setFont(font);

        verticalLayout->addWidget(labelDescription);

        widgetDescription = new CustomTextEdit(SoundfontViewerCenter);
        widgetDescription->setObjectName("widgetDescription");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(widgetDescription->sizePolicy().hasHeightForWidth());
        widgetDescription->setSizePolicy(sizePolicy1);
        widgetDescription->setFrameShape(QFrame::NoFrame);

        verticalLayout->addWidget(widgetDescription);

        labelDownload = new QLabel(SoundfontViewerCenter);
        labelDownload->setObjectName("labelDownload");
        sizePolicy.setHeightForWidth(labelDownload->sizePolicy().hasHeightForWidth());
        labelDownload->setSizePolicy(sizePolicy);
        labelDownload->setMinimumSize(QSize(0, 28));
        labelDownload->setFont(font);

        verticalLayout->addWidget(labelDownload);

        widgetDownload = new SoundfontDownload(SoundfontViewerCenter);
        widgetDownload->setObjectName("widgetDownload");
        sizePolicy.setHeightForWidth(widgetDownload->sizePolicy().hasHeightForWidth());
        widgetDownload->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(widgetDownload);

        labelComment = new QLabel(SoundfontViewerCenter);
        labelComment->setObjectName("labelComment");
        sizePolicy.setHeightForWidth(labelComment->sizePolicy().hasHeightForWidth());
        labelComment->setSizePolicy(sizePolicy);
        labelComment->setMinimumSize(QSize(0, 28));
        labelComment->setFont(font);

        verticalLayout->addWidget(labelComment);

        widgetComment = new SoundfontComment(SoundfontViewerCenter);
        widgetComment->setObjectName("widgetComment");
        sizePolicy.setHeightForWidth(widgetComment->sizePolicy().hasHeightForWidth());
        widgetComment->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(widgetComment);


        retranslateUi(SoundfontViewerCenter);

        QMetaObject::connectSlotsByName(SoundfontViewerCenter);
    } // setupUi

    void retranslateUi(QWidget *SoundfontViewerCenter)
    {
        labelDescription->setText(QCoreApplication::translate("SoundfontViewerCenter", "Description", nullptr));
        widgetDescription->setPlaceholderText(QCoreApplication::translate("SoundfontViewerCenter", "No description.", nullptr));
        labelDownload->setText(QCoreApplication::translate("SoundfontViewerCenter", "Downloads", nullptr));
        labelComment->setText(QCoreApplication::translate("SoundfontViewerCenter", "Comments", nullptr));
        (void)SoundfontViewerCenter;
    } // retranslateUi

};

namespace Ui {
    class SoundfontViewerCenter: public Ui_SoundfontViewerCenter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTVIEWERCENTER_H
