/********************************************************************************
** Form generated from reading UI file 'dialogrecorder.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOGRECORDER_H
#define UI_DIALOGRECORDER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLCDNumber>
#include <QtWidgets/QPushButton>

QT_BEGIN_NAMESPACE

class Ui_DialogRecorder
{
public:
    QHBoxLayout *horizontalLayout;
    QPushButton *pushRecord;
    QLCDNumber *lcdNumber;
    QPushButton *pushPlayPause;

    void setupUi(QDialog *DialogRecorder)
    {
        if (DialogRecorder->objectName().isEmpty())
            DialogRecorder->setObjectName("DialogRecorder");
        DialogRecorder->resize(210, 46);
        DialogRecorder->setMinimumSize(QSize(210, 46));
        DialogRecorder->setMaximumSize(QSize(210, 46));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogRecorder->setWindowIcon(icon);
        horizontalLayout = new QHBoxLayout(DialogRecorder);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(3, 3, 3, 3);
        pushRecord = new QPushButton(DialogRecorder);
        pushRecord->setObjectName("pushRecord");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pushRecord->sizePolicy().hasHeightForWidth());
        pushRecord->setSizePolicy(sizePolicy);
        pushRecord->setMinimumSize(QSize(40, 40));
        pushRecord->setMaximumSize(QSize(40, 16777215));
        pushRecord->setFocusPolicy(Qt::NoFocus);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/record"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushRecord->setIcon(icon1);
        pushRecord->setIconSize(QSize(32, 32));
        pushRecord->setFlat(true);

        horizontalLayout->addWidget(pushRecord);

        lcdNumber = new QLCDNumber(DialogRecorder);
        lcdNumber->setObjectName("lcdNumber");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lcdNumber->sizePolicy().hasHeightForWidth());
        lcdNumber->setSizePolicy(sizePolicy1);
        lcdNumber->setFrameShape(QFrame::NoFrame);
        lcdNumber->setDigitCount(8);
        lcdNumber->setMode(QLCDNumber::Dec);
        lcdNumber->setSegmentStyle(QLCDNumber::Flat);

        horizontalLayout->addWidget(lcdNumber);

        pushPlayPause = new QPushButton(DialogRecorder);
        pushPlayPause->setObjectName("pushPlayPause");
        pushPlayPause->setEnabled(false);
        sizePolicy.setHeightForWidth(pushPlayPause->sizePolicy().hasHeightForWidth());
        pushPlayPause->setSizePolicy(sizePolicy);
        pushPlayPause->setMinimumSize(QSize(40, 40));
        pushPlayPause->setMaximumSize(QSize(40, 16777215));
        pushPlayPause->setFocusPolicy(Qt::NoFocus);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/play"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushPlayPause->setIcon(icon2);
        pushPlayPause->setIconSize(QSize(32, 32));
        pushPlayPause->setFlat(true);

        horizontalLayout->addWidget(pushPlayPause);


        retranslateUi(DialogRecorder);

        QMetaObject::connectSlotsByName(DialogRecorder);
    } // setupUi

    void retranslateUi(QDialog *DialogRecorder)
    {
        DialogRecorder->setWindowTitle(QCoreApplication::translate("DialogRecorder", "Recorder", nullptr));
        pushRecord->setText(QString());
        pushPlayPause->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DialogRecorder: public Ui_DialogRecorder {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOGRECORDER_H
