/********************************************************************************
** Form generated from reading UI file 'player.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PLAYER_H
#define UI_PLAYER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "controllerarea.h"
#include "pianokeybdcustom.h"
#include "playerpresetlist.h"
#include "rotatingspinner.h"
#include "styledaction.h"

QT_BEGIN_NAMESPACE

class Ui_Player
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *topBar;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QComboBox *comboChannel;
    QLabel *label_2;
    QComboBox *comboMultipleSelection;
    QLabel *label_3;
    QComboBox *comboSelectionByKeys;
    QSpacerItem *horizontalSpacer_5;
    StyledAction *pushPanic;
    StyledAction *pushShowRecorder;
    QStackedWidget *stackedMain;
    QWidget *pageWait;
    QGridLayout *gridLayout_2;
    RotatingSpinner *rotatingSpinner;
    QSpacerItem *horizontalSpacer;
    QLabel *labelWait;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QWidget *pagePlay;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QLabel *label_4;
    QListView *listBank;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    PlayerPresetList *listPreset;
    QHBoxLayout *horizontalLayout_4;
    ControllerArea *controllerArea;
    QWidget *pageError;
    QGridLayout *gridLayout_3;
    QLabel *iconWarning;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *verticalSpacer_4;
    QLabel *labelError;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *horizontalSpacer_3;
    QLabel *labelReason;
    PianoKeybdCustom *keyboard;

    void setupUi(QWidget *Player)
    {
        if (Player->objectName().isEmpty())
            Player->setObjectName("Player");
        Player->resize(651, 268);
        Player->setAutoFillBackground(true);
        verticalLayout = new QVBoxLayout(Player);
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        topBar = new QWidget(Player);
        topBar->setObjectName("topBar");
        topBar->setMinimumSize(QSize(0, 42));
        topBar->setMaximumSize(QSize(16777215, 42));
        horizontalLayout_2 = new QHBoxLayout(topBar);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, 2, 2, 2);
        label = new QLabel(topBar);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        comboChannel = new QComboBox(topBar);
        comboChannel->addItem(QString());
        comboChannel->addItem(QString::fromUtf8("1"));
        comboChannel->addItem(QString::fromUtf8("2"));
        comboChannel->addItem(QString::fromUtf8("3"));
        comboChannel->addItem(QString::fromUtf8("4"));
        comboChannel->addItem(QString::fromUtf8("5"));
        comboChannel->addItem(QString::fromUtf8("6"));
        comboChannel->addItem(QString::fromUtf8("7"));
        comboChannel->addItem(QString::fromUtf8("8"));
        comboChannel->addItem(QString::fromUtf8("9"));
        comboChannel->addItem(QString::fromUtf8("10"));
        comboChannel->addItem(QString::fromUtf8("11"));
        comboChannel->addItem(QString::fromUtf8("12"));
        comboChannel->addItem(QString::fromUtf8("13"));
        comboChannel->addItem(QString::fromUtf8("14"));
        comboChannel->addItem(QString::fromUtf8("15"));
        comboChannel->addItem(QString::fromUtf8("16"));
        comboChannel->addItem(QString());
        comboChannel->setObjectName("comboChannel");

        horizontalLayout_2->addWidget(comboChannel);

        label_2 = new QLabel(topBar);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        comboMultipleSelection = new QComboBox(topBar);
        comboMultipleSelection->addItem(QString());
        comboMultipleSelection->addItem(QString());
        comboMultipleSelection->setObjectName("comboMultipleSelection");

        horizontalLayout_2->addWidget(comboMultipleSelection);

        label_3 = new QLabel(topBar);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        comboSelectionByKeys = new QComboBox(topBar);
        comboSelectionByKeys->addItem(QString());
        comboSelectionByKeys->addItem(QString());
        comboSelectionByKeys->addItem(QString());
        comboSelectionByKeys->setObjectName("comboSelectionByKeys");

        horizontalLayout_2->addWidget(comboSelectionByKeys);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        pushPanic = new StyledAction(topBar);
        pushPanic->setObjectName("pushPanic");

        horizontalLayout_2->addWidget(pushPanic);

        pushShowRecorder = new StyledAction(topBar);
        pushShowRecorder->setObjectName("pushShowRecorder");
        pushShowRecorder->setCheckable(true);

        horizontalLayout_2->addWidget(pushShowRecorder);


        verticalLayout->addWidget(topBar);

        stackedMain = new QStackedWidget(Player);
        stackedMain->setObjectName("stackedMain");
        stackedMain->setAutoFillBackground(true);
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_2 = new QGridLayout(pageWait);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        rotatingSpinner = new RotatingSpinner(pageWait);
        rotatingSpinner->setObjectName("rotatingSpinner");
        rotatingSpinner->setMinimumSize(QSize(32, 32));

        gridLayout_2->addWidget(rotatingSpinner, 3, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 3, 0, 1, 1);

        labelWait = new QLabel(pageWait);
        labelWait->setObjectName("labelWait");
        labelWait->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelWait, 1, 0, 1, 3);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 3, 2, 1, 1);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 4, 1, 1, 1);

        stackedMain->addWidget(pageWait);
        pagePlay = new QWidget();
        pagePlay->setObjectName("pagePlay");
        verticalLayout_4 = new QVBoxLayout(pagePlay);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(9, -1, 9, -1);
        label_4 = new QLabel(pagePlay);
        label_4->setObjectName("label_4");
        QFont font;
        font.setBold(true);
        label_4->setFont(font);

        horizontalLayout->addWidget(label_4);


        verticalLayout_4->addLayout(horizontalLayout);

        listBank = new QListView(pagePlay);
        listBank->setObjectName("listBank");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(listBank->sizePolicy().hasHeightForWidth());
        listBank->setSizePolicy(sizePolicy);
        listBank->setContextMenuPolicy(Qt::NoContextMenu);
        listBank->setAcceptDrops(true);
        listBank->setFrameShape(QFrame::NoFrame);
        listBank->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listBank->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        listBank->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listBank->setProperty("showDropIndicator", QVariant(false));
        listBank->setDragDropMode(QAbstractItemView::NoDragDrop);
        listBank->setDefaultDropAction(Qt::IgnoreAction);
        listBank->setTextElideMode(Qt::ElideLeft);
        listBank->setHorizontalScrollMode(QAbstractItemView::ScrollPerItem);
        listBank->setFlow(QListView::LeftToRight);
        listBank->setResizeMode(QListView::Fixed);
        listBank->setSpacing(2);
        listBank->setSelectionRectVisible(false);

        verticalLayout_4->addWidget(listBank);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(9, -1, 9, -1);
        label_5 = new QLabel(pagePlay);
        label_5->setObjectName("label_5");
        label_5->setFont(font);

        horizontalLayout_3->addWidget(label_5);


        verticalLayout_4->addLayout(horizontalLayout_3);

        listPreset = new PlayerPresetList(pagePlay);
        listPreset->setObjectName("listPreset");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(listPreset->sizePolicy().hasHeightForWidth());
        listPreset->setSizePolicy(sizePolicy1);
        listPreset->setFrameShape(QFrame::NoFrame);
        listPreset->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        listPreset->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listPreset->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listPreset->setProperty("showDropIndicator", QVariant(false));
        listPreset->setDragDropMode(QAbstractItemView::NoDragDrop);
        listPreset->setDefaultDropAction(Qt::IgnoreAction);
        listPreset->setTextElideMode(Qt::ElideLeft);
        listPreset->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
        listPreset->setFlow(QListView::LeftToRight);
        listPreset->setProperty("isWrapping", QVariant(true));
        listPreset->setSelectionRectVisible(false);

        verticalLayout_4->addWidget(listPreset);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(9, -1, 9, -1);
        controllerArea = new ControllerArea(pagePlay);
        controllerArea->setObjectName("controllerArea");

        horizontalLayout_4->addWidget(controllerArea);


        verticalLayout_4->addLayout(horizontalLayout_4);

        verticalLayout_4->setStretch(1, 1);
        verticalLayout_4->setStretch(3, 9999);
        stackedMain->addWidget(pagePlay);
        pageError = new QWidget();
        pageError->setObjectName("pageError");
        gridLayout_3 = new QGridLayout(pageError);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        iconWarning = new QLabel(pageError);
        iconWarning->setObjectName("iconWarning");
        iconWarning->setMinimumSize(QSize(64, 64));
        iconWarning->setMaximumSize(QSize(64, 64));
        iconWarning->setText(QString::fromUtf8(""));

        gridLayout_3->addWidget(iconWarning, 1, 1, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_4, 1, 2, 1, 1);

        verticalSpacer_4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_4, 4, 1, 1, 1);

        labelError = new QLabel(pageError);
        labelError->setObjectName("labelError");
        labelError->setFont(font);
        labelError->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(labelError, 2, 0, 1, 3);

        verticalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 0, 1, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_3, 1, 0, 1, 1);

        labelReason = new QLabel(pageError);
        labelReason->setObjectName("labelReason");
        labelReason->setText(QString::fromUtf8("reason"));
        labelReason->setAlignment(Qt::AlignCenter);

        gridLayout_3->addWidget(labelReason, 3, 0, 1, 3);

        stackedMain->addWidget(pageError);

        verticalLayout->addWidget(stackedMain);

        keyboard = new PianoKeybdCustom(Player);
        keyboard->setObjectName("keyboard");
        sizePolicy.setHeightForWidth(keyboard->sizePolicy().hasHeightForWidth());
        keyboard->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(keyboard);


        retranslateUi(Player);

        stackedMain->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(Player);
    } // setupUi

    void retranslateUi(QWidget *Player)
    {
        label->setText(QCoreApplication::translate("Player", "Channel", nullptr));
        comboChannel->setItemText(0, QCoreApplication::translate("Player", "all", nullptr));
        comboChannel->setItemText(17, QCoreApplication::translate("Player", "detect...", nullptr));

        label_2->setText(QCoreApplication::translate("Player", "Multiple selection", nullptr));
        comboMultipleSelection->setItemText(0, QCoreApplication::translate("Player", "off", nullptr));
        comboMultipleSelection->setItemText(1, QCoreApplication::translate("Player", "on", nullptr));

        label_3->setText(QCoreApplication::translate("Player", "Selection by keys", nullptr));
        comboSelectionByKeys->setItemText(0, QCoreApplication::translate("Player", "off", nullptr));
        comboSelectionByKeys->setItemText(1, QCoreApplication::translate("Player", "on", nullptr));
        comboSelectionByKeys->setItemText(2, QCoreApplication::translate("Player", "toggle", nullptr));

        labelWait->setText(QCoreApplication::translate("Player", "Opening file...", nullptr));
        label_4->setText(QCoreApplication::translate("Player", "Banks (number of instruments)", nullptr));
        label_5->setText(QCoreApplication::translate("Player", "Instruments", nullptr));
        labelError->setText(QCoreApplication::translate("Player", "Cannot open the file", nullptr));
        (void)Player;
    } // retranslateUi

};

namespace Ui {
    class Player: public Ui_Player {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PLAYER_H
