/****************************************************************************
** Meta object code from reading C++ file 'toprightwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mainwindow/toprightwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toprightwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14TopRightWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TopRightWidget::qt_create_metaobjectdata<qt_meta_tag_ZN14TopRightWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TopRightWidget",
        "openClicked",
        "",
        "newClicked",
        "openSettingsClicked",
        "onlineHelpClicked",
        "aboutClicked",
        "closeTabClicked",
        "closeClicked",
        "minimizeClicked",
        "fullScreenTriggered",
        "save",
        "saveAs",
        "userClicked",
        "onTabOpen",
        "canSave",
        "canExport",
        "canClose",
        "userStateChanged",
        "UserManager::ConnectionState",
        "state",
        "on_pushUser_clicked",
        "downloadCleared",
        "progressChanged",
        "percent",
        "soundfontId",
        "soundfontName",
        "finalFileName",
        "downloadCanceled",
        "menuOpened",
        "menuClosed",
        "on_pushClose_clicked",
        "on_pushMinimize_clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'openClicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newClicked'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openSettingsClicked'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onlineHelpClicked'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutClicked'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeTabClicked'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'closeClicked'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimizeClicked'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullScreenTriggered'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'save'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'saveAs'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'userClicked'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTabOpen'
        QtMocHelpers::SlotData<void(bool, bool, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 }, { QMetaType::Bool, 16 }, { QMetaType::Bool, 17 },
        }}),
        // Slot 'userStateChanged'
        QtMocHelpers::SlotData<void(UserManager::ConnectionState)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'on_pushUser_clicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadCleared'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'progressChanged'
        QtMocHelpers::SlotData<void(int, int, QString, QString)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { QMetaType::Int, 25 }, { QMetaType::QString, 26 }, { QMetaType::QString, 27 },
        }}),
        // Slot 'downloadCanceled'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'menuOpened'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'menuClosed'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushClose_clicked'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_pushMinimize_clicked'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TopRightWidget, qt_meta_tag_ZN14TopRightWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TopRightWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TopRightWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TopRightWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14TopRightWidgetE_t>.metaTypes,
    nullptr
} };

void TopRightWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TopRightWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openClicked(); break;
        case 1: _t->newClicked(); break;
        case 2: _t->openSettingsClicked(); break;
        case 3: _t->onlineHelpClicked(); break;
        case 4: _t->aboutClicked(); break;
        case 5: _t->closeTabClicked(); break;
        case 6: _t->closeClicked(); break;
        case 7: _t->minimizeClicked(); break;
        case 8: _t->fullScreenTriggered(); break;
        case 9: _t->save(); break;
        case 10: _t->saveAs(); break;
        case 11: _t->userClicked(); break;
        case 12: _t->onTabOpen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 13: _t->userStateChanged((*reinterpret_cast<std::add_pointer_t<UserManager::ConnectionState>>(_a[1]))); break;
        case 14: _t->on_pushUser_clicked(); break;
        case 15: _t->downloadCleared(); break;
        case 16: _t->progressChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 17: _t->downloadCanceled((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->menuOpened(); break;
        case 19: _t->menuClosed(); break;
        case 20: _t->on_pushClose_clicked(); break;
        case 21: _t->on_pushMinimize_clicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::openClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::newClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::openSettingsClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::onlineHelpClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::aboutClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::closeTabClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::closeClicked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::minimizeClicked, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::fullScreenTriggered, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::save, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::saveAs, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TopRightWidget::*)()>(_a, &TopRightWidget::userClicked, 11))
            return;
    }
}

const QMetaObject *TopRightWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TopRightWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TopRightWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TopRightWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void TopRightWidget::openClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TopRightWidget::newClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TopRightWidget::openSettingsClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TopRightWidget::onlineHelpClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TopRightWidget::aboutClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TopRightWidget::closeTabClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TopRightWidget::closeClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void TopRightWidget::minimizeClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TopRightWidget::fullScreenTriggered()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TopRightWidget::save()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TopRightWidget::saveAs()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TopRightWidget::userClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
