/****************************************************************************
** Meta object code from reading C++ file 'tabmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mainwindow/tabmanager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10TabManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto TabManager::qt_create_metaobjectdata<qt_meta_tag_ZN10TabManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TabManager",
        "keyboardDisplayChanged",
        "",
        "isDisplayed",
        "propagate",
        "recorderDisplayChanged",
        "tabOpen",
        "canSave",
        "canExport",
        "canClose",
        "openConfiguration",
        "openSoundfont",
        "fileName",
        "PlayerOptions*",
        "playerOptions",
        "async",
        "EltID",
        "initialSelection",
        "openNewSoundfont",
        "openRepository",
        "SoundfontFilter*",
        "filter",
        "openRepositorySoundfont",
        "id",
        "openUser",
        "openDirectory",
        "directoryPath",
        "editingDone",
        "source",
        "QList<int>",
        "sf2Indexes",
        "getCurrentSf2",
        "closeCurrentTab",
        "onTabTitleChanged",
        "title",
        "onFilePathChanged",
        "filePath",
        "onTabCloseRequested",
        "QWidget*",
        "widget",
        "onTabIndexChanged",
        "tabIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyboardDisplayChanged'
        QtMocHelpers::SignalData<void(bool, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'keyboardDisplayChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'recorderDisplayChanged'
        QtMocHelpers::SignalData<void(bool, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'recorderDisplayChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'tabOpen'
        QtMocHelpers::SignalData<void(bool, bool, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 }, { QMetaType::Bool, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'openConfiguration'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openSoundfont'
        QtMocHelpers::SlotData<void(QString, PlayerOptions *, bool, EltID)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'openSoundfont'
        QtMocHelpers::SlotData<void(QString, PlayerOptions *, bool)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 },
        }}),
        // Slot 'openNewSoundfont'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openRepository'
        QtMocHelpers::SlotData<void(SoundfontFilter *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'openRepositorySoundfont'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'openUser'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openDirectory'
        QtMocHelpers::SlotData<void(QString)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'editingDone'
        QtMocHelpers::SlotData<void(QString, QList<int>)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'getCurrentSf2'
        QtMocHelpers::SlotData<int()>(31, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'closeCurrentTab'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTabTitleChanged'
        QtMocHelpers::SlotData<void(QString)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Slot 'onFilePathChanged'
        QtMocHelpers::SlotData<void(QString)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Slot 'onTabCloseRequested'
        QtMocHelpers::SlotData<void(QWidget *)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'onTabIndexChanged'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'openSoundfont'
        QtMocHelpers::SlotData<void(QString, EltID)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { 0x80000000 | 16, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TabManager, qt_meta_tag_ZN10TabManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TabManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TabManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TabManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10TabManagerE_t>.metaTypes,
    nullptr
} };

void TabManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TabManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyboardDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->keyboardDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->recorderDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->recorderDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->tabOpen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 5: _t->openConfiguration(); break;
        case 6: _t->openSoundfont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlayerOptions*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<EltID>>(_a[4]))); break;
        case 7: _t->openSoundfont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PlayerOptions*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 8: _t->openNewSoundfont(); break;
        case 9: _t->openRepository((*reinterpret_cast<std::add_pointer_t<SoundfontFilter*>>(_a[1]))); break;
        case 10: _t->openRepositorySoundfont((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->openUser(); break;
        case 12: _t->openDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->editingDone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 14: { int _r = _t->getCurrentSf2();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->closeCurrentTab(); break;
        case 16: _t->onTabTitleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->onFilePathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->onTabCloseRequested((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 19: _t->onTabIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->openSoundfont((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<EltID>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EltID >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EltID >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TabManager::*)(bool , bool )>(_a, &TabManager::keyboardDisplayChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabManager::*)(bool , bool )>(_a, &TabManager::recorderDisplayChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TabManager::*)(bool , bool , bool )>(_a, &TabManager::tabOpen, 4))
            return;
    }
}

const QMetaObject *TabManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TabManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TabManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TabManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void TabManager::keyboardDisplayChanged(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void TabManager::recorderDisplayChanged(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 4
void TabManager::tabOpen(bool _t1, bool _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
