/********************************************************************************
** Form generated from reading UI file 'editordialoginsertlink.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITORDIALOGINSERTLINK_H
#define UI_EDITORDIALOGINSERTLINK_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_EditorDialogInsertLink
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *label_2;
    QLineEdit *lineText;
    QLineEdit *lineLink;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *EditorDialogInsertLink)
    {
        if (EditorDialogInsertLink->objectName().isEmpty())
            EditorDialogInsertLink->setObjectName("EditorDialogInsertLink");
        EditorDialogInsertLink->resize(298, 138);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        EditorDialogInsertLink->setWindowIcon(icon);
        gridLayout = new QGridLayout(EditorDialogInsertLink);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(EditorDialogInsertLink);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(EditorDialogInsertLink);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        lineText = new QLineEdit(EditorDialogInsertLink);
        lineText->setObjectName("lineText");

        gridLayout->addWidget(lineText, 0, 1, 1, 1);

        lineLink = new QLineEdit(EditorDialogInsertLink);
        lineLink->setObjectName("lineLink");

        gridLayout->addWidget(lineLink, 1, 1, 1, 1);

        buttonBox = new QDialogButtonBox(EditorDialogInsertLink);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 3, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 1, 1, 1);


        retranslateUi(EditorDialogInsertLink);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, EditorDialogInsertLink, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, EditorDialogInsertLink, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(EditorDialogInsertLink);
    } // setupUi

    void retranslateUi(QDialog *EditorDialogInsertLink)
    {
        EditorDialogInsertLink->setWindowTitle(QCoreApplication::translate("EditorDialogInsertLink", "Insert a link", nullptr));
        label->setText(QCoreApplication::translate("EditorDialogInsertLink", "Text to display", nullptr));
        label_2->setText(QCoreApplication::translate("EditorDialogInsertLink", "Link", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditorDialogInsertLink: public Ui_EditorDialogInsertLink {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITORDIALOGINSERTLINK_H
