/********************************************************************************
** Form generated from reading UI file 'toolloopcrossfade_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLLOOPCROSSFADE_GUI_H
#define UI_TOOLLOOPCROSSFADE_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolLoopCrossfade_gui
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushOk;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushCancel;
    QSpinBox *spinDuration;

    void setupUi(QWidget *ToolLoopCrossfade_gui)
    {
        if (ToolLoopCrossfade_gui->objectName().isEmpty())
            ToolLoopCrossfade_gui->setObjectName("ToolLoopCrossfade_gui");
        ToolLoopCrossfade_gui->resize(237, 79);
        ToolLoopCrossfade_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolLoopCrossfade_gui);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(ToolLoopCrossfade_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushOk = new QPushButton(ToolLoopCrossfade_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushCancel = new QPushButton(ToolLoopCrossfade_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 1, 0, 1, 2);

        spinDuration = new QSpinBox(ToolLoopCrossfade_gui);
        spinDuration->setObjectName("spinDuration");
        spinDuration->setMinimum(1);
        spinDuration->setMaximum(2000);
        spinDuration->setValue(100);

        gridLayout->addWidget(spinDuration, 0, 1, 1, 1);


        retranslateUi(ToolLoopCrossfade_gui);

        QMetaObject::connectSlotsByName(ToolLoopCrossfade_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolLoopCrossfade_gui)
    {
        label->setText(QCoreApplication::translate("ToolLoopCrossfade_gui", "Crossfade duration", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolLoopCrossfade_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolLoopCrossfade_gui", "&Cancel", nullptr));
        spinDuration->setSuffix(QCoreApplication::translate("ToolLoopCrossfade_gui", " ms", "Unit for milliseconds"));
        (void)ToolLoopCrossfade_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolLoopCrossfade_gui: public Ui_ToolLoopCrossfade_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLLOOPCROSSFADE_GUI_H
