/********************************************************************************
** Form generated from reading UI file 'configsectiongeneral.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSECTIONGENERAL_H
#define UI_CONFIGSECTIONGENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigSectionGeneral
{
public:
    QGridLayout *gridLayout;
    QLabel *labelSubTitle2;
    QLabel *label_23;
    QComboBox *comboBufferSize;
    QLabel *labelSubTitle1;
    QCheckBox *checkBoucle;
    QCheckBox *checkBlanc;
    QComboBox *comboAudioOuput;
    QLabel *label;
    QLabel *label_21;
    QLabel *label_4;
    QSpacerItem *horizontalSpacer;
    QComboBox *comboMidiInput;
    QLabel *label_24;
    QCheckBox *checkRepercussionStereo;

    void setupUi(QWidget *ConfigSectionGeneral)
    {
        if (ConfigSectionGeneral->objectName().isEmpty())
            ConfigSectionGeneral->setObjectName("ConfigSectionGeneral");
        ConfigSectionGeneral->resize(598, 278);
        gridLayout = new QGridLayout(ConfigSectionGeneral);
        gridLayout->setObjectName("gridLayout");
        labelSubTitle2 = new QLabel(ConfigSectionGeneral);
        labelSubTitle2->setObjectName("labelSubTitle2");
        QFont font;
        font.setBold(true);
        labelSubTitle2->setFont(font);

        gridLayout->addWidget(labelSubTitle2, 4, 0, 1, 3);

        label_23 = new QLabel(ConfigSectionGeneral);
        label_23->setObjectName("label_23");
        QFont font1;
        font1.setBold(false);
        label_23->setFont(font1);

        gridLayout->addWidget(label_23, 6, 1, 1, 1);

        comboBufferSize = new QComboBox(ConfigSectionGeneral);
        comboBufferSize->addItem(QString());
        comboBufferSize->addItem(QString::fromUtf8("16"));
        comboBufferSize->addItem(QString::fromUtf8("32"));
        comboBufferSize->addItem(QString::fromUtf8("64"));
        comboBufferSize->addItem(QString::fromUtf8("128"));
        comboBufferSize->addItem(QString::fromUtf8("256"));
        comboBufferSize->addItem(QString::fromUtf8("512"));
        comboBufferSize->addItem(QString::fromUtf8("1024"));
        comboBufferSize->addItem(QString::fromUtf8("2048"));
        comboBufferSize->addItem(QString::fromUtf8("4096"));
        comboBufferSize->setObjectName("comboBufferSize");

        gridLayout->addWidget(comboBufferSize, 2, 2, 1, 1);

        labelSubTitle1 = new QLabel(ConfigSectionGeneral);
        labelSubTitle1->setObjectName("labelSubTitle1");
        labelSubTitle1->setFont(font);

        gridLayout->addWidget(labelSubTitle1, 0, 0, 1, 3);

        checkBoucle = new QCheckBox(ConfigSectionGeneral);
        checkBoucle->setObjectName("checkBoucle");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(checkBoucle->sizePolicy().hasHeightForWidth());
        checkBoucle->setSizePolicy(sizePolicy);
        checkBoucle->setFont(font1);

        gridLayout->addWidget(checkBoucle, 6, 2, 1, 1);

        checkBlanc = new QCheckBox(ConfigSectionGeneral);
        checkBlanc->setObjectName("checkBlanc");
        sizePolicy.setHeightForWidth(checkBlanc->sizePolicy().hasHeightForWidth());
        checkBlanc->setSizePolicy(sizePolicy);
        checkBlanc->setFont(font1);

        gridLayout->addWidget(checkBlanc, 7, 2, 1, 1);

        comboAudioOuput = new QComboBox(ConfigSectionGeneral);
        comboAudioOuput->setObjectName("comboAudioOuput");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(comboAudioOuput->sizePolicy().hasHeightForWidth());
        comboAudioOuput->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(comboAudioOuput, 1, 2, 1, 1);

        label = new QLabel(ConfigSectionGeneral);
        label->setObjectName("label");
        label->setMaximumSize(QSize(150, 16777215));

        gridLayout->addWidget(label, 1, 1, 1, 1);

        label_21 = new QLabel(ConfigSectionGeneral);
        label_21->setObjectName("label_21");

        gridLayout->addWidget(label_21, 2, 1, 1, 1);

        label_4 = new QLabel(ConfigSectionGeneral);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 0, 1, 1);

        comboMidiInput = new QComboBox(ConfigSectionGeneral);
        comboMidiInput->setObjectName("comboMidiInput");

        gridLayout->addWidget(comboMidiInput, 3, 2, 1, 1);

        label_24 = new QLabel(ConfigSectionGeneral);
        label_24->setObjectName("label_24");
        label_24->setFont(font1);

        gridLayout->addWidget(label_24, 5, 1, 1, 1);

        checkRepercussionStereo = new QCheckBox(ConfigSectionGeneral);
        checkRepercussionStereo->setObjectName("checkRepercussionStereo");
        sizePolicy.setHeightForWidth(checkRepercussionStereo->sizePolicy().hasHeightForWidth());
        checkRepercussionStereo->setSizePolicy(sizePolicy);
        checkRepercussionStereo->setFont(font1);

        gridLayout->addWidget(checkRepercussionStereo, 5, 2, 1, 1);

        QWidget::setTabOrder(comboAudioOuput, comboBufferSize);
        QWidget::setTabOrder(comboBufferSize, comboMidiInput);
        QWidget::setTabOrder(comboMidiInput, checkRepercussionStereo);
        QWidget::setTabOrder(checkRepercussionStereo, checkBoucle);
        QWidget::setTabOrder(checkBoucle, checkBlanc);

        retranslateUi(ConfigSectionGeneral);

        QMetaObject::connectSlotsByName(ConfigSectionGeneral);
    } // setupUi

    void retranslateUi(QWidget *ConfigSectionGeneral)
    {
        labelSubTitle2->setText(QCoreApplication::translate("ConfigSectionGeneral", "Sample management", nullptr));
        label_23->setText(QCoreApplication::translate("ConfigSectionGeneral", "Sample import", nullptr));
        comboBufferSize->setItemText(0, QCoreApplication::translate("ConfigSectionGeneral", "Default", nullptr));

        labelSubTitle1->setText(QCoreApplication::translate("ConfigSectionGeneral", "Input / Output", nullptr));
#if QT_CONFIG(tooltip)
        checkBoucle->setToolTip(QCoreApplication::translate("ConfigSectionGeneral", "When importing a sample, data located after the end of loop are automatically removed.\n"
"(same effect as the tool \"Trim to end of loop\")", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoucle->setText(QCoreApplication::translate("ConfigSectionGeneral", "trim to loop", nullptr));
#if QT_CONFIG(tooltip)
        checkBlanc->setToolTip(QCoreApplication::translate("ConfigSectionGeneral", "When importing a sample, any blank area located before the beginning of the attack is automatically removed.\n"
"(same effect as the tool \"Remove blank at start\")", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBlanc->setText(QCoreApplication::translate("ConfigSectionGeneral", "remove blank at start", nullptr));
        label->setText(QCoreApplication::translate("ConfigSectionGeneral", "Audio backend", nullptr));
        label_21->setText(QCoreApplication::translate("ConfigSectionGeneral", "Buffer size", nullptr));
        label_4->setText(QCoreApplication::translate("ConfigSectionGeneral", "MIDI input", nullptr));
        label_24->setText(QCoreApplication::translate("ConfigSectionGeneral", "Stereo editing", nullptr));
        checkRepercussionStereo->setText(QCoreApplication::translate("ConfigSectionGeneral", "change linked sample", nullptr));
        (void)ConfigSectionGeneral;
    } // retranslateUi

};

namespace Ui {
    class ConfigSectionGeneral: public Ui_ConfigSectionGeneral {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSECTIONGENERAL_H
