/********************************************************************************
** Form generated from reading UI file 'directorybrowser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIRECTORYBROWSER_H
#define UI_DIRECTORYBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "directoryelementlistview.h"
#include "directorylistview.h"
#include "directorysortmenu.h"
#include "rotatingspinner.h"

QT_BEGIN_NAMESPACE

class Ui_DirectoryBrowser
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *stackedWidget;
    QWidget *pageWait;
    QGridLayout *gridLayout_2;
    QWidget *widgetColored2;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer_2;
    RotatingSpinner *spinner;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_3;
    QWidget *pageFailed;
    QGridLayout *gridLayout_3;
    QWidget *widgetColored;
    QSpacerItem *verticalSpacer_4;
    QLabel *labelError;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *pushRetry;
    QSpacerItem *horizontalSpacer_5;
    QSpacerItem *verticalSpacer_5;
    QWidget *pageSoundfonts;
    QGridLayout *gridLayout_4;
    QWidget *widgetLeft;
    QVBoxLayout *verticalLayout;
    QLabel *labelNoResults;
    DirectoryListView *listView;
    QWidget *widgetRight;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *pushShowSamples;
    QLabel *label;
    QLabel *iconSample;
    DirectoryElementListView *listSamples;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *pushShowInstruments;
    QLabel *label_2;
    QLabel *iconInstrument;
    DirectoryElementListView *listInstruments;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *pushShowPresets;
    QLabel *label_3;
    QLabel *iconPreset;
    DirectoryElementListView *listPresets;
    QSpacerItem *verticalSpacer;
    QFrame *frameTop;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineSearch;
    DirectorySortMenu *widgetSortMenu;

    void setupUi(QWidget *DirectoryBrowser)
    {
        if (DirectoryBrowser->objectName().isEmpty())
            DirectoryBrowser->setObjectName("DirectoryBrowser");
        DirectoryBrowser->resize(550, 447);
        DirectoryBrowser->setWindowTitle(QString::fromUtf8("Form"));
        DirectoryBrowser->setAutoFillBackground(true);
        gridLayout = new QGridLayout(DirectoryBrowser);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(DirectoryBrowser);
        stackedWidget->setObjectName("stackedWidget");
        pageWait = new QWidget();
        pageWait->setObjectName("pageWait");
        gridLayout_2 = new QGridLayout(pageWait);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        widgetColored2 = new QWidget(pageWait);
        widgetColored2->setObjectName("widgetColored2");
        widgetColored2->setMinimumSize(QSize(0, 6));

        gridLayout_2->addWidget(widgetColored2, 0, 0, 1, 3);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 1, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 2, 0, 1, 1);

        spinner = new RotatingSpinner(pageWait);
        spinner->setObjectName("spinner");
        spinner->setMinimumSize(QSize(32, 32));
        spinner->setMaximumSize(QSize(32, 32));

        gridLayout_2->addWidget(spinner, 2, 1, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_3, 2, 2, 1, 1);

        verticalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_3, 3, 1, 1, 1);

        stackedWidget->addWidget(pageWait);
        pageFailed = new QWidget();
        pageFailed->setObjectName("pageFailed");
        gridLayout_3 = new QGridLayout(pageFailed);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        widgetColored = new QWidget(pageFailed);
        widgetColored->setObjectName("widgetColored");
        widgetColored->setMinimumSize(QSize(0, 6));

        gridLayout_3->addWidget(widgetColored, 0, 0, 1, 3);

        verticalSpacer_4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_4, 1, 1, 1, 1);

        labelError = new QLabel(pageFailed);
        labelError->setObjectName("labelError");
        labelError->setText(QString::fromUtf8("Error..."));
        labelError->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(labelError, 2, 0, 1, 3);

        horizontalSpacer_4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_4, 3, 0, 1, 1);

        pushRetry = new QPushButton(pageFailed);
        pushRetry->setObjectName("pushRetry");
        pushRetry->setIconSize(QSize(16, 16));
        pushRetry->setFlat(true);

        gridLayout_3->addWidget(pushRetry, 3, 1, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_5, 3, 2, 1, 1);

        verticalSpacer_5 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_5, 4, 1, 1, 1);

        stackedWidget->addWidget(pageFailed);
        pageSoundfonts = new QWidget();
        pageSoundfonts->setObjectName("pageSoundfonts");
        gridLayout_4 = new QGridLayout(pageSoundfonts);
        gridLayout_4->setSpacing(0);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        widgetLeft = new QWidget(pageSoundfonts);
        widgetLeft->setObjectName("widgetLeft");
        widgetLeft->setMinimumSize(QSize(400, 0));
        verticalLayout = new QVBoxLayout(widgetLeft);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        labelNoResults = new QLabel(widgetLeft);
        labelNoResults->setObjectName("labelNoResults");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelNoResults->sizePolicy().hasHeightForWidth());
        labelNoResults->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        labelNoResults->setFont(font);
        labelNoResults->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(labelNoResults);

        listView = new DirectoryListView(widgetLeft);
        listView->setObjectName("listView");
        listView->setFrameShape(QFrame::Shape::NoFrame);
        listView->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAsNeeded);
        listView->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        listView->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        listView->setProperty("showDropIndicator", QVariant(false));
        listView->setDragDropOverwriteMode(false);
        listView->setAlternatingRowColors(true);
        listView->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
        listView->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        listView->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);

        verticalLayout->addWidget(listView);


        gridLayout_4->addWidget(widgetLeft, 1, 0, 1, 1);

        widgetRight = new QWidget(pageSoundfonts);
        widgetRight->setObjectName("widgetRight");
        widgetRight->setMinimumSize(QSize(150, 0));
        verticalLayout_2 = new QVBoxLayout(widgetRight);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, -1, 0, 0);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(-1, -1, 9, -1);
        pushShowSamples = new QPushButton(widgetRight);
        pushShowSamples->setObjectName("pushShowSamples");
        pushShowSamples->setMinimumSize(QSize(24, 24));
        pushShowSamples->setMaximumSize(QSize(24, 24));
        pushShowSamples->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushShowSamples->setCheckable(true);
        pushShowSamples->setChecked(true);
        pushShowSamples->setFlat(true);

        horizontalLayout_4->addWidget(pushShowSamples);

        label = new QLabel(widgetRight);
        label->setObjectName("label");
        label->setFont(font);

        horizontalLayout_4->addWidget(label);

        iconSample = new QLabel(widgetRight);
        iconSample->setObjectName("iconSample");
        iconSample->setMinimumSize(QSize(13, 16));
        iconSample->setMaximumSize(QSize(13, 16));
        iconSample->setPixmap(QPixmap(QString::fromUtf8(":/icons/sample.svg")));
        iconSample->setScaledContents(true);

        horizontalLayout_4->addWidget(iconSample);


        verticalLayout_2->addLayout(horizontalLayout_4);

        listSamples = new DirectoryElementListView(widgetRight);
        listSamples->setObjectName("listSamples");
        listSamples->setFrameShape(QFrame::Shape::NoFrame);
        listSamples->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        listSamples->setProperty("showDropIndicator", QVariant(false));
        listSamples->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        listSamples->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);

        verticalLayout_2->addWidget(listSamples);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(-1, -1, 9, -1);
        pushShowInstruments = new QPushButton(widgetRight);
        pushShowInstruments->setObjectName("pushShowInstruments");
        pushShowInstruments->setMinimumSize(QSize(24, 24));
        pushShowInstruments->setMaximumSize(QSize(24, 24));
        pushShowInstruments->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushShowInstruments->setCheckable(true);
        pushShowInstruments->setChecked(true);
        pushShowInstruments->setFlat(true);

        horizontalLayout_5->addWidget(pushShowInstruments);

        label_2 = new QLabel(widgetRight);
        label_2->setObjectName("label_2");
        label_2->setFont(font);

        horizontalLayout_5->addWidget(label_2);

        iconInstrument = new QLabel(widgetRight);
        iconInstrument->setObjectName("iconInstrument");
        iconInstrument->setMinimumSize(QSize(13, 16));
        iconInstrument->setMaximumSize(QSize(13, 16));
        iconInstrument->setPixmap(QPixmap(QString::fromUtf8(":/icons/instrument.svg")));
        iconInstrument->setScaledContents(true);

        horizontalLayout_5->addWidget(iconInstrument);


        verticalLayout_2->addLayout(horizontalLayout_5);

        listInstruments = new DirectoryElementListView(widgetRight);
        listInstruments->setObjectName("listInstruments");
        listInstruments->setFrameShape(QFrame::Shape::NoFrame);
        listInstruments->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        listInstruments->setProperty("showDropIndicator", QVariant(false));
        listInstruments->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        listInstruments->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);

        verticalLayout_2->addWidget(listInstruments);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalLayout_6->setContentsMargins(-1, -1, 9, -1);
        pushShowPresets = new QPushButton(widgetRight);
        pushShowPresets->setObjectName("pushShowPresets");
        pushShowPresets->setMinimumSize(QSize(24, 24));
        pushShowPresets->setMaximumSize(QSize(24, 24));
        pushShowPresets->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushShowPresets->setCheckable(true);
        pushShowPresets->setChecked(true);
        pushShowPresets->setFlat(true);

        horizontalLayout_6->addWidget(pushShowPresets);

        label_3 = new QLabel(widgetRight);
        label_3->setObjectName("label_3");
        label_3->setFont(font);

        horizontalLayout_6->addWidget(label_3);

        iconPreset = new QLabel(widgetRight);
        iconPreset->setObjectName("iconPreset");
        iconPreset->setMinimumSize(QSize(13, 16));
        iconPreset->setMaximumSize(QSize(13, 16));
        iconPreset->setPixmap(QPixmap(QString::fromUtf8(":/icons/preset.svg")));
        iconPreset->setScaledContents(true);

        horizontalLayout_6->addWidget(iconPreset);


        verticalLayout_2->addLayout(horizontalLayout_6);

        listPresets = new DirectoryElementListView(widgetRight);
        listPresets->setObjectName("listPresets");
        listPresets->setFrameShape(QFrame::Shape::NoFrame);
        listPresets->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        listPresets->setProperty("showDropIndicator", QVariant(false));
        listPresets->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        listPresets->setVerticalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);

        verticalLayout_2->addWidget(listPresets);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout_2->addItem(verticalSpacer);


        gridLayout_4->addWidget(widgetRight, 1, 1, 1, 1);

        frameTop = new QFrame(pageSoundfonts);
        frameTop->setObjectName("frameTop");
        frameTop->setMinimumSize(QSize(0, 42));
        frameTop->setMaximumSize(QSize(16777215, 42));
        frameTop->setFrameShape(QFrame::Shape::NoFrame);
        horizontalLayout = new QHBoxLayout(frameTop);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(6, 5, 6, 6);
        lineSearch = new QLineEdit(frameTop);
        lineSearch->setObjectName("lineSearch");
        sizePolicy.setHeightForWidth(lineSearch->sizePolicy().hasHeightForWidth());
        lineSearch->setSizePolicy(sizePolicy);
        lineSearch->setClearButtonEnabled(true);

        horizontalLayout->addWidget(lineSearch);

        widgetSortMenu = new DirectorySortMenu(frameTop);
        widgetSortMenu->setObjectName("widgetSortMenu");

        horizontalLayout->addWidget(widgetSortMenu);


        gridLayout_4->addWidget(frameTop, 0, 0, 1, 2);

        stackedWidget->addWidget(pageSoundfonts);

        gridLayout->addWidget(stackedWidget, 0, 0, 1, 1);

        QWidget::setTabOrder(lineSearch, listView);
        QWidget::setTabOrder(listView, pushShowSamples);
        QWidget::setTabOrder(pushShowSamples, listSamples);
        QWidget::setTabOrder(listSamples, pushShowInstruments);
        QWidget::setTabOrder(pushShowInstruments, listInstruments);
        QWidget::setTabOrder(listInstruments, pushShowPresets);
        QWidget::setTabOrder(pushShowPresets, listPresets);
        QWidget::setTabOrder(listPresets, pushRetry);

        retranslateUi(DirectoryBrowser);

        stackedWidget->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(DirectoryBrowser);
    } // setupUi

    void retranslateUi(QWidget *DirectoryBrowser)
    {
        pushRetry->setText(QString());
        labelNoResults->setText(QCoreApplication::translate("DirectoryBrowser", "No results", nullptr));
        pushShowSamples->setText(QString());
        label->setText(QCoreApplication::translate("DirectoryBrowser", "Samples", nullptr));
        iconSample->setText(QString());
        pushShowInstruments->setText(QString());
        label_2->setText(QCoreApplication::translate("DirectoryBrowser", "Instruments", nullptr));
        iconInstrument->setText(QString());
        pushShowPresets->setText(QString());
        label_3->setText(QCoreApplication::translate("DirectoryBrowser", "Presets", nullptr));
        iconPreset->setText(QString());
        lineSearch->setPlaceholderText(QCoreApplication::translate("DirectoryBrowser", "Search a file, a sample, an instrument or a preset...", nullptr));
        (void)DirectoryBrowser;
    } // retranslateUi

};

namespace Ui {
    class DirectoryBrowser: public Ui_DirectoryBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIRECTORYBROWSER_H
