/***************************************************************************
**                                                                        **
**  Polyphone, a soundfont editor                                         **
**  Copyright (C) 2013-2024 Davy Triponney                                **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses/.    **
**                                                                        **
****************************************************************************
**           Author: Davy Triponney                                       **
**  Website/Contact: https://www.polyphone.io                             **
**             Date: 01.01.2013                                           **
***************************************************************************/

#include "samplereaderfactory.h"
#include "samplereadersf.h"
#include "samplereaderfile.h"
#include <QFileInfo>

SampleReader * SampleReaderFactory::getSampleReader(QString filename)
{
    QFileInfo fileInfo(filename);
    QString ext = fileInfo.suffix().toLower();

    if (filename.isEmpty())
        return nullptr;

    // Soundfont?
    if (ext.compare("sf2") == 0 || ext.compare("sf3") == 0)
        return new SampleReaderSf(filename);

    // Other kind of file => libsndfile
    return new SampleReaderFile(filename);
}

QStringList SampleReaderFactory::getPossibleExtensions()
{
    return QStringList() << "aif"
                         << "aifc"
                         << "aiff"
                         << "flac"
                         << "mp3"
                         << "ogg"
                         << "snd"
                         << "wav";
}
