/********************************************************************************
** Form generated from reading UI file 'editordialoginsertimage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITORDIALOGINSERTIMAGE_H
#define UI_EDITORDIALOGINSERTIMAGE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_EditorDialogInsertImage
{
public:
    QGridLayout *gridLayout;
    QLabel *labelWarning;
    QLineEdit *lineUrl;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *EditorDialogInsertImage)
    {
        if (EditorDialogInsertImage->objectName().isEmpty())
            EditorDialogInsertImage->setObjectName("EditorDialogInsertImage");
        EditorDialogInsertImage->resize(426, 121);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        EditorDialogInsertImage->setWindowIcon(icon);
        gridLayout = new QGridLayout(EditorDialogInsertImage);
        gridLayout->setObjectName("gridLayout");
        labelWarning = new QLabel(EditorDialogInsertImage);
        labelWarning->setObjectName("labelWarning");
        QFont font;
        font.setBold(true);
        labelWarning->setFont(font);

        gridLayout->addWidget(labelWarning, 1, 0, 1, 1);

        lineUrl = new QLineEdit(EditorDialogInsertImage);
        lineUrl->setObjectName("lineUrl");

        gridLayout->addWidget(lineUrl, 0, 0, 1, 1);

        buttonBox = new QDialogButtonBox(EditorDialogInsertImage);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 3, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);


        retranslateUi(EditorDialogInsertImage);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, EditorDialogInsertImage, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, EditorDialogInsertImage, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(EditorDialogInsertImage);
    } // setupUi

    void retranslateUi(QDialog *EditorDialogInsertImage)
    {
        EditorDialogInsertImage->setWindowTitle(QCoreApplication::translate("EditorDialogInsertImage", "Insert an image", nullptr));
        labelWarning->setText(QCoreApplication::translate("EditorDialogInsertImage", "Invalid URL (example: https://www.website.com/image.png)", nullptr));
        lineUrl->setPlaceholderText(QCoreApplication::translate("EditorDialogInsertImage", "URL of the image to insert...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditorDialogInsertImage: public Ui_EditorDialogInsertImage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITORDIALOGINSERTIMAGE_H
