/********************************************************************************
** Form generated from reading UI file 'toolpresetlist_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLPRESETLIST_GUI_H
#define UI_TOOLPRESETLIST_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ToolPresetList_gui
{
public:
    QGridLayout *gridLayout;
    QPlainTextEdit *plainTextEdit;
    QPushButton *pushCopy;
    QSpacerItem *horizontalSpacer;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QRadioButton *radioCsv;
    QRadioButton *radioHtml;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *pushClose;

    void setupUi(QWidget *ToolPresetList_gui)
    {
        if (ToolPresetList_gui->objectName().isEmpty())
            ToolPresetList_gui->setObjectName("ToolPresetList_gui");
        ToolPresetList_gui->resize(347, 278);
        ToolPresetList_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolPresetList_gui);
        gridLayout->setObjectName("gridLayout");
        plainTextEdit = new QPlainTextEdit(ToolPresetList_gui);
        plainTextEdit->setObjectName("plainTextEdit");
        plainTextEdit->setMinimumSize(QSize(0, 200));

        gridLayout->addWidget(plainTextEdit, 1, 0, 1, 5);

        pushCopy = new QPushButton(ToolPresetList_gui);
        pushCopy->setObjectName("pushCopy");

        gridLayout->addWidget(pushCopy, 2, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(34, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 1, 1, 1);

        label = new QLabel(ToolPresetList_gui);
        label->setObjectName("label");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(label, 2, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(35, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 2, 3, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        radioCsv = new QRadioButton(ToolPresetList_gui);
        radioCsv->setObjectName("radioCsv");
        radioCsv->setChecked(true);

        horizontalLayout->addWidget(radioCsv);

        radioHtml = new QRadioButton(ToolPresetList_gui);
        radioHtml->setObjectName("radioHtml");

        horizontalLayout->addWidget(radioHtml);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 5);

        pushClose = new QPushButton(ToolPresetList_gui);
        pushClose->setObjectName("pushClose");

        gridLayout->addWidget(pushClose, 2, 4, 1, 1);

        QWidget::setTabOrder(radioCsv, radioHtml);
        QWidget::setTabOrder(radioHtml, plainTextEdit);
        QWidget::setTabOrder(plainTextEdit, pushCopy);
        QWidget::setTabOrder(pushCopy, pushClose);

        retranslateUi(ToolPresetList_gui);

        QMetaObject::connectSlotsByName(ToolPresetList_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolPresetList_gui)
    {
        pushCopy->setText(QCoreApplication::translate("ToolPresetList_gui", "Copy", nullptr));
        label->setText(QCoreApplication::translate("ToolPresetList_gui", "text copied in the clipboard", nullptr));
        radioCsv->setText(QCoreApplication::translate("ToolPresetList_gui", "csv format", nullptr));
        radioHtml->setText(QCoreApplication::translate("ToolPresetList_gui", "html table", nullptr));
        pushClose->setText(QCoreApplication::translate("ToolPresetList_gui", "Close", nullptr));
        (void)ToolPresetList_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolPresetList_gui: public Ui_ToolPresetList_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLPRESETLIST_GUI_H
