/********************************************************************************
** Form generated from reading UI file 'soundfontcellfull.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOUNDFONTCELLFULL_H
#define UI_SOUNDFONTCELLFULL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "attributeflow.h"
#include "elidedlabel.h"
#include "fivestarswidget.h"

QT_BEGIN_NAMESPACE

class Ui_SoundfontCellFull
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *line1;
    QHBoxLayout *layoutLine1;
    ElidedLabel *labelTitle;
    QLabel *iconComment;
    QLabel *labelCommentNumber;
    QLabel *iconDownload;
    QLabel *labelDownloadNumber;
    FiveStarsWidget *widgetStars;
    QFrame *line2;
    QHBoxLayout *layoutLine2;
    QLabel *iconAuthor;
    QLabel *labelAuthor;
    QLabel *iconDate;
    QLabel *labelDate;
    QLabel *iconLicense;
    ElidedLabel *labelLicense;
    QSpacerItem *horizontalSpacer;
    AttributeFlow *line3;

    void setupUi(QWidget *SoundfontCellFull)
    {
        if (SoundfontCellFull->objectName().isEmpty())
            SoundfontCellFull->setObjectName("SoundfontCellFull");
        SoundfontCellFull->resize(400, 98);
        SoundfontCellFull->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(SoundfontCellFull);
        verticalLayout->setObjectName("verticalLayout");
        line1 = new QFrame(SoundfontCellFull);
        line1->setObjectName("line1");
        line1->setMinimumSize(QSize(0, 24));
        line1->setMaximumSize(QSize(16777215, 24));
        layoutLine1 = new QHBoxLayout(line1);
        layoutLine1->setObjectName("layoutLine1");
        layoutLine1->setContentsMargins(0, 0, 0, 0);
        labelTitle = new ElidedLabel(line1);
        labelTitle->setObjectName("labelTitle");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelTitle->sizePolicy().hasHeightForWidth());
        labelTitle->setSizePolicy(sizePolicy);
        QFont font;
        font.setBold(true);
        labelTitle->setFont(font);
        labelTitle->setText(QString::fromUtf8("labelTitle"));

        layoutLine1->addWidget(labelTitle);

        iconComment = new QLabel(line1);
        iconComment->setObjectName("iconComment");
        iconComment->setMinimumSize(QSize(24, 24));
        iconComment->setMaximumSize(QSize(24, 24));

        layoutLine1->addWidget(iconComment);

        labelCommentNumber = new QLabel(line1);
        labelCommentNumber->setObjectName("labelCommentNumber");
        labelCommentNumber->setMinimumSize(QSize(30, 0));
        labelCommentNumber->setText(QString::fromUtf8("0"));
        labelCommentNumber->setAlignment(Qt::AlignCenter);

        layoutLine1->addWidget(labelCommentNumber);

        iconDownload = new QLabel(line1);
        iconDownload->setObjectName("iconDownload");
        iconDownload->setMinimumSize(QSize(24, 24));
        iconDownload->setMaximumSize(QSize(24, 24));

        layoutLine1->addWidget(iconDownload);

        labelDownloadNumber = new QLabel(line1);
        labelDownloadNumber->setObjectName("labelDownloadNumber");
        labelDownloadNumber->setMinimumSize(QSize(30, 0));
        labelDownloadNumber->setText(QString::fromUtf8("0"));
        labelDownloadNumber->setAlignment(Qt::AlignCenter);

        layoutLine1->addWidget(labelDownloadNumber);

        widgetStars = new FiveStarsWidget(line1);
        widgetStars->setObjectName("widgetStars");

        layoutLine1->addWidget(widgetStars);


        verticalLayout->addWidget(line1);

        line2 = new QFrame(SoundfontCellFull);
        line2->setObjectName("line2");
        line2->setMinimumSize(QSize(0, 20));
        line2->setMaximumSize(QSize(16777215, 20));
        layoutLine2 = new QHBoxLayout(line2);
        layoutLine2->setObjectName("layoutLine2");
        layoutLine2->setContentsMargins(0, 0, 0, 0);
        iconAuthor = new QLabel(line2);
        iconAuthor->setObjectName("iconAuthor");
        iconAuthor->setMinimumSize(QSize(16, 16));
        iconAuthor->setMaximumSize(QSize(16, 16));

        layoutLine2->addWidget(iconAuthor);

        labelAuthor = new QLabel(line2);
        labelAuthor->setObjectName("labelAuthor");
        labelAuthor->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        labelAuthor->setText(QString::fromUtf8("labelAuthor"));
        labelAuthor->setTextFormat(Qt::RichText);

        layoutLine2->addWidget(labelAuthor);

        iconDate = new QLabel(line2);
        iconDate->setObjectName("iconDate");
        iconDate->setMinimumSize(QSize(16, 16));
        iconDate->setMaximumSize(QSize(16, 16));

        layoutLine2->addWidget(iconDate);

        labelDate = new QLabel(line2);
        labelDate->setObjectName("labelDate");
        labelDate->setText(QString::fromUtf8("labelDate"));

        layoutLine2->addWidget(labelDate);

        iconLicense = new QLabel(line2);
        iconLicense->setObjectName("iconLicense");
        iconLicense->setMinimumSize(QSize(16, 16));
        iconLicense->setMaximumSize(QSize(16, 16));

        layoutLine2->addWidget(iconLicense);

        labelLicense = new ElidedLabel(line2);
        labelLicense->setObjectName("labelLicense");
        labelLicense->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        labelLicense->setText(QString::fromUtf8("labelLicense"));
        labelLicense->setOpenExternalLinks(true);

        layoutLine2->addWidget(labelLicense);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        layoutLine2->addItem(horizontalSpacer);


        verticalLayout->addWidget(line2);

        line3 = new AttributeFlow(SoundfontCellFull);
        line3->setObjectName("line3");

        verticalLayout->addWidget(line3);


        retranslateUi(SoundfontCellFull);

        QMetaObject::connectSlotsByName(SoundfontCellFull);
    } // setupUi

    void retranslateUi(QWidget *SoundfontCellFull)
    {
        iconComment->setText(QString());
        iconDownload->setText(QString());
        iconAuthor->setText(QString());
        iconDate->setText(QString());
        iconLicense->setText(QString());
        (void)SoundfontCellFull;
    } // retranslateUi

};

namespace Ui {
    class SoundfontCellFull: public Ui_SoundfontCellFull {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOUNDFONTCELLFULL_H
