/********************************************************************************
** Form generated from reading UI file 'footersf2.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FOOTERSF2_H
#define UI_FOOTERSF2_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>
#include "elidedlabel.h"

QT_BEGIN_NAMESPACE

class Ui_FooterSf2
{
public:
    QHBoxLayout *horizontalLayout;
    ElidedLabel *label_filename;

    void setupUi(QWidget *FooterSf2)
    {
        if (FooterSf2->objectName().isEmpty())
            FooterSf2->setObjectName("FooterSf2");
        FooterSf2->resize(190, 19);
        FooterSf2->setWindowTitle(QString::fromUtf8(""));
        horizontalLayout = new QHBoxLayout(FooterSf2);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, 0, -1, 0);
        label_filename = new ElidedLabel(FooterSf2);
        label_filename->setObjectName("label_filename");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_filename->sizePolicy().hasHeightForWidth());
        label_filename->setSizePolicy(sizePolicy);
        label_filename->setText(QString::fromUtf8("TextLabel"));
        label_filename->setWordWrap(true);

        horizontalLayout->addWidget(label_filename);


        retranslateUi(FooterSf2);

        QMetaObject::connectSlotsByName(FooterSf2);
    } // setupUi

    void retranslateUi(QWidget *FooterSf2)
    {
        (void)FooterSf2;
    } // retranslateUi

};

namespace Ui {
    class FooterSf2: public Ui_FooterSf2 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FOOTERSF2_H
