/********************************************************************************
** Form generated from reading UI file 'dialog_about.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOG_ABOUT_H
#define UI_DIALOG_ABOUT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DialogAbout
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *labelIcon;
    QLabel *labelNomVersion;
    QStackedWidget *stackedWidget;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *textBrowser;
    QWidget *page_1;
    QVBoxLayout *verticalLayout_3;
    QLabel *labelDescription;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushCredit;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushClose;

    void setupUi(QDialog *DialogAbout)
    {
        if (DialogAbout->objectName().isEmpty())
            DialogAbout->setObjectName("DialogAbout");
        DialogAbout->setWindowModality(Qt::ApplicationModal);
        DialogAbout->resize(350, 320);
        DialogAbout->setMinimumSize(QSize(350, 320));
        DialogAbout->setMaximumSize(QSize(350, 320));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/misc/polyphone.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        DialogAbout->setWindowIcon(icon);
        DialogAbout->setModal(true);
        verticalLayout = new QVBoxLayout(DialogAbout);
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(6, 6, 6, 6);
        labelIcon = new QLabel(DialogAbout);
        labelIcon->setObjectName("labelIcon");
        labelIcon->setMinimumSize(QSize(128, 128));
        labelIcon->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelIcon);

        labelNomVersion = new QLabel(DialogAbout);
        labelNomVersion->setObjectName("labelNomVersion");
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        labelNomVersion->setFont(font);
        labelNomVersion->setText(QString::fromUtf8("Polyphone v..."));
        labelNomVersion->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelNomVersion);

        stackedWidget = new QStackedWidget(DialogAbout);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout_2 = new QVBoxLayout(page_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        textBrowser = new QTextBrowser(page_2);
        textBrowser->setObjectName("textBrowser");
        QFont font1;
        font1.setPointSize(9);
        textBrowser->setFont(font1);
        textBrowser->setFocusPolicy(Qt::NoFocus);
        textBrowser->setOpenExternalLinks(true);
        textBrowser->setOpenLinks(true);

        verticalLayout_2->addWidget(textBrowser);

        stackedWidget->addWidget(page_2);
        page_1 = new QWidget();
        page_1->setObjectName("page_1");
        verticalLayout_3 = new QVBoxLayout(page_1);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        labelDescription = new QLabel(page_1);
        labelDescription->setObjectName("labelDescription");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelDescription->sizePolicy().hasHeightForWidth());
        labelDescription->setSizePolicy(sizePolicy1);
        labelDescription->setText(QString::fromUtf8("Copyright, site internet"));
        labelDescription->setTextFormat(Qt::RichText);
        labelDescription->setAlignment(Qt::AlignCenter);
        labelDescription->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(labelDescription);

        stackedWidget->addWidget(page_1);

        verticalLayout->addWidget(stackedWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pushCredit = new QPushButton(DialogAbout);
        pushCredit->setObjectName("pushCredit");
        pushCredit->setCheckable(true);

        horizontalLayout->addWidget(pushCredit);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushClose = new QPushButton(DialogAbout);
        pushClose->setObjectName("pushClose");

        horizontalLayout->addWidget(pushClose);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(pushCredit, pushClose);

        retranslateUi(DialogAbout);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(DialogAbout);
    } // setupUi

    void retranslateUi(QDialog *DialogAbout)
    {
        DialogAbout->setWindowTitle(QCoreApplication::translate("DialogAbout", "About", nullptr));
        labelIcon->setText(QString());
        pushCredit->setText(QCoreApplication::translate("DialogAbout", "Credit", nullptr));
        pushClose->setText(QCoreApplication::translate("DialogAbout", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DialogAbout: public Ui_DialogAbout {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIALOG_ABOUT_H
