/********************************************************************************
** Form generated from reading UI file 'modulatorcell.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MODULATORCELL_H
#define UI_MODULATORCELL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "modulatorcombocurve.h"
#include "modulatorcombodest.h"
#include "modulatorcombosrc.h"

QT_BEGIN_NAMESPACE

class Ui_ModulatorCell
{
public:
    QGridLayout *gridLayout_2;
    ModulatorComboSrc *comboSource1;
    QSpacerItem *horizontalSpacer_2;
    QLabel *labelFinalRange;
    ModulatorComboCurve *widgetShape2;
    ModulatorComboSrc *comboSource2;
    QSpacerItem *horizontalSpacer;
    QSpinBox *spinAmount;
    QSpacerItem *horizontalSpacer_3;
    ModulatorComboCurve *widgetShape1;
    ModulatorComboDest *comboDestination;
    QComboBox *comboTransform;
    QVBoxLayout *verticalLayout;
    QLabel *labelModNumber;
    QLabel *labelDetails;

    void setupUi(QWidget *ModulatorCell)
    {
        if (ModulatorCell->objectName().isEmpty())
            ModulatorCell->setObjectName("ModulatorCell");
        ModulatorCell->resize(845, 94);
        ModulatorCell->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout_2 = new QGridLayout(ModulatorCell);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setVerticalSpacing(3);
        comboSource1 = new ModulatorComboSrc(ModulatorCell);
        comboSource1->setObjectName("comboSource1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comboSource1->sizePolicy().hasHeightForWidth());
        comboSource1->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(comboSource1, 0, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(70, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 1, 5, 1, 1);

        labelFinalRange = new QLabel(ModulatorCell);
        labelFinalRange->setObjectName("labelFinalRange");

        gridLayout_2->addWidget(labelFinalRange, 1, 9, 1, 1);

        widgetShape2 = new ModulatorComboCurve(ModulatorCell);
        widgetShape2->setObjectName("widgetShape2");

        gridLayout_2->addWidget(widgetShape2, 1, 4, 1, 1);

        comboSource2 = new ModulatorComboSrc(ModulatorCell);
        comboSource2->setObjectName("comboSource2");

        gridLayout_2->addWidget(comboSource2, 1, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 1, 3, 1, 1);

        spinAmount = new QSpinBox(ModulatorCell);
        spinAmount->setObjectName("spinAmount");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(spinAmount->sizePolicy().hasHeightForWidth());
        spinAmount->setSizePolicy(sizePolicy1);
        spinAmount->setPrefix(QString::fromUtf8("\303\227 "));
        spinAmount->setMinimum(-32768);
        spinAmount->setMaximum(32767);

        gridLayout_2->addWidget(spinAmount, 0, 6, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_3, 1, 8, 1, 1);

        widgetShape1 = new ModulatorComboCurve(ModulatorCell);
        widgetShape1->setObjectName("widgetShape1");

        gridLayout_2->addWidget(widgetShape1, 0, 4, 1, 1);

        comboDestination = new ModulatorComboDest(ModulatorCell);
        comboDestination->setObjectName("comboDestination");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(comboDestination->sizePolicy().hasHeightForWidth());
        comboDestination->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(comboDestination, 0, 9, 1, 1);

        comboTransform = new QComboBox(ModulatorCell);
        comboTransform->addItem(QString());
        comboTransform->addItem(QString());
        comboTransform->setObjectName("comboTransform");

        gridLayout_2->addWidget(comboTransform, 0, 7, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        labelModNumber = new QLabel(ModulatorCell);
        labelModNumber->setObjectName("labelModNumber");
        labelModNumber->setText(QString::fromUtf8("Modulator\n"
"#1"));
        labelModNumber->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelModNumber);

        labelDetails = new QLabel(ModulatorCell);
        labelDetails->setObjectName("labelDetails");
        QFont font;
        font.setPointSize(7);
        font.setBold(true);
        labelDetails->setFont(font);
        labelDetails->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(labelDetails);


        gridLayout_2->addLayout(verticalLayout, 0, 1, 2, 1);

        QWidget::setTabOrder(comboSource1, comboSource2);
        QWidget::setTabOrder(comboSource2, spinAmount);
        QWidget::setTabOrder(spinAmount, comboTransform);
        QWidget::setTabOrder(comboTransform, comboDestination);

        retranslateUi(ModulatorCell);

        QMetaObject::connectSlotsByName(ModulatorCell);
    } // setupUi

    void retranslateUi(QWidget *ModulatorCell)
    {
        labelFinalRange->setText(QString());
        comboTransform->setItemText(0, QCoreApplication::translate("ModulatorCell", "no operation", nullptr));
        comboTransform->setItemText(1, QCoreApplication::translate("ModulatorCell", "absolute value", nullptr));

        labelDetails->setText(QString());
        (void)ModulatorCell;
    } // retranslateUi

};

namespace Ui {
    class ModulatorCell: public Ui_ModulatorCell {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MODULATORCELL_H
