/********************************************************************************
** Form generated from reading UI file 'configtoc.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGTOC_H
#define UI_CONFIGTOC_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "configtocbutton.h"

QT_BEGIN_NAMESPACE

class Ui_ConfigToc
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *frame;
    QVBoxLayout *verticalLayout_5;
    ConfigTocButton *buttonGeneral;
    ConfigTocButton *buttonInterface;
    ConfigTocButton *buttonSound;
    ConfigTocButton *buttonKeyboard;
    ConfigTocButton *buttonRepository;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ConfigToc)
    {
        if (ConfigToc->objectName().isEmpty())
            ConfigToc->setObjectName("ConfigToc");
        ConfigToc->resize(143, 206);
        ConfigToc->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(ConfigToc);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(-1, 9, 0, 0);
        frame = new QFrame(ConfigToc);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_5 = new QVBoxLayout(frame);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        buttonGeneral = new ConfigTocButton(frame);
        buttonGeneral->setObjectName("buttonGeneral");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonGeneral->sizePolicy().hasHeightForWidth());
        buttonGeneral->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(buttonGeneral);

        buttonInterface = new ConfigTocButton(frame);
        buttonInterface->setObjectName("buttonInterface");
        sizePolicy.setHeightForWidth(buttonInterface->sizePolicy().hasHeightForWidth());
        buttonInterface->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(buttonInterface);

        buttonSound = new ConfigTocButton(frame);
        buttonSound->setObjectName("buttonSound");
        sizePolicy.setHeightForWidth(buttonSound->sizePolicy().hasHeightForWidth());
        buttonSound->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(buttonSound);

        buttonKeyboard = new ConfigTocButton(frame);
        buttonKeyboard->setObjectName("buttonKeyboard");
        sizePolicy.setHeightForWidth(buttonKeyboard->sizePolicy().hasHeightForWidth());
        buttonKeyboard->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(buttonKeyboard);

        buttonRepository = new ConfigTocButton(frame);
        buttonRepository->setObjectName("buttonRepository");
        sizePolicy.setHeightForWidth(buttonRepository->sizePolicy().hasHeightForWidth());
        buttonRepository->setSizePolicy(sizePolicy);

        verticalLayout_5->addWidget(buttonRepository);


        verticalLayout->addWidget(frame);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(ConfigToc);

        QMetaObject::connectSlotsByName(ConfigToc);
    } // setupUi

    void retranslateUi(QWidget *ConfigToc)
    {
        buttonGeneral->setText(QCoreApplication::translate("ConfigToc", "General", nullptr));
        buttonInterface->setText(QCoreApplication::translate("ConfigToc", "Interface", nullptr));
        buttonSound->setText(QCoreApplication::translate("ConfigToc", "Sound", nullptr));
        buttonKeyboard->setText(QCoreApplication::translate("ConfigToc", "Keyboard", nullptr));
        buttonRepository->setText(QCoreApplication::translate("ConfigToc", "Online\n"
"repository", nullptr));
        (void)ConfigToc;
    } // retranslateUi

};

namespace Ui {
    class ConfigToc: public Ui_ConfigToc {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGTOC_H
