/********************************************************************************
** Form generated from reading UI file 'toprightwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOPRIGHTWIDGET_H
#define UI_TOPRIGHTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "downloadprogressbutton.h"
#include "rotatingspinner.h"

QT_BEGIN_NAMESPACE

class Ui_TopRightWidget
{
public:
    QHBoxLayout *horizontalLayout;
    RotatingSpinner *spinner;
    QLabel *labelWarning;
    DownloadProgressButton *toolButtonDownload;
    QPushButton *pushUser;
    QToolButton *toolButton;
    QPushButton *pushMinimize;
    QPushButton *pushClose;

    void setupUi(QWidget *TopRightWidget)
    {
        if (TopRightWidget->objectName().isEmpty())
            TopRightWidget->setObjectName("TopRightWidget");
        TopRightWidget->resize(294, 42);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TopRightWidget->sizePolicy().hasHeightForWidth());
        TopRightWidget->setSizePolicy(sizePolicy);
        TopRightWidget->setMinimumSize(QSize(0, 42));
        TopRightWidget->setMaximumSize(QSize(16777215, 42));
        horizontalLayout = new QHBoxLayout(TopRightWidget);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        spinner = new RotatingSpinner(TopRightWidget);
        spinner->setObjectName("spinner");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(spinner->sizePolicy().hasHeightForWidth());
        spinner->setSizePolicy(sizePolicy1);
        spinner->setMinimumSize(QSize(42, 0));
        spinner->setMaximumSize(QSize(42, 20));

        horizontalLayout->addWidget(spinner);

        labelWarning = new QLabel(TopRightWidget);
        labelWarning->setObjectName("labelWarning");
        sizePolicy1.setHeightForWidth(labelWarning->sizePolicy().hasHeightForWidth());
        labelWarning->setSizePolicy(sizePolicy1);
        labelWarning->setMinimumSize(QSize(42, 0));
        labelWarning->setMaximumSize(QSize(42, 16777215));

        horizontalLayout->addWidget(labelWarning);

        toolButtonDownload = new DownloadProgressButton(TopRightWidget);
        toolButtonDownload->setObjectName("toolButtonDownload");
        sizePolicy1.setHeightForWidth(toolButtonDownload->sizePolicy().hasHeightForWidth());
        toolButtonDownload->setSizePolicy(sizePolicy1);
        toolButtonDownload->setMinimumSize(QSize(42, 0));
        toolButtonDownload->setMaximumSize(QSize(42, 16777215));
        toolButtonDownload->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        toolButtonDownload->setText(QString::fromUtf8(""));
        toolButtonDownload->setIconSize(QSize(28, 28));
        toolButtonDownload->setPopupMode(QToolButton::InstantPopup);

        horizontalLayout->addWidget(toolButtonDownload);

        pushUser = new QPushButton(TopRightWidget);
        pushUser->setObjectName("pushUser");
        sizePolicy1.setHeightForWidth(pushUser->sizePolicy().hasHeightForWidth());
        pushUser->setSizePolicy(sizePolicy1);
        pushUser->setMinimumSize(QSize(42, 0));
        pushUser->setMaximumSize(QSize(42, 16777215));
        pushUser->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushUser->setIconSize(QSize(28, 28));
        pushUser->setFlat(true);

        horizontalLayout->addWidget(pushUser);

        toolButton = new QToolButton(TopRightWidget);
        toolButton->setObjectName("toolButton");
        sizePolicy1.setHeightForWidth(toolButton->sizePolicy().hasHeightForWidth());
        toolButton->setSizePolicy(sizePolicy1);
        toolButton->setMinimumSize(QSize(42, 0));
        toolButton->setMaximumSize(QSize(42, 16777215));
        toolButton->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        toolButton->setText(QString::fromUtf8(""));
        toolButton->setIconSize(QSize(28, 28));
        toolButton->setPopupMode(QToolButton::InstantPopup);

        horizontalLayout->addWidget(toolButton);

        pushMinimize = new QPushButton(TopRightWidget);
        pushMinimize->setObjectName("pushMinimize");
        sizePolicy1.setHeightForWidth(pushMinimize->sizePolicy().hasHeightForWidth());
        pushMinimize->setSizePolicy(sizePolicy1);
        pushMinimize->setMinimumSize(QSize(42, 0));
        pushMinimize->setMaximumSize(QSize(42, 16777215));
        pushMinimize->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushMinimize->setIconSize(QSize(28, 28));
        pushMinimize->setFlat(true);

        horizontalLayout->addWidget(pushMinimize);

        pushClose = new QPushButton(TopRightWidget);
        pushClose->setObjectName("pushClose");
        sizePolicy1.setHeightForWidth(pushClose->sizePolicy().hasHeightForWidth());
        pushClose->setSizePolicy(sizePolicy1);
        pushClose->setMinimumSize(QSize(42, 0));
        pushClose->setMaximumSize(QSize(42, 16777215));
        pushClose->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushClose->setIconSize(QSize(28, 28));
        pushClose->setFlat(true);

        horizontalLayout->addWidget(pushClose);


        retranslateUi(TopRightWidget);

        QMetaObject::connectSlotsByName(TopRightWidget);
    } // setupUi

    void retranslateUi(QWidget *TopRightWidget)
    {
        labelWarning->setText(QString());
#if QT_CONFIG(tooltip)
        toolButtonDownload->setToolTip(QCoreApplication::translate("TopRightWidget", "Download status", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushUser->setToolTip(QCoreApplication::translate("TopRightWidget", "User area", nullptr));
#endif // QT_CONFIG(tooltip)
        pushUser->setText(QString());
#if QT_CONFIG(tooltip)
        toolButton->setToolTip(QCoreApplication::translate("TopRightWidget", "Main menu", nullptr));
#endif // QT_CONFIG(tooltip)
        pushMinimize->setText(QString());
#if QT_CONFIG(tooltip)
        pushClose->setToolTip(QCoreApplication::translate("TopRightWidget", "Close", nullptr));
#endif // QT_CONFIG(tooltip)
        pushClose->setText(QString());
        (void)TopRightWidget;
    } // retranslateUi

};

namespace Ui {
    class TopRightWidget: public Ui_TopRightWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOPRIGHTWIDGET_H
