/********************************************************************************
** Form generated from reading UI file 'configsectioninterface.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSECTIONINTERFACE_H
#define UI_CONFIGSECTIONINTERFACE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigSectionInterface
{
public:
    QGridLayout *gridLayout_3;
    QLabel *label_6;
    QLabel *label_22;
    QLabel *labelSubTitle3;
    QComboBox *comboColorTheme;
    QComboBox *comboStyle;
    QLabel *labelBorders;
    QLabel *labelSubTitle2;
    QSpacerItem *horizontalSpacer;
    QLabel *label_4;
    QLabel *label_2;
    QLabel *label;
    QCheckBox *checkDecorations;
    QLabel *label_3;
    QComboBox *comboKeyName;
    QComboBox *comboLangue;
    QGridLayout *gridLayoutColors;
    QLabel *labelColorText;
    QPushButton *pushColorListBackground;
    QPushButton *pushColorWindowText;
    QLabel *labelColorWindow;
    QPushButton *pushColorWindowBackground;
    QPushButton *pushColorButtonText;
    QPushButton *pushColorButtonBackground;
    QLabel *labelColorButton;
    QLabel *labelColorBackground;
    QPushButton *pushColorSelectionText;
    QLabel *labelColorSelection;
    QPushButton *pushColorSelectionBackground;
    QPushButton *pushColorListAlternativeBackground;
    QLabel *labelColorList;
    QPushButton *pushColorListText;
    QLabel *labelRestart;
    QLabel *label_5;
    QCheckBox *checkBorders;
    QLabel *labelColors;
    QLabel *labelSubTitle1;
    QCheckBox *checkUniqueInstance;
    QComboBox *comboSortDivisions;
    QComboBox *comboStartupView;

    void setupUi(QWidget *ConfigSectionInterface)
    {
        if (ConfigSectionInterface->objectName().isEmpty())
            ConfigSectionInterface->setObjectName("ConfigSectionInterface");
        ConfigSectionInterface->resize(560, 472);
        ConfigSectionInterface->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout_3 = new QGridLayout(ConfigSectionInterface);
        gridLayout_3->setObjectName("gridLayout_3");
        label_6 = new QLabel(ConfigSectionInterface);
        label_6->setObjectName("label_6");

        gridLayout_3->addWidget(label_6, 9, 1, 1, 1);

        label_22 = new QLabel(ConfigSectionInterface);
        label_22->setObjectName("label_22");
        label_22->setMaximumSize(QSize(150, 16777215));

        gridLayout_3->addWidget(label_22, 2, 1, 1, 1);

        labelSubTitle3 = new QLabel(ConfigSectionInterface);
        labelSubTitle3->setObjectName("labelSubTitle3");
        QFont font;
        font.setBold(true);
        labelSubTitle3->setFont(font);

        gridLayout_3->addWidget(labelSubTitle3, 8, 0, 1, 3);

        comboColorTheme = new QComboBox(ConfigSectionInterface);
        comboColorTheme->setObjectName("comboColorTheme");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comboColorTheme->sizePolicy().hasHeightForWidth());
        comboColorTheme->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(comboColorTheme, 10, 2, 1, 1);

        comboStyle = new QComboBox(ConfigSectionInterface);
        comboStyle->addItem(QString::fromUtf8("Fusion"));
        comboStyle->setObjectName("comboStyle");

        gridLayout_3->addWidget(comboStyle, 9, 2, 1, 1);

        labelBorders = new QLabel(ConfigSectionInterface);
        labelBorders->setObjectName("labelBorders");

        gridLayout_3->addWidget(labelBorders, 12, 1, 1, 1);

        labelSubTitle2 = new QLabel(ConfigSectionInterface);
        labelSubTitle2->setObjectName("labelSubTitle2");
        labelSubTitle2->setFont(font);

        gridLayout_3->addWidget(labelSubTitle2, 3, 0, 1, 3);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer, 1, 0, 1, 1);

        label_4 = new QLabel(ConfigSectionInterface);
        label_4->setObjectName("label_4");

        gridLayout_3->addWidget(label_4, 4, 1, 1, 1);

        label_2 = new QLabel(ConfigSectionInterface);
        label_2->setObjectName("label_2");

        gridLayout_3->addWidget(label_2, 1, 1, 1, 1);

        label = new QLabel(ConfigSectionInterface);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 6, 1, 1, 1);

        checkDecorations = new QCheckBox(ConfigSectionInterface);
        checkDecorations->setObjectName("checkDecorations");

        gridLayout_3->addWidget(checkDecorations, 13, 2, 1, 1);

        label_3 = new QLabel(ConfigSectionInterface);
        label_3->setObjectName("label_3");

        gridLayout_3->addWidget(label_3, 13, 1, 1, 1);

        comboKeyName = new QComboBox(ConfigSectionInterface);
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->addItem(QString());
        comboKeyName->setObjectName("comboKeyName");
        sizePolicy.setHeightForWidth(comboKeyName->sizePolicy().hasHeightForWidth());
        comboKeyName->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(comboKeyName, 2, 2, 1, 1);

        comboLangue = new QComboBox(ConfigSectionInterface);
        comboLangue->setObjectName("comboLangue");

        gridLayout_3->addWidget(comboLangue, 1, 2, 1, 1);

        gridLayoutColors = new QGridLayout();
        gridLayoutColors->setObjectName("gridLayoutColors");
        labelColorText = new QLabel(ConfigSectionInterface);
        labelColorText->setObjectName("labelColorText");
        labelColorText->setFont(font);
        labelColorText->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        labelColorText->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayoutColors->addWidget(labelColorText, 1, 0, 1, 1);

        pushColorListBackground = new QPushButton(ConfigSectionInterface);
        pushColorListBackground->setObjectName("pushColorListBackground");
        pushColorListBackground->setMinimumSize(QSize(24, 24));
        QFont font1;
        font1.setBold(false);
        pushColorListBackground->setFont(font1);
        pushColorListBackground->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorListBackground->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorListBackground->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));

        gridLayoutColors->addWidget(pushColorListBackground, 2, 4, 1, 1);

        pushColorWindowText = new QPushButton(ConfigSectionInterface);
        pushColorWindowText->setObjectName("pushColorWindowText");
        pushColorWindowText->setMinimumSize(QSize(24, 24));
        pushColorWindowText->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorWindowText->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorWindowText->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorWindowText->setFlat(true);

        gridLayoutColors->addWidget(pushColorWindowText, 1, 1, 1, 1);

        labelColorWindow = new QLabel(ConfigSectionInterface);
        labelColorWindow->setObjectName("labelColorWindow");
        labelColorWindow->setFont(font);
        labelColorWindow->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayoutColors->addWidget(labelColorWindow, 0, 1, 1, 1);

        pushColorWindowBackground = new QPushButton(ConfigSectionInterface);
        pushColorWindowBackground->setObjectName("pushColorWindowBackground");
        pushColorWindowBackground->setMinimumSize(QSize(24, 24));
        pushColorWindowBackground->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorWindowBackground->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorWindowBackground->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorWindowBackground->setFlat(true);

        gridLayoutColors->addWidget(pushColorWindowBackground, 2, 1, 1, 1);

        pushColorButtonText = new QPushButton(ConfigSectionInterface);
        pushColorButtonText->setObjectName("pushColorButtonText");
        pushColorButtonText->setMinimumSize(QSize(24, 24));
        pushColorButtonText->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorButtonText->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorButtonText->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorButtonText->setFlat(true);

        gridLayoutColors->addWidget(pushColorButtonText, 1, 2, 1, 1);

        pushColorButtonBackground = new QPushButton(ConfigSectionInterface);
        pushColorButtonBackground->setObjectName("pushColorButtonBackground");
        pushColorButtonBackground->setMinimumSize(QSize(24, 24));
        pushColorButtonBackground->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorButtonBackground->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorButtonBackground->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorButtonBackground->setFlat(true);

        gridLayoutColors->addWidget(pushColorButtonBackground, 2, 2, 1, 1);

        labelColorButton = new QLabel(ConfigSectionInterface);
        labelColorButton->setObjectName("labelColorButton");
        labelColorButton->setFont(font);
        labelColorButton->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayoutColors->addWidget(labelColorButton, 0, 2, 1, 1);

        labelColorBackground = new QLabel(ConfigSectionInterface);
        labelColorBackground->setObjectName("labelColorBackground");
        labelColorBackground->setFont(font);
        labelColorBackground->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        labelColorBackground->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayoutColors->addWidget(labelColorBackground, 2, 0, 1, 1);

        pushColorSelectionText = new QPushButton(ConfigSectionInterface);
        pushColorSelectionText->setObjectName("pushColorSelectionText");
        pushColorSelectionText->setMinimumSize(QSize(24, 24));
        pushColorSelectionText->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorSelectionText->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorSelectionText->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorSelectionText->setFlat(true);

        gridLayoutColors->addWidget(pushColorSelectionText, 1, 3, 1, 1);

        labelColorSelection = new QLabel(ConfigSectionInterface);
        labelColorSelection->setObjectName("labelColorSelection");
        labelColorSelection->setFont(font);
        labelColorSelection->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayoutColors->addWidget(labelColorSelection, 0, 3, 1, 1);

        pushColorSelectionBackground = new QPushButton(ConfigSectionInterface);
        pushColorSelectionBackground->setObjectName("pushColorSelectionBackground");
        pushColorSelectionBackground->setMinimumSize(QSize(24, 24));
        pushColorSelectionBackground->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorSelectionBackground->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorSelectionBackground->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));
        pushColorSelectionBackground->setFlat(true);

        gridLayoutColors->addWidget(pushColorSelectionBackground, 2, 3, 1, 1);

        pushColorListAlternativeBackground = new QPushButton(ConfigSectionInterface);
        pushColorListAlternativeBackground->setObjectName("pushColorListAlternativeBackground");
        pushColorListAlternativeBackground->setMinimumSize(QSize(24, 24));
        pushColorListAlternativeBackground->setFont(font1);
        pushColorListAlternativeBackground->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorListAlternativeBackground->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorListAlternativeBackground->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));

        gridLayoutColors->addWidget(pushColorListAlternativeBackground, 2, 5, 1, 1);

        labelColorList = new QLabel(ConfigSectionInterface);
        labelColorList->setObjectName("labelColorList");
        labelColorList->setFont(font);
        labelColorList->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayoutColors->addWidget(labelColorList, 0, 4, 1, 2);

        pushColorListText = new QPushButton(ConfigSectionInterface);
        pushColorListText->setObjectName("pushColorListText");
        pushColorListText->setMinimumSize(QSize(24, 24));
        pushColorListText->setFont(font1);
        pushColorListText->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        pushColorListText->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        pushColorListText->setStyleSheet(QString::fromUtf8("QPushButton\n"
"{\n"
"	border: 1px solid #888;\n"
"}"));

        gridLayoutColors->addWidget(pushColorListText, 1, 4, 1, 2);

        gridLayoutColors->setColumnStretch(1, 2);
        gridLayoutColors->setColumnStretch(2, 2);
        gridLayoutColors->setColumnStretch(3, 2);
        gridLayoutColors->setColumnStretch(4, 1);
        gridLayoutColors->setColumnStretch(5, 1);

        gridLayout_3->addLayout(gridLayoutColors, 11, 2, 1, 1);

        labelRestart = new QLabel(ConfigSectionInterface);
        labelRestart->setObjectName("labelRestart");
        labelRestart->setFont(font);
        labelRestart->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_3->addWidget(labelRestart, 14, 0, 1, 3);

        label_5 = new QLabel(ConfigSectionInterface);
        label_5->setObjectName("label_5");

        gridLayout_3->addWidget(label_5, 5, 1, 1, 1);

        checkBorders = new QCheckBox(ConfigSectionInterface);
        checkBorders->setObjectName("checkBorders");

        gridLayout_3->addWidget(checkBorders, 12, 2, 1, 1);

        labelColors = new QLabel(ConfigSectionInterface);
        labelColors->setObjectName("labelColors");

        gridLayout_3->addWidget(labelColors, 10, 1, 1, 1);

        labelSubTitle1 = new QLabel(ConfigSectionInterface);
        labelSubTitle1->setObjectName("labelSubTitle1");
        labelSubTitle1->setFont(font);

        gridLayout_3->addWidget(labelSubTitle1, 0, 0, 1, 3);

        checkUniqueInstance = new QCheckBox(ConfigSectionInterface);
        checkUniqueInstance->setObjectName("checkUniqueInstance");

        gridLayout_3->addWidget(checkUniqueInstance, 4, 2, 1, 1);

        comboSortDivisions = new QComboBox(ConfigSectionInterface);
        comboSortDivisions->addItem(QString());
        comboSortDivisions->addItem(QString());
        comboSortDivisions->addItem(QString());
        comboSortDivisions->addItem(QString());
        comboSortDivisions->setObjectName("comboSortDivisions");
        sizePolicy.setHeightForWidth(comboSortDivisions->sizePolicy().hasHeightForWidth());
        comboSortDivisions->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(comboSortDivisions, 6, 2, 1, 1);

        comboStartupView = new QComboBox(ConfigSectionInterface);
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->addItem(QString());
        comboStartupView->setObjectName("comboStartupView");

        gridLayout_3->addWidget(comboStartupView, 5, 2, 1, 1);

        QWidget::setTabOrder(comboLangue, comboKeyName);
        QWidget::setTabOrder(comboKeyName, checkUniqueInstance);
        QWidget::setTabOrder(checkUniqueInstance, comboStartupView);
        QWidget::setTabOrder(comboStartupView, comboSortDivisions);
        QWidget::setTabOrder(comboSortDivisions, comboStyle);
        QWidget::setTabOrder(comboStyle, comboColorTheme);
        QWidget::setTabOrder(comboColorTheme, checkBorders);
        QWidget::setTabOrder(checkBorders, checkDecorations);

        retranslateUi(ConfigSectionInterface);

        comboLangue->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(ConfigSectionInterface);
    } // setupUi

    void retranslateUi(QWidget *ConfigSectionInterface)
    {
        label_6->setText(QCoreApplication::translate("ConfigSectionInterface", "Style", nullptr));
        label_22->setText(QCoreApplication::translate("ConfigSectionInterface", "Key names", nullptr));
        labelSubTitle3->setText(QCoreApplication::translate("ConfigSectionInterface", "Theme", nullptr));

        labelBorders->setText(QCoreApplication::translate("ConfigSectionInterface", "Window borders", nullptr));
        labelSubTitle2->setText(QCoreApplication::translate("ConfigSectionInterface", "Behaviour", nullptr));
        label_4->setText(QCoreApplication::translate("ConfigSectionInterface", "Unique instance", nullptr));
        label_2->setText(QCoreApplication::translate("ConfigSectionInterface", "Language", nullptr));
        label->setText(QCoreApplication::translate("ConfigSectionInterface", "Sort divisions", nullptr));
        checkDecorations->setText(QString());
        label_3->setText(QCoreApplication::translate("ConfigSectionInterface", "Decorations", nullptr));
        comboKeyName->setItemText(0, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 60, then 61", nullptr));
        comboKeyName->setItemText(1, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C3, then C\342\231\2573", nullptr));
        comboKeyName->setItemText(2, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C3, then D\342\231\2553", nullptr));
        comboKeyName->setItemText(3, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C4, then C\342\231\2574", nullptr));
        comboKeyName->setItemText(4, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C4, then D\342\231\2554", nullptr));
        comboKeyName->setItemText(5, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C5, then C\342\231\2575", nullptr));
        comboKeyName->setItemText(6, QCoreApplication::translate("ConfigSectionInterface", "Middle C \342\206\222 C5, then D\342\231\2555", nullptr));

        labelColorText->setText(QCoreApplication::translate("ConfigSectionInterface", "Text", nullptr));
        pushColorListBackground->setText(QString());
        pushColorWindowText->setText(QString());
        labelColorWindow->setText(QCoreApplication::translate("ConfigSectionInterface", "Window", nullptr));
        pushColorWindowBackground->setText(QString());
        pushColorButtonText->setText(QString());
        pushColorButtonBackground->setText(QString());
        labelColorButton->setText(QCoreApplication::translate("ConfigSectionInterface", "Button", nullptr));
        labelColorBackground->setText(QCoreApplication::translate("ConfigSectionInterface", "Background", nullptr));
        pushColorSelectionText->setText(QString());
        labelColorSelection->setText(QCoreApplication::translate("ConfigSectionInterface", "Selection", nullptr));
        pushColorSelectionBackground->setText(QString());
        pushColorListAlternativeBackground->setText(QString());
        labelColorList->setText(QCoreApplication::translate("ConfigSectionInterface", "List and table", nullptr));
        pushColorListText->setText(QString());
        labelRestart->setText(QCoreApplication::translate("ConfigSectionInterface", "This modification will be applied during the next start of the software.", nullptr));
        label_5->setText(QCoreApplication::translate("ConfigSectionInterface", "Startup view", nullptr));
        checkBorders->setText(QString());
        labelColors->setText(QCoreApplication::translate("ConfigSectionInterface", "Colors", nullptr));
        labelSubTitle1->setText(QCoreApplication::translate("ConfigSectionInterface", "Text", nullptr));
        checkUniqueInstance->setText(QString());
        comboSortDivisions->setItemText(0, QCoreApplication::translate("ConfigSectionInterface", "by key range", nullptr));
        comboSortDivisions->setItemText(1, QCoreApplication::translate("ConfigSectionInterface", "by velocity range", nullptr));
        comboSortDivisions->setItemText(2, QCoreApplication::translate("ConfigSectionInterface", "in alphabetical order", nullptr));
        comboSortDivisions->setItemText(3, QCoreApplication::translate("ConfigSectionInterface", "none", "speaking of the \"sort divisions\""));

        comboStartupView->setItemText(0, QCoreApplication::translate("ConfigSectionInterface", "general information", nullptr));
        comboStartupView->setItemText(1, QCoreApplication::translate("ConfigSectionInterface", "sample list", nullptr));
        comboStartupView->setItemText(2, QCoreApplication::translate("ConfigSectionInterface", "first sample", nullptr));
        comboStartupView->setItemText(3, QCoreApplication::translate("ConfigSectionInterface", "instrument list", nullptr));
        comboStartupView->setItemText(4, QCoreApplication::translate("ConfigSectionInterface", "first instrument", nullptr));
        comboStartupView->setItemText(5, QCoreApplication::translate("ConfigSectionInterface", "preset list", nullptr));
        comboStartupView->setItemText(6, QCoreApplication::translate("ConfigSectionInterface", "first preset", nullptr));

        (void)ConfigSectionInterface;
    } // retranslateUi

};

namespace Ui {
    class ConfigSectionInterface: public Ui_ConfigSectionInterface {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSECTIONINTERFACE_H
