/********************************************************************************
** Form generated from reading UI file 'configsectionkeyboard.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSECTIONKEYBOARD_H
#define UI_CONFIGSECTIONKEYBOARD_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ConfigSectionKeyboard
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *label;
    QComboBox *comboTemperament;
    QPushButton *pushDefaultReferencePitch;
    QLabel *label_3;
    QLabel *labelSubTitle2;
    QPushButton *pushDefaultTemperament;
    QComboBox *comboFirstC;
    QLabel *labelSubTitle1;
    QTableWidget *tableKeyboardMap;
    QLabel *label_2;
    QComboBox *comboTemperamentRelativeKey;
    QDoubleSpinBox *spinReferencePitch;

    void setupUi(QWidget *ConfigSectionKeyboard)
    {
        if (ConfigSectionKeyboard->objectName().isEmpty())
            ConfigSectionKeyboard->setObjectName("ConfigSectionKeyboard");
        ConfigSectionKeyboard->resize(465, 336);
        ConfigSectionKeyboard->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ConfigSectionKeyboard);
        gridLayout->setObjectName("gridLayout");
        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 3, 0, 1, 1);

        label = new QLabel(ConfigSectionKeyboard);
        label->setObjectName("label");

        gridLayout->addWidget(label, 6, 1, 1, 1);

        comboTemperament = new QComboBox(ConfigSectionKeyboard);
        comboTemperament->setObjectName("comboTemperament");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comboTemperament->sizePolicy().hasHeightForWidth());
        comboTemperament->setSizePolicy(sizePolicy);

        gridLayout->addWidget(comboTemperament, 7, 2, 1, 1);

        pushDefaultReferencePitch = new QPushButton(ConfigSectionKeyboard);
        pushDefaultReferencePitch->setObjectName("pushDefaultReferencePitch");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pushDefaultReferencePitch->sizePolicy().hasHeightForWidth());
        pushDefaultReferencePitch->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(pushDefaultReferencePitch, 6, 4, 1, 1);

        label_3 = new QLabel(ConfigSectionKeyboard);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 4, 1, 1, 1);

        labelSubTitle2 = new QLabel(ConfigSectionKeyboard);
        labelSubTitle2->setObjectName("labelSubTitle2");
        QFont font;
        font.setBold(true);
        labelSubTitle2->setFont(font);

        gridLayout->addWidget(labelSubTitle2, 2, 0, 1, 5);

        pushDefaultTemperament = new QPushButton(ConfigSectionKeyboard);
        pushDefaultTemperament->setObjectName("pushDefaultTemperament");
        sizePolicy1.setHeightForWidth(pushDefaultTemperament->sizePolicy().hasHeightForWidth());
        pushDefaultTemperament->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(pushDefaultTemperament, 7, 4, 1, 1);

        comboFirstC = new QComboBox(ConfigSectionKeyboard);
        comboFirstC->addItem(QString::fromUtf8("0"));
        comboFirstC->addItem(QString::fromUtf8("12"));
        comboFirstC->addItem(QString::fromUtf8("24"));
        comboFirstC->addItem(QString::fromUtf8("36"));
        comboFirstC->addItem(QString::fromUtf8("48"));
        comboFirstC->addItem(QString::fromUtf8("60"));
        comboFirstC->addItem(QString::fromUtf8("72"));
        comboFirstC->addItem(QString::fromUtf8("84"));
        comboFirstC->setObjectName("comboFirstC");

        gridLayout->addWidget(comboFirstC, 4, 2, 1, 3);

        labelSubTitle1 = new QLabel(ConfigSectionKeyboard);
        labelSubTitle1->setObjectName("labelSubTitle1");
        labelSubTitle1->setFont(font);

        gridLayout->addWidget(labelSubTitle1, 5, 0, 1, 5);

        tableKeyboardMap = new QTableWidget(ConfigSectionKeyboard);
        if (tableKeyboardMap->columnCount() < 13)
            tableKeyboardMap->setColumnCount(13);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        __qtablewidgetitem->setText(QString::fromUtf8("Do"));
        tableKeyboardMap->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        __qtablewidgetitem1->setText(QString::fromUtf8("Do#"));
        tableKeyboardMap->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        __qtablewidgetitem2->setText(QString::fromUtf8("Re"));
        tableKeyboardMap->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        __qtablewidgetitem3->setText(QString::fromUtf8("Re#"));
        tableKeyboardMap->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        __qtablewidgetitem4->setText(QString::fromUtf8("Mi"));
        tableKeyboardMap->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        __qtablewidgetitem5->setText(QString::fromUtf8("Fa"));
        tableKeyboardMap->setHorizontalHeaderItem(5, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        __qtablewidgetitem6->setText(QString::fromUtf8("Fa#"));
        tableKeyboardMap->setHorizontalHeaderItem(6, __qtablewidgetitem6);
        QTableWidgetItem *__qtablewidgetitem7 = new QTableWidgetItem();
        __qtablewidgetitem7->setText(QString::fromUtf8("Sol"));
        tableKeyboardMap->setHorizontalHeaderItem(7, __qtablewidgetitem7);
        QTableWidgetItem *__qtablewidgetitem8 = new QTableWidgetItem();
        __qtablewidgetitem8->setText(QString::fromUtf8("Sol#"));
        tableKeyboardMap->setHorizontalHeaderItem(8, __qtablewidgetitem8);
        QTableWidgetItem *__qtablewidgetitem9 = new QTableWidgetItem();
        __qtablewidgetitem9->setText(QString::fromUtf8("La"));
        tableKeyboardMap->setHorizontalHeaderItem(9, __qtablewidgetitem9);
        QTableWidgetItem *__qtablewidgetitem10 = new QTableWidgetItem();
        __qtablewidgetitem10->setText(QString::fromUtf8("La#"));
        tableKeyboardMap->setHorizontalHeaderItem(10, __qtablewidgetitem10);
        QTableWidgetItem *__qtablewidgetitem11 = new QTableWidgetItem();
        __qtablewidgetitem11->setText(QString::fromUtf8("Si"));
        tableKeyboardMap->setHorizontalHeaderItem(11, __qtablewidgetitem11);
        QTableWidgetItem *__qtablewidgetitem12 = new QTableWidgetItem();
        __qtablewidgetitem12->setText(QString::fromUtf8("Do"));
        tableKeyboardMap->setHorizontalHeaderItem(12, __qtablewidgetitem12);
        if (tableKeyboardMap->rowCount() < 4)
            tableKeyboardMap->setRowCount(4);
        QTableWidgetItem *__qtablewidgetitem13 = new QTableWidgetItem();
        tableKeyboardMap->setVerticalHeaderItem(0, __qtablewidgetitem13);
        QTableWidgetItem *__qtablewidgetitem14 = new QTableWidgetItem();
        tableKeyboardMap->setVerticalHeaderItem(1, __qtablewidgetitem14);
        QTableWidgetItem *__qtablewidgetitem15 = new QTableWidgetItem();
        tableKeyboardMap->setVerticalHeaderItem(2, __qtablewidgetitem15);
        QTableWidgetItem *__qtablewidgetitem16 = new QTableWidgetItem();
        tableKeyboardMap->setVerticalHeaderItem(3, __qtablewidgetitem16);
        tableKeyboardMap->setObjectName("tableKeyboardMap");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(tableKeyboardMap->sizePolicy().hasHeightForWidth());
        tableKeyboardMap->setSizePolicy(sizePolicy2);
        tableKeyboardMap->setMaximumSize(QSize(16777215, 140));
        tableKeyboardMap->setFrameShape(QFrame::Shape::StyledPanel);
        tableKeyboardMap->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOn);
        tableKeyboardMap->setProperty("showDropIndicator", QVariant(false));
        tableKeyboardMap->setDragDropOverwriteMode(false);
        tableKeyboardMap->setSelectionMode(QAbstractItemView::SelectionMode::NoSelection);
        tableKeyboardMap->setHorizontalScrollMode(QAbstractItemView::ScrollMode::ScrollPerPixel);
        tableKeyboardMap->setCornerButtonEnabled(false);
        tableKeyboardMap->horizontalHeader()->setHighlightSections(false);
        tableKeyboardMap->verticalHeader()->setDefaultSectionSize(25);
        tableKeyboardMap->verticalHeader()->setHighlightSections(false);
        tableKeyboardMap->verticalHeader()->setStretchLastSection(true);

        gridLayout->addWidget(tableKeyboardMap, 3, 1, 1, 4);

        label_2 = new QLabel(ConfigSectionKeyboard);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 7, 1, 1, 1);

        comboTemperamentRelativeKey = new QComboBox(ConfigSectionKeyboard);
        comboTemperamentRelativeKey->setObjectName("comboTemperamentRelativeKey");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(comboTemperamentRelativeKey->sizePolicy().hasHeightForWidth());
        comboTemperamentRelativeKey->setSizePolicy(sizePolicy3);

        gridLayout->addWidget(comboTemperamentRelativeKey, 7, 3, 1, 1);

        spinReferencePitch = new QDoubleSpinBox(ConfigSectionKeyboard);
        spinReferencePitch->setObjectName("spinReferencePitch");
        sizePolicy.setHeightForWidth(spinReferencePitch->sizePolicy().hasHeightForWidth());
        spinReferencePitch->setSizePolicy(sizePolicy);
        spinReferencePitch->setDecimals(1);
        spinReferencePitch->setMinimum(390.000000000000000);
        spinReferencePitch->setMaximum(470.000000000000000);
        spinReferencePitch->setSingleStep(0.100000000000000);
        spinReferencePitch->setValue(440.000000000000000);

        gridLayout->addWidget(spinReferencePitch, 6, 2, 1, 2);

        QWidget::setTabOrder(tableKeyboardMap, comboFirstC);
        QWidget::setTabOrder(comboFirstC, spinReferencePitch);
        QWidget::setTabOrder(spinReferencePitch, pushDefaultReferencePitch);
        QWidget::setTabOrder(pushDefaultReferencePitch, comboTemperament);
        QWidget::setTabOrder(comboTemperament, comboTemperamentRelativeKey);
        QWidget::setTabOrder(comboTemperamentRelativeKey, pushDefaultTemperament);

        retranslateUi(ConfigSectionKeyboard);

        comboFirstC->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ConfigSectionKeyboard);
    } // setupUi

    void retranslateUi(QWidget *ConfigSectionKeyboard)
    {
        label->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Reference pitch", nullptr));
#if QT_CONFIG(tooltip)
        pushDefaultReferencePitch->setToolTip(QCoreApplication::translate("ConfigSectionKeyboard", "Back to default value", nullptr));
#endif // QT_CONFIG(tooltip)
        pushDefaultReferencePitch->setText(QString());
        label_3->setText(QCoreApplication::translate("ConfigSectionKeyboard", "First C", nullptr));
        labelSubTitle2->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Mapping", nullptr));
#if QT_CONFIG(tooltip)
        pushDefaultTemperament->setToolTip(QCoreApplication::translate("ConfigSectionKeyboard", "Back to default value", nullptr));
#endif // QT_CONFIG(tooltip)
        pushDefaultTemperament->setText(QString());

        labelSubTitle1->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Tuning", nullptr));
        QTableWidgetItem *___qtablewidgetitem = tableKeyboardMap->verticalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Octave #1", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = tableKeyboardMap->verticalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Octave #2", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = tableKeyboardMap->verticalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Octave #3", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = tableKeyboardMap->verticalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Octave #4", nullptr));
        label_2->setText(QCoreApplication::translate("ConfigSectionKeyboard", "Temperament", nullptr));
        (void)ConfigSectionKeyboard;
    } // retranslateUi

};

namespace Ui {
    class ConfigSectionKeyboard: public Ui_ConfigSectionKeyboard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGSECTIONKEYBOARD_H
