/********************************************************************************
** Form generated from reading UI file 'toolglobalsettings_gui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TOOLGLOBALSETTINGS_GUI_H
#define UI_TOOLGLOBALSETTINGS_GUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "global_settings/graphparamglobal.h"

QT_BEGIN_NAMESPACE

class Ui_ToolGlobalSettings_gui
{
public:
    QGridLayout *gridLayout;
    QSpinBox *spinVelMin;
    QLabel *label_3;
    QComboBox *comboModif;
    QSpacerItem *verticalSpacer;
    QSpinBox *spinVelMax;
    QLabel *label_5;
    QDoubleSpinBox *doubleSpinMin;
    QLabel *label;
    QDoubleSpinBox *doubleSpinParam;
    QLabel *label_2;
    QLabel *label_4;
    QSpacerItem *verticalSpacer_2;
    QComboBox *comboPattern;
    QDoubleSpinBox *doubleSpinMax;
    QFrame *frame_1;
    QVBoxLayout *verticalLayout_2;
    GraphParamGlobal *graphParamGlobal;
    QComboBox *comboAttribute;
    QLabel *label_6;
    QLabel *label_7;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushOk;
    QPushButton *pushCancel;

    void setupUi(QWidget *ToolGlobalSettings_gui)
    {
        if (ToolGlobalSettings_gui->objectName().isEmpty())
            ToolGlobalSettings_gui->setObjectName("ToolGlobalSettings_gui");
        ToolGlobalSettings_gui->resize(581, 322);
        ToolGlobalSettings_gui->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(ToolGlobalSettings_gui);
        gridLayout->setObjectName("gridLayout");
        spinVelMin = new QSpinBox(ToolGlobalSettings_gui);
        spinVelMin->setObjectName("spinVelMin");
        spinVelMin->setMaximum(127);

        gridLayout->addWidget(spinVelMin, 8, 2, 1, 1);

        label_3 = new QLabel(ToolGlobalSettings_gui);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 1, 1, 1);

        comboModif = new QComboBox(ToolGlobalSettings_gui);
        comboModif->addItem(QString());
        comboModif->addItem(QString());
        comboModif->addItem(QString());
        comboModif->setObjectName("comboModif");

        gridLayout->addWidget(comboModif, 4, 2, 1, 2);

        verticalSpacer = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(verticalSpacer, 9, 1, 1, 1);

        spinVelMax = new QSpinBox(ToolGlobalSettings_gui);
        spinVelMax->setObjectName("spinVelMax");
        spinVelMax->setMaximum(127);

        gridLayout->addWidget(spinVelMax, 8, 3, 1, 1);

        label_5 = new QLabel(ToolGlobalSettings_gui);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 5, 1, 1, 1);

        doubleSpinMin = new QDoubleSpinBox(ToolGlobalSettings_gui);
        doubleSpinMin->setObjectName("doubleSpinMin");
        doubleSpinMin->setMinimum(-20000.000000000000000);
        doubleSpinMin->setMaximum(20000.000000000000000);
        doubleSpinMin->setValue(0.000000000000000);

        gridLayout->addWidget(doubleSpinMin, 2, 2, 1, 2);

        label = new QLabel(ToolGlobalSettings_gui);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 1, 1, 1);

        doubleSpinParam = new QDoubleSpinBox(ToolGlobalSettings_gui);
        doubleSpinParam->setObjectName("doubleSpinParam");
        doubleSpinParam->setEnabled(false);
        doubleSpinParam->setMinimum(0.000000000000000);
        doubleSpinParam->setMaximum(100.000000000000000);
        doubleSpinParam->setValue(50.000000000000000);

        gridLayout->addWidget(doubleSpinParam, 1, 2, 1, 2);

        label_2 = new QLabel(ToolGlobalSettings_gui);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 3, 1, 1, 1);

        label_4 = new QLabel(ToolGlobalSettings_gui);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 4, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        gridLayout->addItem(verticalSpacer_2, 6, 1, 1, 1);

        comboPattern = new QComboBox(ToolGlobalSettings_gui);
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->addItem(QString());
        comboPattern->setObjectName("comboPattern");

        gridLayout->addWidget(comboPattern, 0, 2, 1, 2);

        doubleSpinMax = new QDoubleSpinBox(ToolGlobalSettings_gui);
        doubleSpinMax->setObjectName("doubleSpinMax");
        doubleSpinMax->setMinimum(-20000.000000000000000);
        doubleSpinMax->setMaximum(20000.000000000000000);
        doubleSpinMax->setValue(1.000000000000000);

        gridLayout->addWidget(doubleSpinMax, 3, 2, 1, 2);

        frame_1 = new QFrame(ToolGlobalSettings_gui);
        frame_1->setObjectName("frame_1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frame_1->sizePolicy().hasHeightForWidth());
        frame_1->setSizePolicy(sizePolicy);
        frame_1->setFrameShape(QFrame::StyledPanel);
        frame_1->setFrameShadow(QFrame::Sunken);
        verticalLayout_2 = new QVBoxLayout(frame_1);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        graphParamGlobal = new GraphParamGlobal(frame_1);
        graphParamGlobal->setObjectName("graphParamGlobal");
        graphParamGlobal->setMinimumSize(QSize(350, 0));

        verticalLayout_2->addWidget(graphParamGlobal);


        gridLayout->addWidget(frame_1, 0, 0, 11, 1);

        comboAttribute = new QComboBox(ToolGlobalSettings_gui);
        comboAttribute->setObjectName("comboAttribute");

        gridLayout->addWidget(comboAttribute, 5, 2, 1, 2);

        label_6 = new QLabel(ToolGlobalSettings_gui);
        label_6->setObjectName("label_6");
        label_6->setText(QString::fromUtf8("-"));

        gridLayout->addWidget(label_6, 1, 1, 1, 1);

        label_7 = new QLabel(ToolGlobalSettings_gui);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 7, 1, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushOk = new QPushButton(ToolGlobalSettings_gui);
        pushOk->setObjectName("pushOk");

        horizontalLayout->addWidget(pushOk);

        pushCancel = new QPushButton(ToolGlobalSettings_gui);
        pushCancel->setObjectName("pushCancel");

        horizontalLayout->addWidget(pushCancel);


        gridLayout->addLayout(horizontalLayout, 10, 1, 1, 3);

        QWidget::setTabOrder(comboPattern, doubleSpinParam);
        QWidget::setTabOrder(doubleSpinParam, doubleSpinMin);
        QWidget::setTabOrder(doubleSpinMin, doubleSpinMax);
        QWidget::setTabOrder(doubleSpinMax, comboModif);
        QWidget::setTabOrder(comboModif, comboAttribute);
        QWidget::setTabOrder(comboAttribute, spinVelMin);
        QWidget::setTabOrder(spinVelMin, spinVelMax);
        QWidget::setTabOrder(spinVelMax, pushOk);
        QWidget::setTabOrder(pushOk, pushCancel);

        retranslateUi(ToolGlobalSettings_gui);

        comboPattern->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ToolGlobalSettings_gui);
    } // setupUi

    void retranslateUi(QWidget *ToolGlobalSettings_gui)
    {
        label_3->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Minimum", nullptr));
        comboModif->setItemText(0, QCoreApplication::translate("ToolGlobalSettings_gui", "Addition", nullptr));
        comboModif->setItemText(1, QCoreApplication::translate("ToolGlobalSettings_gui", "Multiplication", nullptr));
        comboModif->setItemText(2, QCoreApplication::translate("ToolGlobalSettings_gui", "Replacement", nullptr));

        label_5->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Parameter", nullptr));
        label->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Pattern", nullptr));
        label_2->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Maximum", nullptr));
        label_4->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Modification", nullptr));
        comboPattern->setItemText(0, QCoreApplication::translate("ToolGlobalSettings_gui", "Manual", nullptr));
        comboPattern->setItemText(1, QCoreApplication::translate("ToolGlobalSettings_gui", "Linear ascending", nullptr));
        comboPattern->setItemText(2, QCoreApplication::translate("ToolGlobalSettings_gui", "Linear descending", nullptr));
        comboPattern->setItemText(3, QCoreApplication::translate("ToolGlobalSettings_gui", "Exponential ascending", nullptr));
        comboPattern->setItemText(4, QCoreApplication::translate("ToolGlobalSettings_gui", "Exponential descending", nullptr));
        comboPattern->setItemText(5, QCoreApplication::translate("ToolGlobalSettings_gui", "Random", nullptr));

        label_7->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "Apply to a velocity range", nullptr));
        pushOk->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "&Ok", nullptr));
        pushCancel->setText(QCoreApplication::translate("ToolGlobalSettings_gui", "&Cancel", nullptr));
        (void)ToolGlobalSettings_gui;
    } // retranslateUi

};

namespace Ui {
    class ToolGlobalSettings_gui: public Ui_ToolGlobalSettings_gui {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TOOLGLOBALSETTINGS_GUI_H
